<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Menuitem;
use App\Models\Addmenuitem;
use App\Models\Mealtype;
use Carbon\Carbon;
use Carbon\CarbonPeriod;
use App\Models\MenuImages;
use App\Models\ShareMenu;
use AWS\CRT\HTTP\Request as HTTPRequest;
use Exception;
use App\Models\Permissions;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\File;
use App\Models\MenuTimer;
use DB;
use Illuminate\Support\Facades\Validator;
use App\Models\MessageManager;



class AddmenuitemController extends Controller
{


    public function resizeBase64Image($base64Data, $targetSizeInKB)
    {
        // Decode base64 data
        $img = explode(',', $base64Data);
        // dd($img[1]);

        $imageData = base64_decode($img[1]);


        // Create an image resource from the decoded data
        $img = imagecreatefromstring($imageData);

        // Get the original image dimensions
        $width = imagesx($img);
        $height = imagesy($img);

        // Set the desired width and calculate the corresponding height to maintain the aspect ratio
        $targetWidth = 200; // Adjust this value based on your requirements
        $targetHeight = ($targetWidth / $width) * $height;

        // Create a new image resource for the resized image
        $resizedImg = imagecreatetruecolor($targetWidth, $targetHeight);

        // Resize the image
        imagecopyresampled($resizedImg, $img, 0, 0, 0, 0, $targetWidth, $targetHeight, $width, $height);

        // Create a buffer for the resized image
        ob_start();

        // Output the resized image to the buffer
        imagejpeg($resizedImg, null, 90); // Adjust the quality as needed

        // Get the buffer content
        $resizedImageData = ob_get_clean();

        // Check if the resized image meets the target size
        while (strlen($resizedImageData) > ($targetSizeInKB * 1024)) {
            // If the size is still larger than the target, reduce the quality and update the buffer
            ob_start();
            imagejpeg($resizedImg, null, 90);
            $resizedImageData = ob_get_clean();
        }

        // Encode the resized image to base64
        $resizedBase64Data = base64_encode($resizedImageData);

        // Destroy the image resources
        imagedestroy($img);
        imagedestroy($resizedImg);

        return $resizedBase64Data;
    }


    private function checkWhatsAppPermission(): bool
    {
        try {
            $permission = Permissions::where('permissionType', 'Desktop Whatsapp')->value('permission');
            return $permission ?? false; // Return the permission value or false if not found

        } catch (ModelNotFoundException $e) {
            return false; // Handle the case when the permission is not found

        } catch (Exception $e) {
            // Log or handle other exceptions
            return false;
        }
    }
    protected function getMenuType($id)
    {
        $category = Mealtype::where('id', $id)->first();
        return $category->category_name;
    }
    protected function getMenuItems($items)
    {
        $itemNames = [];
        foreach ($items as $item) {
            $menu = Menuitem::where('id', $item)->first();
            if ($menu) {
                $itemNames[] = $menu->menuitemname;
            }
        }
        return implode(",", $itemNames);
    }
    public function menuShare(Request $request)
    {
        $id = $request['menuId'];
        $menu = [];
        $data = ShareMenu::find($id);
        $menuMessage = MessageManager::where('msg_type', 'food_menu')
            ->inRandomOrder()
            ->pluck('msg')
            ->where('status',1)
            ->first();
        if ($data) {
            $menuData = json_decode($data->menu_data, true);
            foreach ($menuData as $mealType => $items) {
                $menu[] = [
                    'mealType' => $mealType,
                    "Item" => implode(',', $items)
                ];
            }
            $sharedata = [
                "menudate" => date("d-M-Y", strtotime($data->date)),
                "menu" => $menu,
                'whatsAppPermission' => $this->checkWhatsAppPermission(),
                'message' => $menuMessage,
            ];
        }
        return response()->json(['data' => $sharedata], 200);
    }
    public function getMeals(Request $request)
    {
        $category = Menuitem::select('id', 'menuitemname')
            ->where('catid', $request->input('id'))
            ->get();

        return response()->json($category);
    }

    public static function getCategoryName($catid)
    {
        $category = Mealtype::where('id', $catid)->first();

        if ($category) {
            return $category->category_name;
        }

        return null; // or you can return a default value if category not found
    }
    public static function getMenuName($catid)
    {
        $menu = Menuitem::where('id', $catid)->first();

        if ($menu) {
            return $menu->menuitemname;
        }

        return null; // or you can return a default value if category not found
    }

    public function store(Request $request)
    {
        if (count($request->menucat) > 1) {
            for ($i = 0; $i < count($request->menucat); $i++) {
                $data = new Menuitem(); // Create a new instance inside the loop
                $data->catid = $request->menucat[$i];
                $data->menuitemcode = $request->menuitemcode;
                $data->menuitemname = $request->menuitemname;
                $data->save();
            }
        } else {
            $data = new Menuitem(); // Create a new instance inside the loop
            $data->catid = implode(',', $request->menucat);
            $data->menuitemcode = $request->menuitemcode;
            $data->menuitemname = $request->menuitemname;
            $data->save();
        }
        return redirect('addmenu')->with('success', 'Menu Item added Successfully');
    }

    public function update(Request $request)
    {
        $menuadd_id = $request->input('menuadd_id');
        // return $request->input();
        $data = Menuitem::find($menuadd_id);
        if ($data) {

            $data->menuitemcode = $request->menuitemcode;
            $data->menuitemname = $request->menuitemname;

            $data->update();
            return back()->with('warning', 'Menu Item Updated successful!');
        } else {
            return back()->with('error', 'Data not found.');
        }
    }
    public function destroy($id)
    {
        $data = Menuitem::find($id);
        $data->delete();
        return back()->with('danger', 'Menu Item Delete Successfully');
    }
    public function addmenuitemplan(Request $request)
    {
        $data = new Addmenuitem();
        $data->menuitemdate = $request->menuitemdate;
        $data->menuitemday = $request->menuitemday;
        $data->menuitemtime = $request->menuitemtime;
        $data->menuitemname = $request->menuitemname;
        //$data->menuitemname = implode(',', $request->input('menuitemname', []));
        if ($request->hasfile('menuitemimage')) {
            $file = $request->file('menuitemimage');
            $extension = $file->getClientOriginalExtension();
            $filename = time() . '.' . rand() . '.' . $extension;
            $file->move('assets/uploads/', $filename);
            $data->menuitemimage = $filename;
        }
        $data->save();
        return back()->with('additem', 'added successfully');
    }
    public function addmenuitemplandelete($id)
    {
        $data = ShareMenu::find($id);
        $data->delete();
        return response()->json(['success' => 'Menu Item Delete Successfully'], 200);
    }
    public function editmenuitem(Request $request)
    {
        $menurepeatdate = $request->repeat_menuitemdate;
        $menuid = $request->repeatid;
        $menuday = $request->repeat_menuitemday;
        $existingMeal = Addmenuitem::find($menuid);

        if (!$existingMeal) {
            return response()->json(['message' => 'Meal not found'], 404);
        }

        $existingMeal->menuitemdate = $menurepeatdate;
        $existingMeal->menuitemday = $menuday;
        $existingMeal->id = null; // Set ID to null to insert a new record
        // Insert the updated record as a new entry
        $newMeal = $existingMeal->replicate();
        $newMeal->save();
        return back()->with('additem', 'Repeated Successfully');
    }


    public function getDataForSpecificWeek($dateString, $weekNumber)
    {
        $date = Carbon::parse($dateString);

        // If it's the first week, adjust the start date to include the previous month's days
        if ($weekNumber == 1) {
            $weekStartDate = $date->copy()->startOfMonth()->startOfWeek();
        } else {
            $weekStartDate = $date->copy()->startOfMonth()->addWeeks($weekNumber - 1)->startOfWeek();
        }

        // Calculate the end date of the specified week
        $weekEndDate = $weekStartDate->copy()->addWeek()->subDay();

        // Ensure the end date does not exceed the month
        if ($weekEndDate->month !== $date->month) {
            $weekEndDate = $date->copy()->endOfMonth();
        }
        // Format the dates
        $weekStartDateFormated = $weekStartDate->format('Y-m-d');
        $weekEndDateFormated = $weekEndDate->format('Y-m-d');

        // Query the database for records within the specified week
        $records = ShareMenu::whereMonth('date', $date->month)
            ->whereBetween('date', [$weekStartDateFormated, $weekEndDateFormated])
            ->orderBy('date', 'ASC')
            ->get();

        return $records;
    }

    public function getExactWeekNumberOfMonth($date = null)
    {
        $date = $date ? Carbon::parse($date) : Carbon::now();
        $firstOfMonth = $date->copy()->startOfMonth();
        $firstDayOfWeek = Carbon::now()->startOfWeek()->dayOfWeek; // Get the index of the first day of the week (e.g., 0 for Sunday or 1 for Monday)

        // Calculate the number of days to add to the first of the month to reach the first day of the week
        $daysToAdd = ($firstOfMonth->dayOfWeek + 7 - $firstDayOfWeek) % 7;

        // Get the date of the first day of the first week of the month
        $firstDayOfFirstWeek = $firstOfMonth->copy()->addDays(-$daysToAdd);

        // Calculate the exact week number by comparing the given date with the first day of the first week
        $weekNumber = $date->diffInWeeks($firstDayOfFirstWeek) + 1;

        return $weekNumber;
    }


    public function MenuView(Request $request)
    {
        $selected_month = $request->selected_month ?? Carbon::now()->format('Y-m');
        $selected_week = $request->selected_week ?? $this->getExactWeekNumberOfMonth();
        // Assuming you have all Mealtype instances
        $mealtypes = Mealtype::all();
        // dd($mealtypes);
        // Initialize an empty array to store the result
        $MenuData = [];
        $weekData = [];

        // Loop through each Mealtype
        foreach ($mealtypes as $mealtype) {
            // Access its related Menuitems
            $categoryName = $mealtype->category_name;
            $menuitems = $mealtype->menuitems;
            $items = [];
            foreach ($menuitems as $item) {
                $items[] = $item->menuitemname;
            }
            $MenuData[$mealtype->category_name] = $items;
        }

        $AllMenuImages = MenuImages::orderBy('id', "DESC")->get();

        if ($selected_month != null && $selected_week != null) {
            $weeklyData = $this->getDataForSpecificWeek($selected_month, $selected_week);

            $date = Carbon::parse($selected_month);

            // If it's the first week, adjust the start date to include the previous month's days
            if ($selected_week == 1) {
                $weekStartDate = $date->copy()->startOfMonth()->startOfWeek();
            } else {
                $weekStartDate = $date->copy()->startOfMonth()->addWeeks($selected_week - 1)->startOfWeek();
            }

            // Calculate the end date of the specified week
            $weekEndDate = $weekStartDate->copy()->addWeek()->subDay();

            $currentDate = $weekStartDate->copy();
            while ($currentDate->lte($weekEndDate)) {
                // Check if there's a record for the current date
                // Check if the current date is within the same month as $dateString
                if ($currentDate->month === $date->month) {
                    $record = $weeklyData->firstWhere('date', $currentDate->format('Y-m-d'));

                    if ($record) {
                        // If there's a record, add it to the week data
                        $weekData[$currentDate->format('Y-m-d')] = $record;
                    } else {
                        // If not, add dummy data for this date
                        $weekData[$currentDate->format('Y-m-d')] = [];
                    }
                }
                // Move to the next day
                $currentDate->addDay();
            }
            $date = Carbon::parse($selected_month);
            $firstDayOfMonth = $date->copy()->startOfMonth();
            $lastDayOfMonth = $date->copy()->endOfMonth();
            $firstWeekNumber = $firstDayOfMonth->weekOfMonth;
            $lastWeekNumber = $lastDayOfMonth->weekOfMonth;
            $totalWeeks = $lastWeekNumber - $firstWeekNumber + 1;
        }
        // Parse the selected month into a Carbon instance
        $selectedDate = Carbon::now()->startOfMonth(); // Use the start of the current month
        $currentMonth = $selectedDate->format('Y-m'); // Current month (e.g., "2024-03")

        $previousMonth = $selectedDate->copy()->subMonth()->format('Y-m'); // Previous month (e.g., "2024-02")
        $nextMonth = $selectedDate->copy()->addMonth()->format('Y-m'); // Next month (e.g., "2024-04")

        // dd($previousMonth, $currentMonth, $nextMonth);

        // Create an associative array with month names as keys and 'Y-m' formatted dates as values
        $totalMonthsFilter = [
            Carbon::parse($previousMonth)->englishMonth . " (Previous)" => $previousMonth,
            Carbon::parse($currentMonth)->englishMonth . " (Current)" => $currentMonth,
            Carbon::parse($nextMonth)->englishMonth . " (Next)" => $nextMonth,
        ];
        return view('fronted.addmenu', compact('mealtypes', "MenuData", "AllMenuImages", 'weekData', 'totalWeeks', 'totalMonthsFilter', 'selected_month', 'selected_week'));
    }

    public function menuUplodeImges(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'menuImages' => 'required|image|mimes:jpeg,png,jpg|max:1024',
                'mealTypeCode' => 'required|string',
                'mealTypeName' => 'required',
            ]);
            if ($validator->fails()) {
                return back()->with('error', $validator->errors()->first());
            }
            $data = new MenuImages();

            $data->mealTypeCode = $request->mealTypeCode;
            $data->mealTypeName = $request->mealTypeName;

            if ($request->hasFile('menuImages')) {
                $file = $request->file('menuImages');
                $extension = $file->getClientOriginalExtension();
                $filename = str_replace(' ', '_', $request->mealTypeCode) . '_' . Carbon::now()->format("YmdHis") . '_' . uniqid() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->croppedImageData, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $data->menuImages = $filename;
            }
            $data->save();
            return back()->with('menuImagesSave', 'Image saved Successfully');
        } catch (Exception $e) {
            return back()->with('menuImagesError', "Somthing went wronge");
        }
    }

    public function menustore(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'menudate' => 'required|date',
            'Breakfast' => 'required|array|min:1',
            'Lunch' => 'required|array|min:1',
            'Dinner' => 'required|array|min:1',
            'Brunch' => 'required|array|min:1',
        ], [
            'Breakfast.min' => 'Please provide at least one item for Breakfast',
            'Lunch.min' => 'Please provide at least one item for Lunch',
            'Dinner.min' => 'Please provide at least one item for Dinner',
            'Brunch.min' => 'Please provide at least one item for Brunch',
        ]);
        if ($validator->fails()) {
            $error =  $validator->errors()->first();
            return back()->with('error', $error);
        }
        $selectedBreakfast = $request->input('Breakfast', []);
        $selectedLunch = $request->input('Lunch', []);
        $selectedBrunch = $request->input('Brunch', []);
        $selectedDinner = $request->input('Dinner', []);
        $selectedImages = $request->input('img', []);
        $menuDate = $request->input('menudate');

        $menu = [
            'Breakfast' => $selectedBreakfast,
            'Lunch' => $selectedLunch,
            'Brunch' => $selectedBrunch,
            'Dinner' => $selectedDinner,
        ];
        $jsonMenudata = json_encode($menu);
        //check if the menu is created or not
        $checkMenu = ShareMenu::where('date', $menuDate)->first();
        if (!empty($checkMenu)) {
            $newMenu = $checkMenu;
        } else {
            $newMenu = new ShareMenu();
        }
        $newMenu->menu_data = $jsonMenudata;
        $newMenu->date = $menuDate;
        $newMenu->auto_update = 0;
        if ($newMenu->save()) {
            return back()->with("menuImagesSave", "Menu Created Successfully");
        } else {
            return back()->with("menuImagesError", "Menu not Created");
        }
    }

    public function deleteMenuImages($id)
    {
        try {
            $imgData =  MenuImages::findOrFail($id);
            $imgData->delete();
            return response()->json(['success' => "Menu Images deleted"], 200);
        } catch (Exception) {
            return response()->json(['error' => "Images Not Deleted"], 500);
        }
    }

    public function getItmeCode(Request $request)
    {
        try {
            $categoryName = $request->categoryName;
            $itemCodes = Mealtype::where('category_name', $categoryName)->with('menuitems')->get();
            return response()->json([
                'data' => $itemCodes
            ]);
        } catch (Exception $e) {
            return response()->json([
                'message' => $e->getMessage(),
            ], 500);
        }
    }


    public function storeOrUpdateMenuTimer(Request $request)
    {
        $autoUpdateTime = $request->input('autoUpdateTime');
        $mealTypeName = $request->input('mealTypeName');
        $daysData = $request->input('data');

        DB::beginTransaction();

        try {
            // Delete existing menu timers for the meal type
            MenuTimer::where('meal_type', $mealTypeName)->delete();

            foreach ($daysData as $day => $times) {
                // Assuming 'from' and 'to' are arrays with the same length
                foreach ($times['from'] as $index => $fromTime) {
                    $toTime = $times['to'][$index];

                    // Create a new MenuTimer instance
                    MenuTimer::create([
                        'menu_day' => $day,
                        'meal_type' => $mealTypeName,
                        'auto_update_time' => $autoUpdateTime,
                        'start_time' => $fromTime,
                        'end_time' => $toTime,
                    ]);
                }
            }

            DB::commit();

            return back()->with('success', 'Menu Timer updated successfully.');
        } catch (\Exception $e) {
            DB::rollback();
            return back()->with('error', 'Error occurred while updating menu timer.');
        }
    }


    public function getMenuTimerData(Request $request)
    {
        $mealType = $request->input('mealType');

        // Fetch the data associated with the selected meal type
        $mealTypeData = MenuTimer::where('meal_type', $mealType)->get();
        $data = [];
        // Assuming your meal type data structure is like this:
        if (!empty($mealTypeData)) {
            foreach ($mealTypeData as $item) {
                $data[] = [
                    $item->menu_day => [
                        'auto_update_time' => $item->auto_update_time,
                        'from' => $item->start_time,
                        'to' => $item->end_time,
                    ],
                ];
            }
        }
        return response()->json(['data' => $data]);
    }

    private function getTargetDate($weekType, $dayName)
    {

        // Set the start of the week to Monday
        Carbon::setWeekStartsAt(Carbon::MONDAY);
        Carbon::setWeekEndsAt(Carbon::SUNDAY);

        // Capitalize the first letter of the day name to match Carbon's format
        $dayName = ucfirst($dayName);

        // Get today's date
        $today = Carbon::now(); // Today is Sunday, July 28, 2024

        // Determine the target date based on weekType and dayName
        switch ($weekType) {
            case 'current':
                $targetDate = $today->copy()->startOfWeek()->modify("next $dayName")->format('Y-m-d');
                break;
            case 'prev':
                $targetDate = $today->copy()->subWeek()->startOfWeek()->modify("next $dayName")->format('Y-m-d');
                break;
            case 'next':
                $targetDate = $today->copy()->addWeek()->startOfWeek()->modify("next $dayName")->format('Y-m-d');
                break;
            default:
                throw new Exception('Invalid weekType');
        }

        // Return the target date
        return $targetDate;
    }
   public function getAllMenusForTenant(Request $request)
    {
        try {

            $dontWasteFoodMessages = [
                "🌾 Don't let food go to waste! Every bite is a celebration of resources. Finish your plate with gratitude. 🙌",
                "🍴 Let's be mindful! Every meal is an opportunity to appreciate the efforts that go into our food. Don't waste, savor! 🍽️",
                "🌱 Reduce food waste, nourish the planet! Your plate can make a difference. Eat responsibly, and let's cherish every bite. 🌏",
                "🌍 Food is a gift! Respect it by finishing what's on your plate. Remember, every meal counts toward a sustainable future. 🍲",
                "🥦 Plate, not waste! Conserve food like you conserve the planet. Finish strong and savor every morsel. 🌱",
                "🍲 Every meal is a chance to make a positive impact. Be mindful, finish your plate, and let's together say 'no' to food waste. 🌍",
                "👨‍🌾 From the farm to your plate, every step matters. Contribute to sustainability by appreciating and not wasting your food. 🚜",
                "🍽️ Your plate tells a story of resources and hard work. Honor it by finishing your meal and saying 'no' to food waste. 📜",
                "🔄 Let's close the loop on food waste! Finish what you started, and let's make each meal a small victory for the environment. 🔄",
                "🧑‍🌾 Every farmer's dream is a plate well-enjoyed. Don't let food be taken for granted; savor, appreciate, and finish strong! 🌾"
            ];
            // Get current date and day in English
            $todayDate = Carbon::now()->format('Y-m-d');
            $currentDay = Carbon::now()->englishDayOfWeek;

            // Determine the target date based on weekType and dayName
            $dayName = ucfirst($request->dayName); // Capitalize the first letter to match Carbon's day format
            $targetDate = $this->getTargetDate($request->weekType, $dayName);
            // Find the menu for the target date
            $menu = ShareMenu::where('date', $targetDate)->first();

            $toDayMenu = [];

            // Check if menu exists for the target date
            if ($menu) {
                // Decode menu data
                $menuItems = json_decode($menu->menu_data, true); // Convert to associative array
                if (!$menuItems) {
                    return response()->json([
                        'status' => false,
                        'statusCode' => 400,
                        'message' => "Menu items not found for the day $dayName and date $targetDate",
                    ], 400);
                }
                // Loop through menu items
                foreach ($menuItems as $mealType => $items) {
                    $randomMessageIndex = array_rand($dontWasteFoodMessages); // Select a random index
                    $randomMessage = $dontWasteFoodMessages[$randomMessageIndex];
                    // Find menu timers for the current meal type and target day
                    $menuTimes = MenuTimer::where('meal_type', $mealType)
                        ->where('menu_day', strtolower($dayName))
                        ->first();
                    $menuImage = MenuImages::getRandomImagePathByMealTypeName($mealType);
                    // Check if menu times exist
                    if ($menuTimes) {
                        // Add start and end times to menu item object
                        $startTime = $menuTimes->start_time;
                        $endTime = $menuTimes->end_time;
                        $toDayMenu[$mealType] = [
                            'startTime' => $startTime,
                            'endTime' => $endTime,
                            'menuImage' => $menuImage,
                            'menuItems' => $items,
                            'menuMessage' => $randomMessage
                        ];
                    } else {
                        $toDayMenu[$mealType] = [
                            'startTime' => null,
                            'endTime' => null,
                            'menuImage' => $menuImage,
                            'menuItems' => $items,
                            'menuMessage' => $randomMessage
                        ];
                    }
                }
            } else {
                return response()->json([
                    'status' => false,
                    'statusCode' => 404 ,
                    'data' => [],
                    'message' => 'Menu not found for the specified day',
                ], 404);
            }

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => $toDayMenu,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }


    public function getAllMonthMenuData()
    {
        try {
            $currentYear = Carbon::now()->year;

            $uniqueMonthDates = ShareMenu::selectRaw('DATE_FORMAT(date, "%Y-%m") as month_date, DATE_FORMAT(date, "%M") as month_name')
                ->whereYear('date', $currentYear) // Filter by current year
                ->distinct()
                ->orderBy('month_date') // Order by month_date in ascending order
                ->pluck('month_date', 'month_name');

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => $uniqueMonthDates
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ]);
        }
    }

    public function getWeekNumbeOfMonth(Request $request)
    {
        try {
            $selected_month = $request->selectedDate;
            $date = Carbon::parse($selected_month);
            $firstDayOfMonth = $date->copy()->startOfMonth();
            $lastDayOfMonth = $date->copy()->endOfMonth();
            $firstWeekNumber = $firstDayOfMonth->weekOfMonth;
            $lastWeekNumber = $lastDayOfMonth->weekOfMonth;
            $totalWeeks = $lastWeekNumber - $firstWeekNumber + 1;
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => $totalWeeks,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }

    public function repeatMenu(Request $request)
    {
        // Validate the incoming request
        $validator = Validator::make($request->all(), [
            'copyWeekMonth' => 'required|date',
            'selectWeekNumber' => 'required|string',
            'selectedMonth' => 'required|date',
            'addOnWeek' => 'required|string',
        ]);

        // Conditional validation for when 'selectWeekNumber' is 'all'
        $validator->sometimes('addOnWeek', 'in:all', function ($input) {
            return $input->selectWeekNumber === 'all';
        });

        // Redirect back with errors if validation fails
        if ($validator->fails()) {
            return redirect()->back()->withErrors($validator)->withInput();
        }

        if ($request->addOnWeek != "all" && $request->selectWeekNumber != "all") {
            // Retrieve the menu data for the specified week of the source month
            $repeatWeekData = $this->getDataForSpecificWeek($request->copyWeekMonth, $request->selectWeekNumber);

            // Initialize arrays to store menu data by day and track missing data
            $repeatWeekDataByDay = [];
            $notExistDataInWeek = [];

            // Parse the target month and calculate the start and end dates of the target week
            $newMonthYear = Carbon::parse($request->selectedMonth);
            $addDataToWeekNumber = (int) $request->addOnWeek;
            $weekStartDate = $newMonthYear->copy()->startOfMonth()->addWeeks($addDataToWeekNumber - 1)->startOfWeek(Carbon::MONDAY);
            $weekEndDate = $weekStartDate->copy()->endOfWeek(Carbon::SUNDAY);

            // Adjust the end date if it's in the next month
            if ($weekEndDate->month !== $newMonthYear->month) {
                $weekEndDate = $newMonthYear->copy()->endOfMonth();
            }

            // Organize the source week's menu data by day of the week
            foreach ($repeatWeekData as $week) {
                $weekDate = Carbon::parse($week->date);
                $dayOfTheWeek = $weekDate->englishDayOfWeek;
                $repeatWeekDataByDay[$dayOfTheWeek] = $week->menu_data;
            }

            // Loop through each day of the target week
            $currentDate = $weekStartDate->copy();
            while ($currentDate->lte($weekEndDate)) {
                $currentWeekDay = $currentDate->englishDayOfWeek;

                // If there's menu data for the current day, create a new menu entry
                if (array_key_exists($currentWeekDay, $repeatWeekDataByDay)) {
                    $existingMenu = ShareMenu::whereDate('date', $currentDate->format('Y-m-d'))->first();
                    if ($existingMenu) {
                        $existingMenu->menu_data = $repeatWeekDataByDay[$currentWeekDay];
                        $existingMenu->save();
                    } else {
                        // Create a new menu entry
                        $newMenu = new ShareMenu();
                        $newMenu->menu_data = $repeatWeekDataByDay[$currentWeekDay];
                        $newMenu->date = $currentDate->format('Y-m-d');
                        $newMenu->save();
                    }
                } else {
                    // If not, add the day to the list of missing data
                    $notExistDataInWeek[] = $currentWeekDay;
                }

                // Move to the next day
                $currentDate->addDay();
            }

            // Return back with a success message, including any missing days
            if (count($notExistDataInWeek) > 0) {
                $missingDay = implode(', ', $notExistDataInWeek);
                return back()->with('success', "Menu Repeated successfully but those Day(s) {$missingDay} not found in selected week, Please Add these days menu manually");
            }
        } else {
            // Parse the target month and calculate the number of weeks
            $newMonthYear = Carbon::parse($request->selectedMonth);

            $firstDayOfMonth = $newMonthYear->copy()->startOfMonth();
            $firstDayOfMonthDayOfWeek = $firstDayOfMonth->dayOfWeek; // Monday = 1, Sunday = 7

            $copyMonthYear = Carbon::parse($request->copyWeekMonth);
            // Get the first day of the source month and its numeric day of the week
            $copyMonthStartDayOfWeek = $copyMonthYear->copy()->startOfMonth()->dayOfWeek; // Monday = 1, Sunday = 7

            // Check if the start day of the target month is a Monday and the source month starts later in the week
            if ($firstDayOfMonthDayOfWeek === Carbon::MONDAY && $firstDayOfMonthDayOfWeek < $copyMonthStartDayOfWeek) {
                // If the condition is met, copy data from the second week of the source month
                $weekToCopy = 2;
                $repeatWeekData = $this->getDataForSpecificWeek($request->copyWeekMonth, $weekToCopy);
                // Organize the source week's menu data by day of the week
                foreach ($repeatWeekData as $week) {
                    $weekDate = Carbon::parse($week->date);
                    $dayOfTheWeek = $weekDate->englishDayOfWeek;
                    $repeatWeekDataByDay[$dayOfTheWeek] = $week->menu_data;
                }
                $this->repeatMenuForWeek($newMonthYear, 1, $repeatWeekDataByDay);
            } else {
                // If the condition is not met, start copying from the first week
                $weekToCopy = 1;
            }
            $repeatWeekDataByDay = [];
            $firstDayOfMonth = $newMonthYear->copy()->startOfMonth();
            $lastDayOfMonth = $newMonthYear->copy()->endOfMonth();

            // Get the week number for the first and last day of the month
            $firstWeekOfMonth = $firstDayOfMonth->weekOfYear;
            $lastWeekOfMonth = $lastDayOfMonth->weekOfYear;
            // Calculate the number of weeks in the month
            // Note: This calculation assumes that the week starts on Monday and that the first week of the year has at least four days.
            $weeksInMonth = $lastWeekOfMonth - $firstWeekOfMonth + 1;

            for ($weekNumber = $weekToCopy; $weekNumber <= $weeksInMonth; $weekNumber++) {
                // creat a week data object that have current week of month data;
                $repeatWeekData = $this->getDataForSpecificWeek($request->copyWeekMonth, $weekNumber);
                // Organize the source week's menu data by day of the week
                foreach ($repeatWeekData as $week) {
                    $weekDate = Carbon::parse($week->date);
                    $dayOfTheWeek = $weekDate->englishDayOfWeek;
                    $repeatWeekDataByDay[$dayOfTheWeek] = $week->menu_data;
                }
                $this->repeatMenuForWeek($newMonthYear, $weekNumber, $repeatWeekDataByDay);
            }
        }

        // If no days were missing, return a simple success message
        return back()->with('success', 'Menu Repeated Successfully');
    }

    private function repeatMenuForWeek($newMonthYear, $weekNumber, $repeatWeekDataByDay)
    {
        // Calculate the start and end dates of the specified week
        $weekStartDate = $newMonthYear->copy()->startOfMonth()->addWeeks($weekNumber - 1)->startOfWeek(Carbon::MONDAY);
        $weekEndDate = $weekStartDate->copy()->endOfWeek(Carbon::SUNDAY);

        // Ensure the end date does not exceed the month
        if ($weekEndDate->month !== $newMonthYear->month) {
            $weekEndDate = $newMonthYear->copy()->endOfMonth();
        }

        // Loop through each day of the week and repeat the menu
        $currentDate = $weekStartDate->copy();
        while ($currentDate->lte($weekEndDate)) {
            $currentWeekDay = $currentDate->englishDayOfWeek;
            if (array_key_exists($currentWeekDay, $repeatWeekDataByDay)) {
                $existingMenu = ShareMenu::whereDate('date', $currentDate->format('Y-m-d'))->first();
                if ($existingMenu) {
                    $existingMenu->menu_data = $repeatWeekDataByDay[$currentWeekDay];
                    $existingMenu->save();
                } else {
                    // Create a new menu entry
                    $newMenu = new ShareMenu();
                    $newMenu->menu_data = $repeatWeekDataByDay[$currentWeekDay];
                    $newMenu->date = $currentDate->format('Y-m-d');
                    $newMenu->save();
                }
            }
            $currentDate->addDay();
        }
    }

    public function updateMenu(Request $request)
    {
        // Validate the date input
        $validator =  Validator::make($request->all(), [
            'menudate' => 'required|date',
            'Breakfast' => 'required|array|min:1',
            'Lunch' => 'required|array|min:1',
            'Dinner' => 'required|array|min:1',
            'Brunch' => 'required|array|min:1',
        ], [
            'Breakfast.min' => 'Please provide at least one item for Breakfast',
            'Lunch.min' => 'Please provide at least one item for Lunch',
            'Dinner.min' => 'Please provide at least one item for Dinner',
            'Brunch.min' => 'Please provide at least one item for Brunch',
        ]);

        if ($validator->fails()) {
            return back()->with('error', $validator->errors()->first());
        }
        // Retrieve selected items or default to an empty array
        $selectedBreakfast = $request->input('Breakfast', []);
        $selectedLunch = $request->input('Lunch', []);
        $selectedBrunch = $request->input('Brunch', []);
        $selectedDinner = $request->input('Dinner', []);

        // Retrieve the date and menu ID from the request
        $menuDate = $request->input('menudate');
        $menuId = $request->input('menuEditId'); // Ensure that this input is being sent in the request

        // Prepare the menu data array
        $menu = [
            'Breakfast' => $selectedBreakfast,
            'Lunch' => $selectedLunch,
            'Brunch' => $selectedBrunch,
            'Dinner' => $selectedDinner,
        ];

        // Encode the menu data as JSON
        $jsonMenudata = json_encode($menu);

        // Find the existing ShareMenu entry or fail with a 404 error
        $newMenu = ShareMenu::findOrFail($menuId);

        // Update the menu data and date
        $newMenu->menu_data = $jsonMenudata;
        $newMenu->date = $menuDate;

        // Attempt to save and return an appropriate response
        if ($newMenu->save()) {
            return back()->with("menuImagesSave", "Menu Updated Successfully");
        } else {
            return back()->with("menuImagesError", "Menu could not be updated");
        }
    }

    public function checkNextDayMenu()
    {
        try {
            $nextDayDate = Carbon::now()->addDay();
            $nextDayDateFormat = $nextDayDate->format('Y-m-d');
            // now check menu is created or not
            $checkMenu = ShareMenu::where('date', $nextDayDateFormat)->first();

            if (is_null($checkMenu)) {
                return response()->json([
                    "status" => 200,
                    'menu_is_added' => false
                ], 200);
            } else {
                return response()->json([
                    "status" => 200,
                    'menu_is_added' => true
                ], 200);
            }
        } catch (Exception $e) {
            return response()->json([
                'error' => $e->getMessage()
            ], 500);
        }
    }

    // private function updateMenuAuto()
    // {
    //     try {
    //         $todayInEnglish = Carbon::now()->englishDayOfWeek;
    //         $autoUpdateTime = MenuTimer::where('menu_day', strtolower($todayInEnglish))->first();

    //         if ($autoUpdateTime) {
    //             $timeForAutoUpdate = Carbon::createFromFormat('H:i:s', $autoUpdateTime->auto_update_time);
    //             $now = Carbon::now();
    //             // Check if the current time is past the scheduled update time
    //             $minutesBefore = 2; // For example, 10 minutes before
    //             // Subtract the minutes from the auto-update time
    //             $updateWindowTime = $timeForAutoUpdate->copy()->subMinutes($minutesBefore);
    //             if ($now->gte($updateWindowTime) && $now->lt($timeForAutoUpdate)) {
    //                 // Get the date for the next day
    //                 $nextDayDate = Carbon::now()->addDay();
    //                 // Get the date for the same day last week
    //                 $sameDayLastWeek = Carbon::now()->subWeek()->startOfWeek()->addDays($nextDayDate->dayOfWeek - 1);

    //                 // Retrieve the menu data for last week's same weekday
    //                 $lastWeekMenu = ShareMenu::where('date', $sameDayLastWeek->format('Y-m-d'))->first();
    //                 if ($lastWeekMenu) {
    //                     $menuData = $lastWeekMenu->menu_data;
    //                     // Check if the menu for the next day already exists and if it should be auto-updated
    //                     $nextDayMenu = ShareMenu::where('date', $nextDayDate->format('Y-m-d'))->where('auto_update', 1)->first();
    //                     if ($nextDayMenu) {
    //                         // Update the next day's menu
    //                         $nextDayMenu->update([
    //                             'menu_data' => $menuData,
    //                             'auto_update' => 0 // Assuming you want to reset the auto_update flag
    //                         ]);
    //                         $message = ("Menu data for {$sameDayLastWeek->toDateString()} has been used to update the next day's menu.");
    //                     }
    //                 } else {
    //                     $message = ("No menu data found for {$sameDayLastWeek->toDateString()}.");
    //                 }
    //             } else {
    //                 $message = ('Not the right time to update the menu.');
    //             }
    //         } else {
    //             $message = ('No auto-update time found for today.');
    //         }
    //     } catch (Exception $e) {
    //         $message = $e->getMessage();
    //     }
    //     return $message; 
    // }

    public function addNextDayMenu()
    {
        try {
            $nextDayDate = Carbon::now()->addDay();
            $nextDayDateFormat = $nextDayDate->format('Y-m-d');
            // now check menu is created or not
            $checkMenu = ShareMenu::where('date', $nextDayDateFormat)->first();
            if (!is_null($checkMenu)) {
                return response()->json([
                    'status' => 200,
                    "is_created" => true,
                    'message' => "menu already created for next day({$nextDayDate->format('d-m-Y')} )."
                ]);
            } else {
                ShareMenu::create([
                    'date' => $nextDayDate,
                    'auto_update' => 1
                ]);
                return response()->json([
                    'status' => 200,
                    "is_created" => false,
                    'message' => 'Mene Auto Updated request is created',
                    // 'message1' => $this->updateMenuAuto(),
                ]);
            }
        } catch (Exception $ex) {
            return response()->json([
                'error' => $ex->getMessage(),
            ], 500);
        }
    }
}
