<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\AdminNoticeToTenant;
class AdminNoticeForTenantController extends Controller
{
    // Display a listing of the notices
    public function index()
    {
        $notices = AdminNoticeToTenant::with('studentDetails')->get();
        return view('fronted.adminNotice',compact('notices'));
    }

    // Show the form for creating a new notice
    public function create()
    {
        return view('admin.notices.create');
    }

    // Store a newly created notice in storage
    public function store(Request $request)
    {
        $request->validate([
            'title' => 'required|string|max:255',
            'body' => 'required|string',
        ]);

        TenantNotice::create([
            'title' => $request->title,
            'body' => $request->body,
        ]);

        return redirect()->route('admin.notices.index')->with('success', 'Notice created successfully.');
    }

    // Display the specified notice
    public function show($id)
    {
        $notice = TenantNotice::findOrFail($id);
        return view('admin.notices.show', compact('notice'));
    }

    // Show the form for editing the specified notice
    public function edit($id)
    {
        $notice = TenantNotice::findOrFail($id);
        return view('admin.notices.edit', compact('notice'));
    }

    // Update the specified notice in storage
    public function update(Request $request, $id)
    {
        $request->validate([
            'title' => 'required|string|max:255',
            'body' => 'required|string',
        ]);

        $notice = TenantNotice::findOrFail($id);
        $notice->update([
            'title' => $request->title,
            'body' => $request->body,
        ]);

        return redirect()->route('admin.notices.index')->with('success', 'Notice updated successfully.');
    }

    // Remove the specified notice from storage
    public function destroy($id)
    {
        $notice = TenantNotice::findOrFail($id);
        $notice->delete();

        return redirect()->route('admin.notices.index')->with('success', 'Notice deleted successfully.');
    }
}
