<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use App\Models\AdminNoticeToTenant;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\Validator;


class AdminNoticeToTenantController extends Controller
{
    public function index()
    {
        $notices = AdminNoticeToTenant::all();
        return response()->json(['notices' => $notices], 200);
    }

    public function getNoticeForStudent($studentId)
    {
        try {

            $studentNotices = AdminNoticeToTenant::where('student_id', $studentId)->get();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'successfully get notices',
                'data' => $studentNotices,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'somthing went wronge',
                'error' => $e->getMessage()
            ], 500);
        }
    }


    public function updateResponse(Request $request, $noticeId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'response_message' => 'required|string',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 402,
                    'message' => 'notice not found',
                    'error' => $validator->errors(),
                ], 404);
            }
            $notice = AdminNoticeToTenant::findOrFail($noticeId);
            $notice->response = $request->response_message;
            $notice->save();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'updated response message',
            ],200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'notice not found',
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function store(Request $request)
    {
        $validated = $request->validate([
            'student_id' => 'required|exists:students,id',
            'noticeType' => 'required',
            'date' => 'required|date',
            'text' => 'required',
            'attachment' => 'nullable|string',
            'response' => 'nullable',
            'status' => 'required|string|max:50',
        ]);

        $notice = AdminNoticeToTenant::create($validated);

        return response()->json(['notice' => $notice], 201);
    }

    public function show($id)
    {
        $notice = AdminNoticeToTenant::findOrFail($id);
        return response()->json(['notice' => $notice], 200);
    }

    public function update(Request $request, $id)
    {
        $validated = $request->validate([
            'student_id' => 'required|exists:students,id',
            'noticeType' => 'required',
            'date' => 'required|date',
            'text' => 'required',
            'attachment' => 'nullable|string',
            'response' => 'nullable',
            'status' => 'required|string|max:50',
        ]);

        $notice = AdminNoticeToTenant::findOrFail($id);
        $notice->update($validated);

        return response()->json(['notice' => $notice], 200);
    }

    public function destroy($id)
    {
        $notice = AdminNoticeToTenant::findOrFail($id);
        $notice->delete();

        return response()->json(['message' => 'Notice deleted successfully'], 200);
    }
}
