<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Http\Middleware\CheckUserType;
use App\Models\Businessregister;
use App\Models\EmployeRegister;
use Illuminate\Support\Facades\Auth;
use App\Models\ApiModels\AdminLogin;
use Illuminate\Support\Facades\Hash;
use App\Models\Salesubscription;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\UserAuthTokens;
use Illuminate\Support\Str;
use Carbon\Carbon;
use Exception;

class ApiLoginController extends Controller
{
    // private function deleteExpiredToken($licenceNo): bool
    // {
    //     $userToken = UserAuthTokens::where('licenceno', $licenceNo)->delete();
    //     return true;
    // }
    private function generateUniqueToken($adminType, $licenceNo, $userId)
    {
        // $this->deleteExpiredToken($licenceNo);
        $randomString = Str::random(32);
        $timestamp = Carbon::now()->timestamp;
        $rawToken = $randomString . $timestamp;
        $hashedToken = Hash::make($rawToken);
        $createTokenRecord = new UserAuthTokens();
        $createTokenRecord->user_id = $userId;
        $createTokenRecord->token = $hashedToken;
        $createTokenRecord->usertype = $adminType;
        $createTokenRecord->licenceno = $licenceNo;
        $createTokenRecord->save();
        return $hashedToken;
    }
    protected function CheckSubscription($key)
    {
        $check = Salesubscription::where('user_id', $key)
            ->where('status', 1)
            ->first();
        if ($check) {
            return true;
        }
        return false;
    }

    protected function CheckDatabase($key)
    {
        $checkDatabase = Businessregister::where("license_no", $key)->where('db_status', 1)->first();
        if ($checkDatabase) {
            return true;
        }
        return false;
    }

    private function getHostelList($userName)
    {
        try {
            $adminlist = [];
            config(['database.default' => env('DB_CONNECTION')]);
            $admin = Businessregister::where(function ($query) use ($userName) {
                $query->where('username', $userName)
                    ->orWhere('username1', $userName);
            })->get();
            if ($admin) {
                foreach ($admin as $data) {
                    $adminlist[] = $data->businessname;
                }
                return $adminlist;
            } else {
                return [];
            }
        } catch (Exception $e) {
            return [];
        }
    }

    public function hostelNames(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'userName' => 'required|string',
                'userType' => 'required|string'
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => 'validation Error',
                    'error' => $validator->errors()
                ], 422);
            };
            $userName = $request->userName;
            $userType = $request->userType;
            $adminlist = [];
            if (strtolower($userType) !== "admin") {
                $emp = EmployeRegister::where(function ($query) use ($userName) {
                    $query->where("email", $userName)
                        ->orwhere("mobile", $userName);
                })
                    ->get();
                if (!empty($emp)) {
                    foreach ($emp as $val) {
                        $businessname = Businessregister::where('license_no', $val->license_no)->value('businessname');
                        if (!in_array($businessname, $adminlist)) {
                            $adminlist[] = $businessname;
                        }
                    }
                }
            } else {
                $adminlist =  $this->getHostelList($userName);
            }

            return response()->json([
                'status' => true,
                'message' => 'hostel lists',
                'data' => [
                    'businessList' => $adminlist
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json(["error" => $e->getMessage()], 400);
        }
    }

    public function login(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'userType' => 'required',
                'businessName' => 'required',
                'username' => 'required|string',
                'password' => 'required|string',
            ]);

            if ($validator->fails()) {
                return response()->json(['error' => $validator->errors()], 422);
            }

            $adminType = $request->input('userType');
            $businessName = $request->input('businessName');
            $username = $request->input('username');
            $password = $request->input('password');



            if (($adminType == "admin" || $adminType == "Admin")) {
                $admin = Businessregister::where(function ($query) use ($username) {
                    $query->where('username', $username)->orWhere('username1', $username);
                })
                    ->where('businessname', $businessName)
                    ->first();

                if (!$admin) {
                    return response()->json(['error' => 'Invalid login credentials'], 401);
                }
                $checkSubscription = $this->CheckSubscription($admin->license_no);

                if (!Hash::check($password, $admin->password)) {
                    return response()->json([
                        'error' => 'Invalid login credentials'
                    ], 401);
                }

                if ($this->CheckDatabase($admin->license_no)) {
                    $userToken = $this->generateUniqueToken($adminType, $admin->license_no, $admin->id);
                    return response()->json(
                        [
                            'status' => true,
                            "data" => [
                                'userId' =>  $admin->id,
                                'username' => $request->username,
                                'token' => $userToken,
                                'userType' => $adminType,
                                'businessName' => $businessName,
                                'permission' => "All",
                            ]
                        ],
                        200
                    );
                }

                return response()->json(['error' => $checkSubscription ? 'Database not created' : 'This is not have subscription'], 401);
            }

            // get the admin data by its hostel name and licence name

           $hostelLicenceNo = Businessregister::where('businessname',$businessName)->value('license_no');

            $emp = EmployeRegister::where(function ($query) use ($username) {
                $query->where("username", $username)->orWhere("username1", $username);
            })
                ->where("user_roll", $adminType)
                ->where("license_no", $hostelLicenceNo)
                ->first();

            if (!$emp || !Hash::check($password, $emp->password)) {
                return response()->json(["error" => $emp ? 'Invalid Password' : 'Invalid User Type'], 401);
            }

            if ($this->CheckDatabase($hostelLicenceNo)) {
                $userToken = $this->generateUniqueToken($adminType, $hostelLicenceNo, $emp->id);
                return response()->json(
                    [
                        'status' => true,
                        "data" => [
                            'userId' => $emp->id,
                            'username' => $request->username,
                            "token" => $userToken,
                            'permission' =>  $emp->permissions,
                            'businessName' => $businessName,
                            'userType' => $emp->user_roll
                        ]
                    ],
                    200
                );
            }
            return response()->json(["error" => "Database not created"], 401);
        } catch (QueryException $e) {
            return response()->json(['error' => 'Database error'], 500);
        } catch (Exception $e) {
            return response()->json(['error' => $e->getMessage()], 500);
        }
    }

    public function logoutUserSession($userId)
    {
        try {
            $user = UserAuthTokens::where('user_id', $userId)->firstOrFail();
            // $deletedRecords = UserAuthTokens::where('user_id', $userId)->delete();
            return response()->json([
                'status' => true,
                'message' => 'Logged out successfully.'
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'message' => 'User not found.',
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function ChangeUserPassword(Request $request, $userId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'userType' => 'required|string',
                'old_password' => 'required',
                "password" => [
                    'required',
                    'string',
                    'min:8',
                    'confirmed',
                    'regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/',
                ],
            ]);
            if ($validator->fails()) {
                return response()->json(['errors' => $validator->errors()], 422);
            }
            $userType = strtolower($request->userType);
            if ($userType == "admin") {
                $user = Businessregister::on('mysql')->findOrFail($userId);
            } else {
                $user = EmployeRegister::on('mysql')->findOrFail($userId);
            }
            if (Hash::check($request->old_password, $user->password)) {
                $user->update([
                    'password' => Hash::make($request->password),
                ]);
            } else {
                return response()->json([
                    'status' => false,
                    'error' => 'Old password does not match.'
                ], 422);
            }

            return response()->json([
                'status' => true,
                'message' => "Password changed successfully",
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'message' => 'User not found.',
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function isValidToken(Request $request)
    {
        try {
            $token = $request->token;
            $userLogged = UserAuthTokens::where('token', $token)->firstOrFail();
            return response()->json([
                'staut' => true,
                'message' => "user token  validated.",
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
