<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Illuminate\Support\Carbon;
use UnexpectedValueException;
use Exception;
use Illuminate\Validation\ValidationException;
use App\Models\ApiModels\Sales;
use App\Models\ApiModels\RecieveAmount;
use App\Models\ApiModels\CustomeSales;
use App\Models\ApiModels\ExpensesItem;
use App\Models\ApiModels\RoomSeats;
use App\Models\ApiModels\BusinessDocument;
use App\Models\Businessregister;
use DB;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use InvalidArgumentException;
use Illuminate\Support\Collection;
use App\Models\ApiModels\ProfilesDocumentsModal;
use App\Models\ApiModels\TermsAndConditionsModal;


class BusinessController extends controller
{

    private function prepareBusinessDocuments()
    {
        try {
            $allDocuments = BusinessDocument::all();

            return collect($allDocuments)->map(function ($row) {
                $document = [
                    'documentName' => $row->upload_name,
                    'documents' => !empty($row->upload_pdf) && file_exists(public_path('assets/uploads/pdf/' . $row->upload_pdf))
                        ? asset('assets/uploads/pdf/' . $row->upload_pdf)
                        : null,
                ];

                return $document;
            })->all();
        } catch (Exception $e) {
            throw $e;
        }
    }

    public function getBusinessProfile()
    {
        try {
            $businessProfileData = [];
            $businessProfile = Businessregister::firstOrFail();
            // Get documents
            $businessDocuments = $this->prepareBusinessDocuments() ?? throw new Exception("Documents not found");
            $businessProfileData = [
                'id' => $businessProfile->id,
                'businessType' => $businessProfile->business_type,
                'hostelId' => $businessProfile->license_no,
                'firstName' => $businessProfile->fname,
                'lastName' => $businessProfile->lname,
                'email' => $businessProfile->email,
                'gstNumber' => $businessProfile->gst_no,
                'mobileNumber' => $businessProfile->mobile,
                'altMobileNumber' => $businessProfile->mobile_secondary,
                'businessName' => $businessProfile->businessname,
                'country' => $businessProfile->country,
                'state' => $businessProfile->state,
                'city' => $businessProfile->city,
                'address' => $businessProfile->address,
                'pincode' => $businessProfile->pincode,
                'businessImg' => $businessProfile->pic ? asset('assets/uploads/' . $businessProfile->pic) : null,
                'businessDocuments' => $businessDocuments,
            ];

            return response()->json([
                'status' => true,
                'data' => $businessProfileData
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'message' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    private function getRemainingBusinessName($username, $businessName)
    {
        try {
            config(['database.default' => env('DB_CONNECTION')]);
            $businesData = Businessregister::where('username', $username)
                ->whereNot('businessname', $businessName)
                ->get();

            if (empty($businesData)) {
                return [];
            } else {
                foreach ($businesData as $data) {
                    $businessNames[] = $data['businessname'];
                }
                return $businessNames;
            }
        } catch (Exception $e) {
            throw new Exception("Error in getting business names : " . $e->getMessage());
        }
    }
    public function getBusinessDetailsForSideBar()
    {
        try {
            $businessData = Businessregister::firstOrFail();
            $username = $businessData->mobile;
            $remainingBusinessNamesIfExist = $this->getRemainingBusinessName($username, $businessData->businessname);

            $businessImgPath = asset('assets/uploads/' . $businessData->pic);
            $businessImg = file_exists(public_path('assets/uploads/' . $businessData->pic)) ? $businessImgPath : asset('assets/images/logoimage.jpeg');

            return response()->json([
                'status' => true,
                'message' => 'Successfully fetch business img and remaining names',
                'data' => [
                    'businessImg' => $businessImg,
                    'remainingBusinessNames' => $remainingBusinessNamesIfExist,
                ],
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function updateProfileImg(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'image' => 'required|image|mimes:jpg,png,jpeg',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            $data = Businessregister::firstOrFail();
            $oldProfileImg = $data->pic;
            if ($request->hasFile('image')) {
                $file = $request->file('image');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;

                if ($file->move(sys_get_temp_dir(), $filename)) {
                    $filePath = sys_get_temp_dir() . '/' . $filename;

                    if (file_exists($filePath)) {
                        $fileSize = filesize($filePath);

                        if ($fileSize > 1600 * 1024) {
                            $image = imagecreatefromstring(file_get_contents($filePath));
                            $compressedImage = imagecreatetruecolor(imagesx($image), imagesy($image));
                            imagecopyresampled($compressedImage, $image, 0, 0, 0, 0, imagesx($image), imagesy($image), imagesx($image), imagesy($image));
                            imagejpeg($compressedImage, $filePath, 75); // Compress with 75% quality
                            imagedestroy($image);
                            imagedestroy($compressedImage);
                        }
                        if (rename($filePath, 'assets/uploads/' . $filename)) {
                            $data->pic = $filename;
                        } else {
                            throw new Exception('img not uploaded');
                        }
                    }
                    if ($data->save()) {
                        $this->removeOldDocumets($oldProfileImg, 'profileImg');
                        $this->updateImgInMainBusinessTable($filename, $data->license_no);
                        return response()->json([
                            'status' => true,
                            'message' => 'Image uploaded',
                        ]);
                    } else {
                        throw new Exception('DB error');
                    }
                } else {
                    return response()->json(['error' => 'No photo provided'], 400);
                }
            }
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }


    private function updateImgInMainBusinessTable($imgName, $license_no):bool
    {
        config(['database.default' => env('DB_CONNECTION')]);
        $data = Businessregister::where('license_no', $license_no)->first();
        if ($data) {
            $data->pic = $imgName;
            $data->update();
        }
        config(['database.default' => 'custom_db']);
        return true;
    }

    public function updateBusinessProfile(Request $request, $id)
    {
        try {
            // Validation
            $validator = Validator::make($request->all(), [
                'firstName' => 'required|string',
                'lastName' => 'required|string',
                'email' => 'required|email',
                'gstNumber' => 'nullable',
                'mobileNumber' => 'required|digits:10',
                'altMobileNumber' => 'nullable|digits:10',
                'businessName' => 'required|string',
                'country' => 'required|string',
                'state' => 'required|string',
                'city' => 'required|string',
                'address' => 'required|string',
                'pincode' => 'required|digits:6',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'errors' => $validator->errors()
                ], 422);
            }

            $businessProfile = Businessregister::findOrFail($id);

            $businessProfile->update([
                'fname' => $request->firstName,
                'lname' => $request->lastName,
                'email' => $request->email,
                'mobile' => $request->mobileNumber,
                'mobile_secondary' => $request->altMobileNumber,
                'businessname' => $request->businessName,
                'country' => $request->country,
                'state' => $request->state,
                'city' => $request->city,
                'address' => $request->address,
                'pincode' => $request->pincode,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'business profile updated successfully'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => 'false',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function removeOldDocumets($document, $type = "")
    {
        if ($type == "profileImg") {
            $oldFilePath = public_path('assets/uploads/' . $document);
        } else {
            $oldFilePath = public_path('assets/uploads/pdf/' . $document);
        }
        if (file_exists($oldFilePath)) {
            @unlink($oldFilePath);
        }
    }

    public function uploadDocumets(Request $request, $id)
    {
        try {
            $validator = Validator::make($request->all(), [
                'documents' => 'required|file|mimes:pdf,jpg,png,jpeg',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'error' => $validator->errors()
                ], 422);
            }
            $document = ProfilesDocumentsModal::findOrFail($id);
            $oldDocuments = $this->removeOldDocumets($document->upload_pdf);
            $file = $request->file('documents');
            $filename = time() . '_' . rand() . '.' . $file->getClientOriginalExtension();
            $destinationPath = public_path('assets/uploads/pdf/');
            $file->move($destinationPath, $filename);
            $document->update([
                'upload_pdf' => $filename
            ]);

            return response()->json([
                'status' => true,
                'message' => 'document uploaded successfully',
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such document type found'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function getBusinessProfileDocumets()
    {
        try {
            $documents = ProfilesDocumentsModal::all();
            return response()->json([
                'status' => true,
                'data' => $documents,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false, 'message' => $e->getMessage()
            ], 500);
        }
    }

    public function addTermsAndConditions(Request $request)
    {
        try {
            $validatedData = $request->validate([
                'termsandconditions' => 'required|string',
            ]);
            $termsAndConditions = TermsAndConditionsModal::create($validatedData);
            return response()->json([
                'status' => true,
                'message' => 'Terms and conditions added successfully',
            ], 201);
        }catch(UnexpectedValueException $e){
            return response()->json([
                'status'=>true,
                'message'=>$e->getMessage(),
            ],200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteTermsAndConditions($id)
    {
        try {
            $termsCondition = TermsAndConditionsModal::findOrFail($id);
            $termsCondition->delete();

            return response()->json([
                'status' => true,
                'message' => "Record deleted successfully"
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => "Record not found with ID: {$id}"
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function getInvoiceTermsAndCondtions()
    {
        try {
            $data = TermsAndConditionsModal::all();
            return response()->json([
                'status' => true,
                'data' => $data,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
