<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use DB;
use Illuminate\Support\Carbon;
use Exception;
use App\Models\RoomSeat;
use App\Models\ReserveRoom;
use App\Models\ElectricityManagement;
use App\Models\Studentregister;
use App\Models\Olddataregister;
use App\Models\StudentElecticityRecords;
use App\Models\ElecticityCalculater;
use App\Models\StudentsElecticityBill;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Log;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\Validator;


class ElectricityManagementController extends Controller
{

    public function getAllElectricityRecordsOfStudent($studentId)
    {
        // get the all electricity records of the student  from database
        try {
            $electricityRecords = StudentElecticityRecords::select('id as elctricityId', 'billDate', 'Amount')->where('userId', $studentId)
                ->whereNot('Amount', 0)
                ->where('status', 0)
                ->get();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => $electricityRecords,
                'message' => 'Successfully get electricity bill records',
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => "Failed to fetch data",
                "error" => $e->getMessage(),
            ], 500);
        }
    }

    public function checkElectricityRecordsBeforStore()
    {
        try {
            // check electricity Records have or not
            $countElectricityRecords = ElectricityManagement::select('BillDate')->orderBy('BillDate', 'asc')->groupBy('BillDate')->count();
            $roomNumbers = RoomSeat::select('roomno')->orderBy('roomno', 'Asc')->get();
            if ($countElectricityRecords > 0) {
                $takePreviousReding = false;
            } else {
                $takePreviousReding = true;
            }

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => [
                    'takePreviousReading'  => $takePreviousReding,
                    'RoomNumberLists'      => $roomNumbers
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Server Error',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    private function getInsertedBillDate()
    {
        $lastRecords =  ElectricityManagement::select('BillDate')
            ->orderBy('id', 'desc')
            ->first();
        return $lastRecords ? $lastRecords->BillDate : null;
    }
    protected function getFormattedBillDates($selectedMonth)
    {
        if (!$selectedMonth) {
            return [null, null];
        }

        $previousReadingDate = ElectricityManagement::select('BillDate')
            ->where('BillDate', '<', $selectedMonth)
            ->orderBy('BillDate', 'DESC')
            ->groupBy('BillDate')
            ->first();

        $presentReadingDate = DB::table('electicity_management')
            ->where('BillDate', '=', $selectedMonth)
            ->first();

        $PrenseBillDate = $presentReadingDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : '';
        $PreviousBillDate = $previousReadingDate ? Carbon::parse($previousReadingDate->BillDate)->format('d-M-Y') : ($PrenseBillDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : "");

        return [$PrenseBillDate, $PreviousBillDate];
    }

    protected function getElectricityRecordsByDateRange($startDate, $endDate)
    {
        return $lastRecords = ElectricityManagement::with('studentElectricityRecords.student')
            ->whereBetween('BillDate', [$startDate, $endDate])
            ->get();
    }

    protected function getElectricityRecordsByMonth($selectedMonth)
    {
        return $electricityRecords = ElectricityManagement::with(['studentElectricityRecords' => function ($query) use ($selectedMonth) {
            $query->with(['student:id,candidatename']);
        }])
            ->where('BillDate', $selectedMonth)
            ->get();
    }

    private function PrepareElectrityRecords($presentReadingDate, $previousReadingDate, $roomno, $month, $units, $readingRate)
    {
        try {
            $previousReading = Carbon::parse($previousReadingDate)->toDateString();
            $presentReading = Carbon::parse($presentReadingDate)->toDateString();
            // Check student room number from old data register table
            $students = Studentregister::where('roomno', $roomno)
                ->where(function ($query) use ($previousReading, $presentReading) {
                    $query->where(function ($subQuery) use ($presentReading, $previousReading) {
                        // Consider date only when join_date is null
                        $subQuery->where(function ($innerSubQuery) use ($presentReading, $previousReading) {
                            $innerSubQuery->Where(function ($checkBetween) use ($presentReading, $previousReading) {
                                $checkBetween->whereBetween('date', [$previousReading, $presentReading])
                                    ->whereNull('join_date');
                            })
                                ->orWhere(function ($checkdate) use ($presentReading, $previousReading) {
                                    $checkdate->where('date', "<", $previousReading)
                                        ->whereNull('join_date');
                                });
                        })
                            ->orWhere(function ($innerSubQuery) use ($presentReading, $previousReading) {
                                // Check leave_date when join_date is not null
                                $innerSubQuery->where(function ($checkBetween) use ($presentReading, $previousReading) {
                                    $checkBetween->whereBetween('join_date', [$previousReading, $presentReading]);
                                })
                                    ->orWhere(function ($checkdate) use ($presentReading, $previousReading) {
                                        $checkdate->where('join_date', "<", $previousReading);
                                    });
                            });
                    })
                        ->where('status', 1)
                        ->whereNull('leave_date');
                })
                ->groupBy('formno')
                ->get();

            foreach ($students as $row) {
                $rowDate = $row->join_date ? Carbon::parse($row->join_date)->format('Y-m-d') : Carbon::parse($row->date)->format('Y-m-d');
                $leaveDate = $row->leave_date ? Carbon::parse($row->leave_date)->format('Y-m-d') : null;
                $newRecords = new StudentsElecticityBill();
                $newRecords->roomno =  $roomno;
                $newRecords->candidate_id = $row->id;
                $newRecords->seat_no = $row->seetno;
                $newRecords->joining_date = $row->join_date ? $row->join_date : $row->date;
                $newRecords->leaving_date = $row->leave_date;
                $newRecords->total_reading = $units;

                if ($rowDate >= $previousReading) {
                    $newRecords->reading_start_date = $rowDate;
                } else {
                    $newRecords->reading_start_date = $previousReading;
                }

                if ($leaveDate == null) {
                    $newRecords->reading_end_date = $presentReading;
                } elseif ($leaveDate <= $presentReading) {
                    $newRecords->reading_end_date = $leaveDate;
                } else {
                    $newRecords->reading_end_date = $presentReading;
                }
                $newRecords->save();
            }
            $oldStudents = Olddataregister::where('roomno', $roomno)
                ->where(function ($query) use ($presentReading, $previousReading) {
                    $query->where('date', "<=", $presentReading)
                        ->where(function ($subQuery) use ($presentReading, $previousReading) {
                            $subQuery->whereBetween('leave_date', [$previousReading, $presentReading])
                                ->orWhere('leave_date', ">=", $presentReading);
                        });
                })
                ->groupBy('formno')
                ->get();
            foreach ($oldStudents as $row) {
                $getActualIdOfCandidate = Studentregister::where("formno", $row->formno)->first();
                $newRecords = new StudentsElecticityBill();
                $newRecords->roomno = $roomno;
                $newRecords->candidate_id = $getActualIdOfCandidate->id;
                $newRecords->seat_no = $row->seetno;
                $newRecords->joining_date = $row->date;
                $newRecords->leaving_date = $row->leave_date;
                $newRecords->total_reading = $units;
                $joindate = Carbon::parse($row->date);
                $leaveDate = $row->leave_date ? Carbon::parse($row->leave_date) : null;
                if ($joindate >= $previousReading) {
                    $newRecords->reading_start_date = $joindate;
                } else {
                    $newRecords->reading_start_date = $previousReading;
                }
                if ($leaveDate == null) {
                    $newRecords->reading_end_date = $presentReading;
                } elseif ($leaveDate <= $presentReading) {
                    $newRecords->reading_end_date = $leaveDate;
                } else {
                    $newRecords->reading_end_date = $presentReading;
                }
                $newRecords->save();
            }
            $queryTotalDays = StudentsElecticityBill::selectRaw("SUM(
                DATEDIFF(
                    LEAST(reading_end_date, '{$presentReading}'),
                    GREATEST(reading_start_date, '{$previousReading}')
                )
            ) AS total_days")
                ->where("roomno", $roomno)
                ->whereRaw("reading_end_date <= '{$presentReading}'")
                ->whereRaw("reading_start_date >= '{$previousReading}'")
                ->first();

            $totalDays = $queryTotalDays->total_days;

            $finalQuery = StudentsElecticityBill::select(
                'id',
                \DB::raw("DATEDIFF(
             LEAST(reading_end_date, '{$presentReading}'),
             GREATEST(reading_start_date, '{$previousReading}')
         ) AS days")
            )
                ->where("roomno", $roomno)
                ->whereRaw("reading_end_date <= '{$presentReading}'")
                ->whereRaw("reading_start_date >= '{$previousReading}'")
                ->get();

            foreach ($finalQuery as $result) {
                $studentId = $result["id"];
                $days = $result['days'];
                if ($totalDays != 0 && $days != 0) {
                    $proportion = $days / $totalDays;
                } else {
                    $proportion = 0;
                }

                $studentUnit = round(($units * $proportion), 2);
                $studentAmount = round(($readingRate * $studentUnit), 2);
                $updateRecords = StudentsElecticityBill::where('roomno', $roomno)->where("id", $studentId)->first();
                if (!empty($updateRecords)) {
                    $updateRecords->total_reading_candidate = $studentUnit;
                    $updateRecords->reading_amount = $studentAmount;
                }
                $updateRecords->update();
            }
            return true;
        } catch (QueryException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }
    private function makeElecticityRecords($presentReading, $previousReading, $billDate)
    {
        try {

            $electricityRecords = ElectricityManagement::where("billdate", $billDate)->get();
            foreach ($electricityRecords as $row) {
                $roomno = $row->roomno;
                $prepareElecticityRecords = $this->PrepareElectrityRecords($presentReading, $previousReading, $roomno, $billDate, $row->discountUnits, $row->readingRate);
                if ($prepareElecticityRecords === true) {
                    $storedRecords = StudentsElecticityBill::all();
                    foreach ($storedRecords as $records) {
                        $newRecords = new StudentElecticityRecords();
                        $newRecords->electricity_management_id = $row->id;
                        $newRecords->roomNo = $records->roomno;
                        $newRecords->seatNo = $records->seat_no;
                        $newRecords->userId = $records->candidate_id;
                        $newRecords->unit = $row->units;
                        $newRecords->prUnitCharge = $records->total_reading_candidate;
                        $newRecords->billDate = $billDate;
                        $newRecords->Amount = $records->reading_amount;
                        $newRecords->save();
                    }
                    StudentsElecticityBill::truncate();
                } else {
                    throw  new Exception($prepareElecticityRecords);
                }
            }
            return true;
        } catch (QueryException $e) {
            throw $e;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function updateElectricityRecords($record)
    {
        try {
            StudentsElecticityBill::chunk(200, function ($storedRecords) use ($record) {
                foreach ($storedRecords as $created) {
                    $newRecords = new StudentElecticityRecords();
                    $newRecords->electricity_management_id = $record->id;
                    $newRecords->roomNo = $created->roomno;
                    $newRecords->seatNo = $created->seat_no;
                    $newRecords->userId = $created->candidate_id;
                    $newRecords->unit = $record->units;
                    $newRecords->prUnitCharge = $created->total_reading_candidate;
                    $newRecords->billDate = $record->BillDate;
                    $newRecords->Amount = $created->reading_amount;
                    $newRecords->save();
                }
            });
            StudentsElecticityBill::truncate();
            return true;
        } catch (Exception $e) {
            // Log or handle the exception as needed
            return false;
        }
    }

    private function updateAllElectricityRecords($record)
    {
        StudentElecticityRecords::where('billDate', $record->BillDate)
            ->Where('electricity_management_id', $record->id)
            ->delete();
        list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($record->BillDate);
        $prepareElecticityRecords = $this->PrepareElectrityRecords($PrenseBillDate, $PreviousBillDate, $record->roomno, $record->BillDate, $record->discountUnits, $record->readingRate);
        if (!$prepareElecticityRecords) {
            throw new Exception("PrepareElectrityRecords failed");
        }
        $updatedRecords = $this->updateElectricityRecords($record);
        if (!$updatedRecords) {
            throw new Exception("updateElectricityRecords failed");
        }
        return;
    }

    private function checkElectricityRecordForNextMonth($record)
    {
        $data = ElectricityManagement::where('BillDate', ">", $record->BillDate)->where('roomno', $record->roomno)->first();
        if (!empty($data)) {
            $data->previous_reading = $record->present_reading;
            $data->save();
            return [
                'status' => true,
                'data' => $data,
            ];
        } else {
            return [
                'status' => false,
                'data' => 'not found',
            ];
        }
    }

    public function storeElectricityRecords(Request $request)
    {

        try {
            $validator = Validator::make($request->all(), [
                'billdate' => 'required|date',
                'readingRate' => 'required|numeric',
                'roomSeatsData' => 'required|array',
                'discountUnits' => 'required|numeric'
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            $validatedData = $validator->validated();

            $checkElectricityRecords = ElectricityManagement::count();
            foreach ($validatedData['roomSeatsData'] as $roomNo => $data) {
                if ($checkElectricityRecords == 0) {
                    ElectricityManagement::create([
                        'BillDate' => $request->billdate,
                        'discountUnits' => ($request->discountUnits),
                        'readingRate' => $validatedData['readingRate'],
                        'roomno' => $roomNo,
                        'present_reading' => $data['present_red'],
                        'previous_reading' => $data['previous_red']
                    ]);
                } else {
                    ElectricityManagement::create([
                        'BillDate' => $request->billdate,
                        'discountUnits' => $request->discountUnits,
                        'readingRate' => $validatedData['readingRate'],
                        'roomno' => $roomNo,
                        'present_reading' => $data['present_red'],
                    ]);
                }
            }

            list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($request->billdate);
            $makeElectricityRecords = $this->makeElecticityRecords($PrenseBillDate, $PreviousBillDate, $request->billdate);
            if ($makeElectricityRecords !== true) {
                throw new Exception('Error in creating electricity records');
            }
            return response()->json([
                'status' => true,
                'stautsCode' => 200,
                'message' => 'Electricity Records created Successfully',
            ]);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'stautsCode' => 422,
                'message' => "Invalid request data.",
                'errors' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'stautsCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateElectricity(Request $request, $id)
    {
        try {

            $validator = Validator::make($request->all(), [
                'billdate' => 'required|date',
                'discountUnits' => 'required|numeric',
                'readingRate' => 'required|numeric',
                'previous_reading' => 'required|numeric',
                'present_reading' => 'required|numeric',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'errors' => $validator->errors(),
                ], 422);
            }

            $record = ElectricityManagement::find($id);
            if (!$record) {
                throw new Exception("No Records found");
            }
            // Update the ElectricityManagement record
            $record->update([
                'BillDate' => $request->billdate,
                'discountUnits' => ($request->discountUnits),
                'readingRate' => $request->readingRate,
                'previous_reading' => $request->previous_reading,
                'present_reading' => $request->present_reading,
            ]);
            $this->updateAllElectricityRecords($record);
            // check have any fill which is greater then this bill only first
            $nextMontRecords = $this->checkElectricityRecordForNextMonth($record);
            if ($nextMontRecords['status'] == true) {
                $this->updateAllElectricityRecords($nextMontRecords['data']);
            }
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => "Update Electricity Record successfully",
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }


    public function ElectricityView(Request $request)
    {
        try {
            $selectedMonth = $request->input('selected_month') ?? $this->getInsertedBillDate(); // this is use for selected records according to
            list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($selectedMonth);
            $electricityRecords = $this->getElectricityRecordsByMonth($selectedMonth);

            $allBillDates = ElectricityManagement::select('BillDate')->orderBy('BillDate', 'asc')->groupBy('BillDate')->get();
            // Organize data for the nested list
            $nestedListData = [];
            foreach ($allBillDates as $billDate) {
                $year = Carbon::parse($billDate->BillDate)->format('Y');
                $nestedListData[$year][] = $billDate->BillDate;
            }
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => [
                    'electricityYearData' => $nestedListData,
                    'selectedMonth' => $selectedMonth,
                    'presentBillDate' => $PrenseBillDate,
                    'previousBillDate' => $PreviousBillDate,
                    'electricityRecords' => $electricityRecords,
                ],
                'message' => 'successfully get the electricity data',
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }
}
