<?php

namespace App\Http\Controllers\Api;

use Illuminate\Support\Facades\Validator;
use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\EmployeRegister;
use Illuminate\Support\Facades\DB;
use Exception;
use App\Models\Businessregister;
use Illuminate\Support\Facades\Hash;
use App\Models\empPaymentSetup;
use Illuminate\Validation\Rule;
use Carbon\Carbon;
use App\Models\Reviews;

class EmployeController extends Controller
{
    private function getBusinessDetails()
    {
        $business = Businessregister::select('license_no', 'email', 'mobile')->firstOrFail();
        return [
            'license_no' => $business->license_no,
            'email' => $business->email,
            'mobile' => $business->mobile,
        ];
    }

    private function resizeBase64Image($base64Data, $targetSizeInKB)
    {
        try {
            // Decode base64 data
            $decodedData = base64_decode($base64Data);

            // Check if decoding was successful
            if ($decodedData === false) {
                throw new Exception('Base64 decoding failed');
            }

            // Create an image resource from the decoded data
            $img = imagecreatefromstring($decodedData);

            // Check if image creation was successful
            if ($img === false) {
                throw new Exception('Image creation failed');
            }

            // Get the original image dimensions
            $width = imagesx($img);
            $height = imagesy($img);

            // Set the desired width and calculate the corresponding height to maintain the aspect ratio
            $targetWidth = 200; // Adjust this value based on your requirements
            $targetHeight = ($targetWidth / $width) * $height;

            // Create a new image resource for the resized image
            $resizedImg = imagecreatetruecolor($targetWidth, $targetHeight);

            // Resize the image
            imagecopyresampled($resizedImg, $img, 0, 0, 0, 0, $targetWidth, $targetHeight, $width, $height);

            // Create a buffer for the resized image
            ob_start();

            // Output the resized image to the buffer
            imagejpeg($resizedImg, null, 90); // Adjust the quality as needed

            // Get the buffer content
            $resizedImageData = ob_get_clean();

            $quality = 90;

            // Check if the resized image meets the target size
            while (strlen($resizedImageData) > ($targetSizeInKB * 1024)) {
                // If the size is still larger than the target, reduce the quality and update the buffer
                $quality -= 10; // Reduce the quality by 10 (adjust as needed)

                ob_start();
                imagejpeg($resizedImg, null, $quality);
                $resizedImageData = ob_get_clean();
            }

            // Encode the resized image to base64
            $resizedBase64Data = base64_encode($resizedImageData);

            // Destroy the image resources
            imagedestroy($img);
            imagedestroy($resizedImg);

            return $resizedBase64Data;
        } catch (Exception $e) {
            // Log or handle the exception as needed
            error_log('Error in resizeBase64Image: ' . $e->getMessage());
            return null; // or handle the error in an appropriate way for your application
        }
    }

    public function employeesView()
    {
        try {
            // Use the stored license number to filter employees
            $businessDetails = $this->getBusinessDetails();
            $liceneNo = $businessDetails['license_no'];
            $emp = EmployeRegister::where('license_no', $liceneNo)->with('paymentSetupData')->get();
            $emp->each(function ($emp) {
                $data = $emp->everyMonthSalaryCalulation();
                $emp->attendanceForCurrentMonth = $data;
            });
            return response()->json([
                'status' => true,
                'message' => "Employee Details Fetched Successfully",
                'data' => $emp,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function addEmployee(Request $request)
    {
        try {
            $businessDetails = $this->getBusinessDetails();
            config(['database.default' => env('DB_CONNECTION')]);
            $validator = Validator::make($request->all(), [
                'form_no' => 'required|string|unique:emp_management,form_no',
                'registration_date' => 'required|date',
                'candidate_name' => 'required|string',
                'email' => 'required|email|unique:emp_management,email',
                'mobile' => 'required|digits:10',
                'dob' => 'required|date',
                'aadharcard_no' => 'required|digits:12|unique:emp_management,aadharcard_no',
                'blood_group' => 'required',
                'helth_issue' => 'required',
                'helth_description' => 'nullable|required_if:healthCheck,Yes|string',
                'marital_status' => 'required|string',
                'vehical_check' => 'required',
                'vehical_no' => 'nullable|required_if:vehivalCheck,Yes|string',
                'father_name' => 'nullable|string',
                'current_address' => 'required|string',
                'current_state' => 'required|string',
                'current_pincode' => 'required|digits:6',
                'parmanent_address' => 'required|string',
                'parmanent_state' => 'required|string',
                'parmanent_pincode' => 'required|digits:6',
                'bank_name' => 'required|string',
                'holder_name' => 'required|string',
                'Ifcs_code' => 'required',
                'account_no' => 'required|string|min:6',
                'companyDetails' => ['nullable', 'string', 'json'],
                'current_designation' => 'required|string',
                'current_department' => 'required|string',
                'current_salary' => 'required|numeric',
                'candidate_signature' => 'required|image|mimes:jpeg,png,jpg',
                'candidate_photo' => 'required|image|mimes:jpeg,png,jpg',
                'aadhar_front' => 'required|image|mimes:jpeg,png,jpg',
                'aadhar_back' => 'required|image|mimes:jpeg,png,jpg',
                'experienceLatter' => 'nullable|file|mimes:pdf|max:5048',
                'bankDetailPdf' => 'nullable|file|mimes:pdf|max:5048',
                'otherDocumets' => 'nullable|file|mimes:pdf|max:5048',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Error',
                    'error' => $validator->errors()
                ], 422);
            }

            $data = $validator->validated();

            // Handle the image upload
            if ($request->hasFile('aadhar_front')) {
                $data['aadhar_front'] = $this->uploadImage($request->file('aadhar_front'));
            }
            if ($request->hasFile('aadhar_back')) {
                $data['aadhar_back'] = $this->uploadImage($request->file('aadhar_back'));
            }
            if ($request->hasFile('candidate_signature')) {
                $data['candidate_signature'] = $this->uploadImage($request->file('candidate_signature'));
            }
            if ($request->hasFile('candidate_photo')) {
                $data['candidate_photo'] = $this->uploadImage($request->file('candidate_photo'));
            }
            $compananyDetails = json_decode($request->input('companyDetails'), true);
            if (!is_array($compananyDetails)) {
                throw new Exception($compananyDetails);
            }
            $companyName = [];
            $companyMobileNo = [];
            $designation = [];
            $startDate = [];
            $endDate = [];

            if (!empty($compananyDetails)) {
                foreach ($compananyDetails as $key => $value) {
                    // Assuming $value is an associative array representing each company detail
                    $companyName[] = $value['companyName'];
                    $companyMobileNo[] = $value['contectNo']; // Assuming it's "contectNo" instead of "contactNo"
                    $designation[] = $value['designation'];
                    $startDate[] = $value['startDate'];
                    $endDate[] = $value['endDate'];
                }
            }

            // Now you have separate arrays for each property
            $data['company_name'] = implode(",", $companyName);
            $data['company_contect'] = implode(",", $companyMobileNo);
            $data['designation'] = implode(",", $designation);
            $data['startDate'] =  implode(",", $startDate);
            $data['endDate'] =  implode(",", $endDate);

            // uploade business details
            $data['username'] = $businessDetails['mobile'];
            $data['username1'] = $businessDetails['email'];
            $data['license_no'] = $businessDetails['license_no'];


            if ($request->hasFile('experienceLatter')) {
                $file = $request->file('experienceLatter');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $data['experiece_letter'] = $filename;
            };
            if ($request->hasFile('bankDetailPdf')) {
                $file = $request->file('bankDetailPdf');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $data['bank_doc'] = $filename;
            };
            if ($request->hasFile('otherDocumets')) {
                $file = $request->file('otherDocumets');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $data['other_doc'] = $filename;
            };
            // Create a new employee record with the validated data
            $employee = EmployeRegister::create($data);
            $this->PaymentSetup($employee->id, $request->current_salary);

            // Compress the images after storing the record
            $this->compressImage($employee->aadhar_front);
            $this->compressImage($employee->aadhar_back);
            $this->compressImage($employee->candidate_signature);
            $this->compressImage($employee->candidate_photo);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Employee registered successfully',
            ], 201);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'somthing went wronge',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function PaymentSetup($id, $salary)
    {
        empPaymentSetup::create([
            'date' => Carbon::now()->format('Y-m-d'),
            'emp_id' => $id,
            'monthly_payment' => $salary,
        ]);
    }

    private function uploadImage($image)
    {
        // Generate a unique name for the image
        $imageName = time() . '_' . uniqid() . '.' . $image->getClientOriginalExtension();

        // Specify the path where the image will be stored
        $path = public_path('assets/uploads/');

        // Move the image to the specified path
        $image->move($path, $imageName);

        // Return the name of the image file
        return $imageName;
    }

    private function compressImage($imageName)
    {
        // Check if image name is provided
        if (!$imageName) {
            return;
        }

        // Specify the path where the image is stored
        $path = public_path('assets/uploads/' . $imageName);

        // Check if the image file exists
        if (!file_exists($path)) {
            return;
        }

        // Get the image content
        $imageContent = file_get_contents($path);

        // Compress the image
        $compressedImageBase64 = $this->resizeBase64Image(base64_encode($imageContent), 100);

        // Decode the compressed image data
        $compressedImageData = base64_decode($compressedImageBase64);

        // Replace the original image with the compressed image
        file_put_contents($path, $compressedImageData);
    }

    public function updateEmployee(Request $request, $id)
    {
        try {
            config(['database.default' => env('DB_CONNECTION')]);
            $employee = EmployeRegister::findOrFail($id);
            $validator = Validator::make($request->all(), [
                'form_no' =>  [
                    'required',
                    'string',
                    Rule::unique('emp_management')->ignore($id),
                ],
                'registration_date' => 'required|date',
                'candidate_name' => 'required|string',
                'email' => [
                    'required',
                    'email',
                    Rule::unique('emp_management')->ignore($id),
                ],
                'mobile' => 'required|digits:10',
                'dob' => 'required|date',
                'aadharcard_no' =>  [
                    'required',
                    'digits:12',
                    Rule::unique('emp_management')->ignore($id),
                ],
                'blood_group' => 'required',
                'helth_issue' => 'required',
                'helth_description' => 'nullable|required_if:healthCheck,Yes|string',
                'marital_status' => 'required|string',
                'vehical_check' => 'required',
                'vehical_no' => 'nullable|required_if:vehivalCheck,Yes|string',
                'father_name' => 'nullable|string',
                'current_designation' => 'required|string',
                'current_department' => 'required|string',
                'current_address' => 'required|string',
                'current_state' => 'required|string',
                'current_pincode' => 'required|digits:6',
                'parmanent_address' => 'required|string',
                'parmanent_state' => 'required|string',
                'parmanent_pincode' => 'required|digits:6',
                'bank_name' => 'required|string',
                'holder_name' => 'required|string',
                'Ifcs_code' => 'required',
                'account_no' => 'required|string|min:6',
                'companyDetails' => ['nullable', 'string', 'json'],
                'aadhar_front' => 'nullable|image|mimes:jpeg,png,jpg',
                'aadhar_back' => 'nullable|image|mimes:jpeg,png,jpg',
                'candidate_signature' => 'nullable|image|mimes:jpeg,png,jpg',
                'candidate_photo' => 'nullable|image|mimes:jpeg,png,jpg',
                'experienceLatter' => 'nullable|file|mimes:pdf|max:5048',
                'bankDetailPdf' => 'nullable|file|mimes:pdf|max:5048',
                'otherDocumets' => 'nullable|file|mimes:pdf|max:5048',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Error',
                    'error' => $validator->errors()
                ], 422);
            }
            $data = $validator->validated();
            // Handle the image upload and update only if new images are provided
            if ($request->hasFile('aadhar_front')) {
                $data['aadhar_front'] = $this->uploadImage($request->file('aadhar_front'));
                $this->compressImage($data['aadhar_front']);
            }
            if ($request->hasFile('aadhar_back')) {
                $data['aadhar_back'] = $this->uploadImage($request->file('aadhar_back'));
                $this->compressImage($data['aadhar_back']);
            }
            if ($request->hasFile('candidate_signature')) {
                $data['candidate_signature'] = $this->uploadImage($request->file('candidate_signature'));
                $this->compressImage($data['candidate_signature']);
            }
            if ($request->hasFile('candidate_photo')) {
                $data['candidate_photo'] = $this->uploadImage($request->file('candidate_photo'));
                $this->compressImage($data['candidate_photo']);
            }
            $compananyDetails = json_decode($request->input('companyDetails'), true);
            if (!is_array($compananyDetails)) {
                throw new Exception($compananyDetails);
            }
            $companyName = [];
            $companyMobileNo = [];
            $designation = [];
            $startDate = [];
            $endDate = [];

            if (!empty($compananyDetails)) {
                foreach ($compananyDetails as $key => $value) {
                    // Assuming $value is an associative array representing each company detail
                    $companyName[] = $value['companyName'];
                    $companyMobileNo[] = $value['contectNo']; // Assuming it's "contectNo" instead of "contactNo"
                    $designation[] = $value['designation'];
                    $startDate[] = $value['startDate'];
                    $endDate[] = $value['endDate'];
                }
            }

            // Now you have separate arrays for each property
            $data['company_name'] = implode(",", $companyName);
            $data['company_contect'] = implode(",", $companyMobileNo);
            $data['designation'] = implode(",", $designation);
            $data['startDate'] =  implode(",", $startDate);
            $data['endDate'] =  implode(",", $endDate);


            if ($request->hasFile('experienceLatter')) {
                $file = $request->file('experienceLatter');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $data['experiece_letter'] = $filename;
            };
            if ($request->hasFile('bankDetailPdf')) {
                $file = $request->file('bankDetailPdf');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $data['bank_doc'] = $filename;
            };
            if ($request->hasFile('otherDocumets')) {
                $file = $request->file('otherDocumets');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $data['other_doc'] = $filename;
            };

            $employee->update($data);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => "Employee Information Updated Successfully!"
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => "Employee Not Found!",
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => "Somthing Went  Wrong!",
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function addUserRoll(Request $request)
    {
        try {
            $businessDetails = $this->getBusinessDetails();
            config(['database.default' => env('DB_CONNECTION')]);

            $validator = Validator::make($request->all(), [
                'form_no' => 'required|string|unique:emp_management,form_no',
                'candidate_name' => 'required|string',
                'email' => 'required|email|unique:emp_management,email',
                'mobile' => 'required|digits:10',
                'user_roll' => 'required|string',
                'permissions' => 'required|array|min:1',
                'permissions.*' => 'string',
                'password' => [
                    'required',
                    'string',
                    'min:8',
                    'confirmed',
                    'regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d)(?=.*[\W_]).+$/',
                ],
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'error' => $validator->errors()
                ], 422);
            }

            $data = $validator->validated();

            // Convert the permissions array to a comma-separated string
            $data['permissions'] = implode(',', $data['permissions']);

            // Add business details
            $data['username'] = $businessDetails['mobile'];
            $data['username1'] = $businessDetails['email'];
            $data['license_no'] = $businessDetails['license_no'];

            // Hash the password before saving
            $data['password'] = Hash::make($data['password']);

            // Create a new user role with the validated data
            $userRole = EmployeRegister::create($data);

            return response()->json([
                'status' => true,
                'message' => 'User role added successfully',
            ], 201);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function updateAddUserRoll(Request $request, $id)
    {
        try {
            config(['database.default' => env('DB_CONNECTION')]);
            $employee = EmployeRegister::findOrFail($id);
            $validator = Validator::make($request->all(), [
                'form_no' => 'required|string|unique:emp_management,form_no,' . $id,
                'candidate_name' => 'required|string',
                'email' => 'required|email|unique:emp_management,email,' . $id,
                'mobile' => 'required|digits:10',
                'user_roll' => 'required|string',
                'permissions' => 'required|array|min:1',
                'permissions.*' => 'string',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'error' => $validator->errors()
                ], 422);
            }
            $data = $validator->validated();
            $data['permissions'] = implode(',', $data['permissions']);
            $employee->update($data);
            return response()->json([
                'status' => true,
                'message' => 'User role updated successfully',
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'Employee not found'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteEmploye(Request $request, $id)
    {
        try {
            $validator = Validator::make($request->all(), [
                'payment_terms_rating' => 'required|numeric',
                'tenant_behavior_rating' => 'required|numeric',
                'responsibilities_rating' => 'required|numeric',
                'additional_comments' => 'nullable|string',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Error',
                    'error' => $validator->errors()
                ], 422);
            }
            // business details
            $businessDetails =  $this->getBusinessDetails();

            $employes = EmployeRegister::findOrFail($id);
            $addNewReview = new Reviews();
            $addNewReview->name = $employes->candidate_name;
            $addNewReview->mobile = $employes->mobile;
            $addNewReview->email =   $employes->email;
            $addNewReview->aadhaar = $employes->aadharcard_no;
            $addNewReview->payment_terms = $request->payment_terms_rating;
            $addNewReview->tenant_behavior = $request->tenant_behavior_rating;
            $addNewReview->responsibilities = $request->responsibilities_rating;
            $addNewReview->additional = $request->additional_comments;
            $addNewReview->hostelname = $businessDetails['license_no'];
            $addNewReview->save();
            //Delete employes
            $employes->delete();
            return response()->json([
                "status" => true,
                'statusCode' => 200,
                "message" => "Employee Records deleted Successfully"
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'Employee not found'
            ], 404);
        } catch (Exception $e) {
            return response()->json(["status" => false, "error" => $e->getMessage()], 500);
        }
    }

    public function generateUniqueFormNo()
    {
        try {
            $latestFormNo = EmployeRegister::orderBy('id','desc')
            ->value('form_no');
            $newFormNo = $latestFormNo ? $latestFormNo + 1 : 1;
            return response()->json([
                'status' => true,
                'message' => 'successfuly get the employe form number',
                'data' => ['formNumber' => $newFormNo]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function makeAllPermission(Request $request)
    {
        try {
            $permissions = [
                [
                    'name' => 'Home',
                    'value' => 'Home'
                ],
                [
                    'name' => 'Room/Seats',
                    'value' => 'RoomSeats'
                ],
                [
                    'name' => 'Registration',
                    'value' => 'Registration'
                ],
                [
                    'name' => 'Business Dashboard',
                    'value' => 'BusinessDashboard'
                ],
                [
                    'name' => 'Expenses',
                    'value' => 'Expenses'
                ],
                [
                    'name' => 'Payments',
                    'value' => 'Payments'
                ],
                [
                    'name' => 'Sale/Tenant',
                    'value' => 'SaleTenant'
                ],
                [
                    'name' => 'Tenant Board',
                    'value' => 'TenantBoard',
                ],
                [
                    'name' => 'Tenant Complaint',
                    'value' => 'TenantComplaint',
                ],
                [
                    'name' => 'Tenant Notice',
                    'value' => 'TenantNotice'
                ],
                [
                    'name' => 'Lead Manager',
                    'value' => 'LeadManager'
                ],
                // [
                //     'name' => 'Reports',
                //     'value' => 'Reports'
                // ],
                // [
                //     'name' => 'Sales Reports',
                //     'value' => 'SalesReports'
                // ],
                // [
                //     'name' => 'Expense Reports',
                //     'value' => 'Expenses Reports'
                // ],
                [
                    'name' => 'Online Business',
                    'value' => 'OnlineBusiness'
                ],
                [
                    'name' => 'Backup/Restore',
                    'value' => 'BackupRestore'
                ],
                [
                    'name' => 'Download Report',
                    'value' => 'ReportsDownload'
                ],
                [
                    'name' => 'Electricity',
                    'value' => 'Electricity',
                ],
                [
                    'name' => 'Setting',
                    'value' => 'Setting'
                ],
                [
                    'name' => 'Employees',
                    'value' => 'Employees',
                ],
                [
                    "name" => 'Add User Roll',
                    'value' => 'addUserRoll',
                ],
                [
                    'name' => 'Change Theme',
                    'value' => 'ChangeTheme'
                ],
                [
                    'name' => 'Notification',
                    'value' => 'Notifiction'
                ],
                [
                    'name' => 'Languages',
                    'value' => 'Languages'
                ],
                [
                    'name' => 'Change Password',
                    'value' => 'ChangePassword'
                ],
                [
                    'name' => 'Permisions',
                    'value' => 'Permissions'
                ],
                [
                    'name' => 'Help & Support',
                    'value' => 'HelpSupport'
                ],
                [
                    'name' => 'Refer & Earn',
                    'value' => 'ReferEarn'
                ],
                [
                    'name' => 'Policies',
                    'value' => 'Policies'
                ],
                //quick action
                [
                    'name' => 'Add Attendance',
                    'value' => 'addAttendance'
                ],
                [
                    'name' => 'Add Expenses',
                    'value' => 'AddExpenses'
                ],
                [
                    'name' => 'Take Payment',
                    'value' => 'TakePayment'
                ],
                [
                    'name' => 'Add Menu',
                    'value' => 'AddMenu'
                ],
                [
                    'name' => 'Student Booking',
                    'value' => 'Booking'
                ],
                [
                    'name' => 'Add Sale',
                    'value' => 'AddSale'
                ],
                [
                    'name' => 'Add Items',
                    'value' => 'AddItems'
                ],
                [
                    'name' => 'Uploads',
                    'value' => 'Uploads'
                ],
                [
                    'name' => 'Stock Management',
                    'value' => 'StockManagement'
                ],
                [
                    'name' => 'Add Hostel/PG',
                    'value' => 'AddHostelPG'
                ],
                [
                    'name' => 'Subscription',
                    'value' => 'Subscription'
                ],
                [
                    'name' => 'DownLoad Reports',
                    'value' => 'ReportsDownload'
                ]
            ];
            $userPermissions =  request()->attributes->get('permissions');
            $ValidPermissions = [];
            foreach ($permissions as $permission) {
                if (in_array($permission['value'], $userPermissions)) {
                    array_push($ValidPermissions, $permission);
                }
            }
            return response()->json([
                'status' => true,
                'data' => $ValidPermissions,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
