<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use Illuminate\Support\Str;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Exception;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Carbon;
use App\Models\ApiModels\ExpensesItem;
use Illuminate\Validation\Rule;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use App\Models\ApiModels\Expense;
use App\Models\ApiModels\ExpenseDetails;


class ExpensesController extends controller
{
    public function getExpensesItems()
    {
        try {
            $data = [];
            $items = ExpensesItem::all();
            if (empty($items)) {
                return response()->json([
                    'status' => false,
                    'error' => 'Data Not Found'
                ], 404);
            }
            foreach ($items as $item) {
                $data[] = [
                    'id' => $item->id,
                    'itemCode' => $item->itemcode,
                    'itemName' => $item->itemname,
                    'rate' => $item->rate,
                    'tax' => $item->tax,
                    'amount' => $item->amount
                ];
            }

            return response()->json([
                'status' => true,
                'data' => $data,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                "status" => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function expensesCetegorys()
    {
        try {
            $data = Expense::select('id', 'category_name')->get();
            return response()->json([
                'status' => true,
                'data' => $data,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Somthing Went Wronge',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function createItem(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'itemCode' => 'required|unique:items',
            'itemName' => 'required',
            'rate' => 'required|numeric',
            'tax' => 'required|numeric',
            'amount' => 'required|numeric',
        ], [
            'itemCode.required' => 'Item Code is required!',
            'itemCode.unique' => 'Item Code must be unique!',
            'itemName.required' => 'Item Name is required!',
            'rate.required' => 'Rate is required!',
            'rate.numeric' => 'Rate must be a number!',
            'tax.required' => 'Tax is required!',
            'tax.numeric' => 'Tax must be a number!',
            'amount.required' => 'Amount is required',
            'amount.numeric' => 'Amount must be a number!'
        ]);

        try {
            $validator->validate();
            $item = ExpensesItem::create([
                'itemcode' => $request->itemCode,
                'itemname' => $request->itemName,
                'rate' => $request->rate,
                'tax' => $request->tax,
                'amount' => $request->amount,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Item created successfully!',
            ], 201);
        } catch (ValidationException $e) {
            return response()->json(['status' => false, 'error' => $e->errors()], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }



    public function updateItem(Request $request, $itemId)
    {
        try {
            // Get the item by id from the database
            $item = ExpensesItem::findOrFail($itemId);

            // Validation rules
            $validator = Validator::make($request->all(), [
                'itemCode' => [
                    'required',
                    Rule::unique('items')->ignore($itemId),
                ],
                'itemName' => 'required',
                'rate' => 'required|numeric',
                'tax' => 'required|numeric',
                'amount' => 'required|numeric',
            ], [
                'itemCode.required' => 'Item Code is required!',
                'itemCode.unique' => 'Item Code must be unique!',
                'itemName.required' => 'Item Name is required!',
                'rate.required' => 'Rate is required!',
                'rate.numeric' => 'Rate must be a number!',
                'tax.required' => 'Tax is required!',
                'tax.numeric' => 'Tax must be a number!',
                'amount.required' => 'Amount is required',
                'amount.numeric' => 'Amount must be a number!',
            ]);

            // Validate the request data
            $validator->validate();

            // Update the item
            $item->update([
                'itemcode' => $request->itemCode,
                'itemname' => $request->itemName,
                'rate' => $request->rate,
                'tax' => $request->tax,
                'amount' => $request->amount,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Item updated successfully!'
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No item found with the provided id.'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function deleteItem($itemId)
    {
        try {
            // Get the item by id from the database
            $item = ExpensesItem::findOrFail($itemId);

            // Delete the item
            $item->delete();

            return response()->json([
                'status' => true,
                'message' => 'Item deleted successfully!'
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No item found with the provided id.'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    // expenses View

    public function getTotalAmountByCategory(Request $request)
    {
        try {
            $startDate = $request->input('startDate');
            $endDate = $request->input('endDate');

            $TotalAmounts = 0;
            if (!empty($startDate) && !empty($endDate)) {
                $expenses = Expense::with('details')
                    ->whereHas('details', function ($query) use ($startDate, $endDate) {
                        $query->whereBetween('billdate', [$startDate, $endDate]);
                    })
                    ->orderBy("id", 'desc')
                    ->get();
            } else {
                $expenses = Expense::with('details')
                    ->orderBy("id", 'desc')
                    ->get();
            }
            $result = [];
            foreach ($expenses as $expense) {
                // Filter expenses based on date range
                if (!empty($startDate) && !empty($endDate)) {
                    $filteredDetails = $expense->details->whereBetween('billdate', [$startDate, $endDate]);
                    $categoryAmount = $filteredDetails->sum('amount');
                } else {
                    $categoryAmount = $expense->details->sum('amount');
                }

                $TotalAmounts += $categoryAmount;

                $result["TableData"][] = [
                    "id" => $expense->id,
                    'name' => $expense->category_name,
                    "value" => $categoryAmount
                ];
            }

            $result["TotalAmount"] = [
                'name' => "Total Amount",
                "value" => $TotalAmounts,
            ];

            return response()->json([
                'status' => true,
                'data' => $result,
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    public function createExpensesCategory(Request $request)
    {
        try {
            // Validate request data
            $validator = Validator::make($request->all(), [
                'category_name' => 'required|unique:expenses',
            ], [
                'category_name.required' => 'Category name is required!',
                'category_name.unique' => 'Category name must be unique!',
            ]);

            // Validate the request data
            $validator->validate();

            // Create the expense category
            $expense = Expense::create([
                'category_name' => $request->category_name,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Expense category created successfully!',
            ], 201); // Use 201 status code for "Created"


        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateExpenseCategory(Request $request, $categoryId)
    {
        try {
            // Validate request data
            $validator = Validator::make($request->all(), [
                'category_name' => 'required|unique:expenses,category_name,' . $categoryId,
            ], [
                'category_name.required' => 'Category name is required!',
                'category_name.unique' => 'Category name must be unique!',
            ]);

            // Validate the request data
            $validator->validate();

            // Find the expense category
            $expense = Expense::findOrFail($categoryId);

            // Update the category name
            $expense->update([
                'category_name' => $request->category_name,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Expense category updated successfully!',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteExpenseCategory($categoryId)
    {
        try {
            // Find the expense category
            $expense = Expense::findOrFail($categoryId);

            // Check if the category has associated details
            if ($expense->details()->exists()) {
                return response()->json([
                    'status' => false,
                    'error' => 'Cannot delete category with associated details!',
                ], 422);
            }

            // Delete the category
            $expense->delete();

            return response()->json([
                'status' => true,
                'message' => 'Expense category deleted successfully!',
            ], 200);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    // category / items
    // get all items associate to the categoryId
    public function getCategoryItmes(Request $request, $categoryId)
    {
        try {
            $data = [];
            $startDate = $request->input('startDate');
            $endDate = $request->input('endDate');
            $expense = Expense::findOrFail($categoryId);
            if (!empty($startDate) && !empty($endDate)) {
                $details = ExpenseDetails::where('category_id', $categoryId)->whereBetween('billdate', [$startDate, $endDate])->orderBy('id', 'desc')->get();
            } else {
                $details = ExpenseDetails::where('category_id', $categoryId)->orderBy('id', 'desc')->get();
            }
            foreach ($details as $row) {

                $filePath = $row->uploadbill ? asset('assets/uploads/' . $row->uploadbill) : null;

                $data[] = [
                    'id' => $row->id,
                    'quantity' => $row->quantity,
                    'amount' => $row->amount,
                    'billdate' => $row->billdate,
                    'itemname' => $row->itemname,
                    'rate' => $row->rate,
                    'paymentmode' => $row->paymentmode,
                    'uploadbill' => file_exists(public_path('assets/uploads/' . $row->uploadbill)) ? $filePath : null,
                    'category_id' => $row->category_id
                ];
            }
            return response()->json([
                'status' => true,
                'data' => $data,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateCategoryItemDetails(Request $request, $itemId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'category_id' => 'required',
                'quantity' => 'required|numeric',
                'amount' => 'required|numeric',
                'billdate' => 'required|date',
                'itemname' => 'required',
                'rate' => 'required|numeric',
                'paymentmode' => 'required|string',
                'uploadbill' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'error' => $validator->errors(),
                ], 422);
            }

            // Find the expense detail by its ID
            $expenseDetail = ExpenseDetails::findOrFail($itemId);
            // Handle image upload
            $filename = null;
            if ($request->hasFile('uploadbill') && ($request->uploadbill != null)) {
                $uploadedImage = $request->file('uploadbill');
                if ($uploadedImage->getError() !== UPLOAD_ERR_OK) {
                    throw new \Exception("File upload error: " . $uploadedImage->getErrorMessage());
                }
                $filename = Str::uuid() . '.' . rand() . '.' . $uploadedImage->getClientOriginalExtension();
                $uploadedImage->move('assets/uploads/', $filename);
            }
            // Update the expense detail
            $expenseDetail->update([
                'category_id' => $request->category_id,
                'quantity' => $request->quantity,
                'amount' => $request->amount,
                'billdate' => $request->billdate,
                'itemname' => $request->itemname,
                'rate' => $request->rate,
                'paymentmode' => $request->paymentmode,
                'uploadbill' => $request->hasFile('uploadbill') ? $filename : $expenseDetail->uploadbill,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Category item details updated successfully!',
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function createCategoryItemDetails(Request $request)
    {
        try {
            // Validation of request data
            $validator = Validator::make($request->all(), [
                'category_id' => 'required',
                'quantity' => 'required|numeric',
                'amount' => 'required|numeric',
                'billdate' => 'required|date',
                'itemname' => 'required',
                'rate' => 'required|numeric',
                'paymentmode' => 'required|string',
                'uploadbill' => 'nullable|image|mimes:jpeg,png,jpg|max:2048',
            ], [
                'category_id.required' => 'Category ID is required.',
                'quantity.required' => 'Item Quantity is required.',
                'quantity.numeric' => 'Item Quantity must be numeric.',
                'amount.required' => 'Item Amount is required.',
                'amount.numeric' => 'Item Amount must be numeric.',
                'billdate.required' => 'Billing Date is required.',
                'billdate.date' => 'Invalid Billing Date format.',
                'itemname.required' => 'Item Name is required.',
                'rate.required' => 'Rate is required.',
                'rate.numeric' => 'Rate must be numeric.',
                'paymentmode.required' => 'Payment Mode is required.',
                'paymentmode.string' => 'Invalid Payment Mode format.',
                'uploadbill.image' => 'Uploaded file must be an image.',
                'uploadbill.mimes' => 'Invalid file format. Supported formats: jpeg, png, jpg.',
                'uploadbill.max' => 'File size cannot exceed 2048 KB.',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'error' => $validator->errors(),
                ], 422);
            }

            // Handle image upload
            $filename = null;
            if ($request->hasFile('uploadbill')) {
                $uploadedImage = $request->file('uploadbill');
                $filename = Str::uuid() . '.' . rand() . '.' . $uploadedImage->getClientOriginalExtension();
                $uploadedImage->move('assets/uploads/', $filename);
            }

            // Create a new expense detail with the provided data
            $newExpenseDetail = ExpenseDetails::create([
                'category_id' => $request->category_id,
                'quantity' => $request->quantity,
                'amount' => $request->amount,
                'billdate' => $request->billdate,
                'itemname' => $request->itemname,
                'rate' => $request->rate,
                'paymentmode' => $request->paymentmode,
                'uploadbill' => $filename,
            ]);

            return response()->json([
                'status' => true,
                'message' => 'Category item details created successfully!',
            ], 201);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function deleteCategoryItem($itemId)
    {
        try {
            // Find the expense detail by its ID
            $expenseDetail = ExpenseDetails::findOrFail($itemId);

            // Delete the expense detail
            $expenseDetail->delete();

            return response()->json([
                'status' => true,
                'message' => 'Category item deleted successfully!',
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No category item found with the provided ID.',
            ], 404);
        } catch (\Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
