<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Exception;
use Illuminate\Validation\ValidationException;
use App\Models\Businessregister;
use App\Models\ApiModels\RoomSeats;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use App\Models\ApiModels\SelfStudentRegister;
use App\Models\ApiModels\OldStudentsRegister;
use UnexpectedValueException;
use App\Models\ApiModels\StudentBooking;
use App\Models\ApiModels\BusinessDocument;

class infogetController extends controller
{
    public function getRoomNo()
    {
        try {
            $roomdata = [];
            $rooms = RoomSeats::select("roomno")->get();
            if (!empty($rooms)) {
                foreach ($rooms as $roomno) {
                    $roomdata[] = $roomno->roomno;
                }
                return response()->json([
                    'status' => true,
                    'rooms' => $roomdata
                ], 200);
            } else {
                throw new Exception("Records not found");
            }
        } catch (Exception $e) {
            return response()->json([
                "status" => false,
                "error" => $e->getMessage()
            ], 500);
        }
    }

    public function getSeatNo($roomno)
    {
        try {
            $allSeats = RoomSeats::select('numberseats')->where('roomno', $roomno)->first();

            if (empty($allSeats)) {
                throw new UnexpectedValueException("The Room number {$roomno} is not valid.");
            }

            // Get all booked seats for the room
            $studentCheck = StudentRegister::where('roomno', $roomno)
                ->where('status', 1)
                ->pluck('seetno')
                ->toArray();

            // Generate an array of available seats
            $availableSeats = array_values(array_diff(range(1, $allSeats->numberseats), $studentCheck));

            return response()->json([
                'status' => true,
                'data' => $availableSeats,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function checkBusinessEmail($email)
    {
        try {
            // Validate email format
            if (!filter_var($email, FILTER_VALIDATE_EMAIL)) {
                throw new \InvalidArgumentException('Invalid email format');
            }

            $checkEmail = Businessregister::where("email", $email)->first();

            if (empty($checkEmail)) {
                return response()->json([
                    "status" => true,
                    "message" => "This email is available."
                ]);
            } else {
                throw new ModelNotFoundException("This Email is already in use");
            }
        } catch (\InvalidArgumentException $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 400);
        } catch (ModelNotFoundException $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 404);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 500);
        }
    }

    public function CheckBusinessMobile($mobile)
    {
        try {
            // Validate mobile number format (customize as per your requirements)
            if (!preg_match('/^[0-9]{10}$/', $mobile)) {
                throw new \InvalidArgumentException('Invalid mobile number format');
            }

            $checkMobile = Businessregister::where("mobile", $mobile)->first();

            if (empty($checkMobile)) {
                return response()->json([
                    "status" => true,
                    "message" => "This mobile no. is available."
                ]);
            } else {
                throw new ModelNotFoundException("This Mobile No. is already in use");
            }
        } catch (\InvalidArgumentException $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 400);
        } catch (ModelNotFoundException $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 404);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 500);
        }
    }

    public function checkBusinessName($businessName)
    {
        try {
            if (empty($businessName)) {
                throw new \InvalidArgumentException('Business name cannot be empty');
            }

            $checkBusinessName = Businessregister::where("businessname", $businessName)->first();

            if (empty($checkBusinessName)) {
                return response()->json([
                    'status' => true,
                    "message" => "This Business Name is available.",
                ]);
            } else {
                throw new ModelNotFoundException("This Business Name is already in use");
            }
        } catch (\InvalidArgumentException $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 400);
        } catch (ModelNotFoundException $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 404);
        } catch (\Exception $e) {
            return response()->json(['status' => false, 'message' => $e->getMessage()], 500);
        }
    }

    public function getFormNo()
    {
        try {
            $formNo = StudentRegister::select("formno")
                ->orderBy("id", "DESC")
                ->first();
            if (!empty($formNo)) {
                $newFormNo = $formNo->formno + 1;
            } else {
                $newFormNo = 1;
            }
            return response()->json([
                "status" => true,
                "formno" => $newFormNo
            ]);
        } catch (Exception $e) {
            return response()->json(["status" => false, "message" => $e->getMessage()]);
        }
    }

    public function getRegistrationDetails()
    {
        try {
            $totalRegistration = StudentRegister::count();
            $runningRegistration = StudentRegister::where("status", 1)->count();
            $selfStudentRegister = SelfStudentRegister::count();
            $oldStudentRegister = OldStudentsRegister::count();
            $vacantReg = RoomSeats::sum('numberseats');
            $totalVacant = $vacantReg - $runningRegistration;

            return response()->json([
                "status" => true,
                "data" => [
                    [
                        "title" => "Total Registration",
                        'value' => $totalRegistration,
                    ],
                    [
                        "title" => "Running Registration",
                        'value' => $runningRegistration,
                    ],
                    [
                        "title" => "Old Registration",
                        'value' => $oldStudentRegister,
                    ],
                    [
                        "title" => "Vacant Registration",
                        'value' => $totalVacant,
                    ],
                    [
                        "title" => "Registration Request",
                        'value' => $selfStudentRegister,
                    ]
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                "status" => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function getAvailableSeats($roomNumber)
    {
        try {
            // Get the room first 
            $allSeats = RoomSeats::select('numberseats')->where('roomno', $roomNumber)->first();

            if (empty($allSeats)) {
                throw new UnexpectedValueException("The Room number {$roomNumber} is not valid.");
            }

            // Get all booked seats for the room
            $bookedSeats = StudentBooking::where('roomno', $roomNumber)
                ->where('status', 1)
                ->pluck('seatno')
                ->toArray();

            // Generate an array of available seats
            $availableSeats = array_values(array_diff(range(1, $allSeats->numberseats), $bookedSeats));

            return response()->json([
                'status' => true,
                'data' => $availableSeats,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function getTermConditionPDf()
    {
        try {
            $rulesDocument = BusinessDocument::where("upload_name", 'Rules & Regulations')->first();
            $signatureDocument = BusinessDocument::where("upload_name", 'Stamp With Signature')->first();

            if (empty($rulesDocument) && empty($signatureDocument)) {
                throw new ModelNotFoundException('Sorry! Documents do not exist.');
            }

            $rulesFile = ($rulesDocument->upload_pdf) ? asset('assets/uploads/pdf/' . $rulesDocument->upload_pdf) : null;
            $signatureFile = ($signatureDocument->upload_pdf) ? asset('assets/uploads/pdf/' . $signatureDocument->upload_pdf) : null;

            return response()->json([
                'status' => true,
                'rules_file' => $rulesFile,
                'signature_file' => $signatureFile,
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => "getting error",
            ], 500);
        }
    }
}
