<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Carbon;
use UnexpectedValueException;
use Exception;
use Illuminate\Validation\ValidationException;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use App\Models\ApiModels\ExpensesItem;
use Throwable;

class ItemsController  extends Controller
{
    public function getAllItems()
    {
        try {
            $items = ExpensesItem::all();
            return response()->json([
                'status' => true,
                'message' => 'sales item get successfully',
                'data' => $items,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function addItem(Request $request)
    {

        $validator = Validator::make($request->all(), [
            'itemcode' => [
                'required',
                'string',
            ],
            'itemname' => 'required|string',
            'rate' => 'required|numeric',
            'tax' => 'required|numeric',
            'amount' => 'required|numeric'
        ]);

        if ($validator->fails()) {
            return response()->json([
                'status' => false,
                'error' => $validator->errors()
            ], 422);
        }
        try {
            $validatedData = $validator->validated();
            $item = ExpensesItem::create($validatedData);

            return response()->json([
                'status' => true,
                "message" => 'New Item Created Successfully',
            ], 201);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }


    public function updateItem(Request $request, $itemId)
    {
        try {
            $item = ExpensesItem::findOrFail($itemId);

            $validator = Validator::make($request->all(), [
                'itemcode' => 'required|string|unique:items,itemcode,' . $item->id,
                'itemname' => 'required|string',
                'rate' => 'required|numeric',
                'tax' => 'required|numeric',
                'amount' => 'required|numeric'
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'errors' => $validator->errors()
                ], 422);
            }

            $item->update($validator->validated());

            return response()->json([
                'status' => true,
                'message' => 'Item updated successfully',
                'data' => $item
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'Item not found'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function deleteItem($itemId)
    {
        try {
            $item = ExpensesItem::findOrFail($itemId);

            $item->delete();

            return response()->json([
                'status' => true,
                'message' => 'Item deleted successfully'
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'Item not found'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
