<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Illuminate\Support\Carbon;
use UnexpectedValueException;
use Exception;
use Illuminate\Validation\ValidationException;
use DB;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use InvalidArgumentException;
use App\Models\ApiModels\LeadManager;

class LeadController extends controller
{
    public function getLeadData()
    {
        try {
            $leads = LeadManager::all();
            $totallead = $leads->count();
            $acceptedlead = LeadManager::where('leadstatus', 0)->count();
            $unacceptedlead = LeadManager::where('leadstatus', '1')->count();
            $currentDate = date('Y-m-d');
            $todaylead = LeadManager::whereDate('created_at', $currentDate)->count();
            return response()->json([
                'status' => true,
                'data' => [
                    'mainData'=>[
                        [
                            'title' =>'Total Lead',
                            'value' => $totallead
                        ],
                        [
                            'title' =>'Today Lead',
                            'value' => $todaylead 
                        ],
                        [
                            'title' =>'Taken Lead',
                            'value' => $acceptedlead 
                        ],
                        [
                            'title' =>'Not Taken Lead',
                            'value' => $unacceptedlead 
                        ]
                    ],
                    'tableData' => $leads,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function addNewLead(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'leaddate' => 'required|date',
                'leadcandidatename' => 'required|string',
                'leadmobile' => 'required|regex:/^[0-9]{10}$/',
                'leadmail' => 'required|email',
                'requirement' => 'required',
                'budget' => 'required|numeric',
                'location' => 'required',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Your logic for handling a valid request
            LeadManager::create($validator->validated());

            return response()->json([
                'status' => true,
                'message' => 'Lead added successfully.',
            ]);
        } catch (ValidationException $e) {
            // Handle validation errors
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            // Handle other exceptions
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function deletelead($leadId)
    {
        try {
            $lead = LeadManager::findOrFail($leadId);
            $lead->delete();
            return response()->json([
                'status' => true,
                'message' => "Successfully deleted lead."
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such lead found'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function updateLeadStatus($leadId)
    {
        try {
            $lead = LeadManager::findOrFail($leadId);
            $lead->update([
                'leadstatus' => 1,
            ]);
            return response()->json([
                'status' => true,
                'message' => "Successfully updated lead status."
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such lead found'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }
}
