<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Exception;
use App\Models\Businessregister;
use DB;
use Illuminate\Support\Facades\Log;
use Dompdf\Dompdf;
use App\Models\ApiModels\BusinessDocument;
use App\Models\ApiModels\Sales;
use App\Models\ApiModels\RecieveAmount;
use App\Models\ApiModels\CustomeSales;
use App\Models\ApiModels\ExpensesItem;
use App\Models\ApiModels\TermsAndConditions;
use App\Models\ApiModels\Expense;
use App\Models\ApiModels\ExpenseDetails;
use App\Models\ApiModels\RoomSeats;
use App\Models\ApiModels\OldStudentsRegister;
use Carbon\Carbon;
use PhpParser\Node\Expr\FuncCall;
use App\Models\RoomSeat;
use App\Models\ElectricityManagement;
use App\Models\StockItem;
use App\Models\stock_detail;
use App\Models\StockCategory;
use App\Models\Uploadprofile;
use App\Models\EmpManagement;
use Dompdf\Options;




require_once public_path('assets/fpdi/src/autoload.php');

$dompdfPath = public_path('assets/dompdf');
set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
require_once $dompdfPath . '/autoload.inc.php';

class PdfController extends Controller
{
    private function checkWhatsAppPermission(): bool
    {
        try {
            $permission = Permissions::where('permissionType', 'Desktop Whatsapp')->value('permission');
            return $permission ?? false; // Return the permission value or false if not found

        } catch (ModelNotFoundException $e) {
            return false; // Handle the case when the permission is not found

        } catch (Exception $e) {
            // Log or handle other exceptions
            return false;
        }
    }
    private function numberToWords($number)
    {
        $words = [
            'Zero', 'One', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight', 'Nine', 'Ten',
            'Eleven', 'Twelve', 'Thirteen', 'Fourteen', 'Fifteen', 'Sixteen', 'Seventeen', 'Eighteen', 'Nineteen'
        ];

        $tens = [
            '', '', 'Twenty', 'Thirty', 'Forty', 'Fifty', 'Sixty', 'Seventy', 'Eighty', 'Ninety'
        ];

        if ($number < 20) {
            return $words[$number];
        } elseif ($number < 100) {
            return $tens[(int)($number / 10)] . (($number % 10 !== 0) ? ' ' . $words[$number % 10] : '');
        } elseif ($number < 1000) {
            return $words[(int)($number / 100)] . ' Hundred' . (($number % 100 !== 0) ? ' and ' . $this->numberToWords($number % 100) : '');
        } elseif ($number < 100000) {
            return $this->numberToWords((int)($number / 1000)) . ' Thousand' . (($number % 1000 !== 0) ? ' ' . $this->numberToWords($number % 1000) : '');
        } elseif ($number < 10000000) {
            return $this->numberToWords((int)($number / 100000)) . ' Lakh' . (($number % 100000 !== 0) ? ' ' . $this->numberToWords($number % 100000) : '');
        } else {
            return 'Number out of range';
        }
    }
    private function prepareSalesItemList($sales)
    {
        $salesItems = [];
        $itemIds = explode(",", $sales->item);
        $itemRates = explode(",", $sales->rate);
        $itemQuantities = explode(",", $sales->quantity);
        $itemTax = explode(",", $sales->tax);
        $itemDiscount = explode(",", $sales->discount);
        $itemAmount = explode(",", $sales->amount);
        foreach ($itemIds as $key => $item) {
            if ($item === "0") {
                $itemName = "Room Rent";
            } elseif ($item === "e01") {
                $itemName = 'Electricity Bill';
            } else {
                $itemName = ExpensesItem::where('id', $item)->value('itemname');
            }
            $salesItems[] = [
                'itemId' => $item,
                'itemName' => $itemName,
                'itemRate' => $itemRates[$key],
                'itemQuantity' => $itemQuantities[$key],
                'itemTax' => $itemTax[$key],
                'itemDiscount' => $itemDiscount[$key],
                'itemAmount' => $itemAmount[$key],
            ];
        }

        return $salesItems;
    }
    public function createStudentRegFormDownload($studentId)
    {
        try {
            $studentData = StudentRegister::findOrFail($studentId);
            $bussinessData = Businessregister::firstOrFail();
            $stapWithSign = BusinessDocument::where('upload_name', 'Stamp With Signature')->first();
            // Load the HTML content from the Blade view
            $view = view('pdf.studentRegistrationForm-pdf', compact('bussinessData', 'studentData', 'stapWithSign'))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'registration_form_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);
            file_put_contents($filePath, $output);

            // Check the size of the PDF file
            $pdfSize = filesize($filePath);

            // If the size exceeds 1MB, resize it
            if ($pdfSize > 1048576) { // 1MB in bytes
                // Calculate the resize percentage (you can adjust this as needed)
                $resizePercentage = 0.5; // 50%

                // Resize the PDF
                $this->resizePDF($filePath, $resizePercentage);
            }

            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => 'Registration form PDF created successfully.',
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }

    private function resizePDF($pdfPath, $resizePercentage)
    {
        // Load the PDF using DomPDF
        $dompdf = new Dompdf();
        $dompdf->loadHtml(file_get_contents($pdfPath));

        // Resize the PDF
        $canvas = $dompdf->get_canvas();
        $canvas->scale($resizePercentage);

        // Render the resized PDF
        $dompdf->render();

        // Save the resized PDF back to the same file
        file_put_contents($pdfPath, $dompdf->output());
    }

    public function downloadStudentDocuments($studentId)
    {
        try {
            $studentData = StudentRegister::findOrFail($studentId);
            $bussinessData = Businessregister::firstOrFail();
            $view = view('pdf.studentDocuments-pdf', compact('bussinessData', 'studentData'))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'studentDocuments_' . implode("_", explode(" ", $studentData->candidatename)) . '_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);
            file_put_contents($filePath, $output);

            // Optionally, you can save the filename to the database or log it

            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Student Documents Pdf created successfully for student id {$studentId}.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }
    public function genrateSalesInvoice($invoiceNo)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $termAndCondition = TermsAndConditions::all();
            $stapWithSign = BusinessDocument::where('upload_name', 'Stamp With Signature')->first();
            $saleData = Sales::where('invoiceno', $invoiceNo)->with('student')->firstOrFail();
            $studentData = $saleData->student;
            $numberToWrods = $this->numberToWords($saleData->total);
            $itemList = $this->prepareSalesItemList($saleData);

            $totalReceivedAmountInallBills = Sales::where('studentid', $saleData->studentid)
                ->where('roomno', $saleData->roomno)
                ->sum('remaining_amount');

            $PreviousBalance = $totalReceivedAmountInallBills - ($saleData->remaining_amount);
            $CurrentBalance =  $totalReceivedAmountInallBills - ($saleData->remaining_amount) + ($saleData->remaining_amount);

            $view = view('pdf.studentSalesInvoice-pdf', compact('bussinessData', 'saleData', 'itemList', 'studentData', 'numberToWrods', 'termAndCondition', 'PreviousBalance', 'CurrentBalance', 'stapWithSign'))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'SaleInvoice_InvoiceNo' . $invoiceNo . '_' . implode("_", explode(" ", $studentData->candidatename)) . '_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/pdf/StudentSalesInvoice/' . $filename);
            file_put_contents($filePath, $output);

            $link = asset('assets/uploads/pdf/StudentSalesInvoice/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Sales Report Create For invoice number {$invoiceNo}.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }
    public function genrateCustomSaleInvoice($invoiceNo)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $termAndCondition = TermsAndConditions::all();
            $stapWithSign = BusinessDocument::where('upload_name', 'Stamp With Signature')->first();
            $saleData = CustomeSales::where('invoiceno', $invoiceNo)->firstOrFail();
            $numberToWrods = $this->numberToWords($saleData->total);
            $itemList = $this->prepareSalesItemList($saleData);
            $studentData = null;
            $PreviousBalance = ($saleData->total) - ($saleData->remaining_amount);
            $CurrentBalance =  ($saleData->total) - ($saleData->remaining_amount);

            $view = view('pdf.studentSalesInvoice-pdf', compact('bussinessData', 'saleData', 'itemList', 'numberToWrods', 'studentData', 'termAndCondition', 'PreviousBalance', 'CurrentBalance', 'stapWithSign'))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'Custom_SaleInvoice_InvoiceNo_' . $invoiceNo . '_' . implode("_", explode(" ", $saleData->name)) . '_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/pdf/StudentSalesInvoice/' . $filename);
            file_put_contents($filePath, $output);

            // Optionally, you can save the filename to the database or log it

            // Generate a shareable link
            $link = asset('assets/uploads/pdf/StudentSalesInvoice/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Sales Report Create For invoice number {$invoiceNo}.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function genrateExpeseReport(Request $request)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $startDate = '';
            $endDate = '';
            if ($request->has('startDate') && $request->has('endDate')) {
                $startDate = $request->input('startDate');
                $endDate = $request->input('endDate');

                $expensesData = Expense::whereHas('details', function ($query) use ($startDate, $endDate) {
                    $query->whereBetween('billdate', [$startDate, $endDate]);
                })->with(['details' => function ($query) use ($startDate, $endDate) {
                    $query->whereBetween('billdate', [$startDate, $endDate]);
                }])->get();
            } else {
                $expensesData = Expense::with('details')->get();
            }
            $view = view('pdf.expenseReport-pdf', compact('bussinessData', 'expensesData', 'startDate', 'endDate'))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'ExpenseReport_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);
            file_put_contents($filePath, $output);

            // Optionally, you can save the filename to the database or log it

            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Expense Report created successfully.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }

    private function getTotalRemaningAmount($studentId)
    {
        return Sales::where('studentid', $studentId)->sum('remaining_amount');
    }
    public function genrateRegistrationReport()
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $RoomSeat = RoomSeats::get();

            $totalRegistration = StudentRegister::count();
            $runningRegistration = StudentRegister::where('status', 1)->count();
            $oldRegistration = OldStudentsRegister::where('status', 0)->count();
            $VacantRegistration = RoomSeats::sum('numberseats');

            $studentData = [];
            foreach ($RoomSeat as $key => $value) {
                $roomDetails = [
                    'roomno' => $value->roomno,
                    'roomType' => $value->roomtype,
                    'totalSeats' => $value->numberseats,
                ];
                $studetnDetails = [];
                $totalSeats = $value->numberseats; // Fix the typo here
                for ($i = 1; $i <= $totalSeats; $i++) {
                    // get student data by its room no and seatno in studentdetails
                    $studentInfo = StudentRegister::where(['roomno' => $value->roomno, 'seetno' => $i])->first();
                    if (!empty($studentInfo)) {
                        $studetnDetails[] = [
                            'formno' => $studentInfo->formno,
                            'roomno' => $studentInfo->roomno,
                            'seatno' => $studentInfo->seetno,
                            'registerDate' => Carbon::parse($studentInfo->date)->format('d-M-Y'),
                            'studentName' => $studentInfo->candidatename,
                            'mobileNo' => $studentInfo->phone,
                            'feeStatus' => $this->getTotalRemaningAmount($studentInfo->id) ?? 0,
                        ];
                    } else {
                        $studetnDetails[] = [
                            'formno' => 'No candidate ',
                            'roomno' => '-',
                            'seatno' => '-',
                            'registerDate' => '-',
                            'studentName' => '-',
                            'mobileNo' => '-',
                            'feeStatus' => '-'
                        ];
                    }
                }
                $roomDetails['students'] = $studetnDetails; // Assign the students array directly
                $studentData[] = $roomDetails;
            }
            $vacantSeats = $VacantRegistration - $runningRegistration;

            $view = view('pdf.studentRegister-pdf', compact(
                'bussinessData',
                'studentData',
                'totalRegistration',
                'runningRegistration',
                'oldRegistration',
                'vacantSeats',
            ))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'RegisterStudentReport_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);
            file_put_contents($filePath, $output);

            // Optionally, you can save the filename to the database or log it

            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Register Student Report created successfully.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }

    private function getRoomRent($roomno)
    {
        return RoomSeats::where('roomno', $roomno)->value('rent');
    }

    private function getActualIdOfStudent($formno)
    {
        return StudentRegister::where('formno', $formno)->value('id');
    }

    public function generateOldStudentRegisterReport()
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $totalRegistration = StudentRegister::count();
            $runningRegistration = StudentRegister::where('status', 1)->count();
            $oldRegistration = OldStudentsRegister::where('status', 0)->count();
            $VacantRegistration = RoomSeats::sum('numberseats');
            $vacantSeats = $VacantRegistration - $runningRegistration;
            $studentData = [];

            // Fetch all old student register records
            $oldStudentRegister = OldStudentsRegister::all();

            foreach ($oldStudentRegister as $key => $value) {
                $studentData[] = [
                    'roomno' => $value->roomno,
                    'formno' => $value->formno,
                    'regDate' => Carbon::parse($value->date)->format('d-M-Y'),
                    'leavingDate' => Carbon::parse($value->leave_date)->format('d-M-Y'),
                    'name' => $value->candidatename,
                    'mobileNo' => $value->phone,
                    'tenure' => Carbon::parse($value->leave_date)->diffInDays(Carbon::parse($value->date)),
                    'seatno' => $value->seetno,
                    'rent' => $this->getRoomRent($value->roomno) ?? 0,
                    'feestatus' => $this->getTotalRemaningAmount($this->getActualIdOfStudent($value->formno)) ?? 0,
                ];
            }

            // Generate PDF
            $view = view('pdf.oldStudentRegister-pdf', compact(
                'bussinessData',
                'studentData',
                'totalRegistration',
                'runningRegistration',
                'oldRegistration',
                'vacantSeats',
            ))->render();

            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);
            $dompdf->setPaper('A4', 'portrait');
            $dompdf->render();

            // Save PDF to storage
            $filename = 'OldStudentRegisterReport_' . now()->format('YmdHis') . '.pdf';
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);
            file_put_contents($filePath, $output);

            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Old Student Report created successfully.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }

    public function generateSalesReport(Request $request)
    {
        try {

            $bussinessData = Businessregister::firstOrFail();
            $totalSalesTxn = Sales::count();
            $totalCustomTxn = CustomeSales::count();
            $totalTxn = $totalSalesTxn + $totalCustomTxn;

            // Extract start date and end date from the request (if provided)
            $startDate = $request->input('startDate');
            $endDate = $request->input('endDate');

            // Calculate total amounts based on optional date filters
            $totalAmount = $this->calculateTotalAmount($startDate, $endDate);
            $customTotalAmount = $this->calculateCustomTotalAmount($startDate, $endDate);
            $totalSale = number_format(floatval($totalAmount + $customTotalAmount), 2);
            $dueBalance = number_format(floatval($this->calculatePendingAmount($startDate, $endDate)));
            $salesData = $this->fetchSalesData($startDate, $endDate);
            $customSalesData = $this->fetchCustomSalesData($startDate, $endDate);
            $view = view('pdf.salesReport-pdf', compact('bussinessData', 'totalTxn', 'totalSale', 'dueBalance', 'salesData', 'customSalesData', 'startDate', 'endDate'))->render();
            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            ob_end_clean(); // Add this before $dompdf->render();

            $dompdf->render();
            // Generate a unique filename for the PDF
            $filename = 'SalesReport' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);

            file_put_contents($filePath, $output);
            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);
            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Sales Report created successfully.",
                    'download_link' => $link,
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function calculateTotalAmount($startDate = null, $endDate = null)
    {
        return Sales::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('invoicedate', [$startDate, $endDate]);
        })->sum('total');
    }

    private function calculateCustomTotalAmount($startDate = null, $endDate = null)
    {
        return CustomeSales::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('tpdate', [$startDate, $endDate]);
        })->sum('total');
    }

    private function calculatePendingAmount($startDate = null, $endDate = null)
    {
        return Sales::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('invoicedate', [$startDate, $endDate]);
        })->sum('remaining_amount');
    }
    private function fetchSalesData($startDate = null, $endDate = null)
    {
        $salesData = [];
        $query = Sales::with(['student', 'receiveAmounts']);

        if ($startDate && $endDate) {
            $query->whereBetween('invoicedate', [$startDate, $endDate]);
        }

        $salesRecords = $query->get();

        foreach ($salesRecords as $row) {
            $salesData[] = [
                'invoiceno' => $row->invoiceno,
                'saleType' => 'Reg. Sale',
                'saleDate' => $row->tpdate ? Carbon::parse($row->tpdate)->format('d-M-Y') : Carbon::parse($row->invoicedate)->format('d-M-Y'),
                'studentName' => $row->student->candidatename,
                'roomNo' => $row->roomno,
                'totalAmount' => $row->total,
                'Balance' => $row->remaining_amount,
                'paymentMode' =>  optional($row->receiveAmounts->first())->paymentMode,
                'items' => $this->prepareSalesItemList($row),
            ];
        }
        return $salesData;
    }
    private function fetchCustomSalesData($startDate = null, $endDate = null)
    {
        $customSalesData = [];
        $query = CustomeSales::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('tpdate', [$startDate, $endDate]);
        })->get();
        foreach ($query as  $row) {
            $customSalesData[] = [
                'invoiceno' => $row->invoiceno,
                'saleType' => 'Custom Sale',
                'saleDate' => $row->tpdate ? Carbon::parse($row->tpdate)->format('d-M-Y') : Carbon::parse($row->invoicedate)->format('d-M-Y'),
                'studentName' => $row->name,
                'roomNo' => $row->roomno,
                'totalAmount' => $row->total,
                'Balance' => $row->remaining_amount,
                'paymentMode' => $row->paymentMode,
                'items' => $this->prepareSalesItemList($row),
            ];
        }
        return $customSalesData;
    }

    public function createPaymentInInvoice($paymentInId,)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $termAndCondition = TermsAndConditions::all();
            $stapWithSign = BusinessDocument::where('upload_name', 'Stamp With Signature')->first();
            $data = RecieveAmount::where('id', $paymentInId)->with(['sales' => function ($query) {
                $query->with('student');
            }])->first();

            $studentData =  $data->sales->student;
            $numberToWrods = $this->numberToWords($data->receiveAmount);
            $invoiceNo = $data->invoiceNo;
            $saleData =  $data;

            $view = view('pdf.PaymentInInvoice-pdf', compact('bussinessData', 'saleData', 'studentData', 'numberToWrods', 'stapWithSign'))->render();

            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            $dompdf->render();

            // Generate a unique filename for the PDF
            $filename = 'SaleInvoice_InvoiceNo' . $invoiceNo . '_' . implode("_", explode(" ", $studentData->candidatename)) . '_' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/pdf/StudentSalesInvoice/' . $filename);
            file_put_contents($filePath, $output);

            // Optionally, you can save the filename to the database or log it

            // Generate a shareable link
            $link = asset('assets/uploads/pdf/StudentSalesInvoice/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Sales Report Create For invoice number {$invoiceNo}.",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ]);
        }
    }
    private function getInsertedBillDate()
    {
        $lastRecords =  ElectricityManagement::select('BillDate')
            ->orderBy('id', 'desc')
            ->first();
        return $lastRecords ? $lastRecords->BillDate : null;
    }
    protected function getFormattedBillDates($selectedMonth)
    {
        if (!$selectedMonth) {
            return [null, null];
        }

        $previousReadingDate = ElectricityManagement::select('BillDate')
            ->where('BillDate', '<', $selectedMonth)
            ->orderBy('BillDate', 'DESC')
            ->groupBy('BillDate')
            ->first();

        $presentReadingDate = DB::table('electicity_management')
            ->where('BillDate', '=', $selectedMonth)
            ->first();

        $PrenseBillDate = $presentReadingDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : '';
        $PreviousBillDate = $previousReadingDate ? Carbon::parse($previousReadingDate->BillDate)->format('d-M-Y') : ($PrenseBillDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : "");

        return [$PrenseBillDate, $PreviousBillDate];
    }
    protected function getElectricityRecordsByDateRange($startDate, $endDate)
    {
        return $lastRecords = ElectricityManagement::with('studentElectricityRecords.student')
            ->whereBetween('BillDate', [$startDate, $endDate])
            ->get();
    }
    protected function getElectricityRecordsByMonth($selectedMonth)
    {
        return $electricityRecords = ElectricityManagement::with(['studentElectricityRecords' => function ($query) {
            $query->with('student');
        }])
            ->where('BillDate', $selectedMonth)
            ->get();
    }
    public function DownloadElectricityReports(Request $request)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $selectedMonth = $request->input('selected_month') ?? $this->getInsertedBillDate();
            $startDate = $request->input('startDate');
            $endDate = $request->input('endDate');

            list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($selectedMonth);
            $roomSeats = RoomSeat::with('reserveRoom')->get();
            if ($selectedMonth == 'custom') {
                $PrenseBillDate = Carbon::parse($startDate)->format('d-M-Y');
                $PreviousBillDate = Carbon::parse($endDate)->format('d-M-Y');
                $electricityRecords = $this->getElectricityRecordsByDateRange($request->input('startDate'), $request->input('endDate'));
            } else {
                $electricityRecords = $this->getElectricityRecordsByMonth($selectedMonth);
            }
            $allBillDates = ElectricityManagement::select('BillDate')->orderBy('BillDate', 'Asc')->groupBy('BillDate')->get();

            $view = view('pdf.electricityReport-pdf', compact('bussinessData', 'selectedMonth', 'PrenseBillDate', 'PreviousBillDate', 'roomSeats', 'allBillDates', 'electricityRecords', 'startDate', 'endDate'))->render();

            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);
            $dompdf->setPaper('A4', 'portrait');
            $dompdf->render();
            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filename = 'ElectricityReport_' . uniqid() . '.pdf';
            $filePath = public_path('assets/uploads/pdf/StudentSalesInvoice/' . $filename);
            file_put_contents($filePath, $output);
            $link = asset('assets/uploads/pdf/StudentSalesInvoice/' . $filename);

            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Electricity Report createed successfully",
                    'download_link' => $link,
                ]
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => true,
                'statusCode' => 500,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function sendSaleStatement($studentId)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $salesData = [];
            $query = Sales::where('studentid', $studentId)->with(['student', 'receiveAmounts']);
            $totalTxn = Sales::where('studentid', $studentId)->count();
            $totalSale = Sales::where('studentid', $studentId)->sum("total");
            $dueBalance = Sales::where('studentid', $studentId)->sum('remaining_amount');

            $salesRecords = $query->get();
            foreach ($salesRecords as $row) {
                $salesData[] = [
                    'invoiceno' => $row->invoiceno,
                    'saleType' => 'Reg. Sale',
                    'saleDate' => $row->tpdate ? Carbon::parse($row->tpdate)->format('d-M-Y') : Carbon::parse($row->invoicedate)->format('d-M-Y'),
                    'studentName' => $row->student->candidatename,
                    'roomNo' => $row->roomno,
                    'totalAmount' => $row->total,
                    'Balance' => $row->remaining_amount,
                    'paymentMode' =>  optional($row->receiveAmounts->first())->paymentMode,
                    'items' => $this->prepareSalesItemList($row),
                    'paymentInDetails' => $row->receiveAmounts,
                ];
            }
            $view = view('pdf.sendStatement-pdf', compact('bussinessData', 'totalTxn', 'totalSale', 'dueBalance', 'salesData'))->render();
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);
            $dompdf->setPaper('A4', 'portrait');
            $dompdf->render();
            $filename = 'SaleStatement_' . now()->format('YmdHis') . '.pdf';
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);

            file_put_contents($filePath, $output);
            $link = asset('assets/uploads/' . $filename);
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'data' => [
                    'message' => "Sales statement Report created successfully.",
                    'download_link' => $link,
                    'businessName' => $bussinessData->businessname,
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => true,
                "error" => $e->getMessage(),
                "message" => 'server Error'
            ], 500);
        }
    }

    public function downloadStockReports(Request $request)
    {
        $bussinessData = Businessregister::firstOrFail();
        $month = $request->date ?: Carbon::now()->format('Y-m');
        $previousMonthDate = Carbon::parse($month)->subMonth()->format('Y-m');
        $categoriesWithItemsAndDetails = StockCategory::with(['stockItem' => function ($query) use ($month) {
            $query->with(['itemDetails' => function ($query) use ($month) {
                $query->where('month', $month);
            }]);
        }])->get();
        $categoriesWithItemsAndDetails->each(function ($category) use ($previousMonthDate) {
            $category->stockItem->each(function ($item) use ($category, $previousMonthDate) {
                // Access each item's itemDetails collection
                $previousMonthdata = stock_detail::where('month', $previousMonthDate)->where('item_id', $item->id)->where('category_id', $category->id)->first();
                $item->setAttribute('previousMonthData', $previousMonthdata);
                $itemDetails = $item->itemDetails;
                if (!empty($itemDetails)) {
                    $itemDetails->each(function ($detail) use (&$category, $previousMonthDate, &$item, &$itemDetails) {
                        $category->setAttribute('orderDates', $itemDetails->orderDate);
                    });
                } else {
                    $category->setAttribute('orderDates', []);
                }
            });
        });
        $month = Carbon::parse($month)->format('M-Y');
        $view = view('pdf.stockReport', compact('bussinessData', 'categoriesWithItemsAndDetails', 'month'));
        $dompdf = new Dompdf();
        $dompdf->loadHtml($view);
        $dompdf->setPaper('A4', 'portrait');
        $dompdf->render();
        $filename = 'Stock_Reports_' . now()->format('YmdHis') . '.pdf';
        $output = $dompdf->output();
        $filePath = public_path('assets/uploads/' . $filename);

        file_put_contents($filePath, $output);
        $link = asset('assets/uploads/' . $filename);
        return response()->json([
            'status' => true,
            'downloadLink' => $link,
            'fileName' => $filename,
            'message' => 'stock report created.'
        ], 200);
    }


    public function downloadEmpForm($id)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $stampWithSingDoc = Uploadprofile::where('upload_name', 'Stamp With Signature')->first();
            if (!empty($stampWithSingDoc) && file_exists(public_path('assets/uploads/pdf/' . $stampWithSingDoc->upload_pdf))) {
                $stampWithSingDocUrl = asset('assets/uploads/pdf/' . $stampWithSingDoc->upload_pdf);
            } else {
                $stampWithSingDocUrl = "";
            }
            $emp = EmpManagement::findOrFail($id);
            $empName = $emp->candidate_name;
            $view = view('pdf.empFormDownload-pdf', compact('bussinessData', 'emp', 'stampWithSingDocUrl'))->render();
            $options = new Options();
            $options->set('DOMPDF_PDF_IMAGE_QUALITY', 30);
            $dompdf = new Dompdf($options);
            $dompdf->loadHtml($view);
            $dompdf->setPaper('A4', 'portrait');
            $dompdf->render();
            $filename = 'EmpForm_' . implode('_', explode(' ', $empName)) . "_" . now()->format('YmdHis') . '.pdf';
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);
            file_put_contents($filePath, $output);
            $link = asset('assets/uploads/' . $filename);
            return response()->json([
                'status' => true,
                'downloadLink' => $link,
                'fileName' => $filename,
                'message' => 'emp form created Successfully.'
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
