<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use App\Models\ReferEarnSetup;
use Exception;
use Validator;
use App\Models\ReferHostel;
use App\Models\Businessregister;


class ReferEarnSetupController extends Controller
{

    public function getDataForApi()
    {
        try {
            $referEarnSetup = ReferEarnSetup::get();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'refer earn  setup details',
                'data' => $referEarnSetup
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    private function getHostelId()
    {
        return Businessregister::value('license_no');
    }

    public function getHostelAdminSetupData()
    {
        try {
            $referEarnSetup = ReferEarnSetup::where('is_active',1)->first();
            $referedHosteled = ReferHostel::with('studentDetails')->get();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'At the admin panal refer and earn setup for student',
                'data' => [
                    'formData' => $referEarnSetup,
                    'tableData' => $referedHosteled
                ],
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function  saveReferAndEarnSetup(Request $request)
    {
        try {

            $validator = Validator::make($request->all(), [
                'ref_allowed' => 'required|numeric',
                'ref_discounts' => 'required|array',
                'ref_total_months' => 'required|integer',
                'ref_start_date' => 'required|date',
                'ref_end_date' => 'required|date|after_or_equal:ref_start_date',
                'ref_start_time' => 'required|date_format:H:i:s',
                'ref_end_time' => 'required|date_format:H:i:s',
                'terms_and_conditions' => 'required|array',
                'is_active' => 'required|boolean',
                'message' => 'required',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Error',
                    'error' => $validator->errors(),
                ], 422);
            }

            // Get the validated data
            $validated = $validator->validated();

            if (ReferEarnSetup::exists()) {
                $setup = ReferEarnSetup::first();
                $setup->update($validated);
            } else {
                $setup = ReferEarnSetup::create($validated);
            }

            return response()->json([
                "status" => true,
                "statusCode" => 200,
                'message' => "Setup is created"
            ],200);

        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
