<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use App\Models\ApiModels\StudentRegister;
use Exception;
use Illuminate\Validation\ValidationException;
use App\Models\Businessregister;
use Illuminate\Support\Facades\Hash;
use Illuminate\Support\Facades\File;
use App\Models\Salesubscription;
use App\Models\Subscription;
use Illuminate\Support\Carbon;
use App\Models\ApiModels\StudentBooking;
use App\Models\ApiModels\LeadManager;
use UnexpectedValueException;
use App\Models\ApiModels\OldStudentsRegister;
use DB;
use App\Models\ApiModels\SelfStudentRegister;
use App\Models\ApiModels\StudentLogin;
use PDO;
use App\Models\ApiModels\UserAuthTokens;
use App\Models\ReferHostel;
use Log;




class RegistrationController extends controller
{

    private function resizeBase64Image($base64Data, $targetSizeInKB)
    {
        try {
            // Decode base64 data
            $decodedData = base64_decode($base64Data);

            // Check if decoding was successful
            if ($decodedData === false) {
                throw new Exception('Base64 decoding failed');
            }

            // Create an image resource from the decoded data
            $img = imagecreatefromstring($decodedData);

            // Check if image creation was successful
            if ($img === false) {
                throw new Exception('Image creation failed');
            }

            // Get the original image dimensions
            $width = imagesx($img);
            $height = imagesy($img);

            // Set the desired width and calculate the corresponding height to maintain the aspect ratio
            $targetWidth = 200; // Adjust this value based on your requirements
            $targetHeight = ($targetWidth / $width) * $height;

            // Create a new image resource for the resized image
            $resizedImg = imagecreatetruecolor($targetWidth, $targetHeight);

            // Resize the image
            imagecopyresampled($resizedImg, $img, 0, 0, 0, 0, $targetWidth, $targetHeight, $width, $height);

            // Create a buffer for the resized image
            ob_start();

            // Output the resized image to the buffer
            imagejpeg($resizedImg, null, 90); // Adjust the quality as needed

            // Get the buffer content
            $resizedImageData = ob_get_clean();

            $quality = 90;

            // Check if the resized image meets the target size
            while (strlen($resizedImageData) > ($targetSizeInKB * 1024)) {
                // If the size is still larger than the target, reduce the quality and update the buffer
                $quality -= 10; // Reduce the quality by 10 (adjust as needed)

                ob_start();
                imagejpeg($resizedImg, null, $quality);
                $resizedImageData = ob_get_clean();
            }

            // Encode the resized image to base64
            $resizedBase64Data = base64_encode($resizedImageData);

            // Destroy the image resources
            imagedestroy($img);
            imagedestroy($resizedImg);

            return $resizedBase64Data;
        } catch (Exception $e) {
            // Log or handle the exception as needed
            error_log('Error in resizeBase64Image: ' . $e->getMessage());
            return null; // or handle the error in an appropriate way for your application
        }
    }

    public function getSubscriptionPlansData()
    {
        $subscriptionData = [];
        $package_duration = [];
        $packagesTypes = [];

        try {
            $packages = Subscription::where('status', 1)
                ->whereNotIn('package_duration', ['7 Days', '15 Days'])
                ->select('package_title', 'package_duration')
                ->distinct()
                ->get();

            if ($packages->isEmpty()) {
                throw new \Exception("No subscription plans available");
            }

            foreach ($packages as $details) {
                if (!in_array($details->package_duration, $package_duration)) {
                    $package_duration[] = $details->package_duration;
                }

                // Add the package_title to the $packages array if it's not already there
                if (!in_array($details->package_title, $packagesTypes)) {
                    $packagesTypes[] = $details->package_title;
                }
            }

            $packageDetails = Subscription::where("status", 1)->get();

            if ($packageDetails->isEmpty()) {
                throw new \Exception("No subscription plans activet");
            }
            // get one by one more details about plans;
            foreach ($packageDetails as $details) {
                $subscriptionPlansDetils[$details->package_category][] = [
                    'packageDuration' => $details->package_duration,
                    'packageType' => $details->package_title,
                    "PlanType" => $details->package_category,
                    "subscription_price" => $details->subscription_price,
                    "sub_discount" => $details->sub_discount,
                    "subs_price" => $details->subs_price,
                    "tax_rate" => $details->tax_rate,
                    "total_price" => $details->total_price,
                    "packagecode" =>  $details->packagecode,
                ];
            }

            $subscriptionData = [
                "packagType" => $packagesTypes,
                "packegesDuration" => $package_duration,
                "packages" => $subscriptionPlansDetils
            ];

            return  response()->json([
                'status' => true,
                "data" => $subscriptionData
            ], 200);
        } catch (Exception $e) {
            return  response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function updateOldStudentAndAadharImg($updatedStudent): bool
    {
        try {
            if (OldStudentsRegister::where('formno', $updatedStudent->formno)->exists()) {
                OldStudentsRegister::where('formno', $updatedStudent->formno)->update([
                    'uploadaadharfront' => $updatedStudent->uploadaadharfront,
                    'uploadaadharback' => $updatedStudent->uploadaadharback,
                ]);
            }
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    public function uploadStudentAadharImg(Request $request, $studentId)
    {
        try {

            $validator = Validator::make($request->all(), [
                'aadhareFront' => 'required|image|mimes:jpg,jpeg,png',
                'aadhareBack' => 'required|image|mimes:jpg,jpeg,png',
            ], [
                'aadhareFront.image' => 'The :attribute must be an image.',
                'aadhareFront.mimes' => 'The :attribute must be a file of type: jpg, jpeg, png.',
                'aadhareBack.image' => 'The :attribute must be an image.',
                'aadhareBack.mimes' => 'The :attribute must be a file of type: jpg, jpeg, png.',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // check the student is register or not 
            $checkUserExist = StudentRegister::find($studentId);
            if (empty($checkUserExist)) {
                throw new UnexpectedValueException("The student Id {$studentId} is not valid.");
            }
            // Validate images and upload
            $this->removeOldImage($request, 'aadhareFront', 'uploadaadharfront', $checkUserExist);
            $this->removeOldImage($request, 'aadhareBack', 'uploadaadharback', $checkUserExist);
            $this->validateAndUpload($request, 'aadhareFront', 'uploadaadharfront', $checkUserExist);
            $this->validateAndUpload($request, 'aadhareBack', 'uploadaadharback', $checkUserExist);

            if ($checkUserExist->update()) {
                $this->updateOldStudentAndAadharImg($checkUserExist);

                return response()->json([
                    'status' => true,
                    'studentId' => $studentId,
                    'message' => "Aadhar frount and back imgage uploaded successfuly for student id {$studentId}",
                ]);
            } else {
                return response()->json([
                    'status' => false,
                    'message' => 'Failed to upload images',
                ], 500);
            }
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors()
            ], 422);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function updateOldStudentAndParentImg($updatedStudent): bool
    {
        try {
            if (OldStudentsRegister::where('formno', $updatedStudent->formno)->exists()) {
                OldStudentsRegister::where('formno', $updatedStudent->formno)->update([
                    'candidatesign' => $updatedStudent->candidatesign,
                    'Parentsign' => $updatedStudent->Parentsign,
                ]);
            }
            return true;
        } catch (Exception $e) {
            return false;
        }
    }

    public function uploadStudentAndParentSing(Request $request, $studentId)
    {
        try {

            $validator = Validator::make($request->all(), [
                'candidateSing' => 'required|image|mimes:jpg,jpeg,png',
                'parentSing' => 'required|image|mimes:jpg,jpeg,png',
            ], [
                'candidateSing.image' => 'The :attribute must be an image.',
                'candidateSing.mimes' => 'The :attribute must be a file of type: jpg, jpeg, png.',
                'parentSing.image' => 'The :attribute must be an image.',
                'parentSing.mimes' => 'The :attribute must be a file of type: jpg, jpeg, png.',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // check the student is register or not 
            $checkUserExist = StudentRegister::find($studentId);
            if (empty($checkUserExist)) {
                throw new UnexpectedValueException("The student Id {$studentId} is not valid.");
            }
            // Validate images and upload
            $this->removeOldImage($request, 'candidateSing', 'candidatesign', $checkUserExist);
            $this->removeOldImage($request, 'parentSing', 'Parentsign', $checkUserExist);
            $this->validateAndUpload($request, 'candidateSing', 'candidatesign', $checkUserExist);
            $this->validateAndUpload($request, 'parentSing', 'Parentsign', $checkUserExist);
            if ($checkUserExist->update()) {
                $this->updateOldStudentAndParentImg($checkUserExist);
                return response()->json([
                    'status' => true,
                    'studentId' => $studentId,
                    'message' => "Student and Parent signature imgages uploaded successfuly for student id {$studentId}",
                ]);
            } else {
                return response()->json([
                    'status' => false,
                    'message' => 'Failed to upload images',
                ], 500);
            }
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors()
            ], 422);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }


    public function StudentRegister(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'leadId' => 'nullable|string',
                'bookingId' => 'nullable|string',
                'formNumber' => 'required',
                'roomNumber' => 'required',
                'seatNumber' => 'required',
                'registrationDate' => 'required|date',
                'candidateName' => 'required|string',
                'birthDate' => 'required|date',
                'idProof' => 'required|digits:12',
                'candidatePhone' => 'required|regex:/^[0-9]{10}$/',
                'email' => 'required|email',
                'blood_Group' => 'required',
                'courseName' => 'required',
                'courseDescription' => 'required_if:courseName,=,"coursetype"',
                'jobDescription' => 'required_if:courseName,=,"jobtype"',
                'instituteName' => 'required',
                'instituteDescription' => 'required_if:instituteName,=,"institute Name"',
                'companyDescription' => 'required_if:companyName,!=,"institute Name"',
                'stayDuration' => 'required',
                'healthIssue' => 'required',
                'healthDescription' => 'required_if:healthIssue,!=,yes',
                'vehicleNumber' => 'required',
                'vehicleDescription' => 'required_if:vehicleNumber,!=,yes',
                'fatherName' => 'required|string',
                'fatherOccupation' => 'required',
                'motherName' => 'required|string',
                'motherOccupation' => 'required',
                'parentsPhone1' => 'required|regex:/^[0-9]{10}$/',
                'parentsPhone2' => 'nullable|regex:/^[0-9]{10}$/',
                'parentsEmail' => 'nullable|email',
                'parentsAddress' => 'required',
                'state' => 'required|string',
                'pincode' => 'required|regex:/^[0-9]{6}$/',
                'guardianName' => 'required|string',
                'guardianNumber' => 'required|regex:/^[0-9]{10}$/',
                'guardianAddress' => 'required',
                'candidateImg' => 'required|image|mimes:jpg,jpeg,png',
                'referCode' => 'nullable|string',
            ], [
                'candidateImg.required' => "Candidate Image is required.",
                'candidateImg.image' => 'The :attribute must be an image.',
                'candidateImg.mimes' => 'The :attribute must be a file of type: jpg, jpeg, png.',
            ]);
            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            //check if the booking or leadid given this id is correct or not
            if (!empty($request->bookingId)) {
                $booking = StudentBooking::where('id', $request->bookingId)->where('status', 1)->first();
                if (empty($booking)) {
                    throw new UnexpectedValueException('Booking Id is not valid');
                }
            } elseif (!empty($request->leadId)) {
                $lead = LeadManager::where('id', $request->leadId)->where('leadstatus', 0)->first();
                if (empty($lead)) {
                    throw new UnexpectedValueException('Lead Id is not valid');
                }
            }

            // If the validation passes, create a new instance of the model and save the data
            // check formno 
            $checkFormNo = StudentRegister::where("formno", $request->formNumber)->first();
            if (!empty($checkFormNo)) {
                return response()->json(['status' => false, 'message' => 'This Form Number is already registered'], 422);
            }
            // check seat availability
            $checkSeatNo = StudentRegister::where("roomno", $request->roomNumber)
                ->where("seetno", $request->seatNumber)
                ->where("status", 1)
                ->first();

            if (!empty($checkSeatNo)) {
                return response()->json([
                    "status" => false,
                    'message' => "This seat is not available"
                ], 422);
            }

            $data = new Studentregister();
            $data->formno = $request->formNumber;
            $data->roomno = $request->roomNumber;
            $data->seetno = $request->seatNumber;
            $data->date = $request->registrationDate;
            $data->candidatename = $request->candidateName;
            $data->dob = $request->birthDate;
            $data->valid = $request->idProof;
            $data->phone = $request->candidatePhone;
            $data->email = $request->email;
            $data->blood = $request->blood_Group;
            $data->healthissue = $request->healthIssue;
            $data->health_description = $request->healthDescription;
            $data->stydrt = $request->stayDuration;
            $data->vehicalno = $request->vehicleNumber;
            $data->vehicledescription = $request->vehicleDescription;
            $data->coursename = $request->courseName;
            $data->coursedetails = $request->courseDescription;
            $data->jobdetails = $request->jobDescription;
            $data->companyname = $request->companyDescription;
            $data->institute = $request->instituteName;
            $data->institutename = $request->instituteDescription;
            $data->fathername = $request->fatherName;
            $data->fatheroocc = $request->fatherOccupation;
            $data->mothername = $request->motherName;
            $data->motheroocc = $request->motherOccupation;
            $data->tel = $request->parentsPhone1;
            $data->mobilenumber = $request->parentsPhone2;
            $data->pemail = $request->parentsEmail;
            $data->fullAddress = $request->parentsAddress;
            $data->state = $request->state;
            $data->pinno = $request->pincode;
            $data->guardianname = $request->guardianName;
            $data->guardiannumber = $request->guardianNumber;
            $data->guardianaddress = $request->guardianAddress;
            $data->referCode = strtoupper($request->referCode);

            $this->validateAndUpload($request, 'candidateImg', 'uploadimage', $data);

            DB::beginTransaction();
            if ($data->save()) {
                // check this data come from the booking or lead 
                if (($request->bookingId && $request->bookingId != "") || ($request->leadId && $request->leadId != "")) {
                    $statusUpdate = null;

                    if ($request->bookingId && $request->bookingId != "") {
                        $statusUpdate = StudentBooking::where('id', '=', $request->bookingId)->update(['status' => 0]);
                    } elseif ($request->leadId && $request->leadId != "") {
                        $statusUpdate = LeadManager::where('id', $request->leadId)->update(['leadstatus' => 1]);
                    }

                    if (!$statusUpdate) {
                        throw new Exception("Status update failed");
                    }
                }
                DB::commit();


                if ($request->filled('referCode')) {
                    log::info("refer code get for this student $data->id, referCode: $request->referCode");
                    ReferHostel::updateReferralDiscounts($data->id, strtoupper($request->referCode));
                } else {
                    log::info("refer code not get for this student $data->id, referCode: $request->referCode");
                }

                return response()->json([
                    'status' => true,
                    'studentId' => $data->id,
                    'message' => 'Data saved successfully'
                ], 201);
            } else {
                throw new Exception("Data Not Store");
            }
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            DB::rollBack();
            return response()->json([
                "status" => false,
                "message" => $e->getMessage()
            ], 500);
        }
    }

    public function businessRegister(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                "firstName" => "required|string",
                'lastName' => "required|string",
                'email' => "required|email",
                'mobileNumber' => "required|regex:/^[0-9]{10}$/",
                "alternateMobileNumber" => "nullable|regex:/^[0-9]{10}$/",
                "logoImgage" => "required|image|max:1024",
                "businesType" => "required|string",
                "gstNumber" => "nullable|string",
                "businessName" => "required|string",
                "city" => "required|string",
                "state" => "required|string",
                "country" => "required|string",
                "pincode" => "required|regex:/^[0-9]{6}$/",
                "address" => "required|string",
                "password" => [
                    'required',
                    'string',
                    'min:8',
                    'regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/',
                ],
                'conformPassword' => "required|string|same:password",
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            if (Businessregister::where('email', $request->email)->exists()) {
                throw new Exception('Email already exists: ' . $request->email);
            } elseif (Businessregister::where('mobile', $request->mobilNumber)->exists()) {
                throw new Exception('Mobile number already exists: ' . $request->mobilNumber);
            } elseif (Businessregister::where('businessname', $request->businessName)->exists()) {
                throw new Exception('Business name already exists: ' . $request->businessName);
            }

            $data = new Businessregister();
            $data->business_type = $request->businesType;
            $data->fname = $request->firstName;
            $data->lname = $request->lastName;
            $data->email = $request->email;
            $data->mobile = $request->mobilNumber;
            $data->mobile_secondary = $request->alternateMobileNumber;
            $data->gst_no = $request->gstNumber;
            $data->businessname = $request->businessname;
            if ($request->hasfile('logoImgage')) {
                $file = $request->file('logoImgage');
                $filename = time() . '_' . $file->getClientOriginalExtension();
                $file->move(public_path('assets/uploads/'), $filename);
                $data->pic = $filename;
            }
            $data->city = $request->city;
            $data->state = $request->state;
            $data->country = $request->country;
            $data->address = $request->address;
            $data->pincode = $request->pincode;
            $data->username = $request->email;
            $data->username1 = $request->mobilNumber;
            $data->password = Hash::make($request->password);
            $data->conf_password = Hash::make($request->conformPassword);
            $data->save();
            $lastInsertedId = $data->id;
            $cityName = $request->city;
            $firstAlphabet = substr($cityName, 0, 1);
            $lastAlphabet = substr($cityName, -1);
            $output = strtoupper($firstAlphabet . $lastAlphabet);
            $outputInsert = str_pad($lastInsertedId, 4, '0', STR_PAD_LEFT);
            $license_no = $output . $request->pincode . $outputInsert;
            Businessregister::where('id', $lastInsertedId)
                ->update(['license_no' => $license_no]);
            $databaseName = $license_no;
            return response()->json([
                "status" => true,
                "message" => "Registration Successfully",
                'licenseNo' => $databaseName,
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
    public function businessRegisterByUser(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                "businesType" => "required|string",
                "gstNumber" => "nullable|string",
                "businessName" => "required|string",
                "city" => "required|string",
                "state" => "required|string",
                "country" => "required|string",
                "pincode" => "required|regex:/^[0-9]{6}$/",
                "address" => "required|string",
                "password" => [
                    'required',
                    'string',
                    'min:8',
                    'regex:/^(?=.*[a-z])(?=.*[A-Z])(?=.*\d).+$/',
                ],
                'conformPassword' => "required|string|same:password",
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            $busicBusinessDetails = Businessregister::firstOrFail();
            config(['database.default' => env('DB_CONNECTION')]);
            if (Businessregister::where('businessname', $request->businessName)->exists()) {
                throw new Exception('Business name already exists: ' . $request->businessName);
            }
            $data = new Businessregister();
            $data->regdate = Carbon::now()->format("Y-m-d");
            $data->business_type = $request->businesType;
            $data->fname = $busicBusinessDetails->fname;
            $data->lname = $busicBusinessDetails->lname;
            $data->email = $busicBusinessDetails->email;
            $data->mobile = $busicBusinessDetails->mobile;
            $data->mobile_secondary = $busicBusinessDetails->mobile_secondary;
            $data->gst_no = $request->gstNumber;
            $data->businessname = $request->businessName;
            $data->pic = $busicBusinessDetails->pic;
            $data->city = $request->city;
            $data->state = $request->state;
            $data->country = $request->country;
            $data->address = $request->address;
            $data->pincode = $request->pincode;
            $data->username = $busicBusinessDetails->mobile;
            $data->username1 = $busicBusinessDetails->email;
            $data->password = Hash::make($request->password);
            $data->conf_password = Hash::make($request->conformPassword);
            $data->registeration_source = "self";
            $data->save();
            $lastInsertedId = $data->id;
            $cityName = $request->city;
            $firstAlphabet = substr($cityName, 0, 1);
            $lastAlphabet = substr($cityName, -1);
            $output = strtoupper($firstAlphabet . $lastAlphabet);
            $outputInsert = str_pad($lastInsertedId, 4, '0', STR_PAD_LEFT);
            $license_no = $output . $request->pincode . $outputInsert;
            Businessregister::where('id', $lastInsertedId)
                ->update(['license_no' => $license_no]);
            $databaseName = $license_no;

            $subscriptionData = $this->getSubscriptionPlansData();
            config(['database.default' => 'custom_db']);
            return response()->json([
                "status" => true,
                "message" => "Registration Successfully",
                'licenseNo' => $databaseName,
                "subscriptionData" => $subscriptionData,
            ], 200);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
    public function addSubscription(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'customerId' => "required",
                "noOfseats" => "required",
                "totalAmountPayble" => "required",
                "PlansProductId" => "required",
                "PlanMonth" => "required",
                'planPrice' => "required",
                "planName" => "required",
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            function calculateEndDateAndDays($startDate, $planType, $numberOfMonths)
            {
                $startDateObj = Carbon::createFromFormat('Y-m-d', $startDate);
                $startDateObj->addMonths($numberOfMonths);
                switch ($planType) {
                    case 'Month':
                        break;
                    case 'Years':
                        $startDateObj->addYears($numberOfMonths);
                        break;
                    case 'Months':
                        break;
                    case "Days":
                        $startDateObj->addDays($numberOfMonths);
                        break;
                    default:
                        echo 'Invalid plan type';
                        return null;
                }
                $endDate = $startDateObj->subDay()->format('Y-m-d');
                $numberOfDays = $startDateObj->diffInDays(Carbon::createFromFormat('Y-m-d', $startDate));
                return ['endDate' => $endDate, 'numberOfDays' => $numberOfDays];
            }

            $data = $request->all();
            $userID = $request['customerId'];
            $noOfseats = $data['noOfseats'];
            $totalAmountPayble = $data['totalAmountPayble'];
            $PlansProductId = $data['PlansProductId'];
            $PlanMonth = $data['PlanMonth'];
            $planName = $data['planName'];
            $month = explode(" ", $PlanMonth);
            if ($month[1] == "Days" || $month[1] == "days") {
                $startDate = Carbon::today()->toDateString();
                $startDateObj = Carbon::createFromFormat('Y-m-d', $startDate);
                $futureDate = $startDateObj->addDays($month[0]);
                $endDate = $futureDate->format('Y-m-d');
                $noOfDays = $month[0];
            } else {
                $startDate = Carbon::today()->toDateString();
                $result = calculateEndDateAndDays($startDate, $month[1], $month[0]);
                $endDate = $result['endDate'] . PHP_EOL;
                $noOfDays = $result['numberOfDays'];
            }
            $saveData = new Salesubscription();
            $saveData->user_id = $userID;
            $saveData->plan_id = $PlansProductId;
            $saveData->startdate = $startDate;
            $saveData->enddate = $endDate;
            $saveData->counter = $noOfDays;
            $saveData->no_of_seats = $noOfseats;
            $saveData->paid_amount = $totalAmountPayble;
            $saveData->plan_price = $request->planPrice;
            $saveData->package_type = $planName;
            $saveData->status = 1;
            if ($saveData->save()) {
                return response()->json([
                    "status" => true,
                    "message" => "Subscription added successfully",
                ], 200);
            } else {
                throw new Exception("subscription not added. Error" . $saveData->getError());
            }
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ]);
        }
    }


    public function updateStudentRegister(Request $request, $formno)
    {
        try {
            Log::error('start updating');

            $validator = Validator::make($request->all(), [
                'roomNumber' => 'required',
                'seatNumber' => 'required',
                'registrationDate' => 'required|date',
                'candidateName' => 'required|string',
                'birthDate' => 'required|date',
                'idProof' => 'required|digits:12',
                'candidatePhone' => 'required|regex:/^[0-9]{10}$/',
                'email' => 'required|email',
                'blood_Group' => 'required',
                'courseName' => 'required',
                'courseDescription' => 'required_if:courseName,=,"coursetype"',
                'jobDescription' => 'required_if:courseName,=,"jobtype"',
                'instituteName' => 'required',
                'instituteDescription' => 'required_if:instituteName,=,"institute Name"',
                'companyDescription' => 'required_if:companyName,!=,"institute Name"',
                'stayDuration' => 'required',
                'healthIssue' => 'required',
                'healthDescription' => 'required_if:healthIssue,!=,yes',
                'vehicleNumber' => 'required',
                'vehicleDescription' => 'required_if:vehicleNumber,!=,yes',
                'fatherName' => 'required|string',
                'fatherOccupation' => 'required',
                'motherName' => 'required|string',
                'motherOccupation' => 'required',
                'parentsPhone1' => 'required|regex:/^[0-9]{10}$/',
                'parentsPhone2' => 'nullable|regex:/^[0-9]{10}$/',
                'parentsEmail' => 'nullable|email',
                'parentsAddress' => 'required',
                'state' => 'required|string',
                'pincode' => 'required|regex:/^[0-9]{6}$/',
                'guardianName' => 'required|string',
                'guardianNumber' => 'required|regex:/^[0-9]{10}$/',
                'guardianAddress' => 'required',
                'candidateImg' => 'nullable|image|mimes:jpg,jpeg,png',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            DB::beginTransaction();

            $data = StudentRegister::where('formno', $formno)->first();

            if (empty($data)) {
                return response()->json(['status' => false, 'message' => 'Data not found'], 404);
            }

            // remove old img from the public dir if new img upload
            $this->removeOldImage($request, 'candidateImg', 'uploadimage', $data);
            $data->roomno = $request->roomNumber;
            $data->seetno = $request->seatNumber;
            $data->date = $request->registrationDate;
            $data->candidatename = $request->candidateName;
            $data->dob = $request->birthDate;
            $data->valid = $request->idProof;
            $data->phone = $request->candidatePhone;
            $data->email = $request->email;
            $data->blood = $request->blood_Group;
            $data->healthissue = $request->healthIssue;
            $data->health_description = $request->healthDescription;
            $data->stydrt = $request->stayDuration;
            $data->vehicalno = $request->vehicleNumber;
            $data->vehicledescription = $request->vehicleDescription;
            $data->coursename = $request->courseName;
            $data->coursedetails = $request->courseDescription;
            $data->jobdetails = $request->jobDescription;
            $data->companyname = $request->companyDescription;
            $data->institute = $request->instituteName;
            $data->institutename = $request->instituteDescription;
            $data->fathername = $request->fatherName;
            $data->fatheroocc = $request->fatherOccupation;
            $data->mothername = $request->motherName;
            $data->motheroocc = $request->motherOccupation;
            $data->tel = $request->parentsPhone1;
            $data->mobilenumber = $request->parentsPhone2;
            $data->pemail = $request->parentsEmail;
            $data->fullAddress = $request->parentsAddress;
            $data->state = $request->state;
            $data->pinno = $request->pincode;
            $data->guardianname = $request->guardianName;
            $data->guardiannumber = $request->guardianNumber;
            $data->guardianaddress = $request->guardianAddress;

            $this->validateAndUpload($request, 'candidateImg', 'uploadimage', $data);

            if ($data->update()) {
                $candidatImgValue = $data->candidateImg;
                if (!$this->updateOldStudentData($request, $formno, $candidatImgValue)) {
                    throw new Exception("Failed to update old student data");
                }
                DB::commit();
                Log::error('success update: ');

                return response()->json([
                    'status' => true,
                    'message' => 'Data updated successfully',
                ], 200);
            } else {
                throw new Exception("Data not updated");
            }
        } catch (ValidationException $e) {
            DB::rollBack();
            Log::error('Validation error: ' . $e->getMessage());
            return response()->json([
                'status' => false,
                'errors' => $e->errors(),
            ], 422);
        } catch (Exception $e) {
            DB::rollBack();
            Log::error('Error updating student register: ' . $e->getMessage());
            return response()->json([
                "status" => false,
                "message" => $e->getMessage()
            ], 500);
        }
    }

    private function updateOldStudentData($request, $formId, $candidatImgValue): bool
    {
        try {
            if (OldStudentsRegister::where('formno', $formId)->exists()) {
                OldStudentsRegister::where('formno', $formId)->update([
                    'candidatename' => $request->candidateName,
                    'dob' => $request->birthDate,
                    'valid' => $request->idProof,
                    'phone' => $request->candidatePhone,
                    'email' => $request->email,
                    'blood' => $request->blood_Group,
                    'healthissue' => $request->healthIssue,
                    'health_description' => $request->healthDescription,
                    'stydrt' => $request->stayDuration,
                    'vehicalno' => $request->vehicleNumber,
                    'vehicledescription' => $request->vehicleDescription,
                    'coursename' => $request->courseName,
                    'coursedetails' => $request->courseDescription,
                    'jobdetails' => $request->jobDescription,
                    'companyname' => $request->companyDescription,
                    'institute' => $request->instituteName,
                    'institutename' => $request->instituteDescription,
                    'fathername' => $request->fatherName,
                    'fatheroocc' => $request->fatherOccupation,
                    'mothername' => $request->motherName,
                    'motheroocc' => $request->motherOccupation,
                    'tel' => $request->parentsPhone1,
                    'mobilenumber' => $request->parentsPhone2,
                    'pemail' => $request->parentsEmail,
                    'fullAddress' => $request->parentsAddress,
                    'state' => $request->state,
                    'pinno' => $request->pincode,
                    'guardianname' => $request->guardianName,
                    'guardiannumber' => $request->guardianNumber,
                    'guardianaddress' => $request->guardianAddress,
                    'uploadimage' => $candidatImgValue,
                ]);
            }
            return true;
        } catch (Exception $e) {
            Log::error('Error updating old student data: ' . $e->getMessage());
            return false;
        }
    }

    private function generateFileName($file)
    {
        $extension = $file->getClientOriginalExtension();
        $filename = time() . '_' . rand() . '.' . $extension;
        return $filename;
    }
    private function validateFile($file, $allowedExtensions)
    {
        $extension = strtolower($file->getClientOriginalExtension());
        if (!in_array($extension, $allowedExtensions)) {
            throw new \Exception('Invalid file format. Allowed formats: ' . implode(', ', $allowedExtensions));
        }
    }
    private function validateAndUpload($request, $fieldName, $databaseField, $data)
    {
        try {
            if ($request->hasFile($fieldName)) {
                $file = $request->file($fieldName);
                $this->validateFile($file, ['jpg', 'jpeg', 'png']);
                $base64Data = base64_encode(file_get_contents($file->path()));
                $resizedBase64Data = $this->resizeBase64Image($base64Data, 50);

                if ($resizedBase64Data !== null) {
                    $filename = $this->generateFileName($file);
                    File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                    $data->$databaseField = $filename;
                } else {
                    return response()->json(['status' => false, 'message' => 'Failed to resize and upload image'], 500);
                }
            }
        } catch (Exception $e) {
            // Log or handle the exception as needed
            error_log('Error in validateAndUpload: ' . $e->getMessage());
            return response()->json(['status' => false, 'message' => 'An error occurred during image validation and upload'], 500);
        }
    }

    private function removeOldImage($request, $fieldName, $databaseField, $data)
    {
        try {
            if ($request->hasFile($fieldName) && !empty($data->$databaseField) && file_exists(public_path('assets/uploads/' . $data->$databaseField))) {
                unlink(public_path('assets/uploads/' . $data->$databaseField));
            }
        } catch (Exception $e) {
            error_log('Error in removeOldImage: ' . $e->getMessage());
        }
    }

    public function selfStudentRegisterBytenantPanal(Request $request, $id)
    {
        try {
            $validator = Validator::make($request->all(), [
                'regdate' => 'required|date',
                'candidatename' => 'required|string',
                'dob' => 'required|date',
                'aadhar' => 'required|numeric|digits:12',
                'phone' => 'required|numeric|digits:10',
                'email' => 'required|email',
                'blood' => 'required|string',
                'coursename' => 'required|string',
                'coursedetails' => 'nullable|required_if:coursename,coursetype|string',
                'jobdetails' => 'nullable|required_if:coursename,jobtype|string',
                'institute' => 'required|string',
                'institutename' => 'nullable|required_if:institute,instituteName|string',
                'companyname' => 'nullable|required_if:institute,ComapanyName|string',
                'stydrt' => 'required|string',
                'healthissue' => 'required|in:yes,no',
                'health_description' => 'nullable|required_if:healthissue,yes|string',
                'vehicalno' => 'required|in:yes,no',
                'vehicledescription' => 'nullable|required_if:vehicalno,yes|string',
                'fathername' => 'required|string',
                'fatheroocc' => 'required|string',
                'mothername' => 'required|string',
                'motheroocc' => 'required|string',
                'tel' => 'required|numeric|digits:10',
                'mobilenumber' => 'nullable|numeric|digits:10',
                'pemail' => 'nullable|email',
                'fullAddress' => 'required|string',
                'state' => 'required|string',
                'pinno' => 'required|numeric|digits:6',
                'guardianname' => 'required|string',
                'guardiannumber' => 'required|numeric|digits:10',
                'guardianaddress' => 'required|string',
                'checkbox' => 'required|accepted',
                'uploadaadharfront' => 'required|image|mimes:jpeg,jpg,png',
                'uploadaadharback' => 'required|image|mimes:jpeg,jpg,png',
                'candidatesign' => 'required|image|mimes:jpeg,jpg,png',
                'Parentsign' => 'required|image|mimes:jpeg,jpg,png',
                'uploadimage' => 'required|image|mimes:jpeg,jpg,png',
                'termConditions' => 'required|accepted',
                'referCode' => 'nullable|string'
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => "validation Error",
                    'error' => $validator->errors(),
                ], 422);
            }
            DB::beginTransaction(); // Start a transaction
            $NewEntry = new SelfStudentRegister();
            $NewEntry->regdate = $request->regdate;
            $NewEntry->candidatename = $request->candidatename;
            $NewEntry->dob = $request->dob;
            $NewEntry->aadhar = $request->aadhar;
            $NewEntry->phone = $request->phone;
            $NewEntry->email = $request->email;
            $NewEntry->blood = $request->blood;
            $NewEntry->course_job = $request->coursename;
            $NewEntry->coursedetails = $request->coursedetails;
            $NewEntry->jobdetails = $request->jobdetails;
            $NewEntry->institute_company = $request->institute;
            $NewEntry->institutename = $request->institutename;
            $NewEntry->companyname = $request->companyname;
            $NewEntry->stydrt = $request->stydrt;
            $NewEntry->healthissue = $request->healthissue;
            $NewEntry->health_description = $request->health_description;
            $NewEntry->vehicalno = $request->vehicalno;
            $NewEntry->vehicledescription = $request->vehicledescription;
            $NewEntry->fathername = $request->fathername;
            $NewEntry->fatheroocc = $request->fatheroocc;
            $NewEntry->mothername = $request->mothername;
            $NewEntry->motheroocc = $request->motheroocc;
            $NewEntry->parentmobile1 = $request->tel;
            $NewEntry->parentmobile2 = $request->mobilenumber;
            $NewEntry->pemail = $request->pemail;
            $NewEntry->fullAddress = $request->fullAddress;
            $NewEntry->state = $request->fathername;
            $NewEntry->pinno = $request->pinno;
            $NewEntry->guardianname = $request->guardianname;
            $NewEntry->guardiannumber = $request->guardiannumber;
            $NewEntry->guardianaddress = $request->guardianaddress;
            $NewEntry->termsAndCondition = $request->termConditions;
            $NewEntry->referCode = $request->referCode;
            $this->validateAndUpload($request, 'uploadimage', 'studentImg', $NewEntry);
            $this->validateAndUpload($request, 'uploadaadharfront', 'uploadaadharfront', $NewEntry);
            $this->validateAndUpload($request, 'uploadaadharback', 'uploadaadharback', $NewEntry);
            $this->validateAndUpload($request, 'candidatesign', 'candidatesign', $NewEntry);
            $this->validateAndUpload($request, 'Parentsign', 'Parentsign', $NewEntry);
            $NewEntry->save();

            $chnageStatus = $this->changeStatusInLoginTable($NewEntry, $id);
            if (!$chnageStatus['status']) {
                DB::rollBack(); // Roll back the transaction if status change fails
                return response()->json([
                    'status' => false,
                    'statusCode' => 500,
                    'message' => 'Failed to update student status.',
                ], 500);
            }
            DB::commit();
            return response()->json([
                'status' => true,
                'statusCode' => 201,
                'message' => 'Student registration request has been sent successfully.',
                'data' => $chnageStatus['data'],
                'token' => $chnageStatus['token']
            ], 201);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => $e->getMessage(),
                'statusCode' => 500,
            ], 500);
        }
    }

    private function changeStatusInLoginTable($data, $id)
    {
        try {
            $CandidateName = explode(" ", $data->candidatename);
            $loginStudent =  StudentLogin::findOrFail($id);
            $checkHostelToken = UserAuthTokens::where('licenceno', $loginStudent->hostel_id)->where('usertype', 'Admin')->first();

            if (!empty($checkHostelToken)) {
                $token = $checkHostelToken->token;
            } else {
                $userId = $this->getUserIdOfhostel($loginStudent->hostel_id);
                $token = $this->generateUniqueToken('Admin', $loginStudent->hostel_id, $userId);
            }

            $loginStudent->status = 3;
            $loginStudent->first_name = $CandidateName[0] ?? null;
            $loginStudent->last_name = $CandidateName[1] ?? null;

            if ($loginStudent->save()) {
                return [
                    'status' => true,
                    'data' => $loginStudent,
                    'token' => $token
                ];
            } else {
                throw new Exception('Failed to save the student login status.');
            }
        } catch (ModelNotFoundException $e) {
            return [
                'status' => false,
            ];
        } catch (Exception $e) {
            return [
                'status' => false,
            ];;
        }
    }

    private function generateUniqueToken($adminType, $licenceNo, $userId)
    {
        // $this->deleteExpiredToken($licenceNo);
        $randomString = Str::random(32);
        $timestamp = Carbon::now()->timestamp;
        $rawToken = $randomString . $timestamp;
        $hashedToken = Hash::make($rawToken);
        $createTokenRecord = new UserAuthTokens();
        $createTokenRecord->user_id = $userId;
        $createTokenRecord->token = $hashedToken;
        $createTokenRecord->usertype = $adminType;
        $createTokenRecord->licenceno = $licenceNo;
        $createTokenRecord->save();
        return $hashedToken;
    }

    private function getUserIdOfhostel($licenceNo)
    {
        $hostel = Businessregister::where("license_no", $licenceNo)->first();
        return $hostel->id;
    }

    public function updateStudentProfile(Request $request, $studentId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'candidatename' => 'required|string',
                'dob' => 'required|date',
                'aadhar' => 'required|numeric|digits:12',
                'phone' => 'required|numeric|digits:10',
                'email' => 'required|email',
                'blood' => 'required|string',
                'coursename' => 'required|string',
                'coursedetails' => 'nullable|required_if:coursename,coursetype|string',
                'jobdetails' => 'nullable|required_if:coursename,jobtype|string',
                'institute' => 'required|string',
                'institutename' => 'nullable|required_if:institute,institute Name|string',
                'companyname' => 'nullable|required_if:institute,ComapanyName|string',
                'stydrt' => 'required|string',
                'healthissue' => 'required|in:yes,no',
                'health_description' => 'nullable|required_if:healthissue,yes|string',
                'vehicalno' => 'required|in:yes,no',
                'vehicledescription' => 'nullable|required_if:vehicalno,yes|string',
                'fathername' => 'required|string',
                'fatheroocc' => 'required|string',
                'mothername' => 'required|string',
                'motheroocc' => 'required|string',
                'tel' => 'required|numeric|digits:10',
                'mobilenumber' => 'nullable|numeric|digits:10',
                'pemail' => 'nullable|email',
                'fullAddress' => 'required|string',
                'state' => 'required|string',
                'pinno' => 'required|numeric|digits:6',
                'guardianname' => 'required|string',
                'guardiannumber' => 'required|numeric|digits:10',
                'guardianaddress' => 'required|string',
                'checkbox' => 'required|accepted',
                'uploadaadharfront' => 'nullable|image|mimes:jpeg,jpg,png',
                'uploadaadharback' => 'nullable|image|mimes:jpeg,jpg,png',
                'candidatesign' => 'nullable|image|mimes:jpeg,jpg,png',
                'Parentsign' => 'nullable|image|mimes:jpeg,jpg,png',
                'uploadimage' => 'nullable|image|mimes:jpeg,jpg,png',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => "validation Error",
                    'error' => $validator->errors(),
                ], 422);
            }

            $student = StudentRegister::findOrFail($studentId);

            // remove old img 
            $this->removeOldImage($request, 'candidateImg', 'uploadimage', $student);
            $this->removeOldImage($request, 'uploadaadharfront', 'uploadaadharfront', $student);
            $this->removeOldImage($request, 'uploadaadharback', 'uploadaadharback', $student);
            $this->removeOldImage($request, 'candidatesign', 'candidatesign', $student);
            $this->removeOldImage($request, 'Parentsign', 'Parentsign', $student);

            $student->candidatename = $request->candidatename;
            $student->dob = $request->dob;
            $student->valid = $request->valid;
            $student->phone = $request->phone;
            $student->email = $request->email;
            $student->blood = $request->blood;
            $student->healthissue = $request->healthissue;
            $student->health_description = $request->health_description;
            $student->stydrt = $request->stydrt;
            $student->vehicalno = $request->vehicalno;
            $student->vehicledescription = $request->vehicledescription;
            $student->coursename = $request->coursename;
            $student->coursedetails = $request->coursedetails;
            $student->jobdetails = $request->jobdetails;
            $student->companyname = $request->companyname;
            $student->institute = $request->institute;
            $student->institutename = $request->institutename;
            $student->fathername = $request->fathername;
            $student->fatheroocc = $request->fatheroocc;
            $student->mothername = $request->mothername;
            $student->motheroocc = $request->motheroocc;
            $student->tel = $request->tel;
            $student->mobilenumber = $request->mobilenumber;
            $student->pemail = $request->pemail;
            $student->fullAddress = $request->fullAddress;
            $student->state = $request->state;
            $student->pinno = $request->pinno;
            $student->guardianname = $request->guardianname;
            $student->guardiannumber = $request->guardiannumber;
            $student->guardianaddress = $request->guardianaddress;
            $this->validateAndUpload($request, 'uploadimage', 'uploadimage', $student);
            $this->validateAndUpload($request, 'uploadaadharfront', 'uploadaadharfront', $student);
            $this->validateAndUpload($request, 'uploadaadharback', 'uploadaadharback', $student);
            $this->validateAndUpload($request, 'candidatesign', 'candidatesign', $student);
            $this->validateAndUpload($request, 'Parentsign', 'Parentsign', $student);
            $student->save();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Profile Updated Successfully'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Server Error',
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
