<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\Mail;
use Exception;
use App\Models\Businessregister;
use Illuminate\Validation\ValidationException;
use DB;
use Illuminate\Support\Facades\Log;
use FFI\Exception as FFIException;
use App\Models\ApiModels\StudentRegistertoken;
use App\Models\Permissions;
use Illuminate\Support\Str;
use UnexpectedValueException;
use App\Models\ApiModels\StudentLogin;


class ShareController extends controller
{


    private function checkWhatsAppPermission(): bool
    {
        try {
            $permission = Permissions::where('permissionType', 'Desktop Whatsapp')->value('permission');
            return $permission ?? false; // Return the permission value or false if not found

        } catch (ModelNotFoundException $e) {
            return false; // Handle the case when the permission is not found

        } catch (Exception $e) {
            // Log or handle other exceptions
            return false;
        }
    }

    private function sendEmail($email, $token)
    {
        try {
            $failures = [];
            Mail::send('emails.Registration-form', ['token' => $token], function ($message) use ($email, &$failures) {
                $message->to($email);
                $message->subject("Registration Form");
            });

            if (!empty($failures)) {
                // There are failures, handle them as needed
                return [
                    'status' => false,
                    'error' => 'Failed to send email to ' . implode(', ', $failures),
                ];
            } else {
                // Email sent successfully
                return [
                    'status' => true,
                    'data' => [
                        'message' => "Registration Link sent to the email {$email} successfully",
                    ],
                ];
            }
        } catch (Swift_TransportException $e) {
            return $e->getMessage();
        } catch (Exception $e) {
            return $e->getMessage();
        }
    }

    private function sendMobileResponse($mobile, $token, $whatsAppPermission)
    {
        $formPath = url(route('registration.form', ['token' => $token]));

        return [
            'status' => true,
            'data' => [
                'message' => "Share link of Registration is created for mobile number {$mobile}.",
                'registerFormLink' => $formPath,
                'whatsAppPermission' => $whatsAppPermission,
                'mobileNumber' => $mobile,
            ]
        ];
    }

    private function getLicenceNo()
    {
        try {
            // Assuming Businessregister has a unique constraint on licence_no
            $licenceNo = Businessregister::pluck('license_no')->first();

            // If you expect multiple rows, use $licenceNo = Businessregister::pluck('licence_no');
            // and handle it accordingly in your business logic.

            return $licenceNo ?: '';
        } catch (Exception $e) {
            // Log the exception for debugging purposes
            Log::error("Error getting licence number: " . $e->getMessage());

            // Rethrow the exception or handle it as needed
            throw $e;
        }
    }


    public function shareRegistrationForm(Request $request)
    {
        try {
            $whatsAppPermission = $this->checkWhatsAppPermission();
            $licenceNo = $this->getLicenceNo();

            config(['database.default' => env('DB_CONNECTION')]);
            $validator = Validator::make($request->all(), [
                'shareType' => 'required|string',
                'wantTo' => 'required|string',
                'shareEmail' => 'nullable|required_if:shareType,email|email',
                'ShareMobile' => 'nullable|required_if:shareType,mobile|numeric|digits:10',
                'studentId' => 'nullable|required_if:wantTo,update|numeric'
            ], [
                'shareType.required' => 'Please select a share type.',
                'shareEmail.required_if' => 'The email field is required when share type is email.',
                'shareEmail.email' => 'Please enter a valid email address for sharing.',
                'ShareMobile.required_if' => 'The mobile field is required when share type is mobile.',
                'ShareMobile.numeric' => 'The mobile field must be a number.',
                'ShareMobile.digits' => 'The mobile field must be exactly 10 digits.',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'error' => $validator->errors()
                ], 422);
            }

            $form = new StudentRegistertoken();
            $form->licence_no = $licenceNo;
            $token = Str::random(65);
            $form->token = $token;
            $form->email = $request->shareEmail ? $request->shareEmail : $request->ShareMobile;
            $form->studentId = $request->studentId ?? null;
            $form->save();
            if ($request->shareType === "email") {
                $emailResponse = $this->sendEmail($request->shareEmail, $token);
                return response()->json($emailResponse, 200);
            } elseif ($request->shareType === "mobile") {
                $mobileResponse = $this->sendMobileResponse($request->ShareMobile, $token, $whatsAppPermission);
                return response()->json($mobileResponse, 200);
            } else {
                throw new UnexpectedValueException('Share Type Not Valid');
            }
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => Log::error("Error in shareRegistrationForm: " . $e->getMessage())
            ], 500);
        }
    }
}
