<?php

namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Exception;
use Illuminate\Http\Request;
use App\Models\Expense;
use App\Models\StockItem;
use App\Models\stock_detail;
use App\Models\Expensedetail;
use App\Models\StockCategory;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use Carbon\Carbon;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\Eloquent\ModelNotFoundException;

class StockManagementController extends Controller
{
    public function view(Request $request)
    {
        try {
            $filterMonth = $request->MontheFilter ?: Carbon::now()->format('Y-m');
            $previousMonthDate = Carbon::parse($filterMonth)->subMonth()->format('Y-m');
            $categoriesWithItemsAndDetails = StockCategory::with(['stockItem' => function ($query) use ($filterMonth) {
                $query->with(['itemDetails' => function ($query) use ($filterMonth) {
                    $query->where('month', $filterMonth);
                }]);
            }])->get();
            $categoriesWithItemsAndDetails->each(function ($category) use ($previousMonthDate) {
                $category->stockItem->each(function ($item) use ($category, $previousMonthDate) {
                    // Access each item's itemDetails collection
                    $previousMonthdata = stock_detail::where('month', $previousMonthDate)->where('item_id', $item->id)->where('category_id', $category->id)->first();
                    $item->setAttribute('previousMonthData', $previousMonthdata ?? new \stdClass());
                    $itemDetails = $item->itemDetails;
                    // Ensure item_details is set to an empty object if null or empty
                    if (empty($item->itemDetails)) {
                        $item->setAttribute('item_details', new \stdClass());
                        $category->setAttribute('orderDates', new \stdClass());
                    }

                    if (!empty($itemDetails)) {

                        $itemDetails->each(function ($detail) use (&$category, &$itemDetails) {
                            $orderDatas = $itemDetails->orderDate;
                            $orderObject = [];
                            foreach ($orderDatas as $index => $date) {
                                $key = "order" . ($index + 1) . "date";
                                $orderObject[$key] = $date;
                            }
                            $previousItemOrderDate = $category->getAttribute('orderDates');
                            if(empty($orderObject)){  
                                $category->setAttribute('orderDates', new \stdClass());
                            }elseif (!is_array($previousItemOrderDate) && !empty($orderObject)) {
                                $category->setAttribute('orderDates', $orderObject);
                            } elseif (!empty($orderObject) && count($orderObject) > count($previousItemOrderDate)) {
                                $category->setAttribute('orderDates', $orderObject);
                            }
                        });
                    }
                });
            });

            return response()->json([
                'status' => true,
                'message' => 'stock management data',
                'data' => [
                    'MontheFilter' => $filterMonth,
                    'categoriesWithItemsAndDetails' => $categoriesWithItemsAndDetails
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function stockCetegorys()
    {
        try {
            $stockCategory = StockCategory::all();
            return response()->json([
                'status' => true,
                'message' => 'stock categorys',
                'data' => [
                    'category' => $stockCategory
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function storeStockItemAndCategory(Request $request)
    {
        try {
            DB::beginTransaction();

            $rules = [
                'items' => 'required|array',
                'items.*.categoryId' => 'required|numeric',
                'items.*.categoryName' => 'required|string',
                'items.*.itemName' => 'required|string',
                'items.*.itemUnits' => 'required|string',
            ];
            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }

            $tableData = $request->items;
            foreach ($tableData as $data) {
                $categoryID = $data['categoryId'];
                if ($categoryID == 0) {
                    $newCategory = new StockCategory;
                    $newCategory->category_name = $data['categoryName'];
                    $newCategory->save();
                    $categoryID = $newCategory->id;
                }

                $stockItem = new StockItem;
                $stockItem->category_id = $categoryID;
                $stockItem->itemName = $data['itemName'];
                $stockItem->itemUnite = $data['itemUnits'];
                $stockItem->save();
            }

            DB::commit();

            return response()->json([
                'status' => true,
                'message' => 'Stock items created successfully',
            ], 200);
        } catch (Exception $e) {
            DB::rollBack();

            return response()->json([
                'status' => false,
                'message' => 'Something went wrong',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function storeEstimationStock(Request $request)
    {
        try {
            $rules = [
                'data' => 'required|array',
                'data.*.month' => 'required|date_format:Y-m',
                'data.*.categoryId' => 'required|numeric',
                'data.*.itemId' => 'required|numeric',
                'data.*.value' => 'required|numeric',
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }

            $estData = $request->input('data');
            $date = Carbon::now();

            DB::beginTransaction();

            try {
                foreach ($estData as $data) {
                    $existingRecord = stock_detail::where('item_id', $data['itemId'])
                        ->where('category_id', $data['categoryId'])
                        ->where('month', $data['month'])
                        ->first();

                    if ($existingRecord) {
                        $existingRecord->update([
                            'est_stock' => $data['value'],
                            'Date' => $date
                        ]);
                    } else {
                        stock_detail::create([
                            'item_id' => $data['itemId'],
                            'category_id' => $data['categoryId'],
                            'month' => $data['month'],
                            'est_stock' => $data['value'],
                            'Date' => $date
                        ]);
                    }
                }
                DB::commit();

                return response()->json([
                    'status' => true,
                    'message' => 'Estimation stock store successfully',
                ], 500);
            } catch (Exception $e) {
                DB::rollBack();
                return response()->json([
                    'status' => false,
                    'message' => 'Somthing went wronge',
                    'error' => $e->getMessage(),
                ], 500);
            }
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function addOrder(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'orderData' => 'required|array',
                'orderData.*.itemDetailsId' => 'required|numeric',
                'orderData.*.orderValue' => 'required|numeric',
                'orderData.*.orderNumber' => 'required|numeric|between:1,3',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Validation Error',
                    'errors' => $validator->errors(),
                ], 422);
            }

            // stock_detail
            $orderData = $request->orderData;

            foreach ($orderData as $data) {
                $record = stock_detail::find($data['itemDetailsId']);

                if ($record) {
                    $orderDates = $record->orderDate;

                    $orderNumber = $data['orderNumber'];
                    if ($orderNumber >= 1 && $orderNumber <= 3) {
                        if (isset($orderDates[$orderNumber - 1])) {
                            $orderDates[$orderNumber - 1] = Carbon::now()->format('Y-m-d');
                        } else {
                            $orderDates[] =  Carbon::now()->format('Y-m-d');
                        }
                        $orderField = "order" . $orderNumber;
                        $record->update([
                            'orderDate' => implode(',', $orderDates),
                            $orderField => $data['orderValue'],
                        ]);
                    }
                }
            }

            return response()->json([
                'status' => true,
                'message' => 'Order added successfully',
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Something went wrong',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    function addUserStock(Request $request)
    {
        try {
            $rules = [
                'data' => 'required|array',
                'data.*.itemDetailsId' => 'required|numeric',
                'data.*.value' => 'required|numeric',
            ];

            $validator = Validator::make($request->all(), $rules);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Validation errors',
                    'errors' => $validator->errors()
                ], 422);
            }

            DB::beginTransaction();
            $usedStockData = $request->data;
            try {
                foreach ($usedStockData as $data) {
                    $existingRecord = stock_detail::find($data['itemDetailsId']);

                    if ($existingRecord) {
                        $existingRecord->update([
                            'used_stock' => $data['value'],
                        ]);
                    };
                }
                DB::commit();

                return response()->json([
                    'status' => true,
                    'message' => 'used stock store successfully',
                ], 500);
            } catch (Exception $e) {
                DB::rollBack();
                return response()->json([
                    'status' => false,
                    'message' => 'Somthing went wronge',
                    'error' => $e->getMessage(),
                ], 500);
            }
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    public function destroy(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'stockId' => 'required|numeric',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'message' => 'Validation Error',
                    'errors' => $validator->errors(),
                ], 422);
            }

            $stock = stock_detail::findOrFail($request->stockId);
            $stock->delete();

            return response()->json([
                'status' => true,
                'message' => 'Stock record deleted successfully',
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'message' => 'Stock record not found',
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'message' => 'Something went wrong',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
