<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Exception;
use UnexpectedValueException;
use Illuminate\Validation\ValidationException;
use App\Models\ApiModels\StudentRegister;
use App\Models\Businessregister;
use Illuminate\Support\Facades\Validator;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Models\ApiModels\StudentLogin;
use Illuminate\Support\Facades\Hash;
use App\Models\ApiModels\UserAuthTokens;
use Carbon\Carbon;
use Illuminate\Support\Str;
use Illuminate\Support\Facades\Mail;
use App\Models\ApiModels\Sales;


class StudentManagementController extends Controller
{

    public function deleteEmptyPasswords()
    {
        $recordsToDelete = StudentLogin::whereNull('password')->orWhere('password', '=', '')->get();
        foreach ($recordsToDelete as $record) {
            $record->delete();
        }

        return response()->json(['message' => 'Records deleted successfully']);
    }

    private function checkStudentInHostel($businessDetails, $studentMobile)
    {
        try {
            $connectionName = 'custom_db';
            config([
                "database.connections.$connectionName" => [
                    'driver'   => 'mysql',
                    'host'     => $businessDetails->db_host,
                    'port'     => $businessDetails->db_port,
                    'database' => $businessDetails->db_name,
                    'username' => $businessDetails->db_username,
                    'password' => $businessDetails->db_password,
                ],
            ]);

            DB::setDefaultConnection($connectionName);

            // check database name
            //check student in the database in runing registration Table
            $studentDetails = StudentRegister::where('phone', $studentMobile)->where('status', 1)->first();
            DB::reconnect('mysql');
            if (!empty($studentDetails)) {
                return [
                    'status' => true,
                    'student_id' => $studentDetails->id,
                    'studentName' => $studentDetails->candidatename,
                    'email' => $studentDetails->email,
                ];
            } else {
                return [
                    'status' => false,
                    'message' => 'The provided mobile number is not registered.'
                ];
            }
        } catch (\Exception $e) {
            Log::error('Database connection error', ['exception' => $e->getMessage()]);
            return response()->json(['error' => 'Database connection error'], 500);
        }
    }

    public function registerHostelList(Request $request)
    {
        try {
            $city = $request->input('city');
            $state = $request->input('state');

            // Normalize city and state inputs
            $normalizedCity = strtolower($city);
            $normalizedState = str_replace(' ', '', strtolower($state));

            $query = Businessregister::select('id', 'businessname')->where('db_status', 1);

            if ($city) {
                $query->whereRaw('LOWER(city) = ?', [$normalizedCity]);
            }

            if ($state) {
                $query->whereRaw('REPLACE(LOWER(state), " ", "") = ?', [$normalizedState]);
            }

            $hostels = $query->get();

            return response()->json([
                'status' => true,
                'data' => $hostels,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                "error" => $e->getMessage(),
            ], 500);
        }
    }

    public function studentRegisterForLogin(Request $request)
    {
        try {
            $validation = Validator::make($request->all(), [
                'hostelId' => 'nullable|string',
                'studentMobile' => 'required|digits:10',
                'email' => 'nullable|email'
            ]);

            $validation->sometimes(['first_name', 'last_name'], 'required|string', function ($input) {
                return is_null($input->hostelId);
            });


            if ($validation->fails()) {
                throw new ValidationException($validation);
            }

            $hosterlId = $request->hostelId;
            // Define the data array
            $data = [];
            $checkMobileNumber = StudentLogin::where('mobile_no', $request->studentMobile)->whereNot('is_active_in_hostel', 2)->first();
            if (!empty($checkMobileNumber)) {
                return response()->json([
                    'status' => 'false',
                    'statusCode' => 409,
                    'message' => 'Student already register'
                ], 409);
            }
            // check hostelid is null or empty
            if (is_null($hosterlId)) {
                $data = [
                    'first_name' => $request->first_name,
                    'last_name' => $request->last_name,
                    'email' => $request->email,
                    'mobile_no' => $request->studentMobile,
                    'status' => 0, // not registered and no hostel id
                ];
            } else {
                // check hostelId is valid or not 
                $hostelDetail = Businessregister::where('license_no', $request->hostelId)->first();
                // Check if the student is registered in the hostel
                if (empty($hostelDetail)) {
                    return response()->json([
                        'status' => 'false',
                        'statusCode' =>  406,
                        'message' => 'Hostel Id is not Valid',
                    ], 406);
                }
                $student = $this->checkStudentInHostel($hostelDetail, $request->studentMobile);
                if (!empty($student) && $student['status'] == true) {
                    $studentName = explode(' ', $student['studentName']);
                    // Assign first name
                    $firstName = $studentName[0] ?? null;
                    // Assign last name if available
                    $lastName = $studentName[1] ?? null;
                    $data = [
                        'student_id' => $student['student_id'],
                        'first_name' => $firstName,
                        'last_name' => $lastName,
                        'email' => $student['email'],
                        'hostel_id' => $request->hostelId,
                        'mobile_no' => $request->studentMobile,
                        'status' => 2, //student register 
                        'is_active_in_hostel' => 1
                    ];
                } else {
                    $data = [
                        'mobile_no' => $request->studentMobile,
                        'email' => $request->email,
                        'hostel_id' => $request->hostelId,
                        'status' => 1,
                        'is_active_in_hostel' => 0
                    ];
                }
            }
            $studentRegister = StudentLogin::create($data);
            return response()->json([
                'status' => true,
                'statusCode' => 201,
                'message' => 'Student Registered Successfully',
                'student_login_id' => $studentRegister->id,
            ], 201);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation Error',
                'error' => $e->errors(),
            ], 422);
        } catch (\Exception $e) {
            Log::error('Exception', ['exception' => $e->getMessage()]);
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage(),
            ], 500);
        }
    }

    public function createStudentPassword(Request $request, $loginId)
    {
        try {
            // Validation rules
            $validation = Validator::make($request->all(), [
                'password' => [
                    'required',
                    'min:8', // At least 8 characters
                    'regex:/[A-Z]/', // At least one uppercase letter
                    'regex:/[0-9]/', // At least one number
                ],
                'confirm_password' => 'required_with:password|same:password' // Confirm password field
            ]);

            // Check if validation fails
            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation errors',
                    'errors' => $validation->errors(),
                ], 422);
            }

            // Find the student login by ID
            $studentLogin = StudentLogin::whereNot('is_active_in_hostel', 2)->find($loginId);
            if (!$studentLogin) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 404,
                    'message' => 'Student login not found',
                ], 404);
            }

            // Hash and save the new password
            $studentLogin->password = Hash::make($request->password);
            $studentLogin->save();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Password updated successfully',
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Server Error',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function loginStudents(Request $request)
    {
        try {
            $validation = Validator::make($request->all(), [
                'username' => 'required|string',
                'password' => 'required',
            ]);

            // Check if validation fails
            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation errors',
                    'errors' => $validation->errors(),
                ], 422);
            }

            // Check if the input username is an email or mobile number
            $isEmail = filter_var($request->username, FILTER_VALIDATE_EMAIL);
            $field = $isEmail ? 'email' : 'mobile_no';

            $studentData = StudentLogin::where($field, $request->username)->whereNull('hostel_id')->where('status', 0)->first();
            if (empty($studentData)) {
                $studentData =  StudentLogin::where($field, $request->username)->whereNot('is_active_in_hostel', 2)->first();
            }
            if (empty($studentData)) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Invalid username and password',
                ], 422);
            }
            if (!Hash::check($request->password, $studentData->password)) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Invalid username and password',
                ], 422);
            }

            if ($studentData->hostel_id == null) {
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'message' => 'login successfully',
                    'hostelStatus' => $studentData->status,
                    'token' => null,
                    'userType' => 'student',
                    'studentData' => $studentData,
                ], 200);
            } else {
                $businessDetails = Businessregister::where('license_no', $studentData->hostel_id)->firstOrFail();
                // Check if the student is registered in the hostel
                $student = $this->checkStudentInHostel($businessDetails, $studentData->mobile_no);

                if ($student['status'] == true) {
                    $studentName = explode(' ', $student['studentName']);
                    // Assign first name
                    $firstName = $studentName[0] ?? null;
                    // Assign last name if available
                    $lastName = $studentName[1] ?? null;
                    $data = [
                        'student_id' => $student['student_id'],
                        'first_name' => $firstName,
                        'last_name' => $lastName,
                        'email' => $student['email'],
                        'status' => 2, //student register 
                    ];
                } elseif ($studentData->status == 3) {
                    $data = [
                        'status' => 3, // student registration request panding
                    ];
                } else {
                    $data = [
                        'status' => 1, // student not registered but has hostel id
                    ];
                }
                // Switch back to the default database specified in .env
                config(['database.default' => env('DB_CONNECTION')]);
                $studentRegister = $studentData->update($data);
                $userId = $this->getUserIdOfhostel($studentData->hostel_id);
                //then genrate token
                $token = $this->generateUniqueToken('Admin', $studentData->hostel_id, $userId);
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'message' => 'login Successfully',
                    'studentRegister' => $studentData->student_id,
                    'token' => $token,
                    'hostelStatus' => $studentData->status,
                    'userType' => 'student',
                    'studentData' => $studentData,
                ], 200);
            }
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'server Error',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    private function getUserIdOfhostel($licenceNo)
    {
        $hostel = Businessregister::where("license_no", $licenceNo)->first();
        return $hostel->id;
    }

    private function generateUniqueToken($adminType, $licenceNo, $userId)
    {
        // $this->deleteExpiredToken($licenceNo);
        $randomString = Str::random(32);
        $timestamp = Carbon::now()->timestamp;
        $rawToken = $randomString . $timestamp;
        $hashedToken = Hash::make($rawToken);
        $createTokenRecord = new UserAuthTokens();
        $createTokenRecord->user_id = $userId;
        $createTokenRecord->token = $hashedToken;
        $createTokenRecord->usertype = $adminType;
        $createTokenRecord->licenceno = $licenceNo;
        $createTokenRecord->save();
        return $hashedToken;
    }

    public function studentRoomMeteLists($studentId)
    {
        try {
            // Check student 
            $student = StudentRegister::findOrFail($studentId); // Pass $studentId to findOrFail
            $studentRoomNo = $student->roomno;

            // Get student roommates

            $studentRoomMates = StudentRegister::where('id', '!=', $studentId)
                ->where('roomno', $studentRoomNo)
                ->where('status', 1)
                ->get();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Student roommate list',
                'data' => $studentRoomMates,
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Student not found'
            ], 404);
        } catch (Exception $e) {
            // Catch all other exceptions
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    public function forgetStudentPassword(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'email' => ['required', 'string', 'email', 'max:255'],
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation error',
                    'error' => $validator->errors()
                ], 422);
            }
            // Find the student by email
            $student = StudentLogin::where('email', $request->email)->firstOrFail();

            // Generate 4-digit OTP
            $otp = str_pad(random_int(1111, 9999), 4, '0', STR_PAD_LEFT);

            // store this otp to the table 
            $student->update([
                'forget_password_otp' => $otp,
            ]);
            // Send email with OTP
            Mail::send('emails.forget-student-password', ['otp' => $otp], function ($message) use ($request) {
                $message->to($request->email);
                $message->subject("Forget Password OTP");
            });

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'OTP sent to your email.',
            ], 200);
        } catch (ModelNotFoundException $ex) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Email does not exist!',
                'error' => $ex->getMessage(),
            ], 404);
        } catch (Exception $ex) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Server Error!',
                'error' => $ex->getMessage(),
            ], 500);
        }
    }

    public function verifyForgetPasswordOtp(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'email' => 'required|string|email',
                'otp' => 'required|digits:4',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => "Validation error",
                    'errors' => $validator->errors(),
                ]);
            }
            // Check the OTP and email 
            $checkOtp = StudentLogin::where('email', $request->email)->firstOrFail();
            if ($request->otp == $checkOtp->forget_password_otp) {
                $checkOtp->update([
                    'forget_password_otp' => null,
                ]);
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'message' => "OTP verified",
                    'data' => [
                        'studentId' => $checkOtp->id,
                    ]
                ], 200);
            } else {
                return response()->json([
                    'status' => false,
                    'statusCode' => 200,
                    'message' => "Invalid OTP!"
                ], 200);
            }
        } catch (ModelNotFoundException $ex) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Invalid email',
                'error' => $ex->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Server Error!',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
    public function  resetPassword(Request $request, $studentId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'password' => [
                    'required',
                    'min:8',
                    'regex:/[A-Z]/',
                    'regex:/[0-9]/',
                ],
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation error',
                    'error' => $validator->errors()
                ], 422);
            }
            // check student id is exist or not
            $student = StudentLogin::findOrFail($studentId);
            // update password in database
            $student->update([
                'password' => Hash::make($request->input('password')),
            ]);
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'password reset succssfully',
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Server Error!',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function oldRecordsOfTheStudent(Request $request)
    {
        try {
            $validation = Validator::make($request->all(), [
                'studentMobile' => 'required|numeric',
            ]);

            if ($validation->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 402,
                    'message' => 'Validation Error',
                    'error' => $validation->errors(),
                ]);
            }

            $studentMobile = $request->studentMobile;
            $oldStudentRecordsArray = [];

            // Ensure we are using the main database connection

            // Get the student's old records from the student login table where is_active_in_hostel has value 2
            $studentRecords = StudentLogin::where('mobile_no', $studentMobile)
                ->where('is_active_in_hostel', 2)
                ->get();
            DB::setDefaultConnection('mysql');

            if ($studentRecords->isEmpty()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 404,
                    'message' => 'Student not found',
                ], 404);
            }

            foreach ($studentRecords as $student) {
                $hostelId = $student->hostel_id;
                $studentId = $student->student_id;
                $hostelDetails = Businessregister::where('license_no', $hostelId)->first();

                if ($hostelDetails) {
                    $this->switchDatabaseConnection($hostelDetails);

                    $oldStudentRecords = StudentRegister::with('roomRent')->find($studentId);

                    if ($oldStudentRecords) {
                        $totalRemaningAmount = $this->getTotalRemaningAmount($oldStudentRecords->id);

                        $oldStudentRecords->setAttribute('total_remaining_amount', $totalRemaningAmount);
                        $oldStudentRecordsArray[] = $oldStudentRecords;
                    }
                    // Reset the connection back to the default (main database)
                    DB::setDefaultConnection(config('database.default'));
                }
            }

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Student old records fetched successfully',
                'data' => $oldStudentRecordsArray,
            ]);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Something went wrong',
                'error' => $e->getMessage()
            ]);
        }
    }

    protected function switchDatabaseConnection($hostel)
    {
        $db_name = $hostel->db_name;
        $db_username = $hostel->db_username;
        $db_password = $hostel->db_password;
        $db_host = $hostel->db_host;
        $db_port = $hostel->db_port;

        config(['database.connections.custom_db' => [
            'driver' => 'mysql',
            'host' => $db_host,
            'port' => $db_port,
            'database' => $db_name,
            'username' => $db_username,
            'password' => $db_password,
        ]]);

        DB::purge('custom_db');
        DB::setDefaultConnection('custom_db');
    }

    protected function getTotalRemaningAmount($studentId)
    {
        $allSalesRecords = Sales::where('studentid', $studentId)->get();
        $totalRemaningAmount = 0;

        foreach ($allSalesRecords as $records) {
            $totalRemaningAmount += $records->remaining_amount;
        }

        return $totalRemaningAmount;
    }
}
