<?php


namespace App\Http\Controllers\Api;

use App\Http\Controllers\Controller;
use Illuminate\Http\Request;
use App\Models\SubscriptionFeatures;
use App\Models\SubscriptionPermission;
use App\Models\Subscription;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use App\Models\Salesubscription;
use App\Models\ApiModels\RoomSeats;
use Exception;
use App\Models\ApiModels\BussinessModel;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Carbon\Carbon;

class SubscriptionController extends Controller
{
    private function makeSubscriptionFeaturesModify($allFeatures)
    {
        if (!is_array($allFeatures) || count($allFeatures) <= 6) {
            return $allFeatures; // No modification needed
        }

        // Extract first 5 features and last feature
        $firstFive = array_slice($allFeatures, 0, 5);
        $lastFeature = end($allFeatures);

        // Combine first five features with the last one
        $Features = array_merge($firstFive, [$lastFeature]);

        return $Features;
    }

    public function getAllSubscriptionPlanDetails()
    {
        try {
            $subscriptionData = [];
            $planDescriptionTitle = [
                "S-mart Basic" => [
                    'planTitle' => "Quick Start",
                    'backgroundColor' => '#92cddb',
                    'borderColor' => '#92cddf',
                    'footerBackgroundColor' => '#dbeef4',
                ],
                "S-mart Silver" => [
                    'planTitle' => "Value For Many",
                    'backgroundColor' => '#d8d8d8',
                    'borderColor' => '#a3ad89',
                    'footerBackgroundColor' => "#f2f2f2",
                ],
                "S-mart Gold" => [
                    'planTitle' => "Most Populer",
                    'backgroundColor' => '#fcf276',
                    'borderColor' => '#ffd83a',
                    'footerBackgroundColor' => "#fafad4",
                ],
                "S-mart Platinum" => [
                    'planTitle' => "Extreemly Popular",
                    'backgroundColor' => '#fcf276',
                    'borderColor' => '#ffd83a',
                    'footerBackgroundColor' => "#fff",
                ],
                "S-mart Premium" => [
                    'planTitle' => "Premium Plan",
                    'backgroundColor' => '#fcf276',
                    'borderColor' => '#ffd83a',
                    'footerBackgroundColor' => "#fff",
                ],
            ];
            $getsubscriptiondata = Subscription::where("status", 1)
                ->orderBy('subs_price', 'asc')
                ->whereNotIn('package_duration', ['7 Days', '15 Days'])
                ->groupBy('package_category')
                ->get();

            if (empty($getsubscriptiondata)) {
                throw new Exception("Subscription Modal are not created yet");
            }

            foreach ($getsubscriptiondata as $key => $value) {
                $subscriptionData[] = [
                    'styleAndTitel' => $planDescriptionTitle[$value->package_category],
                    'planName' => $value->package_category,
                    'perBedAmount' => $value->subscription_price,
                    'perBedDisAmount' => $value->subs_price,
                    'discount' => $value->sub_discount,
                    'features' => $this->makeSubscriptionFeaturesModify($value->subscription_features),
                ];
            }
            return response()->json([
                'status' => true,
                'message' => 'Successfully Get All Subscription Plans Details!',
                'statusCode' => 200,
                'data' => $subscriptionData,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => "Somthing wronge",
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function getSelectedPlanDetails()
    {
        $subscriptonId =  BussinessModel::value('license_no');
        // check its subscription
        if (is_null($subscriptonId)) {
            return null;
        } else {
            $ActivePlanDetails =  Salesubscription::where('user_id', $subscriptonId)
                ->where('status', 1)
                ->first();
            return $ActivePlanDetails;
        }
    }

    public function subscriptionFeaturesDetailsByFilter(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'duration' => 'required|string',
                'planType' => 'required|string'
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => "validation failed",
                    'errors' => $validator->errors()->toArray()
                ], 422);
            }
            $activPlanDetails = $this->getSelectedPlanDetails();
            $subscriptionData = [];
            $getsubscriptiondata = Subscription::where("status", 1)
                ->where('package_title', $request->planType)
                ->where('package_duration', $request->duration)
                ->orWhere('package_title', 'trial')
                ->orderBy('subs_price', 'asc')
                ->get();

            if (empty($getsubscriptiondata)) {
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'data' => $subscriptionData,
                    'message' => 'Subscriptions Modals are not created yet',
                ]);
            }
            $selectedPlandMessage = "Upgrade seats and  duration with same plan from here.";
            $plansFooterMessages = [
                "S-mart Basic" => "Change Your plan in S-mart Basic with same selection of current plan and also upgrade seats, duration and type of plan from here.",
                "S-mart Silver" => "Change Your plan in S-mart Silver with same selection of current plan and also upgrade seats, duration and  type of plan from here.",
                "S-mart Gold" => "Change Your plan in S-mart Gold with same selection of current plan and also upgrade seats, duration and  type of plan from here.",
                "S-mart Platinum" => "Change Your plan in S-mart Platinum with Premium selection of current plan and also upgrade seats, duration and  type of plan from here.",
                "S-mart Premium" => "Change Your plan in S-mart Premium with same selection of current plan and also upgrade seats, duration and  type of plan from here.",
            ];
            $planDescriptionTitle = [
                "S-mart Basic" => [
                    "planTitle" => "Quick Start",
                    'backgroundColor' => '#9AF3F3',
                    'borderColor' => '#1ee6e7',
                ],
                "S-mart Silver" => [
                    "planTitle" =>  "Value For Many",
                    'backgroundColor' => '#F0E7E7',
                    'borderColor' => '#6c757d',
                ],
                "S-mart Gold" => [
                    "planTitle" =>   "Most Populer",
                    'backgroundColor' => '#Ffd700',
                    'borderColor' => '#ffc107',
                ],
                "S-mart Platinum" => [
                    "planTitle" => "Extreemly Popular",
                    'backgroundColor' => '#Ffd700',
                    'borderColor' => '#ffc107',
                ],
                "S-mart Premium" => [
                    "planTitle" =>  "Premium Plan",
                    'backgroundColor' => '#Ffd700',
                    'borderColor' => '#ffc107',
                ],
            ];

            foreach ($getsubscriptiondata as $key => $value) {
                $subscriptionData[] = [
                    'id' => $value->id,
                    'plansFooterMessages' => $activPlanDetails ? $selectedPlandMessage : $plansFooterMessages[$value->package_category],
                    'styleAndPlanTitle' => $planDescriptionTitle[$value->package_category],
                    'packageName' => $value->package_category,
                    'disSubsPrice' => $value->subs_price,
                    'subsPrice' => $value->subscription_price,
                    'subDiscount' => $value->sub_discount,
                    'features' =>  $value->subscription_features,
                    'planType' => $value->package_title,
                    'planDuration' => $value->package_duration,
                    "packagecode" =>  $value->packagecode,
                    'selectedPlandDetails' => $activPlanDetails,
                    'currentTotalSeats' => $this->getCurrentActiveRoomSeats(),
                ];
            }
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'successfully get the data',
                'data' => $subscriptionData
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Somthing Wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function filterSubscriptionDetails()
    {
        try {
            $package_title = Subscription::where('status', 1)
                ->whereNotIn('package_duration', ['7 Days', '15 Days'])
                ->select('package_title', 'package_duration')
                ->distinct()
                ->get();

            $filterData = [
                'planType' => [],
                'durations' => []
            ];

            if ($package_title->isNotEmpty()) {
                $planTypes = [];
                $durations = [];

                foreach ($package_title as $val) {
                    // Check if the plan type is not already in the array
                    if (array_search($val->package_title, $planTypes) === false) {
                        $planTypes[] = $val->package_title;
                    }

                    // Check if the duration is not already in the array
                    if (array_search($val->package_duration, $durations) === false) {
                        $durations[] = $val->package_duration;
                    }
                }

                // Update filterData with the collected plan types and durations
                $filterData['planType'] = $planTypes;
                $filterData['durations'] = $durations;
            }

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Successfully Get The Package filter List.',
                'data' => $filterData,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Something went wrong.',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function subscriptionFeatureAndPermissionCompare()
    {
        try {
            $allFeatures = SubscriptionFeatures::whereNotNull('all_features')
                ->where('all_features', '<>', '')
                ->pluck('all_features')
                ->toArray();
            $S_mart_Basic = SubscriptionFeatures::whereNotNull('S_mart_Basic')
                ->where('S_mart_Basic', '<>', '')
                ->pluck('S_mart_Basic')
                ->toArray();
            $S_mart_Silver = SubscriptionFeatures::whereNotNull('S_mart_Silver')
                ->where('S_mart_Silver', '<>', '')
                ->pluck('S_mart_Silver')->toArray();

            $S_mart_Gold = SubscriptionFeatures::whereNotNull('S_mart_Gold')
                ->where('S_mart_Gold', '<>', '')
                ->pluck('S_mart_Gold')->toArray();
            $S_mart_Platinum = SubscriptionFeatures::whereNotNull('S_mart_Premium')
                ->where('S_mart_Premium', '<>', '')

                ->pluck('S_mart_Platinum')->toArray();
            $S_mart_Premium = SubscriptionFeatures::whereNotNull('S_mart_Premium')
                ->where('S_mart_Premium', '<>', '')
                ->pluck('S_mart_Premium')->toArray();

            $allPermissions = SubscriptionPermission::whereNotNull('all_permissions')
                ->where('all_permissions', '<>', '')
                ->pluck('all_permissions')
                ->toArray();

            $S_mart_Basic_permission = SubscriptionPermission::whereNotNull('S_mart_Basic')
                ->where('S_mart_Basic', '<>', '')
                ->pluck('S_mart_Basic')
                ->toArray();
            $S_mart_Silver_permission = SubscriptionPermission::whereNotNull('S_mart_Silver')
                ->where('S_mart_Silver', '<>', '')
                ->pluck('S_mart_Silver')->toArray();

            $S_mart_Gold_permission = SubscriptionPermission::whereNotNull('S_mart_Gold')
                ->where('S_mart_Gold', '<>', '')
                ->pluck('S_mart_Gold')->toArray();
            $S_mart_Platinum_permission = SubscriptionPermission::whereNotNull('S_mart_Premium')
                ->where('S_mart_Premium', '<>', '')
                ->pluck('S_mart_Platinum')->toArray();

            $S_mart_Premium_permission = SubscriptionPermission::whereNotNull('S_mart_Premium')
                ->where('S_mart_Premium', '<>', '')
                ->pluck('S_mart_Premium')->toArray();


            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'successfully get data.',
                'data' => [
                    'featureCompareData' => [
                        'allFeatures' => $allFeatures,
                        'S_mart_Basic' => $S_mart_Basic,
                        'S_mart_Silver' => $S_mart_Silver,
                        'S_mart_Gold' => $S_mart_Gold,
                        'S_mart_Platinum' => $S_mart_Platinum,
                        'S_mart_Premium' => $S_mart_Premium,
                    ],
                    'permissionCompareData' => [
                        'allPermissions' => $allPermissions,
                        'S_mart_Basic_permission' => $S_mart_Basic_permission,
                        'S_mart_Silver_permission' => $S_mart_Silver_permission,
                        'S_mart_Gold_permission' => $S_mart_Gold_permission,
                        'S_mart_Platinum_permission' => $S_mart_Platinum_permission,
                        'S_mart_Premium_permission' => $S_mart_Premium_permission,
                    ]
                ]
            ], 200);

            return view('fronted.compareSubscriptionPlan', compact());
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Somthing Wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    private function getCurrentActiveRoomSeats()
    {
        return RoomSeats::sum('numberseats');
    }

    public function getSubscriptionAmout(Request $request, $selectPlanId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'wantToupgradeSeats' => 'required|numeric',
                'currentPlanPackageCode' => 'nullable|string',
                'selectedPlanId' => 'nullable|numeric',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 402,
                    'message' => 'validation Error',
                    'error' => $validator->errors(),
                ], 422);
            }
            $selectedPlanId = $request->selectedPlanId;
            $subscriptionDetails = [];
            $calculatationData = [];
            $currentPlan = $request->currentPlanPackageCode;
            $registerSeats = $this->getCurrentActiveRoomSeats();
            $wantToUpgradeSeats = $request->wantToupgradeSeats;

            $getsubscriptiondata = Subscription::findOrFail($selectPlanId);

            // if the selecte plan is trial then 
            if ($getsubscriptiondata->package_title  == "Trial") {
                $subscriptionDetails = [
                    "PlanName" => $getsubscriptiondata->package_category,
                    "PlanDuration" => $getsubscriptiondata->package_duration,
                    "PlanType" => $getsubscriptiondata->package_title,
                    "subscription_price" => $getsubscriptiondata->subscription_price,
                    "sub_discount" => $getsubscriptiondata->sub_discount,
                    "subs_price" => $getsubscriptiondata->subs_price,
                    "tax_rate" => 0,
                    "total_price" => 1,
                    "packagecode" =>  $getsubscriptiondata->packagecode,
                    'calculationData' => [
                        "status" => true,
                        "totalUpgradedSeats" => $registerSeats + $wantToUpgradeSeats,
                        "totalUpgradeAmount" => 0,
                        "remaningUpgradedAmount" => 0,
                        'payAbleAmountForUpgrade' => 1,
                    ],
                    'totalSeats' => ($registerSeats + $wantToUpgradeSeats)
                ];
                return response()->json([
                    "status" => true,
                    "statusCode" => 200,
                    'message' => 'Successfully fetched subscription details',
                    'data' => $subscriptionDetails
                ], 200);
            }


            // check if the user have same plans or not have any plan
            if (($currentPlan === $getsubscriptiondata->packagecode)) {
                //if user want to upgrade seats then calculate amount based on number of seats
                $calculatationData = $this->calculateAmountForSeatUpgrade($wantToUpgradeSeats, $getsubscriptiondata->total_price, $selectedPlanId);
            } else {
                $totalSeats = $registerSeats + $wantToUpgradeSeats;
                $calculatationData = $this->calculateAmountForPlanUpgrade($totalSeats, $getsubscriptiondata->package_category, $getsubscriptiondata->package_duration, $getsubscriptiondata->package_title, $selectedPlanId);
            }
            $subscriptionDetails = [
                "PlanName" => $getsubscriptiondata->package_category,
                "PlanDuration" => $getsubscriptiondata->package_duration,
                "PlanType" => $getsubscriptiondata->package_title,
                "subscription_price" => $getsubscriptiondata->subscription_price,
                "sub_discount" => $getsubscriptiondata->sub_discount,
                "subs_price" => $getsubscriptiondata->subs_price,
                "tax_rate" => $getsubscriptiondata->tax_rate,
                "total_price" => $getsubscriptiondata->total_price,
                "packagecode" =>  $getsubscriptiondata->packagecode,
                'calculationData' => $calculatationData,
                'totalSeats' => ($registerSeats + $wantToUpgradeSeats)
            ];

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Subscription Calculation Details.',
                'data' => $subscriptionDetails,
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Selected Pland Is Not Valid',
                'error' => $e->getMessage()
            ], 500);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'server error',
                'error' => $e->getMessage()
            ], 500);
        }
    }

    private function calculateAmountForSeatUpgrade($newUpgradeSeats, $plansPrice, $selectedPlanId)
    {

        try {
            if (!$selectedPlanId) {
                throw new Exception("current Pland not selected there was the calculation not found");
            }

            $selectedPlanId = (int) $selectedPlanId;

            $subscription = Salesubscription::findOrFail($selectedPlanId);

            // Extract relevant subscription data
            $oldSeats = $subscription->no_of_seats;
            $paidAmount = $subscription->paid_amount;
            $planStartDate = Carbon::parse($subscription->startdate);
            $planEndDate = Carbon::Parse($subscription->enddate);
            $prSetAmount = $subscription->plan_price;

            // Calculate total days in the subscription plan
            $totalDays = $planStartDate->diffInDays($planEndDate) + 1; // Including both start and end dates

            // Calculate days used in the subscription plan
            $daysUsed = $planStartDate->diffInDays(now()) + 1; // Including current day

            // Calculate remaining days in the subscription plan
            $remainingDays = max(0, $totalDays - $daysUsed);
            //calculate per day amount 
            $prdayAmount = $paidAmount / $totalDays;
            // calculate cost of used day
            $usedCost = round(($prdayAmount * $daysUsed), 2);
            //calculate reamanign amount 
            $remaningAmontOfplans = round(($paidAmount  - $usedCost), 2);
            // calcualte  new amount for total seat (old + new);
            $totalSeats = $oldSeats + $newUpgradeSeats;
            $newAmount = round(($totalSeats * $plansPrice), 2);
            return [
                "status" => true,
                "totalUpgradedSeats" => $totalSeats,
                "totalUpgradeAmount" => round($newAmount, 2),
                "remaningUpgradedAmount" => round($remaningAmontOfplans, 2),
                'payAbleAmountForUpgrade' => round(($newAmount - $remaningAmontOfplans), 2),
            ];
        } catch (Exception $e) {
            return [];
        }
    }

    private function calculateAmountForPlanUpgrade($registerSeats, $planName, $duration, $plan, $selectedPlanId)
    {
        try {
            // Retrieve subscription data
            if (!$selectedPlanId) {
                throw new Exception("current Pland not selected there was the calculation not found");
            }
            $selectedPlanId = (int) $selectedPlanId;
            $subscription = Salesubscription::findOrFail($selectedPlanId);

            $UpgradePlanDetails = Subscription::where('package_category', $planName)->where('package_title', $plan)->where('package_duration', $duration)->where("status", 1)->first();
            $upgradePlanPrice = $UpgradePlanDetails->total_price;
            // Extract relevant subscription data
            $paidAmount = $subscription->paid_amount;
            $planStartDate = Carbon::parse($subscription->startdate);
            $planEndDate = Carbon::Parse($subscription->enddate);
            // Calculate total days in the subscription plan
            $totalDays = $planStartDate->diffInDays($planEndDate) + 1; // Including both start and end dates

            // Calculate days used in the subscription plan
            $daysUsed = $planStartDate->diffInDays(now()) + 1; // Including current day

            // Calculate remaining days in the subscription plan
            $remainingDays = max(0, $totalDays - $daysUsed);
            //calculate per day amount 
            $prdayAmount = $paidAmount / $totalDays;
            // calculate cost of used day
            $usedCost = round(($prdayAmount * $daysUsed), 2);
            //calculate reamanign amount 
            $remaningAmontOfplans = round(($paidAmount  - $usedCost), 2);
            // calcualte  new amount for total seat (old + new);
            $totalSeats = $registerSeats;
            $newAmount = round(($totalSeats * $upgradePlanPrice), 2);
            return [
                "status" => true,
                "totalUpgradedSeats" => $totalSeats,
                "totalUpgradeAmount" => round($newAmount, 2),
                "remaningUpgradedAmount" => round($remaningAmontOfplans, 2),
                'payAbleAmountForUpgrade' => round(($newAmount - $remaningAmontOfplans), 2),
            ];
        } catch (ModelNotFoundException $e) {
            return [];
        } catch (Exception $e) {
            return [];
        }
    }

    private function getLicenceNo()
    {
        return BussinessModel::value('license_no');
    }


    private function calculateEndDateAndDays($startDate, $planType, $numberOfUnits)
    {
        $startDateObj = Carbon::createFromFormat('Y-m-d', $startDate);
        $endDateObj = $startDateObj->copy(); // Create a copy for calculating end date

        switch ($planType) {
            case 'Month':
                $endDateObj->addMonths($numberOfUnits);
                break;
            case 'Years':
            case 'Year':
                $endDateObj->addYears($numberOfUnits);
                // Handle leap year (add an extra day for each leap year)
                $numberOfLeapYears = intval($endDateObj->diffInYears($startDateObj) / 4); // Count leap years
                $endDateObj->addDays($numberOfLeapYears); // Add extra days for leap years
                break;
            case 'Months':
                $endDateObj->addMonths($numberOfUnits);
                break;
            case 'Days':
                $endDateObj->addDays($numberOfUnits);
                break;
            default:
                echo 'Invalid plan type';
                return null;
        }

        // Calculate total number of days
        $numberOfDays = $endDateObj->diffInDays($startDateObj);

        // Adjust the end date to be the last day of the subscription period
        $endDateObj->subDay(); // Subtract 1 day to get the last day of the period
        $endDate = $endDateObj->format('Y-m-d');

        return ['endDate' => $endDate, 'numberOfDays' => $numberOfDays];
    }

    public function addSubscription(Request $request)
    {
        DB::beginTransaction();
        try {
            $validator = Validator::make($request->all(), [
                'totalSeats' => 'required|numeric',
                'packageCode' => 'required|string',
                'packageName' => 'required|string',
                'planDuration' => 'required|string',
                'planPrice' => 'required|numeric',
                'totalPaybleAmount' => 'required|numeric',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Failed',
                    'errors' => $validator->errors()
                ], 422);
            }
            // first check its have  any active subscription or not.
            $licenceNo = $this->getLicenceNo();
            $userActiveSubscriptions = Salesubscription::where('user_id', $licenceNo)->where('status', 1)->first();
            // add transition here
            if (!empty($userActiveSubscriptions)) {
                $userActiveSubscriptions->update([
                    'status' => 0,
                ]);
            }

            $planDuration = $request->planDuration;
            //  calculate the end date for the subscription on the base of month day year
            $planDurationArry = explode(' ', $planDuration);
            if ($planDurationArry[1] == "Days" || $planDurationArry[1] == "days") {
                $startDate = Carbon::today()->toDateString();
                $startDateObj = Carbon::createFromFormat('Y-m-d', $startDate);
                $futureDate = $startDateObj->addDays($planDurationArry[0]);
                $endDate = $futureDate->format('Y-m-d');
                $noOfDays = $planDurationArry[0];
            } else {
                $startDate = Carbon::today()->toDateString();
                $result = $this->calculateEndDateAndDays($startDate, $planDurationArry[1], $planDurationArry[0]);
                $endDate = $result['endDate'] . PHP_EOL;
                $noOfDays = $result['numberOfDays'];
            }

            Salesubscription::create([
                'user_id' => $licenceNo,
                'plan_id' => $request->packageCode,
                'startdate' => $startDate,
                'enddate' => $endDate,
                'counter' => $noOfDays,
                'no_of_seats' => $request->totalSeats,
                'paid_amount' => $request->totalPaybleAmount,
                'package_type' => $request->packageName,
                'plan_price' => $request->planPrice,
                'status' => 1
            ]);
            DB::commit();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'subscription add successffuly'
            ], 200);
        } catch (Exception $e) {
            DB::rollBack();

            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'Somthing went wronge',
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
