<?php

namespace App\Http\Controllers\Api;

use Illuminate\Http\Request;
use App\Http\Controllers\Controller;
use Illuminate\Database\Eloquent\Model;
use DB;
use App\Models\TanantComplaint;
use Exception;
use App\Models\TanantNotice;
use Carbon\Carbon;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\Validator;
use Illuminate\Validation\ValidationException;
use Illuminate\Support\Facades\File;

class TanantComplaintController extends Controller
{
    private function resizeBase64Image($base64Data, $targetSizeInKB)
    {
        try {
            // Decode base64 data
            $decodedData = base64_decode($base64Data);

            // Check if decoding was successful
            if ($decodedData === false) {
                throw new Exception('Base64 decoding failed');
            }

            // Create an image resource from the decoded data
            $img = imagecreatefromstring($decodedData);

            // Check if image creation was successful
            if ($img === false) {
                throw new Exception('Image creation failed');
            }

            // Get the original image dimensions
            $width = imagesx($img);
            $height = imagesy($img);

            // Set the desired width and calculate the corresponding height to maintain the aspect ratio
            $targetWidth = 200; // Adjust this value based on your requirements
            $targetHeight = ($targetWidth / $width) * $height;

            // Create a new image resource for the resized image
            $resizedImg = imagecreatetruecolor($targetWidth, $targetHeight);

            // Resize the image
            imagecopyresampled($resizedImg, $img, 0, 0, 0, 0, $targetWidth, $targetHeight, $width, $height);

            // Create a buffer for the resized image
            ob_start();

            // Output the resized image to the buffer
            imagejpeg($resizedImg, null, 90); // Adjust the quality as needed

            // Get the buffer content
            $resizedImageData = ob_get_clean();

            $quality = 90;

            // Check if the resized image meets the target size
            while (strlen($resizedImageData) > ($targetSizeInKB * 1024)) {
                // If the size is still larger than the target, reduce the quality and update the buffer
                $quality -= 10; // Reduce the quality by 10 (adjust as needed)

                ob_start();
                imagejpeg($resizedImg, null, $quality);
                $resizedImageData = ob_get_clean();
            }

            // Encode the resized image to base64
            $resizedBase64Data = base64_encode($resizedImageData);

            // Destroy the image resources
            imagedestroy($img);
            imagedestroy($resizedImg);

            return $resizedBase64Data;
        } catch (Exception $e) {
            // Log or handle the exception as needed
            error_log('Error in resizeBase64Image: ' . $e->getMessage());
            return null; // or handle the error in an appropriate way for your application
        }
    }
    public function complaintView(Request $request)
    {
        try {
            $filterType = $request['complaint-fillter-type'];
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];
            $complaintStatus = $request['complaint'];
            if (!empty($filterType) && $filterType !== "all") {
                if ($filterType == "custom" && !empty($startDate) && !empty($endDate)) {
                    $complaints = TanantComplaint::with('student')
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->get();
                } else {
                    $complaints = TanantComplaint::with('student')
                        ->where('complaint_type', $filterType)
                        ->get();
                }
            } elseif (!empty($complaintStatus)) {
                $complaints = TanantComplaint::with('student')
                    ->where('action', $complaintStatus)
                    ->get();
            } else {
                $complaints = TanantComplaint::with('student')->get();
            }

            $activeComplaint = TanantComplaint::where('action', 1)->count();
            $closedComplaint = TanantComplaint::where('action', 2)->count();
            $unseenComplaint = TanantComplaint::where('action', 0)->count();
            $totalComplaint = TanantComplaint::count();

            return response()->json([
                'status' => true,
                'message' => 'get all the complaint succsessfully',
                'data' => [
                    'complaintData' =>  [
                        [
                            'title' => 'Total Complaints',
                            'value' => $totalComplaint ?? 0,
                            'status' => null,
                        ],
                        [
                            'title' => 'Active Complaints',
                            'value' => $activeComplaint ?? 0,
                            'status' => 1,
                        ],
                        [
                            'title' => 'Closed Complaints',
                            'value' => $closedComplaint ?? 0,
                            'status' => 2,
                        ],
                        [
                            'title' => 'Unseen Complaints',
                            'value' => $unseenComplaint ?? 0,
                            'status' => 0,
                        ],
                    ],
                    'tableData' => $complaints
                ],
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateComplaintAction(Request $request, $complaintId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'action' => 'required|string',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'errors' => $validator->errors()
                ], 422);
            }
            $complaint = TanantComplaint::findOrFail($complaintId);
            $complaint->action = $request->input('action');
            $complaint->update();
            return response()->json([
                'status' => true,
                'message' => 'Action updated successfully'
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such complaints found'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function giveComplaintResponse(Request $request, $complaintId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'response' => 'required|string',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'errors' => $validator->errors()
                ], 422);
            }
            $complaint = TanantComplaint::findOrFail($complaintId);
            $complaint->responseMsg = $request->response;
            $complaint->update();
            return response()->json([
                'status' => true,
                'message' => 'Complaint response add successfully'
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such complaints found'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => 'Complaint response not added.'
            ], 500);
        }
    }

    public function noticeView(Request $request)
    {
        try {
            $filterType = $request['notice-fillter-type'];
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];
            $noticeStatus = $request['notice'];

            if (!empty($filterType) && $filterType !== "all") {
                if ($filterType == "custom" && !empty($startDate) && !empty($endDate)) {
                    $notices = TanantNotice::with('student')
                        ->whereBetween('noticeDate', [$startDate, $endDate])
                        ->get();
                } else {
                    $notices = TanantNotice::with('student')
                        ->where('noticeType', $filterType)
                        ->get();
                }
            } elseif (!empty($noticeStatus)) {
                $notices = TanantNotice::with('student')
                    ->where('action', $noticeStatus)
                    ->get();
            } else {
                $notices = TanantNotice::with('student')->get();
            }

            $activeNotice = TanantNotice::where('action', 1)->count();
            $closedNotice = TanantNotice::where('action', 2)->count();
            $unseenNotice = TanantNotice::where('action', 0)->count();
            $totalNotice = TanantNotice::count();

            return response()->json([
                'status' => true,
                'data' => [
                    'notice' => [
                        [
                            'title' =>  'Total Notice',
                            'value' =>  $totalNotice ?? 0,
                        ],
                        [
                            'title' => 'Active Notice',
                            'value' =>  $activeNotice ?? 0,
                        ],
                        [
                            'title' => 'Closed Notice',
                            'value' =>  $closedNotice ?? 0,
                        ],
                        [
                            'title' => 'UnSeen Notice',
                            'value' => $unseenNotice ?? 0,
                        ],
                    ],
                    'tableData' => $notices
                ],
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function updateNoticeAction(Request $request, $noticeId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'action' => 'required|string',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'errors' => $validator->errors()
                ], 422);
            }
            $TanantNotice = TanantNotice::findOrFail($noticeId);
            $TanantNotice->action = $request->action;
            $TanantNotice->update();
            return response()->json([
                'status' => true,
                'message' => "Successfully updated the action of notice.",
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such notice found'
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function giveNoticeResponse(Request $request, $noticeId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'response' => 'required|string',
            ]);
            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'errors' => $validator->errors()
                ], 422);
            }
            $notice = TanantNotice::findOrFail($noticeId);
            $notice->responseMsg = $request->response;
            $notice->update();
            return response()->json([
                'status' => true,
                'message' => 'Notice response add successfully'
            ]);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => 'No such notice found'
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function createComplaint(Request $request, $studentId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'complaint_date' => 'required|date',
                'complaint_type' => 'required|string',
                'complaint_msg' => 'required|string',
                'document' => 'nullable|image|mimes:jpeg,png,jpg',
            ]);

            if ($validator->fails()) {
                throw new ValidationException($validator);
            }

            // Create a new complaint record
            $complaint = new TanantComplaint();
            $complaint->studentId = $studentId;
            $complaint->complaint_date = $request->complaint_date;
            $complaint->complaint_type = $request->complaint_type;
            $complaint->complaint_msg = $request->complaint_msg;

            // Handle the document upload if it exists
            if ($request->hasFile('document') && $request->file('document')->isValid()) {
                $file = $request->file('document');
                $base64Data = base64_encode(file_get_contents($file->path()));
                $resizedBase64Data = $this->resizeBase64Image($base64Data, 50);
                if ($resizedBase64Data !== null) {
                    $filename = $this->generateFileName($file);
                    File::put(public_path('assets/uploads/complaintDocuments/' . $filename), base64_decode($resizedBase64Data));
                    $complaint->document = $filename;
                }
            }
            // Save the complaint record
            $complaint->save();

            return response()->json([
                'status' => true,
                'statusCode' => 201,
                'message' => 'Complaint created successfully.',
                'data' => $complaint
            ], 201);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation Error',
                'errors' => $e->errors()
            ], 422);
        } catch (Exception $e) {
            // Catch any other exceptions and return a general error response
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred while creating the complaint.',
                'error' => $e->getMessage()
            ], 500);
        }
    }
    private function generateFileName($file)
    {
        $extension = $file->getClientOriginalExtension();
        $filename = time() . '_' . rand() . '.' . $extension;
        return $filename;
    }

    public function getAllComplaintsByStudent($studentId)
    {
        try {
            $studentComlaints =  TanantComplaint::where('studentId', $studentId)->with(['student' => function ($query) {
                $query->select('id', 'roomno', 'seetno');
            }])->get();
            return response()->json([
                "status" => true,
                'statusCode' => 200,
                'message' => 'Students Complain List',
                'data' => $studentComlaints
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'server Error',
            ], 500);
        }
    }


    public function createNoticeRequest(Request $request, $studentId)
    {
        try {

            $validator = Validator::make($request->all(), [
                'noticeDate' => 'required|date',
                'noticeType' => 'required|string',
                'NoticeMsg' => 'required|string',
                'startDate' => 'required|date',
                'endDate' => 'nullable|date',
                'time' => 'nullable|date_format:H:i',
                'document' => 'nullable|image|mimes:jpeg,png,jpg',
            ]);
            if ($validator->fails()) {
                throw new ValidationException($validator);
            }


            // Create a new complaint record
            $notice = new TanantNotice();
            $notice->studentId = $studentId;
            $notice->noticeDate = $request->noticeDate;
            $notice->startDate = $request->startDate;
            $notice->endDate = $request->endDate;
            $notice->time = $request->time;
            $notice->noticeType = $request->noticeType;
            $notice->NoticeMsg = $request->NoticeMsg;

            // Handle the document upload if it exists
            if ($request->hasFile('document') && $request->file('document')->isValid()) {
                $file = $request->file('document');
                $base64Data = base64_encode(file_get_contents($file->path()));
                $resizedBase64Data = $this->resizeBase64Image($base64Data, 50);
                if ($resizedBase64Data !== null) {
                    $filename = $this->generateFileName($file);
                    File::put(public_path('assets/uploads/complaintDocuments/' . $filename), base64_decode($resizedBase64Data));
                    $notice->document = $filename;
                }
            }
            $notice->save();
            return response()->json([
                'status' => true,
                'statusCode' => 201,
                'message' => 'notice$notice created successfully.',
                'data' => $notice
            ], 201);
        } catch (ValidationException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 422,
                'message' => 'Validation Error',
                'errors' => $e->errors()
            ], 422);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    public function getAllNotices($studentId)
    {
        try {
            $studentNotice =  TanantNotice::where('studentId', $studentId)->with(['student' => function ($query) {
                $query->select('id', 'roomno', 'seetno');
            }])->get();
            return response()->json([
                "status" => true,
                'statusCode' => 200,
                'message' => 'Students Notice List',
                'data' => $studentNotice
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'server Error',
            ], 500);
        }
    }

    public function deleteNotice($noticeId)
    {
        try {
            $notice = TanantNotice::findOrFail($noticeId);
            $notice->delete();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => "Notice Deleted Successfully",
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => $e->getMessage()
            ], 500);
        }
    }
}
