<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Models\Permissions;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use App\Models\PermissionSetting;
use App\Models\Item;
use Illuminate\Support\Facades\Validator;

class AutoPermissionController extends Controller
{
    public function view()
    {
        // get sales Items;
        $items = Item::all();
        $permissions = PermissionSetting::pluck('permission_value', 'permission_name')->toArray();
        return view('fronted.auto_permissions', compact('items', 'permissions'));
    }

    public function store(Request $request)
    {
        // Validate the request data
        $validator = Validator::make(
            $request->all(),
            [
                'permissions' => 'required|array|min:1',
                'permissions.*' => 'nullable'
            ]
        );
        if ($validator->fails()) {
            return redirect()->back()->with('error', $validator->errors()->first());
        }

        $validatedData = $validator->validated();

        try {
            // Prepare the data to be saved
            $permissions = $validatedData['permissions'];

            foreach ($permissions as $key => $value) {
                $status = false;

                if (is_array($value)) {
                    $status = true;
                    $value = implode(',', $value);
                } elseif (is_numeric($value)) {
                    $status = true;
                    $value = (int)$value;
                } else {
                    $status = $value === 'on' ? 1 : 0;
                    $value = $status;
                }

                // Ensure $key and $value are properly sanitized
                $key = htmlspecialchars($key, ENT_QUOTES, 'UTF-8');
                $value = htmlspecialchars($value, ENT_QUOTES, 'UTF-8');

                $data = [
                    'permission_name' => $key,
                    'permission_value' => $value,
                    'permission_status' => $status,
                ];

                // Store the permission and its value
                PermissionSetting::updateOrCreate(['permission_name' => $key], $data);
            }

            return redirect()->back()->with('success', 'Permission setting created successfully.');
        } catch (Exception $e) {
            // Handle the exception and redirect back with an error message
            return redirect()->back()->with('error', $e->getMessage());
        }
    }
}
