<?php

namespace App\Http\Controllers;

use App\Http\Middleware\CheckSubscriptionPermission;
use App\Models\Businessregister;
use App\Models\EmployeRegister;
use Exception;
use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\File;
use Illuminate\Support\Facades\Config;
use Intervention\Image\Facades\Image;
use Carbon\Carbon;
use Illuminate\Support\Facades\Hash;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Support\Facades\Storage;
use App\Models\Salesubscription;
use App\Models\BusinessSubscriptionPermission;

class BusinessController extends Controller
{
    protected $permissionsArray;
    public function businessDetails()
    {
        $licenceNo = session("liseneceNo");
        $user_db_user_id = Businessregister::value('id');
        $profile = Businessregister::on('mysql')->with(['salesSubscription' => function ($query) {
            $query->where('status', 1)
                ->with('subscriptionDetail');
        }])->where('license_no',$licenceNo)->first();

        $license_no = $profile->license_no;
        $allSubscriptionDetails = Salesubscription::with('subscriptionDetail')->where('user_id', $license_no)->orderBy('id', 'desc')->get();
        $businessPermissions = BusinessSubscriptionPermission::where('business_id', $license_no)->first();
        $mainPermissions = !empty($businessPermissions) ? json_decode($businessPermissions->main_permissions, true) : [];
        $addOnPermissions = !empty($businessPermissions) ? json_decode($businessPermissions->addOn_permissions, true) : [];

        // employee Profile
        //first check which user login admin or employee
        $userRoll = session('userRoll');
        $empId = session("empId"); //empId which is user login
        $empProfile = null;

        if( strtolower($userRoll) != 'admin'){
            // now get the employee details
            $empProfile = EmployeRegister::where('id', $empId)->first();
        }
        return view('fronted.profile', compact('profile', 'allSubscriptionDetails', 'mainPermissions', 'addOnPermissions','user_db_user_id','empProfile'));
    }


    public function DeleteUserEmp(Request $request)
    {
        $userId = $request['empId'];
        config(['database.default' => env('DB_CONNECTION')]);
        $emp =  EmployeRegister::where("id", $userId)->first();
        if ($emp->delete()) {
            return response()->json(["Success" => "Delted"]);
        } else {
            return response()->json(["error" => "not Delted"], 500);
        }
    }
    public function EmployeRegisterUpdate(Request $request)
    {

        config(['database.default' => env('DB_CONNECTION')]);
        $userName = session("liseneceNo");
        $this->permissionsArray = Session::get('permissions', []);

        $checkBusinessRoll = EmployeRegister::where('license_no', $userName)
            ->where('user_roll', '!=', '')
            ->count();
        // check permission
        if (!$this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 1 member)') || !$this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 3)')) {
            return back()->with("UserRegisterFailed", "You have not permission to add user roll!!");
        } elseif ($this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 1 member)') && $checkBusinessRoll > 1) {
            return back()->with("UserRegisterFailed", "You have not permission to add user roll more then one");
        } elseif ($this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 3)') && $checkBusinessRoll >= 3) {
            return back()->with("UserRegisterFailed", "You have not permission to add user roll more then 3.");
        }

        $permissionDatas = $request['permissions'];
        $businessNameCheck = Businessregister::where('license_no', $userName)->first();
        $username1 = $businessNameCheck->username;
        $userName2 = $businessNameCheck->username1;
        $datasave = EmployeRegister::where("id", $request->EditeUserID)->first();
        $datasave->candidate_name = $request->userName;
        $datasave->email = $request->userEmail;
        $datasave->mobile = $request->userMobile;
        $datasave->user_roll = $request->userRole;
        $datasave->permissions = json_encode($permissionDatas);
        $datasave->username = $username1;
        $datasave->username1 = $userName2;
        $datasave->password = Hash::make($request->Password);
        $datasave->license_no = $businessNameCheck->license_no;
        if ($datasave->update()) {
            return back()->with("UserRegister", "User Registertion updated successfully.");
        } else {
            return back()->with("UserRegisterFailed", "User is not updated !!");
        }
    }
    public function getEmployeRegisterData(Request $request)
    {
        config(['database.default' => env('DB_CONNECTION')]);
        $employeData =  EmployeRegister::where("id", $request->userId)->first();
        if ($employeData) {
            $data = [
                "name" => $employeData->name,
                "email" => $employeData->email,
                'mobile' => $employeData->mobile,
                "userRoll" => $employeData->user_roll,
                "permissions" => explode(",", $employeData->permissions),
                "password" => $employeData->password
            ];
            return response()->json($data, 200);
        } else {
            return response()->json("error", "User ID not valid");
        }
    }
    public function EmployeRegister(Request $request)
    {
        $permissionDatas = $request['permissions'];
        $this->permissionsArray = Session::get('permissions', []);
        $userName = session("liseneceNo");
        config(['database.default' => env('DB_CONNECTION')]);
        $checkBusinessRoll = EmployeRegister::where('license_no', $userName)
            ->where('user_roll', '!=', '')
            ->count();
        // check permission
        if (!$this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 1 member)') || !$this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 3)')) {
            return back()->with("UserRegisterFailed", "You have not permission to add user roll!!");
        } elseif ($this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 1 member)') && $checkBusinessRoll > 1) {
            return back()->with("UserRegisterFailed", "You have not permission to add user roll more then one");
        } elseif ($this->hasPermission($this->permissionsArray, 'admin permission.add user role (upto 3)') && $checkBusinessRoll >= 3) {
            return back()->with("UserRegisterFailed", "You have not permission to add user roll more then 3.");
        }

        $businessNameCheck = Businessregister::where('license_no', $userName)->first();
        $username1 = $businessNameCheck->username;
        $userName2 = $businessNameCheck->username1;
        $datasave = new EmployeRegister;
        $datasave->registration_date = Carbon::now();
        $datasave->form_no = $request->formno;
        $datasave->candidate_name = $request->userName;
        $datasave->email = $request->userEmail;
        $datasave->mobile = $request->userMobile;
        $datasave->user_roll = $request->userRole;
        $datasave->permissions = json_encode($permissionDatas);
        $datasave->username = $username1;
        $datasave->username1 = $userName2;
        $datasave->password = Hash::make($request->Password);
        $datasave->license_no = $businessNameCheck->license_no;
        if ($datasave->save()) {
            return back()->with("UserRegister", "User Registertion successfully completed.");
        } else {
            return back()->with("UserRegisterFailed", "User not Register!!");
        }
    }
    public function getAllBussinessNames(Request $request)
    {
        $userName = session("user");
        $hostelName = [];
        config(['database.default' => env('DB_CONNECTION')]);
        $userType = session('userRoll');
        if (strtolower($userType) !== "admin") {
            $empId = session('empId');
            $emp = EmployeRegister::find($empId);
            $empMobileNumber = $emp->mobile;
            $getHostelList = EmployeRegister::where('mobile', $empMobileNumber)->with('businessRegister')->get();
            if (!empty($getHostelList)) {
                foreach ($getHostelList as $hostels) {
                    if (!in_array($hostels->businessRegister->businessname, $hostelName)) {
                        $hostelName[] = $hostels->businessRegister->businessname;
                    }
                }
            }
        } else {
            $businessNameCheck = Businessregister::where('username', $userName)->get();
            foreach ($businessNameCheck as $row) {
                $hostelName[] = $row->businessname;
            }
        }
        config(['database.default' => "custom"]);
        return response()->json($hostelName, 200);
    }
    public function store(Request $request)
    {
        try {
            config(['database.default' => env('DB_CONNECTION')]);
            $emailExists = Businessregister::where('email',  $request->email);
            $mobileNoExists = Businessregister::where('mobile', $request->mobile);
            $businessNameCheck = Businessregister::where('businessname', $request->businessname);
            if ($emailExists->exists()) {
                return back()->with('emailExist', 'Given Email already Exist,please login to continue');
            } elseif ($mobileNoExists->exists()) {
                return back()->with('MobileExist', 'Given Email already Exist,please login to continue');
            } elseif ($businessNameCheck->exists()) {
                return back()->with('BussinessNameCheck', 'Given Business Name already Exist,Please Change !');
            } else {
                $formno = Businessregister::orderBy('id', 'desc')->value('form_no');
                $formno = $formno + 1;
                $regDate = Carbon::now()->format('Y-m-d');
                $userNames = explode("or", $request->username);
                $candidateCropImg = $request->UserCropImg;
                $data = new Businessregister();
                $data->form_no = $formno;
                $data->regdate  = $regDate;
                $data->business_type = $request->business_type;
                $data->fname = $request->fname;
                $data->lname = $request->lname;
                $data->email = $request->email;
                $data->mobile = $request->mobile;
                $data->mobile_secondary = $request->mobile_secondary;
                $data->gst_no = $request->gst_no;
                $data->businessname = $request->businessname;
                if ($request->hasfile('pic')) {
                    $file = $request->file('pic');
                    $extention = $file->getClientOriginalExtension();
                    $filename = time() . '.' . rand() . '.' . $extention;
                    $imageData = explode(',', $candidateCropImg);
                    $imageData = base64_decode($imageData[1]);
                    File::put(public_path('assets/uploads/' . $filename), $imageData);
                    $finalPath = 'uploads/' . $filename;
                    Storage::disk('external_images')->put($finalPath, file_get_contents('assets/uploads/' . $filename));
                    $data->pic = $filename;
                }
                $data->city = $request->city;
                $data->state = $request->state;
                $data->country = $request->country;
                $data->address = $request->address;
                $data->pincode = $request->pincode;
                $data->username = str_replace(" ", "", $userNames[1]);
                $data->username1 = str_replace(" ", "", $userNames[0]);
                $data->password = Hash::make($request->password);
                $data->conf_password = Hash::make($request->conf_password);
                $data->save();
                $lastInsertedId = $data->id;
                $cityName = $request->city;
                $firstAlphabet = substr($cityName, 0, 1);
                $lastAlphabet = substr($cityName, -1);
                $output = strtoupper($firstAlphabet . $lastAlphabet);
                $outputInsert = str_pad($lastInsertedId, 4, '0', STR_PAD_LEFT);
                $license_no = $output . $request->pincode . $outputInsert;
                Businessregister::where('id', $lastInsertedId)
                    ->update(['license_no' => $license_no]);
                $databaseName = $license_no;
                return back()->with('RegistrationComplete', $license_no);
            }
        } catch (Exception $e) {
            return back()->with('RegisetrationError', 'Registation Failed, Try Again!!');
        }
    }



    public function update(Request $request, $id)
    {
        $data = Businessregister::find($id);
        if (!$data) {
            return back()->with('error', 'Record not found');
        }

        DB::beginTransaction();

        try {
            $data->fname = $request->fname;
            $data->lname = $request->lname;
            $data->email = $request->email;
            $data->mobile = $request->mobile;
            $data->mobile_secondary = $request->mobile_secondary;
            $data->gst_no = $request->gst_no;
            $data->businessname = $request->businessname;
            $data->city = $request->city;
            $data->state = $request->state;
            $data->country = $request->country;
            $data->address = $request->address;
            $data->pincode = $request->pincode;

            if (!$data->save()) {
                throw new Exception("Failed to update.");
            }

            $result = $this->updateBusinessDetailsInMainDatabase($request, $data->license_no);
            if ($result === false) {
                throw new Exception('Failed to update business details in the main database.');
            }
            Session::put('hostel', $data->businessname);

            DB::commit();
            return back()->with('success', 'Updated Successfully');
        } catch (Exception $e) {
            DB::rollBack();
            return back()->with('error', $e->getMessage());
        }
    }



    private function updateBusinessDetailsInMainDatabase($request, $license_no)
    {
        try {
            // Set the default database connection
            config(['database.default' => env('DB_CONNECTION')]);

            // Retrieve the business record by license number
            $business = Businessregister::where('license_no', $license_no)->firstOrFail();

            // Update the business details
            $business->fname = $request->fname;
            $business->lname = $request->lname;
            $business->email = $request->email;
            $business->mobile = $request->mobile;
            $business->mobile_secondary = $request->mobile_secondary;
            $business->gst_no = $request->gst_no;
            $business->businessname = $request->businessname;
            $business->city = $request->city;
            $business->state = $request->state;
            $business->country = $request->country;
            $business->address = $request->address;
            $business->pincode = $request->pincode;
            $business->save(); // Use save() instead of update() when updating individual attributes
            config(['database.default' => 'custom_db']);

            return true; // Return true to indicate success
        } catch (ModelNotFoundException $e) {
            // Handle the case where the business record doesn't exist
            config(['database.default' => 'custom_db']);

            return false;
        } catch (Exception $e) {
            config(['database.default' => 'custom_db']);

            // Handle other possible exceptions
            return false;
        }
    }


    private function updateImgInMainBusinessTable($imgName, $license_no)
    {
        config(['database.default' => env('DB_CONNECTION')]);
        $data = Businessregister::where('license_no', $license_no)->first();
        if ($data) {
            $data->pic = $imgName;
            $data->update();
        }
    }

    public function profilePhoto(Request $request)
    {
        $userId = session('user');
        $data = Businessregister::where('username', $userId)
            ->latest()->first();
        if (!$data) {
            return response()->json(['message' => 'Record not found'], 404);
        }
        if ($request->hasFile('pic')) {
            $file = $request->file('pic');
            $extension = $file->getClientOriginalExtension();
            $filename = time() . '.' . rand() . '.' . $extension;

            if ($file->move(sys_get_temp_dir(), $filename)) {
                $filePath = sys_get_temp_dir() . '/' . $filename;
                if (file_exists($filePath)) {
                    $fileSize = filesize($filePath);
                    if ($fileSize > 1600 * 1024) {
                        $image = imagecreatefromstring(file_get_contents($filePath));
                        $compressedImage = imagecreatetruecolor(imagesx($image), imagesy($image));
                        imagecopyresampled($compressedImage, $image, 0, 0, 0, 0, imagesx($image), imagesy($image), imagesx($image), imagesy($image));
                        imagejpeg($compressedImage, $filePath, 75); // Compress with 75% quality
                        imagedestroy($image);
                        imagedestroy($compressedImage);
                    }
                    // Move the file to the final destination
                    if (rename($filePath, 'assets/uploads/' . $filename)) {
                        $data->pic = $filename;
                        $finalPath = 'uploads/' . $filename;
                        // Save the file directly to the external_images disk
                        Storage::disk('external_images')->put($finalPath, file_get_contents('assets/uploads/' . $filename));
                    } else {
                        return back()->with('error', 'The file could not be moved to the destination.');
                    }
                }
                if ($data->save()) {
                    $this->updateImgInMainBusinessTable($filename, $data->license_no);
                    return response()->json(['message' => 'Image saved successfully']);
                } else {
                    return response()->json(['error' => 'Error saving the image'], 500);
                }
            } else {
                return response()->json(['error' => 'No photo provided'], 400);
            }
        }
    }


    private function hasPermission($permissionsArray, $permission)
    {
        $keys = explode('.', $permission);
        $current = $permissionsArray;
        // dd($keys,$current); 
        foreach ($keys as $key) {
            if (!isset($current[$key])) {
                return false;
            }
            $current = $current[$key];
        }

        return true;
    }

    public function addhostelpg(Request $request)
    {
        $userId = Session::get('user');
        $this->permissionsArray = Session::get('permissions', []);

        config(['database.default' => env('DB_CONNECTION')]);
        $bussinesProfieData = DB::table('businessregisters')->where('username', $userId)
            ->get();
        // check permissions 
        if ($this->hasPermission($this->permissionsArray, "admin permission.add business (Upto 1)")) {
            if (count($bussinesProfieData) >= 1) {
                return back()->with('Error', 'You have not Permission to add business more then 1.');
            }
        } elseif (!$this->hasPermission($this->permissionsArray, "admin permission.add business (unlimited)")) {
            return back()->with('Error', 'You have not Permission to add business');
        } else {
            return back()->with('Error', 'You have not permission to add business');
        }

        if ($bussinesProfieData) {
            foreach ($bussinesProfieData as $row) {
                $firstname = $row->fname;
                $lastName = $row->lname;
                $email = $row->email;
                $mobile = $row->mobile;
                $mobile_secondary = $row->mobile_secondary;
                $pic = $row->pic;
            }
        }
        $data = new Businessregister();
        $userNames = explode("or", $request->username);
        $data->fname  = $firstname;
        $data->lname  = $lastName;
        $data->email  = $email;
        $data->mobile  = $mobile;
        $data->mobile_secondary  = $mobile_secondary;
        $data->pic = $pic;
        $data->business_type = $request->business_type;
        $data->gst_no = $request->gst_no;
        $data->businessname = $request->businessname;
        $data->city = $request->city;
        $data->state = $request->state;
        $data->country = $request->country;
        $data->address = $request->address;
        $data->pincode = $request->pincode;
        $data->username = str_replace(" ", "", $userNames[0]);
        $data->username1 = str_replace(" ", "", $userNames[1]);
        $data->password = Hash::make($request->password);
        $data->conf_password =  Hash::make($request->conf_password);
        if ($data->save()) {
            $lastInsertedId = $data->id;
            $cityName = $request->city;
            $firstAlphabet = substr($cityName, 0, 1);
            $lastAlphabet = substr($cityName, -1);
            $output = strtoupper($firstAlphabet . $lastAlphabet);
            $outputInsert = str_pad($lastInsertedId, 4, '0', STR_PAD_LEFT);
            $license_no = $output . $request->pincode . $outputInsert;
            Businessregister::where('id', $lastInsertedId)
                ->update(['license_no' => $license_no]);
            config(['database.default' => "custom_db"]);
            return redirect()->back()->with('HostelID', $lastInsertedId);
        } else {
            return redirect()->back()->with('Error', 'Details are not valid, try again...');
        }
    }
}
