<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use DB;
use Illuminate\Support\Carbon;
use Exception;
use App\Models\RoomSeat;
use App\Models\ReserveRoom;
use App\Models\ElectricityManagement;
use App\Models\Studentregister;
use App\Models\Olddataregister;
use App\Models\StudentElecticityRecords;
use App\Models\ElecticityCalculater;
use App\Models\StudentsElecticityBill;
use Illuminate\Database\QueryException;
use Illuminate\Support\Facades\Log;
use App\Models\ElectricityMeterChange;
use Illuminate\Support\Facades\Validator;
use App\Models\PermissionSetting;
use App\Models\Sale;

class ElecticityContoller extends Controller
{

   public function index(Request $request)
   {
      $selectedMonth = $request->input('selected_month') ?? $this->getInsertedBillDate(); // this is use for selected records according to
      $startDate = $request->input('startDate');
      $endDate = $request->input('endDate');

      list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($selectedMonth);
      $roomSeats = RoomSeat::with('reserveRoom')->get();
      if ($selectedMonth == 'custom') {
         $electricityRecords = $this->getElectricityRecordsByDateRange($request->input('startDate'), $request->input('endDate'));
      } else {
         $electricityRecords = $this->getElectricityRecordsByMonth($selectedMonth);
      }
      $allBillDates = ElectricityManagement::select('BillDate')->orderBy('BillDate', 'asc')->groupBy('BillDate')->get();
      // Organize data for the nested list
      $nestedListData = [];
      foreach ($allBillDates as $billDate) {
         $year = Carbon::parse($billDate->BillDate)->format('Y');
         $nestedListData[$year][] = Carbon::parse($billDate->BillDate)->format('j-M-Y');
      }
      $roomNumbers = RoomSeat::pluck('roomno');

      // check automatice electricity bill genrate permission is active or not
      $isAutoBillGenratePermissionActive = $this->isAutoBillGenratePermissionActive('autoElectricityBilling');

      return view("fronted.studentElecticityBill", compact('nestedListData', 'selectedMonth', 'PrenseBillDate', 'PreviousBillDate', 'roomSeats', 'allBillDates', 'electricityRecords', 'startDate', 'endDate', 'roomNumbers', 'isAutoBillGenratePermissionActive'));
   }

   private function isAutoBillGenratePermissionActive($permission_name)
   {
      $permissionSetting = PermissionSetting::where('permission_name', $permission_name)->where('permission_status', 1)->first();
      if ($permissionSetting) {
         return true;
      } else {
         return false;
      }
   }

   private function getInsertedBillDate()
   {
      $lastRecords =  ElectricityManagement::select('BillDate')
         ->orderBy('id', 'desc')
         ->first();
      return $lastRecords ? $lastRecords->BillDate : null;
   }
   protected function getFormattedBillDates($selectedMonth)
   {
      if (!$selectedMonth) {
         return [null, null];
      }

      $previousReadingDate = ElectricityManagement::select('BillDate')
         ->where('BillDate', '<', $selectedMonth)
         ->orderBy('BillDate', 'DESC')
         ->groupBy('BillDate')
         ->first();

      $presentReadingDate = DB::table('electicity_management')
         ->where('BillDate', '=', $selectedMonth)
         ->first();

      $PrenseBillDate = $presentReadingDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : '';
      $PreviousBillDate = $previousReadingDate ? Carbon::parse($previousReadingDate->BillDate)->format('d-M-Y') : ($PrenseBillDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : "");

      return [$PrenseBillDate, $PreviousBillDate];
   }

   protected function getElectricityRecordsByDateRange($startDate, $endDate)
   {
      return $lastRecords = ElectricityManagement::with('studentElectricityRecords.student')
         ->whereBetween('BillDate', [$startDate, $endDate])
         ->get();
   }

   protected function getElectricityRecordsByMonth($selectedMonth)
   {
      // Enable query log for debugging purposes
      DB::enableQueryLog();

      // Fetch electricity records with associated student electricity records and students
      $electricityRecords = ElectricityManagement::with(['studentElectricityRecords' => function ($query) {
         $query->with('student');
      }])
         ->where('BillDate', $selectedMonth)
         ->get();

      // Iterate over each electricity record
      $electricityRecords->each(function ($data) {
         // Fetch the most recent meter change record up to the bill date
         $meterChangeRecords = ElectricityMeterChange::where('roomno', $data->roomno)
            ->where('date', '<=', $data->BillDate)
            ->orderBy('id', 'desc')
            ->first();

         // Add the meter change record as a new attribute to the data object
         $data->setAttribute('meterChangeRecords', $meterChangeRecords);
      });

      return $electricityRecords;

      // Dump the query log for debugging
      dd(DB::getQueryLog());
   }


   private function PrepareElectrityRecords($presentReadingDate, $previousReadingDate, $roomno, $month, $units, $readingRate)
   {
      try {
         $previousReading = Carbon::parse($previousReadingDate)->toDateString();
         $presentReading = Carbon::parse($presentReadingDate)->toDateString();
         // Check student room number from old data register table
         $students = Studentregister::where('roomno', $roomno)
            ->where(function ($query) use ($previousReading, $presentReading) {
               $query->where(function ($subQuery) use ($presentReading, $previousReading) {
                  // Consider date only when join_date is null
                  $subQuery->where(function ($innerSubQuery) use ($presentReading, $previousReading) {
                     $innerSubQuery->Where(function ($checkBetween) use ($presentReading, $previousReading) {
                        $checkBetween->whereBetween('date', [$previousReading, $presentReading])
                           ->whereNull('join_date');
                     })
                        ->orWhere(function ($checkdate) use ($presentReading, $previousReading) {
                           $checkdate->where('date', "<", $previousReading)
                              ->whereNull('join_date');
                        });
                  })
                     ->orWhere(function ($innerSubQuery) use ($presentReading, $previousReading) {
                        // Check leave_date when join_date is not null
                        $innerSubQuery->where(function ($checkBetween) use ($presentReading, $previousReading) {
                           $checkBetween->whereBetween('join_date', [$previousReading, $presentReading]);
                        })
                           ->orWhere(function ($checkdate) use ($presentReading, $previousReading) {
                              $checkdate->where('join_date', "<", $previousReading);
                           });
                     });
               })
                  ->where('status', 1)
                  ->whereNull('leave_date');
            })
            ->groupBy('formno')
            ->get();

         foreach ($students as $row) {
            $rowDate = $row->join_date ? Carbon::parse($row->join_date)->format('Y-m-d') : Carbon::parse($row->date)->format('Y-m-d');
            $leaveDate = $row->leave_date ? Carbon::parse($row->leave_date)->format('Y-m-d') : null;
            $newRecords = new StudentsElecticityBill();
            $newRecords->roomno =  $roomno;
            $newRecords->candidate_id = $row->id;
            $newRecords->seat_no = $row->seetno;
            $newRecords->joining_date = $row->join_date ? $row->join_date : $row->date;
            $newRecords->leaving_date = $row->leave_date;
            $newRecords->total_reading = $units;

            if ($rowDate >= $previousReading) {
               $newRecords->reading_start_date = $rowDate;
            } else {
               $newRecords->reading_start_date = $previousReading;
            }

            if ($leaveDate == null) {
               $newRecords->reading_end_date = $presentReading;
            } elseif ($leaveDate <= $presentReading) {
               $newRecords->reading_end_date = $leaveDate;
            } else {
               $newRecords->reading_end_date = $presentReading;
            }
            $newRecords->save();
         }
         $oldStudents = Olddataregister::where('roomno', $roomno)
            ->where(function ($query) use ($presentReading, $previousReading) {
               $query->where('date', "<=", $presentReading)
                  ->where(function ($subQuery) use ($presentReading, $previousReading) {
                     $subQuery->whereBetween('leave_date', [$previousReading, $presentReading])
                        ->orWhere('leave_date', ">=", $presentReading);
                  });
            })
            ->groupBy('formno')
            ->get();
         foreach ($oldStudents as $row) {
            $getActualIdOfCandidate = Studentregister::where("formno", $row->formno)->first();
            $newRecords = new StudentsElecticityBill();
            $newRecords->roomno = $roomno;
            $newRecords->candidate_id = $getActualIdOfCandidate->id;
            $newRecords->seat_no = $row->seetno;
            $newRecords->joining_date = $row->date;
            $newRecords->leaving_date = $row->leave_date;
            $newRecords->total_reading = $units;
            $joindate = Carbon::parse($row->date);
            $leaveDate = $row->leave_date ? Carbon::parse($row->leave_date) : null;
            if ($joindate >= $previousReading) {
               $newRecords->reading_start_date = $joindate;
            } else {
               $newRecords->reading_start_date = $previousReading;
            }
            if ($leaveDate == null) {
               $newRecords->reading_end_date = $presentReading;
            } elseif ($leaveDate <= $presentReading) {
               $newRecords->reading_end_date = $leaveDate;
            } else {
               $newRecords->reading_end_date = $presentReading;
            }
            $newRecords->save();
         }
         $queryTotalDays = StudentsElecticityBill::selectRaw("SUM(
        DATEDIFF(
            LEAST(reading_end_date, '{$presentReading}'),
            GREATEST(reading_start_date, '{$previousReading}')
        )
    ) AS total_days")
            ->where("roomno", $roomno)
            ->whereRaw("reading_end_date <= '{$presentReading}'")
            ->whereRaw("reading_start_date >= '{$previousReading}'")
            ->first();

         $totalDays = $queryTotalDays->total_days;

         $finalQuery = StudentsElecticityBill::select(
            'id',
            \DB::raw("DATEDIFF(
             LEAST(reading_end_date, '{$presentReading}'),
             GREATEST(reading_start_date, '{$previousReading}')
         ) AS days")
         )
            ->where("roomno", $roomno)
            ->whereRaw("reading_end_date <= '{$presentReading}'")
            ->whereRaw("reading_start_date >= '{$previousReading}'")
            ->get();

         foreach ($finalQuery as $result) {
            $studentId = $result["id"];
            $days = $result['days'];
            if ($totalDays != 0 && $days != 0) {
               $proportion = $days / $totalDays;
            } else {
               $proportion = 0;
            }

            $studentUnit = round(($units * $proportion), 2);
            $studentAmount = round(($readingRate * $studentUnit), 2);
            $updateRecords = StudentsElecticityBill::where('roomno', $roomno)->where("id", $studentId)->first();
            if (!empty($updateRecords)) {
               $updateRecords->total_reading_candidate = $studentUnit;
               $updateRecords->reading_amount = $studentAmount;
            }
            $updateRecords->update();
         }
         return true;
      } catch (QueryException $e) {
         Log::error('QueryException: ' . $e->getMessage());
         dd($e->getMessage());
         return $e->getMessage();
      } catch (Exception $e) {
         dd($e->getMessage());
         Log::error('Exception: ' . $e->getMessage());
         return $e->getMessage();
      }
   }
   private function makeElecticityRecords($presentReading, $previousReading, $billDate)
   {
      try {
         $electricityRecords = ElectricityManagement::where("billdate", $billDate)->get();
         foreach ($electricityRecords as $row) {
            $roomno = $row->roomno;
            $prepareElecticityRecords = $this->PrepareElectrityRecords($presentReading, $previousReading, $roomno, $billDate, $row->discountUnits, $row->readingRate);
            if ($prepareElecticityRecords === true) {
               $storedRecords = StudentsElecticityBill::all();
               foreach ($storedRecords as $records) {
                  $newRecords = new StudentElecticityRecords();
                  $newRecords->electricity_management_id = $row->id;
                  $newRecords->roomNo = $records->roomno;
                  $newRecords->seatNo = $records->seat_no;
                  $newRecords->userId = $records->candidate_id;
                  $newRecords->unit = $row->units;
                  $newRecords->prUnitCharge = $records->total_reading_candidate;
                  $newRecords->billDate = $billDate;
                  $newRecords->Amount = $records->reading_amount;
                  $newRecords->save();
               }
               StudentsElecticityBill::truncate();
            } else {
               throw  new Exception($prepareElecticityRecords);
            }
         }
         return true;
      } catch (QueryException $e) {
         Log::error('QueryException: ' . $e->getMessage());
         return $e->getMessage();
      } catch (Exception $e) {
         Log::error('Exception: ' . $e->getMessage());
         return $e->getMessage();
      }
   }
   public function store(Request $request)
   {
      $validatedData = $request->validate([
         'billdate' => 'required|date',
         'readingRate' => 'required|numeric',
         'roomSeatsData' => 'required|array',
         'discountUnits' => "required|numeric"
      ]);
      $checkElectricityRecords = ElectricityManagement::count();
      foreach ($validatedData['roomSeatsData'] as $roomNo => $data) {
         if ($checkElectricityRecords == 0) {
            ElectricityManagement::create([
               'BillDate' => $request->billdate,
               'discountUnits' => ($request->discountUnits),
               'readingRate' => $validatedData['readingRate'],
               'roomno' => $roomNo,
               'present_reading' => $data['present_red'],
               'previous_reading' => $data['previous_red']
            ]);
         } else {
            ElectricityManagement::create([
               'BillDate' => $request->billdate,
               'discountUnits' => $request->discountUnits,
               'readingRate' => $validatedData['readingRate'],
               'roomno' => $roomNo,
               'present_reading' => $data['present_red'],
            ]);
         }
      }
      list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($request->billdate);
      $makeElectricityRecords = $this->makeElecticityRecords($PrenseBillDate, $PreviousBillDate, $request->billdate);
      if ($makeElectricityRecords) {
         return redirect()->back()->with(['Electricity' => [
            'message' => 'electricity bill created Successfully',
            'billDate' => $request->billdate,
         ]]);
      } else {
         return back()->withErrors(['Error' => 'An error occurred while storing data']);
      }
   }

   public function delete(Request $request)
   {
      try {
         $month = $request->month;
         // Start a database transaction
         DB::beginTransaction();

         // Find the record
         $record = ElectricityManagement::where('BillDate', $month)->delete();
         $deleteElectricityRecords = StudentElecticityRecords::where("billDate", $month)->delete();
         DB::commit();
         return response()->json(['status' => 'success'], 200);
      } catch (\Exception $e) {
         // An error occurred, so rollback the transaction
         DB::rollBack();

         // Log the error
         \Log::error('Error deleting records: ' . $e->getMessage());

         // Return a JSON response with an error message and status code
         return response()->json(['error' => 'Error deleting records', 'details' => $e->getMessage()], 500);
      }
   }


   public function getinfo($id)
   {
      $record = ElectricityManagement::find($id); // Replace YourModel with the actual model name
      return response()->json($record);
   }

   private function updateElectricityRecords($record)
   {
      try {
         StudentsElecticityBill::chunk(200, function ($storedRecords) use ($record) {
            foreach ($storedRecords as $created) {
               $newRecords = new StudentElecticityRecords();
               $newRecords->electricity_management_id = $record->id;
               $newRecords->roomNo = $created->roomno;
               $newRecords->seatNo = $created->seat_no;
               $newRecords->userId = $created->candidate_id;
               $newRecords->unit = $record->units;
               $newRecords->prUnitCharge = $created->total_reading_candidate;
               $newRecords->billDate = $record->BillDate;
               $newRecords->Amount = $created->reading_amount;
               $newRecords->save();
            }
         });
         StudentsElecticityBill::truncate();
         return true;
      } catch (Exception $e) {
         // Log or handle the exception as needed
         return false;
      }
   }

   private function checkElectricityRecordForNextMonth($record)
   {
      $data = ElectricityManagement::where('BillDate', ">", $record->BillDate)->where('roomno', $record->roomno)->first();
      if (!empty($data)) {
         $data->previous_reading = $record->present_reading;
         $data->save();
         return [
            'status' => true,
            'data' => $data,
         ];
      } else {
         return [
            'status' => false,
            'data' => 'not found',
         ];
      }
   }

   public function update(Request $request, $id)
   {
      try {
         $record = ElectricityManagement::find($id);

         if (!$record) {
            throw new Exception("No Records found");
         }
         // Update the ElectricityManagement record
         $record->update([
            'BillDate' => $request->billdate,
            'discountUnits' => ($request->discountUnits),
            'readingRate' => $request->readingRate,
            'previous_reading' => $request->previousRed,
            'present_reading' => $request->PresentRed,
         ]);
         $this->updateAllElectricityRecords($record);
         // check have any fill which is greater then this bill only first
         $nextMontRecords = $this->checkElectricityRecordForNextMonth($record);
         if ($nextMontRecords['status'] == true) {
            $this->updateAllElectricityRecords($nextMontRecords['data']);
         }
         return back()->with('Electicity', 'Record updated and related records deleted');
      } catch (Exception $e) {
         return response()->json(["error" => $e->getMessage(), "stackTrace" => $e->getTraceAsString()]);
      }
   }


   private function updateAllElectricityRecords($record)
   {
      StudentElecticityRecords::where('billDate', $record->BillDate)
         ->Where('electricity_management_id', $record->id)
         ->delete();
      list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($record->BillDate);
      $prepareElecticityRecords = $this->PrepareElectrityRecords($PrenseBillDate, $PreviousBillDate, $record->roomno, $record->BillDate, $record->discountUnits, $record->readingRate);
      if (!$prepareElecticityRecords) {
         throw new Exception("PrepareElectrityRecords failed");
      }
      $updatedRecords = $this->updateElectricityRecords($record);
      if (!$updatedRecords) {
         throw new Exception("updateElectricityRecords failed");
      }
      return;
   }


   public function storeRecord(Request $request)
   {
      try {
         ElectricityManagement::create([
            'BillDate' => $request->billdate,
            'readingRate' => $request->readingRate,
            'roomno' => $request->roomno,
            'previous_reading' => $request->previousRed,
            'present_reading' => $request->PresentRed
         ]);
         return back()->with('Electicity', "Electicity Bill Store Successfully");
      } catch (Exception $e) {
         return back()->with('error', "Somthing has been Wrong");
      }
   }

   public function getBillRecords(Request $request)
   {
      $roomno = $request['roomNo'];
      $candidateId = $request['candidateId'];
      $month = $request['ElectricityMonth'];
      try {
         $ElectricityBill = StudentElecticityRecords::select("Amount")
            ->where("userId", $candidateId)
            ->where("billDate", $month)
            ->where("status", 0)
            ->first();
         return response()->json(
            [
               'amount' => $ElectricityBill->Amount,
               'tax' => 0,
               'rate' => $ElectricityBill->Amount,
            ],200);
      } catch (Exception $e) {
         return response()->json(['error' => $e->getMessage()], 404);
      }
   }

   public function getElectricityMonth($id)
   {
      try {
         $totalPandingBillMonth = [];
         $ElectricityMonth = StudentElecticityRecords::select("billDate")
            ->where("userId", $id)
            ->where("status", 0)
            ->whereNot('Amount', 0)
            ->orderBy("billDate", "ASC")
            ->get();
         if (!empty($ElectricityMonth)) {
            foreach ($ElectricityMonth as $month) {
               $totalPandingBillMonth[] = [
                  $month->billDate => Carbon::parse($month->billDate)->format("d-M-Y")
               ];
            }
         }
         return response()->json($totalPandingBillMonth);
      } catch (Exception $e) {
         return respone()->json([
            'status' => false,
            'message' => $e->getMessage()
         ]);
      }
   }


   public function getStudentElectricityBill($studentId)
   {
      try {

         $records = StudentElecticityRecords::where('userId', $studentId)->with('electricityBillDetails')->get();

         return response()->json([
            'status' => true,
            'statusCode' => 200,
            'data' => $records,
            'message' => 'Student Electricity Records fetched successfully'
         ], 200);
      } catch (Exception $e) {
         return response()->json([
            'status' => false,
            'statusCode' => 500,
            'message' => $e->getMessage()
         ], 500);
      }
   }

   public function addMeterChangeRecords(Request $request)
   {
      try {
         // Define validation rules
         $validator = Validator::make($request->all(), [
            'date' => 'required|date',
            'roomno' => 'required|string',
            'old_meter_reading' => 'required|numeric',
            'new_meter_reading' => 'required|numeric',
            'total_old_reading' => 'required|numeric',
         ]);

         // Check if validation fails
         if ($validator->fails()) {
            return back()->with('error', $validator->errors()->first());
         }

         // Create a new ElectricityMeterChange record
         $meterChange = ElectricityMeterChange::create($request->all());

         return back()->with('success', 'Record added successfully.');
      } catch (Exception $e) {
         // Log the exception
         Log::error('Error adding meter change record: ' . $e->getMessage());

         return back()->with('error', 'Something went wrong.');
      }
   }

   public function meterReadingData(Request $request)
   {
      try {
         $roomno = $request->roomNo;
         $data = ElectricityMeterChange::where('roomno', $roomno)
            ->orderBy('id', 'desc')
            ->get();
         $meterChangeHistory = [];
         $oldMeterTotal = 0;
         foreach ($data as $key => $value) {
            $meterChangeHistory[] = (($value->old_meter_reading) + ($value->new_meter_reading)) . " (" . (Carbon::parse($value->date)->format('d-M-Y')) . ")";
            if ($oldMeterTotal == 0) {
               $oldMeterTotal = $value->total_old_reading;
            }
         }
         return response()->json([
            'status' => true,
            'message' => 'meter data get successfully',
            'data' => [
               'meterHistory' => $meterChangeHistory,
               'oldMeterTotal' => $oldMeterTotal
            ],
         ], 200);
      } catch (Exception $e) {
         return response()->json([
            'status' => false,
            'error' => $e->getMessage(),
         ], 500);
      }
   }

   public function autoGenrateElectricityBill(Request $request)
   {
      try {
         $validator = Validator::make($request->all(), [
            'billDate' => 'required|date',
         ]);

         if ($validator->fails()) {
            return response()->json([
               'status' => false,
               'error' => 'Validation failed',
            ], 400);
         }

         // Get the date of the bill to create
         $billDate = $request->billDate;

         // Get all electricity records for the specified bill date
         $allBillOfCreatedAt = StudentElecticityRecords::where('billDate', $billDate)
            ->where('status', 0)
            ->where('Amount', '>', 0)
            ->get();

         DB::beginTransaction();

         foreach ($allBillOfCreatedAt as $bill) {
            $inVoiceNo = $this->getInvoiceNo();
            $createBillResponse = $this->createSaleRecords($bill, $inVoiceNo);
            if (!$createBillResponse['status']) {
               Log::info('Bill not created for the student', ['details' => $createBillResponse]);
               continue;
            }
            $bill->status = 1;
            $bill->invoiceno = $createBillResponse['invoiceNo'];
            $bill->save();
         }

         DB::commit();

         return response()->json([
            'status' => true,
            'message' => 'Automatic bill creation initiated successfully',
         ]);
      } catch (Exception $e) {
         DB::rollBack();
         return response()->json([
            'status' => false,
            'error' => $e->getMessage(),
         ]);
      }
   }

   private function getInvoiceNo()
   {
      $invoiceNo = Sale::orderBy('id', 'desc')->first();
      if ($invoiceNo) {
         $invoiceNo = $invoiceNo->invoiceno + 1;
      } else {
         $invoiceNo = 1;
      }
      return $invoiceNo;
   }

   private function createSaleRecords($billRecords, $invoiceNo)
   {
      try {
         $newSaleRecords = new Sale();
         $newSaleRecords->roomno = $billRecords->roomNo;
         $newSaleRecords->studentid = $billRecords->userId;
         $newSaleRecords->invoiceno = $invoiceNo;
         $newSaleRecords->invoicedate = Carbon::now()->format('Y-m-d');
         $newSaleRecords->tpdate = Carbon::now()->format('Y-m-d');
         $newSaleRecords->item = 'e01';
         $newSaleRecords->quantity = '1';
         $newSaleRecords->rate = $billRecords->Amount;
         $newSaleRecords->tax = 0;
         $newSaleRecords->amount = $billRecords->Amount;
         $newSaleRecords->discount = 0;
         $newSaleRecords->total = $billRecords->Amount;
         $newSaleRecords->receive_amount = 0;
         $newSaleRecords->remaining_amount = $billRecords->Amount;
         $newSaleRecords->unused = 0;
         $newSaleRecords->save();

         Log::info('bill created successfully', ['details' => "invoice No: $newSaleRecords->invoiceno , studentId : $billRecords->userId "]);
         return [
            'status' => true,
            'message' => "Bill created for room no $billRecords->roomNo and student id $billRecords->userId",
            'invoiceNo' => $newSaleRecords->invoiceno,
         ];
      } catch (Exception $e) {
         Log::info('bill not created created successfully', ['details' => "invoice No: $newSaleRecords->invoiceno , studentId : $billRecords->userId ", 'error' => $e->getMessage()]);

         return [
            'status' => false,
            'error' => $e->getMessage(),
         ];
      }
   }
}
