<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\EmpManagement;
use Carbon\Carbon;
use Intervention\Image\Facades\Image;
use Illuminate\Support\Facades\File;
use Exception;
use App\Models\EmpAttendance;
use App\Models\Businessregister;
use Illuminate\Support\Facades\Validator;
use App\Models\Uploadprofile;
use App\Models\Reviews;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Validation\Rule;
use App\Models\empPaymentSetup;
use App\Models\OldEmployee;
use DB;


class EmpController extends Controller
{
    public function resizeBase64Image($base64Data, $targetSizeInKB)
    {
        // Decode base64 data
        $img = explode(',', $base64Data);
        $imageData = base64_decode($img[1]);


        // Create an image resource from the decoded data
        $img = imagecreatefromstring($imageData);

        // Get the original image dimensions
        $width = imagesx($img);
        $height = imagesy($img);

        // Set the desired width and calculate the corresponding height to maintain the aspect ratio
        $targetWidth = 200; // Adjust this value based on your requirements
        $targetHeight = ($targetWidth / $width) * $height;

        // Create a new image resource for the resized image
        $resizedImg = imagecreatetruecolor($targetWidth, $targetHeight);

        // Resize the image
        imagecopyresampled($resizedImg, $img, 0, 0, 0, 0, $targetWidth, $targetHeight, $width, $height);

        // Create a buffer for the resized image
        ob_start();

        // Output the resized image to the buffer
        imagejpeg($resizedImg, null, 90); // Adjust the quality as needed

        // Get the buffer content
        $resizedImageData = ob_get_clean();

        // Check if the resized image meets the target size
        while (strlen($resizedImageData) > ($targetSizeInKB * 1024)) {
            // If the size is still larger than the target, reduce the quality and update the buffer
            ob_start();
            imagejpeg($resizedImg, null, 90);
            $resizedImageData = ob_get_clean();
        }

        // Encode the resized image to base64
        $resizedBase64Data = base64_encode($resizedImageData);

        // Destroy the image resources
        imagedestroy($img);
        imagedestroy($resizedImg);

        return $resizedBase64Data;
    }
    public function ResisterEmp()
    {
        config(['database.default' => env('DB_CONNECTION')]);
        $liceneNo = session('liseneceNo');
        $latestFormNo = EmpManagement::max('form_no');

        $allEmployees = EmpManagement::where('license_no', $liceneNo)->get();
        config(['database.default' => "custom_db"]);
        $formno =  $latestFormNo ? $latestFormNo + 1 : 1;
        $allPermission = session('mainPermissions');
        return view("fronted.employeeregisteration", compact('formno', 'allEmployees', 'allPermission'));
    }
    public function attendaceView(Request $request)
    {
        $selectedDate = $request->selectedMonth ? Carbon::now()->setMonth($request->selectedMonth) : Carbon::now();
        $liceneNo = session('liseneceNo');
        $allEmployees = EmpManagement::where('license_no', $liceneNo)->where('is_active', 1)->get();

        $allEmployees->each(function ($emp) use ($selectedDate) {
            $data = $emp->everyMonthSalaryCalulation($selectedDate);
            $emp->attendanceForCurrentMonth = $data;
        });
        $date = Carbon::now();
        return view("fronted.emp_attendance", compact('allEmployees', "date"));
    }
    public function index()
    {
        $liceneNo = session('liseneceNo');
        $latestFormNo = EmpManagement::max('form_no');
        $allEmployees = EmpManagement::where('license_no', $liceneNo)->where('is_active', 1)->get();
        $selectedDate = Carbon::now();
        $allEmployees->each(function ($emp) use ($selectedDate) {
            $data = $emp->everyMonthSalaryCalulation($selectedDate);
            $emp->attendanceForCurrentMonth = $data;
        });
        $runningReg = $allEmployees->count();
        $oldReg =  OldEmployee::count();
        $totalReg = EmpManagement::where('license_no', $liceneNo)->count();
        return view("fronted.emp_management", compact('allEmployees', 'runningReg', 'oldReg', 'totalReg'));
    }

    public function oldEmpView()
    {
        $liceneNo = session('liseneceNo');
        $allEmployees = OldEmployee::where('license_no', $liceneNo)->get();
        $selectedDate = Carbon::now();
        $allEmployees->each(function ($emp) use ($selectedDate) {
            $data = $emp->everyMonthSalaryCalulation($selectedDate);
            $emp->attendanceForCurrentMonth = $data;
        });
        $runningReg = EmpManagement::where('license_no', $liceneNo)->where('is_active',1)->count();
        $oldReg =  OldEmployee::count();
        $totalReg = EmpManagement::where('license_no', $liceneNo)->count();
        return view("fronted.old_emp_management", compact('allEmployees', 'runningReg', 'oldReg', 'totalReg'));
    }


    public function empRegistraionFormView()
    {
        $filePath = '';
        $empTermsAndCondtion = Uploadprofile::select('upload_pdf')->where('upload_name', 'T&C_for_emp')->first();
        if ($empTermsAndCondtion && file_exists(public_path('assets/uploads/pdf/' . $empTermsAndCondtion->upload_pdf))) {
            $filePath = asset('assets/uploads/pdf/' . $empTermsAndCondtion->upload_pdf);
        }
        $latestFormNo = EmpManagement::max('form_no');
        $formno =  $latestFormNo ? $latestFormNo + 1 : 1;
        return view("fronted.empRegistrationForm", compact('formno', 'filePath'));
    }

    public function editEmpRegistrationFormView($empId)
    {
        try {
            $filePath = '';
            $empTermsAndCondtion = Uploadprofile::select('upload_pdf')->where('upload_name', 'T&C_for_emp')->first();
            if ($empTermsAndCondtion && file_exists(public_path('assets/uploads/pdf/' . $empTermsAndCondtion->upload_pdf))) {
                $filePath = asset('assets/uploads/pdf/' . $empTermsAndCondtion->upload_pdf);
            }
            $empDetails = EmpManagement::with('paymentSetupData')->findOrFail($empId);
            return view("fronted.editEmpRegistrationForm", compact('empDetails', 'filePath'));
        } catch (Exception $e) {
            return redirect()->route('employees.show')->with('error', $e->getMessage());
        }
    }

    public function store(Request $request)
    {
        config(['database.default' => env('DB_CONNECTION')]);
        $validatedData = $request->validate([
            'formno' => 'required|string|unique:emp_management,form_no',
            'regdate' => 'required|date',
            'empName' => 'required|string',
            'userEmail' => 'required|email|unique:emp_management,email',
            'userMobile' => 'required|digits:10',
            'userDob' => 'required|date',
            'userAadhar' => 'required|digits:12|unique:emp_management,aadharcard_no',
            'bloodGroup' => 'required',
            'healthCheck' => 'required',
            'healthDesc' => 'nullable|required_if:healthCheck,Yes|string',
            'maritalStatus' => 'required|string',
            'vehicalCheck' => 'required',
            'vehicalNo' => 'nullable|required_if:vehivalCheck,Yes|string',
            'userFatherName' => 'nullable|string',
            'Designation' => 'required|string',
            'currentDepartment' => 'required|string',
            'salaryPerMonth' => 'required|numeric',
            'currentAddress' => 'required|string',
            'currentState' => 'required|string',
            'currentPin' => 'required|digits:6',
            'parmanaetAddress' => 'required|string',
            'parmanaetState' => 'required|string',
            'parmanaetPin' => 'required|digits:6',
            'userBankName' => 'required|string',
            'holderName' => 'required|string',
            'ifscCode' => 'required',
            'BankAccount' => 'required|string|min:6',
            'companyName' => 'nullable|array',
            'companyMobileNo' => 'nullable|array',
            'designation' => 'nullable|array',
            'startDate' => 'nullable|array',
            'endDate' => 'nullable|array',
            'uploadaadharfront' => 'required|image|mimes:jpeg,png,jpg',
            'uploadaadharback' => 'required|image|mimes:jpeg,png,jpg',
            'employeeSignature' => 'required|image|mimes:jpeg,png,jpg',
            'empImg' => 'required|image|mimes:jpeg,png,jpg',
            'experienceLatter' => 'file|mimes:pdf|max:5048',
            'bankDetailPdf' => 'file|mimes:pdf|max:5048',
            'otherDocumets' => 'file|mimes:pdf|max:5048',
        ], [
            'userEmail.unique' => 'This email is registered with other account.',
            'userAadhar.unique' => 'This Aadhar number is already registered with other account.',
            'healthDesc.required_if' => 'Health description is required if health check is "Yes".',
            'vehicleNo.required_if' => 'Vehicle number is required if vehicle check is "Yes".',
        ]);
        if ($validatedData) {
            $newEmp = new EmpManagement();
            $newEmp->form_no = $request->formno;
            $newEmp->registration_date = $request->regdate;
            $newEmp->join_date = $request->regdate;
            $newEmp->candidate_name = $request->empName;
            $newEmp->mobile = $request->userMobile;
            $newEmp->email  = $request->userEmail;
            $newEmp->dob = $request->userDob;
            $newEmp->aadharcard_no  = $request->userAadhar;
            $newEmp->helth_issue = $request->healthCheck;
            $newEmp->helth_description = $request->healthDesc;
            $newEmp->marital_status = $request->maritalStatus;
            $newEmp->vehical_check = $request->vehicalCheck;
            $newEmp->vehical_no = $request->vehicalNo;
            $newEmp->blood_group = $request->bloodGroup;
            $newEmp->father_name = $request->userFatherName;
            $newEmp->current_address = $request->currentAddress;
            $newEmp->current_state = $request->currentState;
            $newEmp->current_pincode = $request->currentPin;
            $newEmp->parmanent_address = $request->parmanaetAddress;
            $newEmp->parmanent_state = $request->parmanaetState;
            $newEmp->parmanent_pincode = $request->parmanaetPin;
            $newEmp->company_name =  implode(",", $request->companyName);
            $newEmp->company_contect = implode(",", $request->companyMobileNo);
            $newEmp->designation =  implode(",", $request->designation);
            $newEmp->startDate =  implode(",", $request->startDate);
            $newEmp->endDate =   implode(",", $request->endDate);
            $newEmp->bank_name = $request->userBankName;
            $newEmp->holder_name = $request->holderName;
            $newEmp->Ifcs_code = $request->ifscCode;
            $newEmp->account_no = $request->BankAccount;
            $newEmp->user_roll = $request->userRole;
            $newEmp->current_designation = $request->Designation;
            $newEmp->current_department = $request->currentDepartment;
            $newEmp->current_salary = $request->salaryPerMonth;
            $newEmp->license_no = session('liseneceNo');

            if ($request->hasFile('uploadaadharfront')) {
                $file = $request->file('uploadaadharfront');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->AadharFrontCrop, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->aadhar_front = $filename;
            };

            if ($request->hasFile('uploadaadharback')) {
                $file = $request->file('uploadaadharback');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->AadharBackCrop, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->aadhar_back = $filename;
            };

            if ($request->hasFile('employeeSignature')) {
                $file = $request->file('employeeSignature');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->candidatesignCropImg, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->candidate_signature = $filename;
            };

            if ($request->hasFile('experienceLatter')) {
                $file = $request->file('experienceLatter');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $newEmp->experiece_letter = $filename;
            };
            if ($request->hasFile('bankDetailPdf')) {
                $file = $request->file('bankDetailPdf');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $newEmp->bank_doc = $filename;
            };
            if ($request->hasFile('otherDocumets')) {
                $file = $request->file('otherDocumets');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $newEmp->other_doc = $filename;
            };

            if ($request->hasFile('empImg')) {
                $file = $request->file('empImg');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->ParentCropImg, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->candidate_photo = $filename;
            };

            if ($newEmp->save()) {
                // add this payment details in payment setup
                $this->PaymentSetup($newEmp->id, $request->salaryPerMonth);
                return back()->with(['success' => "Employee has been registered."], 200);
            } else {
                return back()->with(['error' => "Employee has not been registered."], 500);
            }
        }
    }

    // payment setup 
    private function PaymentSetup($id, $salary)
    {
        empPaymentSetup::create([
            'date' => Carbon::now()->format('Y-m-d'),
            'emp_id' => $id,
            'monthly_payment' => $salary,
        ]);
    }

    public function update(Request $request, $id)
    {
        config(['database.default' => env('DB_CONNECTION')]);
        $request->validate([
            'form_no' => [
                'required',
                'string',
                Rule::unique('emp_management')->ignore($id),
            ],
            'regdate' => 'required|date',
            'empName' => 'required|string',
            'email' => [
                'required',
                'email',
                Rule::unique('emp_management')->ignore($id),
            ],
            'userMobile' => 'required|digits:10',
            'userDob' => 'required|date',
            'aadharcard_no' =>  [
                'required',
                'digits:12',
                Rule::unique('emp_management')->ignore($id),
            ],
            'bloodGroup' => 'required',
            'healthCheck' => 'required',
            'healthDesc' => 'nullable|required_if:healthCheck,Yes|string',
            'maritalStatus' => 'required|string',
            'vehicalCheck' => 'required',
            'vehicalNo' => 'nullable|required_if:vehivalCheck,Yes|string',
            'userFatherName' => 'nullable|string',
            'Designation' => 'required|string',
            'currentDepartment' => 'required|string',
            'salaryPerMonth' => 'required|numeric',
            'currentAddress' => 'required|string',
            'currentState' => 'required|string',
            'currentPin' => 'required|digits:6',
            'parmanaetAddress' => 'required|string',
            'parmanaetState' => 'required|string',
            'parmanaetPin' => 'required|digits:6',
            'userBankName' => 'required|string',
            'holderName' => 'required|string',
            'ifscCode' => 'required',
            'BankAccount' => 'required|string|min:6',
            'companyName' => 'nullable|array',
            'companyMobileNo' => 'nullable|array',
            'designation' => 'nullable|array',
            'startDate' => 'nullable|array',
            'endDate' => 'nullable|array',
            'uploadaadharfront' => 'nullable|image|mimes:jpeg,png,jpg',
            'uploadaadharback' => 'nullable|image|mimes:jpeg,png,jpg',
            'employeeSignature' => 'nullable|image|mimes:jpeg,png,jpg',
            'empImg' => 'nullable|image|mimes:jpeg,png,jpg',
            'experienceLatter' => 'file|mimes:pdf|max:5048',
            'bankDetailPdf' => 'file|mimes:pdf|max:5048',
            'otherDocumets' => 'file|mimes:pdf|max:5048',
        ], [
            'userEmail.unique' => 'This email is registered with other account.',
            'userAadhar.unique' => 'This Aadhar number is already registered with other account.',
            'healthDesc.required_if' => 'Health description is required if health check is "Yes".',
            'vehicleNo.required_if' => 'Vehicle number is required if vehicle check is "Yes".',
        ]);

        try {
            $newEmp = EmpManagement::findOrFail($id);
            $newEmp->form_no = $request->form_no;
            $newEmp->registration_date = $request->regdate;
            $newEmp->candidate_name = $request->empName;
            $newEmp->mobile = $request->userMobile;
            $newEmp->email  = $request->email;
            $newEmp->dob = $request->userDob;
            $newEmp->aadharcard_no  = $request->aadharcard_no;
            $newEmp->helth_issue = $request->healthCheck;
            $newEmp->helth_description = $request->healthDesc;
            $newEmp->marital_status = $request->maritalStatus;
            $newEmp->vehical_check = $request->vehicalCheck;
            $newEmp->vehical_no = $request->vehicalNo;
            $newEmp->blood_group = $request->bloodGroup;
            $newEmp->father_name = $request->userFatherName;
            $newEmp->current_address = $request->currentAddress;
            $newEmp->current_state = $request->currentState;
            $newEmp->current_pincode = $request->currentPin;
            $newEmp->parmanent_address = $request->parmanaetAddress;
            $newEmp->parmanent_state = $request->parmanaetState;
            $newEmp->parmanent_pincode = $request->parmanaetPin;
            $newEmp->company_name =  implode(",", $request->companyName);
            $newEmp->company_contect = implode(",", $request->companyMobileNo);
            $newEmp->designation =  implode(",", $request->designation);
            $newEmp->startDate =  implode(",", $request->startDate);
            $newEmp->endDate =   implode(",", $request->endDate);
            $newEmp->bank_name = $request->userBankName;
            $newEmp->holder_name = $request->holderName;
            $newEmp->Ifcs_code = $request->ifscCode;
            $newEmp->account_no = $request->BankAccount;
            $newEmp->user_roll = $request->userRole;
            $newEmp->current_designation = $request->Designation;
            $newEmp->current_department = $request->currentDepartment;
            $newEmp->license_no = session('liseneceNo');

            if ($request->hasFile('uploadaadharfront')) {
                $file = $request->file('uploadaadharfront');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->AadharFrontCrop, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->aadhar_front = $filename;
            };

            if ($request->hasFile('uploadaadharback')) {
                $file = $request->file('uploadaadharback');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->AadharBackCrop, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->aadhar_back = $filename;
            };

            if ($request->hasFile('employeeSignature')) {
                $file = $request->file('employeeSignature');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->candidatesignCropImg, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->candidate_signature = $filename;
            };

            if ($request->hasFile('experienceLatter')) {
                $file = $request->file('experienceLatter');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $newEmp->experiece_letter = $filename;
            };
            if ($request->hasFile('bankDetailPdf')) {
                $file = $request->file('bankDetailPdf');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $newEmp->bank_doc = $filename;
            };
            if ($request->hasFile('otherDocumets')) {
                $file = $request->file('otherDocumets');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '_' . rand() . '.' . $extension;
                $file->move('assets/uploads/', $filename);
                $newEmp->other_doc = $filename;
            };

            if ($request->hasFile('empImg')) {
                $file = $request->file('empImg');
                $extension = $file->getClientOriginalExtension();
                $filename = time() . '.' . rand() . '.' . $extension;
                $resizedBase64Data = $this->resizeBase64Image($request->ParentCropImg, 50);
                File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                $newEmp->candidate_photo = $filename;
            };
            if ($newEmp->update()) {
                return back()->with(['success' => "Employee details updated successfully."], 200);
            } else {
                return back()->with(['error' => "Employee details not updated."], 500);
            }
        } catch (Exception $e) {
            return back()->with(['error' => $e->getMessage()]);
        }
    }
    public function permanentDelete($empId)
    {
        try {
            $employee = OldEmployee::find($empId);
            if (!$employee) {
                return response()->json(['message' => 'Employee not found'], 404);
            }
            $employee->delete();
            return response()->json(['message' => 'Employee deleted successfully'], 200);

        } catch (Exception $e) {
            return response()->json(['message' => 'somting went wronge'], 500);
        }
    }
    public function getdata($empId)
    {
        $employeeData = EmpManagement::find($empId);
        if ($employeeData) {
            $permission = explode(",", $employeeData->permissions);
            if (!empty($employeeData->company_name)) {
                $companyNames = explode(",", $employeeData->company_name);
                $company_contect = explode(",", $employeeData->company_contect);
                $designation = explode(",", $employeeData->designation);
                $startDate  = explode(",", $employeeData->startDate);
                $endDate  = explode(",", $employeeData->endDate);
                if (count($companyNames) > 1) {
                    $firstCompanyName = $companyNames[0];
                    $firstCompanyMobile = $company_contect[0];
                    $firstCompanydesignation = $designation[0];
                    $firststartDate = $startDate[0];
                    $firstendDate = $endDate[0];
                    $sencandCompanyName = $companyNames[1];
                    $sencandCompanyMobile = $company_contect[1];
                    $sencandCompanydesignation = $designation[1];
                    $sencandstartDate = $startDate[1];
                    $sencandendDate = $endDate[1];
                } else {
                    $firstCompanyName = $companyNames[0];
                    $firstCompanyMobile = $company_contect[0];
                    $firstCompanydesignation = $designation[0];
                    $firststartDate = $startDate[0];
                    $firstendDate = $endDate[0];
                    $sencandCompanyName = "";
                    $sencandCompanyMobile =  "";
                    $sencandCompanydesignation = "";
                    $sencandstartDate = "";
                    $sencandendDate = "";
                }
            } else {
                $firstCompanyName = "";
                $firstCompanyMobile = "";
                $firstCompanydesignation = "";
                $firststartDate = "";
                $firstendDate = "";
                $sencandCompanyName = "";
                $sencandCompanyMobile =  "";
                $sencandCompanydesignation = "";
                $sencandstartDate = "";
                $sencandendDate = "";
            }
            if (!empty($employeeData->aadhar_front)) {
                $aadhar_front_path = ('assets/uploads/' . $employeeData->aadhar_front);
            } else {
                $aadhar_front_path = "";
            }
            if (!empty($employeeData->aadhar_back)) {
                $aadhar_back_path = ('assets/uploads/' . $employeeData->aadhar_back);
            } else {
                $aadhar_back_path = "";
            }
            if (!empty($employeeData->candidate_signature)) {
                $candidate_signature_path = ('assets/uploads/' . $employeeData->candidate_signature);
            } else {
                $candidate_signature_path = "";
            }
            if (!empty($employeeData->candidate_photo)) {
                $candidate_photo_path = ('assets/uploads/' . $employeeData->candidate_photo);
            } else {
                $candidate_photo_path = "";
            }
            $empdata = [
                'formno' => $employeeData->form_no,
                'regdate' => $employeeData->registration_date,
                'empName' => $employeeData->candidate_name,
                'userEmail' => $employeeData->email,
                'userMobile' => $employeeData->mobile,
                'userDob' => $employeeData->dob,
                'userAadhar' => $employeeData->aadharcard_no,
                'bloodGroup' => $employeeData->blood_group,
                'healthCheck' => $employeeData->helth_issue,
                'healthDesc' => $employeeData->helth_description,
                'maritalStatus' => $employeeData->marital_status,
                'vehicalCheck' => $employeeData->vehical_check,
                'vehicalNo' => $employeeData->vehical_no,
                'userFatherName' => $employeeData->father_name,
                'currentdesignation' => $employeeData->current_designation,
                'currentAddress' => $employeeData->current_address,
                'currentState' =>  $employeeData->current_state,
                'currentPin' => $employeeData->current_pincode,
                'parmanaetAddress' => $employeeData->parmanent_address,
                'parmanaetState' => $employeeData->parmanent_state,
                'parmanaetPin' => $employeeData->parmanent_pincode,
                'userBankName' => $employeeData->bank_name,
                'holderName' => $employeeData->holder_name,
                'ifscCode' => $employeeData->Ifcs_code,
                'BankAccount' => $employeeData->account_no,
                'firstCompanyName' => $firstCompanyName,
                'firstCompanyMobile' => $firstCompanyMobile,
                'userDesignation' => $firstCompanydesignation,
                'startDate' => $firststartDate,
                'endDate' => $firstendDate,
                'secandCompanyName' => $sencandCompanyName,
                'secandCompanyMobile' => $sencandCompanyMobile,
                'userDesignationSecand' => $sencandCompanydesignation,
                'startDateSecand' => $sencandstartDate,
                'endDateSecand' => $sencandendDate,
                'uploadaadharfront' => $aadhar_front_path,
                'uploadaadharback' => $aadhar_back_path,
                'employeeSignature' => $candidate_signature_path,
                'empImg' => $candidate_photo_path,
                'permissions' => isset($permission) ? $permission : [],
                'userRoll' => $employeeData->user_roll,
                'password' => $employeeData->password,
            ];

            return response()->json($empdata, 200);
        } else {
            return response()->json(["error" => "Somthing has been wrong"], 500);
        }
    }
    // attendance view;
    public function Attendacestore(Request $request)
    {
        $attendanceDate = $request->input('attendance_date');
        $attendanceData = $request->input('attendance');
        // Loop through the attendance data and save it for each employee
        try {
            foreach ($attendanceData as $employeeId => $status) {
                $attendanceRecord = EmpAttendance::where("employee_id", $employeeId)->where("date", $attendanceDate)->first();
                if ($attendanceRecord) {
                    return redirect()->back()->with("error", "Attendance already stored for " . $attendanceDate);
                } else {
                    // Create a new record
                    $newAttendanceRecord = new EmpAttendance();
                    $newAttendanceRecord->date = $attendanceDate;
                    $newAttendanceRecord->employee_id = $employeeId;
                    $newAttendanceRecord->status = $status;
                    $newAttendanceRecord->save();
                }
            }
            return redirect()->back()->with('attendaneSaved', 'Attendance saved successfully');
        } catch (Exception $e) {
            return redirect()->back()->with("error", "Attendance already stored for " . $attendanceDate);
        }
    }


    public function view($id)
    {
        try {
            $attendanceData = EmpAttendance::where("employee_id", $id)->get();
            $totalPresent =  EmpAttendance::where("employee_id", $id)->where("status", "P")->count();
            $totalAbsenet =  EmpAttendance::where("employee_id", $id)->where("status", "A")->count();
            $totalLeaves =  EmpAttendance::where("employee_id", $id)->where("status", "L")->count();
            $totalHalfDays =  EmpAttendance::where("employee_id", $id)->where("status", "H")->count();
            $attendanceEvents = array();
            foreach ($attendanceData as $attendance) {
                $attendanceEvents[] = [
                    'title' => $attendance->status,
                    'start' => $attendance->date,
                    'color' => $attendance->status === 'P' ? 'green' : ($attendance->status === 'A' ? 'orange' : ($attendance->status === "H" ? 'yellow' : 'red')),
                ];
            }
            // Retrieve the associated employee data
            $employee = null;
            if ($attendance) {
                $employee = EmpManagement::find($id);
            }
            // get the PaymentSetupd details 
            $paymentSetups = empPaymentSetup::where('emp_id', $id)->get();
            return view('fronted.emp_attendacne_view', [
                'attendance' => $attendanceEvents,
                'employeeDetails' => $employee,
                'empId' => $id,
                'dataNotFound' => count($attendanceData) === 0,
                'persent' => $totalPresent,
                'absent' => $totalAbsenet,
                'leaves' => $totalLeaves,
                'halfdays' => $totalHalfDays,
                'paymentSetup' => $paymentSetups
            ]);
        } catch (Exception $e) {
            return back()->with(["error" => "Attendance Record Not Found"]);
        }
    }

    public function updateAttendace(Request $request)
    {
        $employeeId = $request['empid'];
        $attendanceDate = $request['date'];
        $status = $request['status'];
        $attendanceRecord = EmpAttendance::firstOrNew([
            'date' => $attendanceDate,
            'employee_id' => $employeeId,
        ]);
        $attendanceRecord->status = $status;
        if ($attendanceRecord->save()) {
            return response()->json(['message' => 'Employee deleted successfully'], 200);
        } else {
            return response()->json(['message' => 'Employee deleted successfully'], 500);
        }
    }

    public function delete($id, $date)
    {
        try {
            $record = EmpAttendance::where('employee_id', $id)
                ->where('date', $date)
                ->firstOrFail();
            $record->delete();
            return response()->json(['message' => 'Attendance Record deleted successfully']);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Failed to delete record'], 500);
        }
    }

    public function deleteAndAddReview(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'leave_date' => 'required|date',
            'condidateId' => 'required'
        ]);

        if ($validator->fails()) {
            return back()->with('error', $validator->errors()->first());
        }

        DB::beginTransaction(); // Start the transaction
        try {
            $empId = $request->condidateId; // Ensure the correct key name, e.g., candidateId
            $employes = EmpManagement::findOrFail($empId);
            $addNewReview = new Reviews();
            $addNewReview->name = $employes->candidate_name;
            $addNewReview->mobile = $employes->mobile;
            $addNewReview->email = $employes->email;
            $addNewReview->aadhaar = $employes->aadharcard_no;
            $addNewReview->payment_terms = $request->payment_terms_rating;
            $addNewReview->tenant_behavior = $request->tenant_behavior_rating;
            $addNewReview->responsibilities = $request->responsibilities_rating;
            $addNewReview->additional = $request->additional_comments;
            $addNewReview->hostelname = session("hostel");
            $addNewReview->save();

            // Mark the employee as inactive
            $employes->update(['is_active' => 0]);

            // Copy employee data to old_management_table
            $oldEmployeeData = $employes->makeHidden(['attendanceForCurrentMonth'])->toArray();
            $oldEmployeeData['candidate_photo'] = $employes->getAttributes()['candidate_photo'];
            $oldEmployeeData['candidate_signature'] = $employes->getAttributes()['candidate_signature'];
            $oldEmployeeData['aadhar_front'] = $employes->getAttributes()['aadhar_front'];
            $oldEmployeeData['aadhar_back'] = $employes->getAttributes()['aadhar_back'];
            $oldEmployeeData['experiece_letter'] = $employes->getAttributes()['experiece_letter'];
            $oldEmployeeData['bank_doc'] = $employes->getAttributes()['bank_doc'];
            $oldEmployeeData['other_doc'] = $employes->getAttributes()['other_doc'];

            $oldEmployeeData['leave_date'] = $request->leave_date;
            $oldEmployeeData['emp_management_id'] = $employes->id;
            OldEmployee::create($oldEmployeeData); // Correct way to create a new record

            DB::commit(); // Commit the transaction

            return back()->with('success', "Employee status changed successfully and review added");
        } catch (ModelNotFoundException $e) {
            DB::rollBack(); // Rollback the transaction on error
            return back()->with('error', "Invalid User");
        } catch (Exception $e) {
            DB::rollBack(); // Rollback the transaction on error
            dd($e->getMessage());
            return back()->with('error', $e->getMessage());
        }
    }

    public function restoreEmployee(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'id' => 'required|string',
                'restore_date' => 'required|date',
            ]);
            if ($validator->fails()) {
                return back()->with('error', $validator->errors()->first());
            }
            $emp = EmpManagement::findOrFail($request->id);
            $emp->update([
                'join_date' => $request->restore_date,
                'is_active' => 1
            ]);
            return back()->with('success', "Employee restore successfully");
        } catch (ModelNotFoundException $e) {
            return back()->with('error', "Invalid User");
        } catch (Exception $e) {
            return back()->with('error', $e->getMessage());
        }
    }
}
