<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Carbon\Carbon;
use Exception;
use Illuminate\Support\Facades\Validator;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Illuminate\Validation\Rule;
use DB;
use App\Models\MessageManager;

class MessageManagerController extends Controller
{
    public function view()
    {
        $data = MessageManager::get();
        return view('fronted.message_manager', compact('data'));
    }


    private function createMessageCode($messageType)
    {
        // Handle the specific case for 'food_menu'
        if ($messageType == 'food_menu') {
            $predefinedCodes = [
                'food_menu_001',
                'food_menu_002',
                'food_menu_003',
                'food_menu_004',
                'food_menu_005',
            ];

            // Get existing codes for 'food_menu'
            $existingCodes = MessageManager::where('msg_type', 'food_menu')->pluck('msg_code')->toArray();

            // Find the first available predefined code
            foreach ($predefinedCodes as $code) {
                if (!in_array($code, $existingCodes)) {
                    return $code;
                }
            }

            // If all predefined codes are used, return an error (or handle as needed)
            throw new Exception('Food Menu message limit exceeded.');
        }

        // Handle other message types
        $existingCodes = MessageManager::where('msg_type', $messageType)->pluck('msg_code')->toArray();

        // Extract numerical part from existing codes and find the next available number
        $existingNumbers = array_map(function ($code) use ($messageType) {
            return (int) str_replace($messageType . '_', '', $code);
        }, $existingCodes);

        $nextNumber = empty($existingNumbers) ? 1 : max($existingNumbers) + 1;

        // Format the new code
        return $messageType . '_' . str_pad($nextNumber, 3, '0', STR_PAD_LEFT);
    }

    public function store(Request $request)
    {
        try {
            // Validate the request data
            $validator = Validator::make($request->all(), [
                'msg_type' => 'required|string|max:255',
                'msg' => 'required|string',
                'status' => 'required|boolean',
            ]);

            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator)->withInput();
            }

            $data = $validator->validated();
            // Store the new message

            $msgCode = $this->createMessageCode($data['msg_type']);

            $data['msg_code'] = $msgCode;
            MessageManager::create($data);

            return redirect()->back()->with('success', 'Message created successfully');
        } catch (Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }

    public function delete($id)
    {
        try {
            $message = MessageManager::findOrFail($id);
            $message->delete();
            return response()->json(['message' => 'Message deleted successfully'], 200);
        } catch (\Exception $e) {
            return response()->json(['error' => 'Failed to delete message'], 500);
        }
    }

    public function update(Request $request)
    {
        try {
            // Validate the request data
            $validator = Validator::make($request->all(), [
                'msgId' => 'required|numeric',
                'msg_type' => 'required|string|max:255',
                'msg' => 'required|string',
                'status' => 'required|boolean',
            ]);

            if ($validator->fails()) {
                return redirect()->back()->withErrors($validator)->withInput();
            }
            $data = $validator->validated();
            $msgCode = $this->createMessageCode($data['msg_type']);

            $data['msg_code'] = $msgCode;
            $findRecords = MessageManager::findOrFail($request->msgId);
            $findRecords->update($data);
            return redirect()->back()->with('success', 'Message updated successfully');
        } catch (ModelNotFoundException $e) {
            return redirect()->back()->with('error', 'Message Records not Found');
        } catch (Exception $e) {
            return redirect()->back()->with('error', $e->getMessage());
        }
    }
}
