<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Support\Facades\DB;
use Dompdf\Dompdf;
use Dompdf\Options;
use Illuminate\Support\Facades\Session;
use Illuminate\Support\Facades\Response;
use App\Models\Sale;
use App\Models\Studentregister;
use App\Models\Item;
use App\Models\Recieve_amount;
use App\Models\Customsalesale;
use App\Models\StudentElecticityRecords;
use Carbon\Carbon;
use App\Models\Permissions;
use Exception;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use App\Models\ElectricityManagement;
use App\Models\Olddataregister;
use App\Models\ElecticityCalculater;
use App\Models\StudentsElecticityBill;
use App\Models\RoomSeat;
use App\Models\ReserveRoom;
use App\Models\Businessregister;
use App\Models\Termsandcondition;
use App\Models\ApiModels\BusinessDocument;
use App\Models\ApiModels\ExpensesItem;
use App\Models\EmpManagement;
use App\Models\Uploadprofile;
use App\Models\stock_detail;
use App\Models\StockCategory;
use App\Models\SubscriptionBillSetting;
use App\Models\Salesubscription;
use App\Models\MessageManager;


require_once public_path('assets/tcpdf/tcpdf.php');
require_once public_path('assets/tcpdf/tcpdi/tcpdi.php');
require_once public_path('assets/fpdi/src/autoload.php');


$dompdfPath = public_path('assets/dompdf');
set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
require_once $dompdfPath . '/autoload.inc.php';

use setasign\Fpdi\Tcpdf\Fpdi;
use TCPDF;

class PDFController extends Controller
{

    private function checkWhatsAppPermission(): bool
    {
        try {
            $permission = Permissions::where('permissionType', 'Desktop Whatsapp')->value('permission');
            return $permission ?? false; // Return the permission value or false if not found

        } catch (ModelNotFoundException $e) {
            return false; // Handle the case when the permission is not found

        } catch (Exception $e) {
            // Log or handle other exceptions
            return false;
        }
    }



    private function numberToWords($number)
    {
        $words = [
            'Zero', 'One', 'Two', 'Three', 'Four', 'Five', 'Six', 'Seven', 'Eight', 'Nine', 'Ten',
            'Eleven', 'Twelve', 'Thirteen', 'Fourteen', 'Fifteen', 'Sixteen', 'Seventeen', 'Eighteen', 'Nineteen'
        ];

        $tens = [
            '', '', 'Twenty', 'Thirty', 'Forty', 'Fifty', 'Sixty', 'Seventy', 'Eighty', 'Ninety'
        ];

        if ($number < 20) {
            return $words[$number];
        } elseif ($number < 100) {
            return $tens[(int)($number / 10)] . (($number % 10 !== 0) ? ' ' . $words[$number % 10] : '');
        } elseif ($number < 1000) {
            return $words[(int)($number / 100)] . ' Hundred' . (($number % 100 !== 0) ? ' and ' . $this->numberToWords($number % 100) : '');
        } elseif ($number < 100000) {
            return $this->numberToWords((int)($number / 1000)) . ' Thousand' . (($number % 1000 !== 0) ? ' ' . $this->numberToWords($number % 1000) : '');
        } elseif ($number < 10000000) {
            return $this->numberToWords((int)($number / 100000)) . ' Lakh' . (($number % 100000 !== 0) ? ' ' . $this->numberToWords($number % 100000) : '');
        } else {
            return 'Number out of range';
        }
    }
    // Download Stock Reports *************************
    public function downloadStockReports(Request $request)
    {
        $bussinessData = Businessregister::firstOrFail();
        $month = $request->date ?: Carbon::now()->format('Y-m');
        $previousMonthDate = Carbon::parse($month)->subMonth()->format('Y-m');
        $categoriesWithItemsAndDetails = StockCategory::with(['stockItem' => function ($query) use ($month) {
            $query->with(['itemDetails' => function ($query) use ($month) {
                $query->where('month', $month);
            }]);
        }])->get();
        $categoriesWithItemsAndDetails->each(function ($category) use ($previousMonthDate) {
            $category->stockItem->each(function ($item) use ($category, $previousMonthDate) {
                // Access each item's itemDetails collection
                $previousMonthdata = stock_detail::where('month', $previousMonthDate)->where('item_id', $item->id)->where('category_id', $category->id)->first();
                $item->setAttribute('previousMonthData', $previousMonthdata);
                $itemDetails = $item->itemDetails;
                if (!empty($itemDetails)) {
                    $itemDetails->each(function ($detail) use (&$category, $previousMonthDate, &$item, &$itemDetails) {
                        $category->setAttribute('orderDates', $itemDetails->orderDate);
                    });
                } else {
                    $category->setAttribute('orderDates', []);
                }
            });
        });
        $month = Carbon::parse($month)->format('M-Y');
        $view = view('pdf.stockReport', compact('bussinessData', 'categoriesWithItemsAndDetails', 'month'));
        $dompdf = new Dompdf();
        $dompdf->loadHtml($view);
        $dompdf->setPaper('A4', 'portrait');
        $dompdf->render();
        $filename = 'Stock_Reports_' . now()->format('YmdHis') . '.pdf';
        $output = $dompdf->output();
        $filePath = public_path('assets/uploads/' . $filename);

        file_put_contents($filePath, $output);
        $link = asset('assets/uploads/' . $filename);
        return response()->json([
            'status' => true,
            'downloadLink' => $link,
            'fileName' => $filename,
            'message' => 'stock report created.'
        ], 200);
    }
    public function downloadRoomSeat()
    {
        // Path to the Dompdf library
        $dompdfPath = public_path('assets/dompdf');
        // Add the Dompdf library to the include path
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
?>
        <html>

        <head>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
            <style>
                h6 {
                    font-size: 0.6rem;
                    font-weight: 500;
                    margin: 0;
                }

                h5 {
                    font-size: 0.8rem;
                    font-weight: 500;
                    margin: 0;
                }

                .table {
                    width: 100%;
                    margin-bottom: 1rem;
                    color: #212529;
                }

                .table-bordered {
                    border: 1px solid #dee2e6;
                }

                .table-striped tbody tr:nth-of-type(odd) {
                    background-color: rgba(0, 0, 0, 0.05);
                }

                .text-center {
                    text-align: center;
                }
            </style>
        </head>

        <body>
            <div class="container">
                <?php
                $user = session('user');
                $businessData = DB::table("businessregisters")->where('username', $user)
                    ->get();
                if ($businessData) {
                    foreach ($businessData as $data) {
                        $email = $data->email;
                        $mobile = $data->mobile;
                        $img = $data->pic;
                        $businessname = $data->businessname;
                        $Address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                    }
                }
                ?>
                <div class="container" style="width:400px; display:inline-block;">
                    <h1><?php echo $businessname ?></h1>
                    <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                    <h6>Address:<?php echo $Address ?></h6>
                </div>
                <div class="container" style="width:250px; display:inline-block;margin-left:30px;">
                    <?php
                    if (file_exists(public_path('assets/uploads/' . $img))) {
                        $imgPath = public_path('assets/uploads/' . $img);
                        $imgData = file_get_contents($imgPath);
                        $imgBase64 = base64_encode($imgData);
                    } else {
                        $imgPath = public_path('assets/images/logoimage.jpeg');
                        $imgData = file_get_contents($imgPath);
                        $imgBase64 = base64_encode($imgData);
                    }
                    ?>
                    <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
                </div>
                <hr>
            </div>
            <table>
                <tr>
                    <h4>
                        <center>Room Seat Availability Report</center>
                    </h4>
                </tr>
            </table>
            <?php
            $roomnumber = DB::table('roomseats')->count();
            $totalseats = DB::table('roomseats')->sum("numberseats");
            $engagedseats = DB::table('studentregisters')->where('status', 1)->count('id');
            $vacantSeats = $totalseats - $engagedseats;
            ?>
            <table class="table table-striped text-center table-bordered mb-4 roomSeatTable" id="example">
                <thead class="align-middle">
                    <tr>
                        <th>Total Room</th>
                        <th>Total Seat</th>
                        <th>Engaged Seat</th>
                        <th>Vacant</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td><?php echo $roomnumber; ?></td>
                        <td><?php echo $totalseats; ?></td>
                        <td><?php echo $engagedseats; ?></td>
                        <td><?php echo $vacantSeats; ?></td>
                    </tr>
                </tbody>
            </table>
            <table class="table table-striped text-center table-bordered mb-4 roomSeatTable" id="example">
                <thead class="align-middle">
                    <tr>
                        <th>S.N0</th>
                        <th>Room No</th>
                        <th>Room Type</th>
                        <th>No.Of Seats</th>
                        <th>No. Of Candidate</th>
                        <th>Rent</th>
                    </tr>
                </thead>
                <tbody class="align-middle">
                    <?php
                    $data = DB::table('roomseats')->orderBy('roomno', 'asc')->get();
                    $count = 0;
                    foreach ($data as $temp) :
                        $count++;
                    ?>
                        <tr>
                            <td><?php echo  $count ?></td>
                            <td><?php echo  $temp->roomno ?></td>
                            <td class="text-capitalize"><?php echo  $temp->roomtype ?></td>
                            <td><?php echo  $temp->numberseats ?></td>
                            <td>
                                <?php
                                $noOfCandidate = DB::table('studentregisters')
                                    ->where('roomno', $temp->roomno)
                                    ->where('status', 1)
                                    ->count();
                                echo $noOfCandidate;
                                ?>
                            </td>
                            <td><?php echo  $temp->rent ?></td>
                        </tr>
                    <?php endforeach; ?>
                </tbody>
            </table>
        </body>

        </html>
    <?php
        $html = ob_get_clean(); // Get the buffered content and clear the buffer
        $dompdf = new Dompdf(); // Create a new Dompdf instance
        $dompdf->loadHtml($html); // Load HTML content
        $dompdf->setPaper('A4', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        $filename = 'RoomSeatDetails.pdf'; // Generate a unique filename for the PDF
        $dompdf->stream($filename, ['Attachment' => true]);
    }
    ///////////////////////////////////////download all expenses report//////////////////////////////
    public function downloadAllExpenses()
    {
        $dompdfPath = public_path('assets/dompdf');
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
    ?>
        <html>

        <head>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
            <style>
                h6 {
                    font-size: 0.6rem;
                    font-weight: 500;
                    margin: 0;
                }

                h5 {
                    font-size: 0.8rem;
                    font-weight: 500;
                    margin: 0;
                }

                .table {
                    width: 100%;
                    margin-bottom: 1rem;
                    color: #212529;
                }

                .table-bordered {
                    border: 1px solid #dee2e6;
                }

                .table-striped tbody tr:nth-of-type(odd) {
                    background-color: rgba(0, 0, 0, 0.05);
                }

                .text-center {
                    text-align: center;
                }

                .item-category {
                    font-size: 1rem;
                    font-weight: 500;
                }

                .items {
                    height: 80vh;
                    overflow: auto;
                }

                #expensesReport thead {
                    font-size: 0.9rem;
                    background-color: #212529;
                    color: #fff;
                }

                #expensesReport tbody {
                    font-size: 0.8rem;
                }

                .totalAmount {
                    font-size: 1rem;
                    font-weight: 600;
                }

                .text-warning {
                    color: "orange!important";
                }
            </style>
        </head>

        <body>
            <?php
            $user = session('user');
            $businessData = DB::table("businessregisters")->where('username', $user)
                ->get();
            if ($businessData) {
                foreach ($businessData as $data) {
                    $email = $data->email;
                    $mobile = $data->mobile;
                    $img = $data->pic;
                    $businessname = $data->businessname;
                    $Address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                }
            }
            ?>
            <div class="container" style="width:400px; display:inline-block;">
                <h1><?php echo $businessname ?></h1>
                <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                <h6>Address:<?php echo $Address ?></h6>
            </div>
            <div class="container" style="width:250px; display:inline-block; margin-left:30px;">
                <?php
                if (file_exists(public_path('assets/uploads/' . $img))) {
                    $imgPath = public_path('assets/uploads/' . $img);
                    $imgData = file_get_contents($imgPath);
                    $imgBase64 = base64_encode($imgData);
                } else {
                    $imgPath = public_path('assets/images/logoimage.jpeg');
                    $imgData = file_get_contents($imgPath);
                    $imgBase64 = base64_encode($imgData);
                }
                ?>
                <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
            </div>
            <hr>
            <table>
                <tr>
                    <h4>
                        <center>Expenses Report</center>
                    </h4>
                </tr>
            </table>
            <div class="container">
                <div class="items border-bottom">
                    <?php
                    $expenses = DB::table('expenses')->get();
                    $count = 1;
                    if ($expenses->count() > 0) {
                        $startDate =  session('ExpensesStartDate');
                        $endDate = session('ExpensesEndDate');
                        foreach ($expenses as $expense) {
                            if ($startDate != '' && $endDate != '') {
                                $expense_details = DB::table('expensedetails')
                                    ->where('category_id', '=', $expense->id)
                                    ->whereBetween('billdate', [$startDate, $endDate])
                                    ->get();
                            } else {
                                $expense_details = DB::table('expensedetails')
                                    ->where('category_id', '=', $expense->id)
                                    ->get();
                            }
                            $totalExpenses = 0;
                            if ($expense_details->count() > 0) {
                                foreach ($expense_details as $item) {
                                    $totalExpenses = $totalExpenses + $item->amount;
                                }
                    ?>
                                <h4 style="background-color:#d9d9d9" class="text-black p-1 item-category">Category Name: <span class="text-warning text-capitalize fw-bold"><?php echo $expense->category_name ?></span> - Rs.
                                    <span class="text-warning text-capitalize fw-bold ms-1"><i class="fa-solid fa-indian-rupee-sign ms-1"></i><?php echo $totalExpenses ?></span>
                                </h4>
                                <div class="table-responsive">
                                    <table border="1" id="expensesReport" class="table text-center table-light">
                                        <thead class="align-middle">
                                            <tr>
                                                <th>S No</th>
                                                <th>Date</th>
                                                <th>Item Name</th>
                                                <th>Quantity</th>
                                                <th>Rate</th>
                                                <th>Amount</th>
                                                <th>Payment Type</th>
                                            </tr>
                                        </thead>
                                        <tbody class="align-middle">
                                            <?php
                                            foreach ($expense_details as $item) {
                                            ?>
                                                <tr>
                                                    <td><?php echo $count ?></td>
                                                    <td><?php echo  \Carbon\Carbon::parse($item->billdate)->format('d-M-Y') ?></td>
                                                    <td class="text-capitalize"><?php echo $item->itemname ?></td>
                                                    <td><?php echo $item->quantity ?></td>
                                                    <td><i class="fa-solid fa-indian-rupee-sign me-1"></i><?php echo $item->rate ?></td>
                                                    <td><i class="fa-solid fa-indian-rupee-sign me-1"></i><?php echo  $item->amount ?></td>
                                                    <td class="text-capitalize"><?php echo $item->paymentmode ?></td>
                                                </tr>
                                            <?php
                                                $count++;
                                            }
                                            $count = 1;
                                            ?>
                                        </tbody>
                                    </table>
                                </div>
                    <?php
                            }
                        }
                    } else {
                        echo "<p>Data not Found</p>";
                    }
                    ?>
                </div>
            </div>
            <div class="container">
                <?php
                if ($startDate != '' && $endDate != '') {
                    $totalAmount = DB::table('expensedetails')->whereBetween('billdate', [$startDate, $endDate])
                        ->sum('amount');
                } else {
                    $totalAmount = DB::table('expensedetails')
                        ->sum('amount');
                }
                //session()->forget('startDate');
                //session()->forget('endDate');
                ?>
                <h4 class="totalAmount bg-white p-1">Total Amount <span class="text-warning"><i class="fa-solid fa-indian-rupee-sign mx-1"></i> <?php echo $totalAmount ?></span>
                </h4>
            </div>
        </body>

        </html>
    <?php
        $html = ob_get_clean(); // Get the buffered content and clear the buffer
        $dompdf = new Dompdf(); // Create a new Dompdf instance
        $dompdf->loadHtml($html); // Load HTML content
        $dompdf->setPaper('A4', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        $filename = 'Expenses.pdf'; // Generate a unique filename for the PDF
        $dompdf->stream($filename, ['Attachment' => true]); // Save the PDF to the public folder and stream it for download
    }
    //////////////////////////////////////download old registration//////////////////////////////////
    public function downloadOldReg()
    {
        // Path to the Dompdf library
        $dompdfPath = public_path('assets/dompdf');
        // Add the Dompdf library to the include path
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
    ?>
        <html>

        <head>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
            <style>
                h6 {
                    font-size: 0.6rem;
                    font-weight: 500;
                    margin: 0;
                }

                h5 {
                    font-size: 0.8rem;
                    font-weight: 500;
                    margin: 0;
                }

                .table {
                    width: 100%;
                    margin-bottom: 1rem;
                    color: #212529;
                }

                .table-bordered {
                    border: 1px solid #dee2e6;
                }

                .table-striped tbody tr:nth-of-type(odd) {
                    background-color: rgba(0, 0, 0, 0.05);
                }

                .text-center {
                    text-align: center;
                }
            </style>
        </head>

        <body>
            <?php
            $user = session('user');
            $businessData = DB::table("businessregisters")->where('username', $user)
                ->get();
            if ($businessData) {
                foreach ($businessData as $data) {
                    $email = $data->email;
                    $mobile = $data->mobile;
                    $img = $data->pic;
                    $businessname = $data->businessname;
                    $Address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                }
            }
            ?>
            <div class="container" style="width:600px; display:inline-block;">
                <h1><?php echo $businessname ?></h1>
                <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                <h6>Address:<?php echo $Address ?></h6>
            </div>
            <div class="container" style="width:250px; display:inline-block; margin-left:170px;">
                <?php
                if (file_exists(public_path('assets/uploads/' . $img))) {
                    $imgPath = public_path('assets/uploads/' . $img);
                    $imgData = file_get_contents($imgPath);
                    $imgBase64 = base64_encode($imgData);
                } else {
                    $imgPath = public_path('assets/images/logoimage.jpeg');
                    $imgData = file_get_contents($imgPath);
                    $imgBase64 = base64_encode($imgData);
                }
                ?>
                <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
            </div>
            <hr>
            <table>
                <tr>
                    <h4>
                        <center>Old Registrations Report</center>
                    </h4>
                </tr>
            </table>
            <?php
            $totalregis = DB::table('studentregisters')->get()->count('id');
            $RunningReg = DB::table('studentregisters')->where('status', 1)->get()->count('id');
            $OldReg = DB::table('olddataregisters')->where('status', 0)->get()->count('id');
            $VacanttReg = DB::table('roomseats')->get()->sum('numberseats');
            $totalVacant = $VacanttReg - $RunningReg;
            ?>
            <table class="table table-striped text-center table-bordered mb-4 roomSeatTable" id="example">
                <thead class="align-middle">
                    <tr>
                        <th>Total Registration</th>
                        <th>Running Registration</th>
                        <th>Old Registration</th>
                        <th>Vacant Registration</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td><?php echo $totalregis; ?></td>
                        <td><?php echo $RunningReg; ?></td>
                        <td><?php echo $OldReg; ?></td>
                        <td><?php echo $totalVacant; ?></td>
                    </tr>
                </tbody>
            </table>
            <table class="table table-striped text-center table-bordered mb-4 roomSeatTable" id="example">
                <thead class="align-middle">
                    <tr>
                        <th>S.No</th>
                        <th>Room NO</th>
                        <th>Reg No</th>
                        <th>Reg Date</th>
                        <th>Leaving Date</th>
                        <th>Candidate</th>
                        <th>Mobile No</th>
                        <th>Tenure</th>
                        <th>Seat No</th>
                        <th>Rent</th>
                        <th>Fee Status</th>
                    </tr>
                </thead>
                <tbody class="align-middle">
                    <?php
                    $data = DB::table('olddataregisters')->get();
                    $count = 1;
                    foreach ($data as $temp) {
                    ?>
                        <tr>
                            <td><?php echo $count++; ?></td>
                            <td><?php echo $temp->roomno; ?></td>
                            <td><?php echo $temp->formno; ?></td>
                            <td><?php echo date('d-M-Y', strtotime($temp->date)); ?></td>
                            <td><?php echo date('d-M-Y', strtotime($temp->leave_date)); ?></td>
                            <td><?php echo  ucfirst(trans($temp->candidatename)); ?></td>
                            <td><?php echo $temp->phone; ?></td>
                            <td>
                                <?php
                                $startDate = \Carbon\Carbon::parse($temp->date);
                                $endDate = \Carbon\Carbon::parse($temp->leave_date);
                                $daysCount = $endDate->diffInDays($startDate);
                                echo $daysCount;
                                ?>
                            </td>
                            <td><?php echo $temp->seetno; ?></td>
                            <?php
                            $rent = DB::table('roomseats')->where('roomno', $temp->roomno)->first();
                            ?>
                            <td>
                                <?php if (!empty($rent)) {
                                    echo $rent->rent;
                                } ?>
                            </td>
                            <td class="fw-bold" style="color:#00ff00">
                                <?php
                                $studentId = $temp->formno;
                                $StudenRemaningAmount = DB::table("sales")
                                    ->select(DB::raw('SUM(sales.remaining_amount) as totalAmount'))
                                    ->where('studentid', $studentId)
                                    ->get();
                                foreach ($StudenRemaningAmount as $sale) {
                                    echo number_format(floatval($sale->totalAmount), 2);
                                }
                                ?>
                            </td>
                        </tr>
                    <?php
                    }
                    ?>
                </tbody>
            </table>
        </body>

        </html>
    <?php
        $html = ob_get_clean(); // Get the buffered content and clear the buffer
        $dompdf = new Dompdf(); // Create a new Dompdf instance
        $dompdf->loadHtml($html); // Load HTML content
        $dompdf->setPaper('A3', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        $filename = 'OldRegistrations.pdf'; // Generate a unique filename for the PDF
        $dompdf->stream($filename, ['Attachment' => true]); // Save the PDF to the public folder and stream it for download
    }
    //////////////////////////////////////download Registration Panel reports
    public function downloadRegistrationPanel()
    {
        // Path to the Dompdf library
        $dompdfPath = public_path('assets/dompdf');
        // Add the Dompdf library to the include path
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
    ?>
        <html>

        <head>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
            <style>
                h6 {
                    font-size: 0.6rem;
                    font-weight: 500;
                    margin: 0;
                }

                h5 {
                    font-size: 0.8rem;
                    font-weight: 500;
                    margin: 0;
                }

                .table {
                    width: 100%;
                    margin-bottom: 1rem;
                    color: #212529;
                }

                .table-bordered {
                    border: 1px solid #dee2e6;
                }

                .table-striped tbody tr:nth-of-type(odd) {
                    background-color: rgba(0, 0, 0, 0.05);
                }

                .text-center {
                    text-align: center;
                }
            </style>
        </head>

        <body>
            <?php
            $user = session('user');
            $businessData = DB::table("businessregisters")->where('username', $user)
                ->get();
            if ($businessData) {
                foreach ($businessData as $data) {
                    $email = $data->email;
                    $mobile = $data->mobile;
                    $img = $data->pic;
                    $businessname = $data->businessname;
                    $Address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                }
            }
            ?>
            <div class="container" style="width:600px; display:inline-block;">
                <h1><?php echo $businessname ?></h1>
                <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                <h6>Address:<?php echo $Address ?></h6>
            </div>
            <div class="container" style="width:250px; display:inline-block; margin-left:170px;">
                <?php
                if (file_exists(public_path('assets/uploads/' . $img))) {
                    $imgPath = public_path('assets/uploads/' . $img);
                    $imgData = file_get_contents($imgPath);
                    $imgBase64 = base64_encode($imgData);
                } else {
                    $imgPath = public_path('assets/images/logoimage.jpeg');
                    $imgData = file_get_contents($imgPath);
                    $imgBase64 = base64_encode($imgData);
                }
                ?>
                <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
            </div>
            <hr>
            <table>
                <tr>
                    <h4>
                        <center> Registration Panel Report</center>
                    </h4>
                </tr>
            </table>
            <?php
            $totalregis = DB::table('studentregisters')->get()->count('id');
            $RunningReg = DB::table('studentregisters')->where('status', 1)->get()->count('id');
            $OldReg = DB::table('olddataregisters')->where('status', 0)->get()->count('id');
            $VacanttReg = DB::table('roomseats')->get()->sum('numberseats');
            $totalVacant = $VacanttReg - $RunningReg;
            ?>
            <table class="table table-striped text-center table-bordered mb-4 roomSeatTable" id="example">
                <thead class="align-middle">
                    <tr>
                        <th>Total Registration</th>
                        <th>Running Registration</th>
                        <th>Old Registration</th>
                        <th>Vacant Registration</th>
                    </tr>
                </thead>
                <tbody>
                    <tr>
                        <td><?php echo $totalregis; ?></td>
                        <td><?php echo $RunningReg; ?></td>
                        <td><?php echo $OldReg; ?></td>
                        <td><?php echo $totalVacant; ?></td>
                    </tr>
                </tbody>
            </table>
            <table class="table table-striped text-center table-bordered mb-4 roomSeatTable" id="example">
                <thead class="align-middle">
                    <tr>
                        <th>S.NO</th>
                        <th>Room NO</th>
                        <th>Room Type</th>
                        <th>Reg No.</th>
                        <th>Candidate</th>
                        <th>Mobile</th>
                        <th>Seat No.</th>
                        <th>Fee Status</th>
                        <th>Reg.Date</th>
                    </tr>
                </thead>
                <tbody class="align-middle">
                    <?php
                    $i = 1;
                    $roomsDetails = DB::table('roomseats')->get();
                    foreach ($roomsDetails as $temp) {
                        $numberSeats = $temp->numberseats;
                        echo '<tr class="align-middle">
                              <td rowspan="' . ($numberSeats + 1) . '">' . $i++ . '</td>
                              <td rowspan="' . ($numberSeats + 1) . '">' . $temp->roomno . '</td>
                              <td rowspan="' . ($numberSeats + 1) . '" class="text-capitalize">' . $temp->roomtype . '</td>';
                        for ($j = 1; $j <= $numberSeats; $j++) {
                            $studentDetails = DB::table('studentregisters')
                                ->where('roomno', $temp->roomno)
                                ->where('seetno', $j)
                                ->where('status', 1)
                                ->first(); // Use first() instead of get(
                            echo '<tr>';
                            if ($studentDetails) {
                                $studentId = $studentDetails->id;
                                $StudenRemaningAmount = DB::table("sales")
                                    ->select(DB::raw('SUM(sales.remaining_amount) as totalAmount'))
                                    ->where('studentid', $studentId)
                                    ->get();
                                foreach ($StudenRemaningAmount as $sale) {
                                    $TotalAmount = $sale->totalAmount;
                                };
                                echo '<td>' . $studentDetails->formno . '</td>
                                      <td class="text-capitalize">' . $studentDetails->candidatename . '</td>
                                      <td>' . $studentDetails->phone . '</td>
                                      <td>' . $studentDetails->seetno . '</td>
                                      <td class="fw-bold" style="color:#00ff00" > ' . number_format(floatval($TotalAmount), 2) . '</td>
                                      <td>' . \Carbon\Carbon::createFromFormat('Y-m-d', $studentDetails->date)->format('d-M-Y') . '</td>';
                            } else {
                                echo '<td>No candidate</td>
                                      <td>-</td>
                                      <td>-</td>
                                      <td>-</td>
                                      <td>-</td>
                                      <td>-</td>';
                            }
                            echo '</tr>';
                        }
                        echo '</tr>';
                    }
                    ?>
                </tbody>
            </table>
        </body>

        </html>
    <?php
        $html = ob_get_clean(); // Get the buffered content and clear the buffer
        $dompdf = new Dompdf(); // Create a new Dompdf instance
        $dompdf->loadHtml($html); // Load HTML content
        $dompdf->setPaper('A3', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        $filename = 'RegistrationPanel.pdf'; // Generate a unique filename for the PDF
        $dompdf->stream($filename, ['Attachment' => true]); // Save the PDF to the public folder and stream it for download
    }
    // ***************************************************************************************************************
    // sales report
    private function calculateTotalAmount($startDate = null, $endDate = null)
    {
        return Sale::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('invoicedate', [$startDate, $endDate]);
        })->sum('total');
    }

    private function calculateCustomTotalAmount($startDate = null, $endDate = null)
    {
        return Customsalesale::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('tpdate', [$startDate, $endDate]);
        })->sum('total');
    }

    private function calculatePendingAmount($startDate = null, $endDate = null)
    {
        return Sale::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('invoicedate', [$startDate, $endDate]);
        })->sum('remaining_amount');
    }
    private function fetchSalesData($startDate = null, $endDate = null)
    {
        $salesData = [];
        $query = Sale::with(['student', 'receiveAmounts']);

        if ($startDate && $endDate) {
            $query->whereBetween('invoicedate', [$startDate, $endDate]);
        }

        $salesRecords = $query->get();

        foreach ($salesRecords as $row) {
            $salesData[] = [
                'invoiceno' => $row->invoiceno,
                'saleType' => 'Reg. Sale',
                'saleDate' => $row->tpdate ? Carbon::parse($row->tpdate)->format('d-M-Y') : Carbon::parse($row->invoicedate)->format('d-M-Y'),
                'studentName' => $row->student->candidatename,
                'roomNo' => $row->roomno,
                'totalAmount' => $row->total,
                'Balance' => $row->remaining_amount,
                'paymentMode' =>  optional($row->receiveAmounts->first())->paymentMode,
                'items' => $this->prepareSalesItemList($row),
            ];
        }
        return $salesData;
    }
    private function fetchCustomSalesData($startDate = null, $endDate = null)
    {
        $customSalesData = [];
        $query = Customsalesale::when($startDate && $endDate, function ($query) use ($startDate, $endDate) {
            $query->whereBetween('tpdate', [$startDate, $endDate]);
        })->get();
        foreach ($query as  $row) {
            $customSalesData[] = [
                'invoiceno' => $row->invoiceno,
                'saleType' => 'Custom Sale',
                'saleDate' => $row->tpdate ? Carbon::parse($row->tpdate)->format('d-M-Y') : Carbon::parse($row->invoicedate)->format('d-M-Y'),
                'studentName' => $row->name,
                'roomNo' => $row->roomno,
                'totalAmount' => $row->total,
                'Balance' => $row->remaining_amount,
                'paymentMode' => $row->paymentMode,
                'items' => $this->prepareSalesItemList($row),
            ];
        }
        return $customSalesData;
    }

    public function downloadAllSales(Request $request)
    {
        try {
            $bussinessData = Businessregister::firstOrFail();
            $totalSalesTxn = Sale::count();
            $totalCustomTxn = Customsalesale::count();
            $totalTxn = $totalSalesTxn + $totalCustomTxn;

            // Extract start date and end date from the request (if provided)
            $startDate =  session('startDate');
            $endDate = session('endDate');

            // Calculate total amounts based on optional date filters
            $totalAmount = $this->calculateTotalAmount($startDate, $endDate);
            $customTotalAmount = $this->calculateCustomTotalAmount($startDate, $endDate);
            $totalSale = number_format(floatval($totalAmount + $customTotalAmount), 2);
            $dueBalance = number_format(floatval($this->calculatePendingAmount($startDate, $endDate)));
            $salesData = $this->fetchSalesData($startDate, $endDate);
            $customSalesData = $this->fetchCustomSalesData($startDate, $endDate);
            $view = view('pdf.salesReport-pdf', compact('bussinessData', 'totalTxn', 'totalSale', 'dueBalance', 'salesData', 'customSalesData', 'startDate', 'endDate'))->render();
            // Create a new instance of Dompdf
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);

            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');

            // Render the HTML content to PDF
            ob_end_clean(); // Add this before $dompdf->render();

            $dompdf->render();
            // Generate a unique filename for the PDF
            $filename = 'SalesReport' . now()->format('YmdHis') . '.pdf';

            // Save the PDF to the storage path
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);

            file_put_contents($filePath, $output);
            // Generate a shareable link
            $link = asset('assets/uploads/' . $filename);
            $dompdf->stream($filename, ['Attachment' => true]);
            // Include the link in the response
            return response()->json([
                'status' => true,
                'data' => [
                    'message' => "Sales Report created successfully.",
                    'download_link' => $link,
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
    //////////////////////////////////////////////candidate form details//////////////////////////////
    public function downloadcandidateDetails()
    {
        $dompdfPath = public_path('assets/dompdf');
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
    ?>
        <html>

        <head>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
        </head>

        <body>
            <div class="container-fluid">
                <style>
                    .container-fluid {
                        margin: 0;
                    }

                    .sign {
                        display: flex;
                    }

                    .form-group label {
                        font-weight: bold;
                    }

                    .personal {
                        padding: 6px;
                        width: 300px;
                        color: #fff;
                        font-size: 1rem;
                    }

                    .headings {
                        background: #4c4747;
                        color: #fff;
                        margin: 0;
                        padding: 0;
                    }

                    .headings h4 {
                        font-size: 1rem;
                        padding: 3px 2px;
                        margin: 0;
                    }

                    .parents {
                        padding: 6px;
                        width: 300px;
                        color: #fff;
                        font-size: 1rem;
                    }

                    .bx {
                        margin: 1em auto;
                        padding: 2em 1em;
                        border: solid 1px #808080;
                        display: inline-block;
                        position: relative;
                        font-size: 18px;
                        max-width: 350px;
                    }

                    .col-form-label {
                        font-size: 0.8rem;
                    }

                    #form-container input {
                        font-size: 0.8rem;
                        margin: 4px 0;
                    }

                    #form-container input:focus {
                        box-shadow: none;
                    }

                    #form-container label {
                        font-size: 0.8rem;
                    }

                    .quoteAuthor {
                        padding-top: 20px
                    }

                    .sing-box {
                        width: 220px;
                        height: 90px;
                        position: relative;
                        background-color: #fcfffc;
                        border-radius: 20px;
                        border: 1px solid #808080;
                    }

                    .sing-box p {
                        position: absolute;
                        bottom: 0;
                        left: 20%;
                    }

                    .declaration-text {
                        font-size: 0.8rem;
                        text-transform: capitalize;
                    }

                    .form-control:focus {
                        box-shadow: none;
                    }

                    #rules {
                        height: 100vh;
                    }

                    #Terms {
                        height: 100vh;
                    }

                    .adminhostel {
                        display: inline-block;
                        font-size: 0.8rem;
                        margin: 0;
                        padding: 0;
                    }

                    .input-group {
                        display: inline-block;
                        width: 120px;
                        border: 1px solid red;
                        width: 100px;
                    }

                    td input {
                        border: none;
                    }

                    .form-label-sm::after {
                        content: ":";
                    }

                    .form-label-sm {
                        margin-right: 10px;
                        white-space: nowrap;
                    }

                    .form-control {
                        border: 1px solid #000;
                        border-radius: 5px;
                        padding: 5px 5px;
                        min-height: 20px;
                    }

                    .container {
                        margin: 0;
                        padding: 0;
                    }

                    .first {
                        border: 1px solid #000;
                        border-radius: 5px;
                        background-color: #cccccc;
                        ;
                        padding: 12px 10px;
                        margin: 0;
                    }

                    .second {
                        border: 1px solid #000;
                        border-radius: 5px;
                        background-color: #cccccc;
                        padding: 10px 10px;
                        margin: 4px 0;
                    }
                </style>
                <?php
                $path = public_path('assets/uploads');
                $publicFolder = 'public';
                $desiredPath = substr($path, strpos($path, $publicFolder) + strlen($publicFolder) + 5);
                ?>
                <?php
                $id = request()->segment(count(request()->segments()));
                $user = session('user');
                $userData = DB::table('businessregisters')->where("username", $user)->get();
                if ($userData) {
                    foreach ($userData as $data) {
                        $bussinessName = $data->businessname;
                        $bussinessemail = $data->email;
                        $mobile1 = $data->mobile;
                        $mobile2 = $data->mobile_secondary;
                        $address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                        $img = $data->pic;
                        $imgPath = asset('assets/uploads/' . $img);
                    }
                }
                $student = DB::table("studentregisters")->where('id', $id)->get();
                if ($student) {
                    foreach ($student as $info) {
                        $formNo = $info->formno;
                        $roomNo = $info->roomno;
                        $date = $info->date;
                        $name = $info->candidatename;
                        $dob = $info->dob;
                        $mobile = $info->phone;
                        $email = $info->email;
                        $addhare = $info->valid;
                        $bloodGroup = $info->blood;
                        $healthissue = $info->healthissue;
                        $stay = $info->stydrt;
                        $vehicalno = $info->vehicalno;
                        $coursename = $info->coursename;
                        $coursedetails = $info->coursedetails;
                        $jobdetails = $info->jobdetails;
                        $companyname = $info->companyname;
                        $institute = $info->institutename;
                        $candidatesign = $info->candidatesign;
                        $Parentsign = $info->Parentsign;
                    }
                    if ($coursename) {
                        if ($coursename == 'jobtype') {
                            $cjtype = $jobdetails;
                        }
                        if ($coursename == 'coursetype') {
                            $cjtype = $coursedetails;
                        }
                    }
                    if ($companyname != null) {
                        $company = $companyname;
                    } else {
                        $company = $institute;
                    }
                }
                ?>
                <div class="first">
                    <div class="adminhostel" style="width:390px;margin:0 120px 0 0;padding:0;">
                        <h4 style="margin:6px 0; padding:0"><?php echo $bussinessName; ?></h4>
                        <span>Email:
                            <?php echo $bussinessemail . " Mobile: " . $mobile1 . "," . $mobile2 . "<br> Address: " . $address; ?></span>
                    </div>
                    <div class="adminhostel" style="width:150px;margin:0">
                        <?php
                        if (file_exists(public_path('assets/uploads/' . $img))) {
                            $imgPath = public_path('assets/uploads/' . $img);
                            $imgData = file_get_contents($imgPath);
                            $imgBase64 = base64_encode($imgData);
                        } else {
                            $imgPath = public_path('assets/images/logoimage.jpeg');
                            $imgData = file_get_contents($imgPath);
                            $imgBase64 = base64_encode($imgData);
                        }
                        ?>
                        <img class="img-responsive object-fit-contain" src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:100%; height:70px;">
                    </div>
                </div>
                <div class="second">
                    <table style="margin: 0;padding:0;">
                        <tr style="margin: 0;padding:0">
                            <td> <label class="form-label-sm">Form No</label></td>
                            <td>
                                <input style="width:138px;" type="text" class="form-control" value="<?php echo $formNo; ?>" id="formNo" placeholder="" readonly>
                            </td>
                            <td> <label class="form-label-sm">Room No</label>
                            </td>
                            <td> <input style="width:138px;" type="text" class="form-control" id="RoomNo" value="<?php echo $roomNo; ?>" placeholder="" readonly>
                            </td>
                            <td><label class="form-label-sm" for="Date">Date</label>
                            </td>
                            <td>
                                <input style="width:138px;" type="text" class="form-control" id="Date" value="<?php echo  \Carbon\Carbon::createFromFormat('Y-m-d', $date)->format('d-M-Y'); ?>" placeholder="" readonly>
                            </td>
                        </tr>
                    </table>
                    <table style="margin:0;padding:0">
                        <thead>
                            <tr style="margin:0px;padding:0px">
                                <td colspan="6">
                                    <div class="headings">
                                        <h4>Personal Details</h4>
                                    </div>
                                </td>
                            </tr>
                        </thead>
                        <tbody class="align-middle">
                            <tr>
                                <td><label class="form-label-sm" for="name">Candidate Name</label> </td>
                                <td colspan="2"><input style="width:95%" class="form-control" type="text" name="" value="<?php echo $name; ?>" id="name" placeholder="Full Name" readonly></td>
                                <td rowspan="5" style="width:160px;">
                                    <?php
                                    if (file_exists(public_path('assets/uploads/' . $info->uploadimage))) {
                                        $imgPath = public_path('assets/uploads/' . $info->uploadimage);
                                        $imgData = file_get_contents($imgPath);
                                        $imgBase64 = base64_encode($imgData);
                                    ?>
                                        <img class="img-responsive object-fit-contain" src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="Parent Signature" style="width:100%; height:200px;object-fit:contain">
                                    <?php
                                    }
                                    ?>
                                </td>
                            </tr>
                            <tr>
                                <td rowspan="1"><label class="form-label-sm" for="name">Contact Detail</label></td>
                                <td rowspan="1"> <input style="width:90%" class="form-control" type="text" name="" value="<?php echo $mobile; ?>" id="mobile" placeholder="Phone" readonly>
                                </td>
                                <td rowspan="1"> <input class="form-control" type="text" id="email" value="<?php echo $email; ?>" placeholder="email" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td><label class="form-label-sm" for="name">Birth Detail</label></td>
                                <td> <input style="width:90%" class="form-control" type="text" value="<?php echo \Carbon\Carbon::createFromFormat('Y-m-d', $dob)->format('d-M-Y'); ?>" id="Birth" readonly>
                                </td>
                                <td> <input class="form-control" type="text" id="aadhar" value="<?php echo $addhare; ?>" placeholder="Adhar/VT/DL ID Proof" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td><label class="form-label-sm" for="name">Medical Detail</label></td>
                                <td> <input style="width:90%" class="form-control" type="text" name="" value="<?php echo $bloodGroup; ?>" id="blood-group" placeholder="Blood Group" readonly>
                                </td>
                                <td> <input class="form-control" type="text" id="health-issue" value="<?php echo $healthissue == "no" ? "No Health Issue" : $healthissue ?>" placeholder="Any Health issue" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td> <label class="form-label-sm" for="name">Other Detail</label></td>
                                <td> <input style="width:90%" class="form-control" type="text" name="" value="<?php echo $stay; ?>" id="stay-duration" placeholder="Stay Duration" readonly>
                                </td>
                                <td> <input class="form-control" type="text" id="vehical" value="<?php echo $vehicalno == "no" ? "No Vehical" : $vehicalno; ?>" placeholder="Vehical Name & No" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td><label class="form-label-sm" for="name">Course/Job Type</label></td>
                                <td colspan="4"> <input style="width:92%" class="form-control" type="text" value="<?php echo $cjtype; ?>" id="courseType" placeholder="" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td> <label class="form-label-sm" for="name">Institute/Company</label>
                                </td>
                                <td colspan="4"> <input style="width:92%" class="form-control" type="text" value="<?php echo $company; ?>" id="Cpy-type" placeholder="" readonly>
                                </td>
                            </tr>
                        </tbody>
                    </table>
                    <table style="margin:0;padding:0">
                        <thead>
                            <tr style="margin:0px;padding:0px">
                                <td colspan="6">
                                    <div class="headings">
                                        <h4>Parent's/Guardian Details</h4>
                                    </div>
                                </td>
                            </tr>
                        </thead>
                        <tbody>
                            <tr>
                                <td><label class="form-label-sm" for="name">Father Name</label></td>
                                <td><input class="form-control" type="text" name="" id="father-name" value="<?php echo $info->fathername; ?>" placeholder="Father Name" readonly></td>
                                <td>
                                    <label class="form-label-sm" for="name">Occupation</label>
                                </td>
                                <td colspan="2">
                                    <input class="form-control" type="text" name="" id="father-occupation" value="<?php echo $info->fatheroocc; ?>" placeholder="Father Occupation" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td> <label class="form-label-sm" for="name">Mother Name</label></td>
                                <td>
                                    <input class="form-control" type="text" name="" value="<?php echo $info->mothername; ?>" id="mother-name" placeholder="Mother Name" readonly>
                                </td>
                                <td>
                                    <label class="form-label-sm" for="">Occupation</label>
                                </td>
                                <td colspan="2">
                                    <input class="form-control" type="text" name="" id="mother-occupation" value="<?php echo $info->motheroocc; ?>" placeholder="mother Occupation" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td>
                                    <label class="form-label-sm" for="">Contact No</label>
                                </td>
                                <td>
                                    <input class="form-control" type="text" name="" value="<?php echo $info->tel; ?>" id="Tel" placeholder="Tel" readonly>
                                </td>
                                <td></td>
                                <td colspan="2">
                                    <input class="form-control" type="text" name="" value="<?php echo $info->mobilenumber; ?>" id="Mobile" placeholder="Mobile" readonly>
                                </td>
                            </tr>
                            <tr>
                                <td> <label class="form-label-sm" for="">Email Id</label></td>
                                <td colspan="4">
                                    <input style="width:94%;padding:3px;;" class="form-control" type="text" id="parent-email" value="<?php echo $info->pemail; ?>">
                                </td>
                            </tr>
                            <tr>
                                <td><label class="form-label-sm" for="">Full Address</label></td>
                                <td colspan="4"><input style="width:94%" class="form-control" value="<?php echo $info->fullAddress . "," . $info->state . "," . $info->pinno; ?>" type="text">
                                </td>
                            </tr>
                            <tr>
                                <td> <label class="form-label-sm">Guardian Name</label></td>
                                <td> <input type="text" class="form-control" value="<?php echo $info->guardianname; ?>">
                                </td>
                                <td><label class="form-label-sm" for="">Contact No</label> </td>
                                <td> <input type="text" value="<?php echo $info->guardiannumber; ?>" id="" class="form-control"> </td>
                            </tr>
                            <tr>
                                <td>
                                    <label class="form-control-sm">Address</label>
                                </td>
                                <td colspan="4">
                                    <input style="width:94%" type="text" value=" <?php echo $info->guardianaddress; ?>" class="form-control">
                                </td>
                            </tr>
                        </tbody>
                    </table>
                    <table>
                        <tr>
                            <td class="headings">
                                <h4>Disclamer</h4>
                            </td>
                        </tr>
                        <tr>
                            <td>
                                <p class="declaration-text">
                                    I DECLARE THAT THE INFORMATION GIVEN ABOVE IS TRUE TO THE BEST OF MY KNOWLEDGE.
                                    I AGREE THAT IF ANY INFORMATION FURNISHED ABOVE FOUND INCORRECT MY ADMISSION IS LIABLE
                                    TO BE CANCELLED. I HAVE READ AND ACCEPTED ALL TERMS & CONDITIONS ATTACHED WITH THIS FORM.
                                </p>
                            </td>
                        </tr>
                    </table>
                    <table>
                        <tr>
                            <td class="sing-box" style="margin-right:10px">
                                <?php
                                if ($candidatesign != null && file_exists(public_path('assets/uploads/' . $candidatesign))) {
                                    $imgPath = public_path('assets/uploads/' . $candidatesign);
                                    $imgData = file_get_contents($imgPath);
                                    $imgBase64 = base64_encode($imgData);
                                ?>
                                    <center>
                                        <img class="img-responsive object-fit-contain" src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="candidate signature" style="width:80px; height:80px;">
                                    </center>
                                <?php
                                }
                                ?>
                                <p class="text-center">Candidate Sign </p>
                            </td>
                            <td class="sing-box" style="margin-right:10px">
                                <?php

                                if ($Parentsign != null && file_exists(public_path('assets/uploads/' . $Parentsign))) {
                                    $imgPath = public_path('assets/uploads/' . $info->Parentsign);
                                    $imgData = file_get_contents($imgPath);
                                    $imgBase64 = base64_encode($imgData);
                                ?>
                                    <center>
                                        <img class="img-responsive object-fit-contain" src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="Parent Signature" style="width:80px; height:80px;">
                                    </center>
                                <?php
                                }
                                ?>

                                <p class="text-center">Parent/Guardian Sign </p>
                            </td>
                            <td class="sing-box shadow-sm">
                                <?php $documents = DB::table('uploadprofiles')->where('upload_name', 'Stamp With Signature')->first(); ?>
                                <?php
                                if (file_exists(public_path('assets/uploads/pdf/' . $documents->upload_pdf))) {
                                    $imgPath = public_path('assets/uploads/pdf/' . $documents->upload_pdf);
                                    $imgData = file_get_contents($imgPath);
                                    $imgBase64 = base64_encode($imgData);
                                ?>
                                    <center><img class="img-responsive object-fit-contain" src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="Parent Signature" style="width:80px; height:80px;">
                                    </center>
                                <?php
                                }
                                ?>
                                <p class="text-center">Rector/Warden & Sign </p>
                            </td>
                        </tr>
                    </table>
                </div>
            </div>
        </body>

        </html>
    <?php
        $documents = DB::table('uploadprofiles')->where('upload_name', 'Rules & Regulations')->first();
        $html = ob_get_clean();
        $dompdf = new Dompdf();
        $dompdf->loadHtml($html);
        $dompdf->setPaper('A4', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        $dompdf->set_option('isRemoteEnabled', false);
        $filename = implode("_", explode(" ", $name)) . '.pdf'; // Generate a unique filename for the PDF
        $dompdf->stream($filename, ['Attachment' => true]); // Save the PDF to the public folder and stream it for download
    }
    public function DownLoadTermsCondition()
    {
        $documents = DB::table('uploadprofiles')->where('upload_name', 'Rules & Regulations')->first();
        $existingPdfPath = public_path('assets/uploads/pdf/' . $documents->upload_pdf);
        if (file_exists($existingPdfPath)) {
            return Response::download($existingPdfPath, 'TermsAndConditions.pdf', [
                'Content-Type' => 'application/pdf',
                'Content-Disposition' => 'attachment; filename="TermsAndConditions.pdf"',
            ]);
        } else {
            return back()->with("error", "You have not uploded terms and conditions");
        }
    }
    public function CandidateDocument()
    {
        // Path to the Dompdf library
        $dompdfPath = public_path('assets/dompdf');
        // Add the Dompdf library to the include path
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
    ?>
        <html>

        <head>
            <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
            <style>
                .item-category {
                    font-size: 1rem;
                    font-weight: 500;
                }

                .items {
                    height: 80vh;
                    overflow: auto;
                }

                #expensesReport thead {
                    font-size: 0.9rem;
                }

                #expensesReport tbody {
                    font-size: 0.8rem;
                }

                .totalAmount {
                    font-size: 1rem;
                    font-weight: 600;
                }

                #SalesReportsPdf h6 {
                    font-size: 0.6rem;
                    font-weight: 500;
                    margin: 0;
                }

                #SalesReportsPdf h5 {
                    font-size: 0.8rem;
                    font-weight: 500;
                    margin: 0;
                }

                .table {
                    width: 100%;
                    margin-bottom: 1rem;
                    color: #212529;
                }

                .table-bordered {
                    border: 1px solid #dee2e6;
                }

                .table-striped tbody tr:nth-of-type(odd) {
                    background-color: rgba(0, 0, 0, 0.05);
                }

                .text-center {
                    text-align: center;
                }

                .card {
                    display: inline-block;
                    width: 100px;
                    height: auto;
                    margin-bottom: 10px;
                    border: 1px solid #212529;
                    padding: 5px;
                    text-align: center;
                    border-radius: 5px;
                    margin-right: 30px;
                }

                .text-capitalize {
                    text-transform: capitalize;
                }

                .table-light thead {
                    background-color: #212529;
                    color: #fff;
                }
            </style>
        </head>

        <body>
            <div class="container-fluid" id="SalesReportsPdf">
                <?php
                $user = session('user');
                $businessData = DB::table("businessregisters")->where('username', $user)
                    ->get();
                if ($businessData) {
                    foreach ($businessData as $data) {
                        $email = $data->email;
                        $mobile = $data->mobile;
                        $img = $data->pic;
                        $businessname = $data->businessname;
                        $Address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                    }
                }
                ?>
                <div class="container" style="width:600px; display:inline-block;">
                    <h1><?php echo $businessname ?></h1>
                    <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                    <h6>Address:<?php echo $Address ?></h6>
                </div>
                <div class="container" style="width:250px; display:inline-block; margin-left:170px;">
                    <?php
                    if (file_exists(public_path('assets/uploads/' . $img))) {
                        $imgPath = public_path('assets/uploads/' . $img);
                        $imgData = file_get_contents($imgPath);
                        $imgBase64 = base64_encode($imgData);
                    } else {
                        $imgPath = public_path('assets/images/logoimage.jpeg');
                        $imgData = file_get_contents($imgPath);
                        $imgBase64 = base64_encode($imgData);
                    }
                    ?>
                    <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
                </div>
                <hr>
                <div class="container">
                    <?php
                    $id = request()->segment(count(request()->segments()));
                    $documents = DB::table("studentregisters")->where("id", $id)->select('uploadaadharfront', 'uploadaadharback', 'formno', 'roomno', 'seetno', 'candidatename')->first();
                    $filename = $documents->formno . "-" . $documents->roomno . "-" . $documents->seetno . "-" . implode("_", explode(" ", $documents->candidatename));
                    if ($documents) {
                        $aadharFront = $documents->uploadaadharfront;
                        $aadharBack = $documents->uploadaadharback;
                        $imageDir = public_path('assets/uploads/');
                        $aadharfrountPath = public_path('assets/uploads/' . $aadharFront);
                        $aadharBackPath = public_path('assets/uploads/' . $aadharBack);
                        if (file_exists($aadharfrountPath) && file_exists($aadharBackPath)) {
                            $FruntimgData = file_get_contents($aadharfrountPath);
                            $aadharF = base64_encode($FruntimgData);
                            $BackimgData = file_get_contents($aadharBackPath);
                            $aadharB = base64_encode($BackimgData);
                    ?>
                            <div class="container m-2" style="margin:10px">
                                <img style="width:300px; height:300px; margin:20px" src="data:image/png;base64,<?php echo $aadharF; ?>" alt="Img is not uploaded">
                            </div>
                            <div class="container m-2" style="margin:10px">
                                <img style="width:300px; height:300px; margin:20px" src="data:image/png;base64,<?php echo $aadharB; ?>" alt="Img is not uploaded">
                            </div>
                        <?php
                        } else {
                        ?>
                            <p>Document Not Found</p>
                        <?php
                        }
                        ?>

                    <?php
                    }
                    ?>
                </div>
            </div>
        </body>
        </html>
        <?php
        $html = ob_get_clean();
        $dompdf = new Dompdf(); // Create a new Dompdf instance
        $dompdf->loadHtml($html); // Load HTML content
        $dompdf->setPaper('A3', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        $dompdf->set_option('isRemoteEnabled', false);
        $dompdf->stream($filename, ['Attachment' => true]); // Save the PDF to the public folder and stream it for download
    }
    public function shareAllExpenses()
    {
        $dompdfPath = public_path('assets/dompdf');
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
        ?>
            <html>

            <head>
                <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
                <style>
                    h6 {
                        font-size: 0.6rem;
                        font-weight: 500;
                        margin: 0;
                    }

                    h5 {
                        font-size: 0.8rem;
                        font-weight: 500;
                        margin: 0;
                    }

                    .table {
                        width: 100%;
                        margin-bottom: 1rem;
                        color: #212529;
                    }

                    .table-bordered {
                        border: 1px solid #dee2e6;
                    }

                    .table-striped tbody tr:nth-of-type(odd) {
                        background-color: rgba(0, 0, 0, 0.05);
                    }

                    .text-center {
                        text-align: center;
                    }

                    .item-category {
                        font-size: 1rem;
                        font-weight: 500;
                    }

                    .items {
                        height: 80vh;
                        overflow: auto;
                    }

                    #expensesReport thead {
                        font-size: 0.9rem;
                        background-color: #212529;
                        color: #fff;
                    }

                    #expensesReport tbody {
                        font-size: 0.8rem;
                    }

                    .totalAmount {
                        font-size: 1rem;
                        font-weight: 600;
                    }

                    .text-warning {
                        color: "orange!important";
                    }
                </style>
            </head>

            <body>
                <?php
                $user = session('user');
                $businessData = DB::table("businessregisters")->where('username', $user)
                    ->get();
                if ($businessData) {
                    foreach ($businessData as $data) {
                        $email = $data->email;
                        $mobile = $data->mobile;
                        $img = $data->pic;
                        $businessname = $data->businessname;
                        $Address = $data->address . "," . $data->city . "," . $data->state . "," . $data->pincode;
                    }
                }
                ?>
                <div class="container" style="width:400px; display:inline-block;">
                    <h1><?php echo $businessname ?></h1>
                    <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                    <h6>Address:<?php echo $Address ?></h6>
                </div>
                <div class="container" style="width:250px; display:inline-block; margin-left:30px;">
                    <?php
                    $imgPath = public_path('assets/uploads/' . $img);
                    if (file_exists($imgPath)) {
                        $imgData = file_get_contents($imgPath);
                        $imgBase64 = base64_encode($imgData);
                    } else {
                        $imgPath = public_path('assets/images/logoimage.jpeg');
                        $imgData = file_get_contents($imgPath);
                        $imgBase64 = base64_encode($imgData);
                    }
                    ?>
                    <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
                </div>
                <hr>
                <table>
                    <tr>
                        <h4>
                            <center>Expenses Report</center>
                        </h4>
                    </tr>
                </table>
                <div class="container">
                    <div class="items border-bottom">
                        <?php
                        $expenses = DB::table('expenses')->get();
                        $count = 1;
                        if ($expenses->count() > 0) {
                            $startDate =  session('startDate');
                            $endDate = session('endDate');
                            foreach ($expenses as $expense) {
                                if ($startDate != '' && $endDate != '') {
                                    $expense_details = DB::table('expensedetails')
                                        ->where('category_id', '=', $expense->id)
                                        ->whereBetween('billdate', [$startDate, $endDate])
                                        ->get();
                                } else {
                                    $expense_details = DB::table('expensedetails')
                                        ->where('category_id', '=', $expense->id)
                                        ->get();
                                }
                                $totalExpenses = 0;
                                if ($expense_details->count() > 0) {
                                    foreach ($expense_details as $item) {
                                        $totalExpenses = $totalExpenses + $item->amount;
                                    }
                        ?>
                                    <h4 style="background-color:#d9d9d9" class="text-black p-1 item-category">Category Name: <span class="text-warning text-capitalize fw-bold"><?= $expense->category_name ?></span> - Rs.
                                        <span class="text-warning text-capitalize fw-bold ms-1"><i class="fa-solid fa-indian-rupee-sign ms-1"></i><?= $totalExpenses ?></span>
                                    </h4>
                                    <div class="table-responsive">
                                        <table border="1" id="expensesReport" class="table text-center table-light">
                                            <thead class="align-middle">
                                                <tr>
                                                    <th>S No</th>
                                                    <th>Date</th>
                                                    <th>Item Name</th>
                                                    <th>Quantity</th>
                                                    <th>Rate</th>
                                                    <th>Amount</th>
                                                    <th>Payment Type</th>
                                                </tr>
                                            </thead>
                                            <tbody class="align-middle">
                                                <?php
                                                foreach ($expense_details as $item) {
                                                ?>
                                                    <tr>
                                                        <td><?= $count ?></td>
                                                        <td><?= \Carbon\Carbon::parse($item->billdate)->format('d-M-Y') ?></td>
                                                        <td class="text-capitalize"><?= $item->itemname ?></td>
                                                        <td><?= $item->quantity ?></td>
                                                        <td><i class="fa-solid fa-indian-rupee-sign me-1"></i><?= $item->rate ?></td>
                                                        <td><i class="fa-solid fa-indian-rupee-sign me-1"></i><?= $item->amount ?></td>
                                                        <td class="text-capitalize"><?= $item->paymentmode ?></td>
                                                    </tr>
                                                <?php
                                                    $count++;
                                                }
                                                $count = 1;
                                                ?>
                                            </tbody>
                                        </table>
                                    </div>
                        <?php
                                }
                            }
                        } else {
                            echo "<p>Data not Found</p>";
                        }
                        ?>
                    </div>
                </div>
                <div class="container">
                    <?php
                    if ($startDate != '' && $endDate != '') {
                        $totalAmount = DB::table('expensedetails')->whereBetween('billdate', [$startDate, $endDate])
                            ->sum('amount');
                    } else {
                        $totalAmount = DB::table('expensedetails')
                            ->sum('amount');
                    }
                    //session()->forget('startDate');
                    //session()->forget('endDate');
                    ?>
                    <h4 class="totalAmount bg-white p-1">Total Amount <span class="text-warning"><i class="fa-solid fa-indian-rupee-sign mx-1"></i> <?= $totalAmount ?></span>
                    </h4>
                </div>
            </body>

            </html>
        <?php
        $html = ob_get_clean(); // Get the buffered content and clear the buffer
        $dompdf = new Dompdf(); // Create a new Dompdf instance
        $dompdf->loadHtml($html); // Load HTML content
        $dompdf->setPaper('A4', 'portrait'); // (Optional) Set paper size and orientation
        $dompdf->render(); // Render the HTML content to PDF
        // Generate a random file name
        $name = 'table_data_' . uniqid() . '.pdf';
        $filename = public_path('share/' . $name);
        // Save the PDF file locally
        $output = $dompdf->output();
        file_put_contents($filename, $output);
        //echo  $fileUrl = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'] . '?file=' . $name . '&action=download';
        $fileUrl =  asset('/share/' . $name);
        // Construct the WhatsApp share link with the file URL
        $whatsappLink = 'whatsapp://send?text=' . urlencode("👋 Hi Dear,\r\n \r\nYour Expenses Report Link is here! 📊🔗 Please click on the link below to download 👇\r\n\r\n$fileUrl\r\n\r\nRegards, \r\n$businessname");
        header("Location: $whatsappLink");
        //$dompdf->stream($filename, ['Attachment' => true]); // Save the PDF to the public folder and stream it for download
    }
    //////////////////////////////////share sale report on whatsapp
    function shareAllSales(Request $request)
    {
        // Path to the Dompdf library
        $dompdfPath = public_path('assets/dompdf');
        // Add the Dompdf library to the include path
        set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
        require_once $dompdfPath . '/autoload.inc.php';
        ob_start(); // Start output buffering
        ?>
            <html>

            <head>
                <link href="https://cdn.jsdelivr.net/npm/bootstrap@5.0.2/dist/css/bootstrap.min.css" rel="stylesheet" integrity="sha384-EVSTQN3/azprG1Anm3QDgpJLIm9Nao0Yz1ztcQTwFspd3yD65VohhpuuCOmLASjC" crossorigin="anonymous">
                <style>
                    .item-category {
                        font-size: 1rem;
                        font-weight: 500;
                    }

                    .items {
                        height: 80vh;
                        overflow: auto;
                    }

                    #expensesReport thead {
                        font-size: 0.9rem;
                    }

                    #expensesReport tbody {
                        font-size: 0.8rem;
                    }

                    .totalAmount {
                        font-size: 1rem;
                        font-weight: 600;
                    }

                    #SalesReportsPdf h6 {
                        font-size: 0.6rem;
                        font-weight: 500;
                        margin: 0;
                    }

                    #SalesReportsPdf h5 {
                        font-size: 0.8rem;
                        font-weight: 500;
                        margin: 0;
                    }

                    .table {
                        width: 100%;
                        margin-bottom: 1rem;
                        color: #212529;
                    }

                    .table-bordered {
                        border: 1px solid #dee2e6;
                    }

                    .table-striped tbody tr:nth-of-type(odd) {
                        background-color: rgba(0, 0, 0, 0.05);
                    }

                    .text-center {
                        text-align: center;
                    }

                    .card {
                        display: inline-block;
                        width: 100px;
                        height: auto;
                        margin-bottom: 10px;
                        border: 1px solid #212529;
                        padding: 5px;
                        text-align: center;
                        border-radius: 5px;
                        margin-right: 30px;
                    }

                    .text-capitalize {
                        text-transform: capitalize;
                    }

                    .table-light thead {
                        background-color: #212529;
                        color: #fff;
                    }
                </style>
            </head>

            <body>
                <div class="container-fluid" id="SalesReportsPdf">
                    <?php
                    $user = session('user');
                    $businessData = DB::table("businessregisters")->where('username', $user)
                        ->get();
                    if ($businessData) {
                        foreach ($businessData as $data) {
                            $email = $data->email;
                            $mobile = $data->mobile;
                            $img = $data->pic;
                            $businessname = $data->businessname;
                            $Address = $data->address . "," . $data->state . "," . $data->pincode;
                        }
                    }
                    ?>
                    <div class="container" style="width:600px; display:inline-block;">
                        <h1><?php echo $businessname ?></h1>
                        <h6><span>Email:<?php echo $email ?></span><span> Mobile:<?php echo $mobile ?></span></h6>
                        <h6>Address:<?php echo $Address ?></h6>
                    </div>
                    <div class="container" style="width:250px; display:inline-block; margin-left:170px;">
                        <?php
                        $imgPath = public_path('assets/uploads/' . $img);
                        if (file_exists($imgPath)) {
                            $imgData = file_get_contents($imgPath);
                            $imgBase64 = base64_encode($imgData);
                        } else {
                            $imgPath = public_path('assets/images/logoimage.jpeg');
                            $imgData = file_get_contents($imgPath);
                            $imgBase64 = base64_encode($imgData);
                        }
                        ?>
                        <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="" style="width:250px; height:100px;">
                    </div>
                    <hr>
                    <div class="container">
                        <div class="row mt-4 mb-3" style="margin-top:5px">
                            <div class="col-lg-5">
                                <div class="d-flex justify-content-between align-item center">
                                    <?php
                                    $totalRows = DB::table("sales")->count();
                                    $customRow = DB::table("customsalesales")->count();
                                    ?>
                                    <div class="card shadow-sm">
                                        <div class="card-body py-1 ">
                                            <h6 class="">No of Txn</h6>
                                            <h5><?php echo $totalRows +  $customRow ?></h5>
                                        </div>
                                    </div>
                                    <div class="card shadow-sm ">
                                        <?php
                                        $startDate =  session('startDate');
                                        $endDate = session('endDate');
                                        if ($startDate != "" && $endDate != '') {
                                            $totalAmount = DB::table('sales')
                                                ->select(DB::raw('SUM(sales.total) as totalAmount'))
                                                ->whereBetween('sales.invoicedate', [$startDate, $endDate])
                                                ->get();
                                            $customeTotalAmount = DB::table("customsalesales")
                                                ->select(DB::raw('SUM(customsalesales.total) as totalAmount'), (DB::raw('SUM(customsalesales.remaining_amount) as remainingAmount')))
                                                ->whereBetween('customsalesales.tpdate', [$startDate, $endDate])
                                                ->get();
                                            $pandingAmount = DB::table('sales')
                                                ->select(DB::raw('SUM(sales.remaining_amount) as totalAmount'))
                                                ->whereBetween('sales.invoicedate', [$startDate, $endDate])
                                                ->get();
                                            Session::put('startDate', $startDate);
                                            Session::put('endDate', $endDate);
                                        } else {
                                            $totalAmount = DB::table('sales')
                                                ->select(DB::raw('SUM(sales.total) as totalAmount'))
                                                ->get();
                                            $customeTotalAmount = DB::table("customsalesales")
                                                ->select(DB::raw('SUM(customsalesales.total) as totalAmount'), (DB::raw('SUM(customsalesales.remaining_amount) as remainingAmount')))
                                                ->get();
                                            $pandingAmount = DB::table('sales')
                                                ->select(DB::raw('SUM(sales.remaining_amount) as totalAmount'))
                                                ->get();
                                        }
                                        if ($totalAmount && $customeTotalAmount) {
                                            foreach ($totalAmount as $sale) {
                                                $SaleTotal = $sale->totalAmount;
                                            }
                                            foreach ($customeTotalAmount as $custom) {
                                                $CustomeTotal = $custom->totalAmount;
                                            }
                                            $TotalAmount = $SaleTotal + $CustomeTotal;
                                        } else if ($totalAmount) {
                                            foreach ($totalAmount as $sale) {
                                                $TotalAmount = $sale->totalAmount;
                                            }
                                        } else if ($customeTotalAmount) {
                                            foreach ($customeTotalAmount as $custom) {
                                                $TotalAmount = $custom->totalAmount;
                                            }
                                        }
                                        ?>
                                        <div class="card-body py-1">
                                            <h6>Total Sale</h6>
                                            <h5> <?php echo $TotalAmount ?></h5>
                                        </div>
                                    </div>
                                    <div class="card shadow-sm">
                                        <?php
                                        if ($pandingAmount && $customeTotalAmount) {
                                            foreach ($pandingAmount as $sale) {
                                                $pandingAmount = $sale->totalAmount;
                                            }
                                            foreach ($customeTotalAmount as $custom) {
                                                $totalCustomPanding = $custom->remainingAmount;
                                            }
                                            $TotalpandingAmount = $pandingAmount + $totalCustomPanding;
                                        }
                                        ?>
                                        <div class="card-body py-1">
                                            <h6>Due Balance</h6>
                                            <h5><?php echo $TotalpandingAmount ?></h5>
                                        </div>
                                    </div>
                                </div>
                            </div>
                        </div>
                    </div>
                    <div class="container">
                        <div class="items border-bottom">
                            <div class="table-responsive">
                                <table border="1" id="expensesReport" class="table text-center table-light table-bordered">
                                    <thead class="align-middle">
                                        <th>S No</th>
                                        <th>Invoice No</th>
                                        <th>Sale Type</th>
                                        <th>Date</th>
                                        <th>Name</th>
                                        <th>RoomNo</th>
                                        <th>Total Amount</th>
                                        <th>Balance</th>
                                        <th>Payment Mode</th>
                                        <th>Item</th>
                                        <th>Quantity</th>
                                        <th>Amount</th>
                                    </thead>
                                    <tbody class="align-middle">
                                        <?php
                                        if ($startDate != "" && $endDate != '') {
                                            $data = DB::table('sales')
                                                ->join('studentregisters', 'sales.studentid', '=', 'studentregisters.id')
                                                ->whereBetween('sales.invoicedate', [$startDate, $endDate])
                                                ->select('sales.*', 'studentregisters.*')
                                                ->get();
                                            $customeData = DB::table('customsalesales')->whereBetween('tpdate', [$startDate, $endDate])
                                                ->get();
                                        } else {
                                            $data = DB::table('sales')
                                                ->join('studentregisters', 'sales.studentid', '=', 'studentregisters.id')
                                                ->select('sales.*', 'studentregisters.*')
                                                ->get();
                                            $customeData = DB::table('customsalesales')
                                                ->get();
                                        }
                                        //session()->forget('startDate');
                                        //session()->forget('endDate');
                                        $count = 1;
                                        foreach ($data as $items) {
                                            if ($items->tpdate != null) {
                                                $PaymentMode = DB::table('recieve_amounts')->where('receiveDate', $items->tpdate)
                                                    ->where('candidateID', $items->id)
                                                    ->select('paymentMode')
                                                    ->orderBy('id', 'DESC')
                                                    ->first();
                                                if ($PaymentMode) {
                                                    $mode = $PaymentMode->paymentMode;
                                                } else {
                                                    $mode = "";
                                                }
                                            }
                                            $itemID = explode(',', $items->item);
                                            $itemQuantity = explode(',', $items->quantity);
                                            $itemAmount = explode(',', $items->amount);
                                            $itemArraySize = sizeof($itemID);
                                        ?>
                                            <tr>
                                                <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $count  ?></td>
                                                <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $items->invoiceno  ?>
                                                </td>
                                                <td rowspan="<?php echo $itemArraySize ?>">Reg. Sale</td>
                                                <td rowspan="<?php echo $itemArraySize ?>">
                                                    <?php echo \Carbon\Carbon::parse($items->invoicedate)->format('d-M-Y')   ?>
                                                </td>
                                                <td rowspan="<?php echo $itemArraySize ?>" class="text-capitalize">
                                                    <?php echo  $items->candidatename  ?></td>
                                                <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $items->roomno  ?></td>
                                                <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $items->total  ?></td>
                                                <td rowspan="<?php echo $itemArraySize ?>"><span>
                                                        <?php echo $items->remaining_amount  ?></span></td>
                                                <td rowspan="<?php echo $itemArraySize ?>" class="text-capitalize">
                                                    <?php echo $mode ?></td>
                                                <?php
                                                for ($i = 0; $i < $itemArraySize; $i++) {
                                                    if ($itemID[$i] == '0') {
                                                        $itemName = "Room Rent";
                                                    } elseif ($itemID[$i] == "e01" || $itemID[$i] == "E01") {
                                                        $electricityRecords = DB::table("students_electricity_bill")->where("invoiceno", $items->invoiceno)->first();
                                                        $itemName = "Electricity Bill( " . \Carbon\Carbon::parse($electricityRecords->billDate)->format("d-M-Y") . " )";
                                                    } else {
                                                        $getItemName = DB::table('items')
                                                            ->where('id', $itemID[$i])
                                                            ->select('itemname')
                                                            ->first();
                                                        if ($getItemName) {
                                                            $itemName = $getItemName->itemname;
                                                        } else {
                                                            $itemName = $itemID[$i];
                                                        }
                                                    }
                                                    if ($i == 0) {
                                                ?>
                                                        <td class="text-capitalize"><?php echo $itemName ?></td>
                                                        <td><?php echo $itemQuantity[$i] ?></td>
                                                        <td><span> <?php echo $itemAmount[$i] ?></span></td>
                                            </tr>
                                        <?php
                                                    } else {
                                        ?>
                                            </tr>
                                            <tr>
                                                <td class="text-capitalize"><?php echo $itemName; ?></td>
                                                <td><?php echo $itemQuantity[$i] ?></td>
                                                <td><span> <?php echo $itemAmount[$i] ?></span></td>
                                            </tr>
                                        <?php
                                                    }
                                        ?>
                                <?php
                                                }
                                                $count++;
                                            }
                                ?>
                                <?php
                                if ($customeData) {
                                    foreach ($customeData as $items) {
                                        $itemID = explode(',', $items->item);
                                        $itemQuantity = explode(',', $items->quantity);
                                        $itemAmount = explode(',', $items->amount);
                                        $itemArraySize = sizeof($itemID);
                                        $invoiceNo = $items->invoiceno;
                                        $invoiceDate = $items->tpdate;
                                        $name = $items->name;
                                        $roomno = $items->roomno;
                                        $totalAmount = $items->total;
                                        $PaymentMode = $items->paymentMode;
                                        $remaningAmount = $items->remaining_amount;
                                ?>
                                        <tr>
                                            <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $count   ?></td>
                                            <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $invoiceNo  ?></td>
                                            <td rowspan="<?php echo $itemArraySize ?>">Custom Sale</td>
                                            <td rowspan="<?php echo $itemArraySize ?>">
                                                <?php echo \Carbon\Carbon::parse($invoiceDate)->format('d-M-Y')  ?></td>
                                            <td rowspan="<?php echo $itemArraySize ?>" class="text-capitalize">
                                                <?php echo  $name   ?></td ?>
                                            <td rowspan="<?php echo $itemArraySize ?>"><?php echo  $roomno   ?></td>
                                            <td rowspan="<?php echo $itemArraySize ?>"><i class="fa-solid fa-indian-rupee-sign me-1"></i><?php echo  $totalAmount   ?>
                                            </td>
                                            <td rowspan="<?php echo $itemArraySize ?>"><span><i class="fa-solid fa-indian-rupee-sign me-1"></i><?php echo  $remaningAmount   ?></span>
                                            </td>
                                            <td rowspan="<?php echo $itemArraySize ?>" class="text-capitalize">
                                                <?php echo $PaymentMode ?></td>
                                            <?php
                                            for ($i = 0; $i < $itemArraySize; $i++) {
                                                if ($itemID[$i] == '0') {
                                                    $itemName = "Room Rent";
                                                } elseif ($itemID[$i] == "e01" || $itemID[$i] == "E01") {
                                                    $electricityRecords = DB::table("students_electricity_bill")->where("invoiceno", $items->invoiceno)->first();
                                                    $itemName = "Electricity Bill( " . \Carbon\Carbon::parse($electricityRecords->billDate)->format("d-M-Y") . " )";
                                                } else {
                                                    $getItemName = DB::table('items')
                                                        ->where('id', $itemID[$i])
                                                        ->select('itemname')
                                                        ->first();
                                                    if ($getItemName) {
                                                        $itemName = $getItemName->itemname;
                                                    } else {
                                                        $itemName = $itemID[$i];
                                                    }
                                                }
                                                if ($i == 0) {
                                            ?>
                                                    <td class="text-capitalize"><?php echo $itemName ?></td>
                                                    <td><?php echo $itemQuantity[$i] ?></td>
                                                    <td><span><i class="fa-solid fa-indian-rupee-sign me-1"></i><?php echo $itemAmount[$i] ?></span>
                                                    </td>
                                        </tr>
                                    <?php
                                                } else {
                                    ?> </tr>
                                        <tr>
                                            <td class="text-capitalize"><?php echo $itemName ?></td>
                                            <td><?php echo $itemQuantity[$i] ?></td>
                                            <td><span><i class="fa-solid fa-indian-rupee-sign me-1"></i><?php echo $itemAmount[$i] ?></span>
                                            </td>
                                        </tr>
                                    <?php
                                                }
                                    ?>
                        <?php
                                            }
                                            $count++;
                                        }
                                    }
                        ?>
                                    </tbody>
                                </table>
                            </div>
                        </div>
                    </div>
                </div>
            <?php
            $html = ob_get_clean();
            $dompdf = new Dompdf(); // Create a new Dompdf instance
            $dompdf->loadHtml($html); // Load HTML content
            $dompdf->setPaper('A3', 'portrait'); // (Optional) Set paper size and orientation
            $dompdf->render(); // Render the HTML content to PDF
            // Generate a random file name
            $name = 'saleReport_' . uniqid() . '.pdf';
            $filename = public_path('share/' . $name);
            // Save the PDF file locally
            $output = $dompdf->output();
            file_put_contents($filename, $output);
            //echo  $fileUrl = 'http://' . $_SERVER['HTTP_HOST'] . $_SERVER['PHP_SELF'] . '?file=' . $name . '&action=download';
            $fileUrl =  asset('/share/' . $name);
            // Construct the WhatsApp share link with the file URL
            $whatsappLink = 'whatsapp://send?text=' . urlencode("👋 Hi Dear,\r\n \r\nYour Sales Report Link is here! 📊🔗 Please click on the link below to download 👇\r\n\r\n$fileUrl\r\n\r\nRegards, \r\n$businessname");
            header("Location: $whatsappLink");
        }

        private function prepareSalesItemList($sales)
        {
            $salesItems = [];
            $itemIds = explode(",", $sales->item);
            $itemRates = explode(",", $sales->rate);
            $itemQuantities = explode(",", $sales->quantity);
            $itemTax = explode(",", $sales->tax);
            $itemDiscount = explode(",", $sales->discount);
            $itemAmount = explode(",", $sales->amount);
            foreach ($itemIds as $key => $item) {
                if ($item === "0") {
                    $itemName = "Room Rent";
                } elseif ($item === "e01") {
                    $itemName = 'Electricity Bill';
                } else {
                    $itemName = ExpensesItem::where('id', $item)->value('itemname');
                }
                $salesItems[] = [
                    'itemId' => $item,
                    'itemName' => $itemName,
                    'itemRate' => $itemRates[$key],
                    'itemQuantity' => $itemQuantities[$key],
                    'itemTax' => $itemTax[$key],
                    'itemDiscount' => $itemDiscount[$key],
                    'itemAmount' => $itemAmount[$key],
                ];
            }
            return $salesItems;
        }
        ///////////////////////////////////////////////////sale invoice share and generate
        public function generateInvoiceSale(Request $request)
        {
            $roomNo = $request['roomNo'];
            $fullMonthName = "";
            $invoiceNo = $request['invoiceNo'];
            $bussinessData = Businessregister::firstOrFail();
            $termAndCondition = Termsandcondition::all();
            $stapWithSign = BusinessDocument::where('upload_name', 'Stamp With Signature')->first();
            $saleData = Sale::where('invoiceno', $invoiceNo)->with('student')->firstOrFail();
            $studentData = $saleData->student;
            $candidatMobile = $studentData->phone;
            $numberToWrods = $this->numberToWords($saleData->total);
            $itemList = $this->prepareSalesItemList($saleData);
            $totalReceivedAmountInallBills = Sale::where('studentid', $saleData->studentid)
                ->where('roomno', $saleData->roomno)
                ->sum('remaining_amount');
            $PreviousBalance = $totalReceivedAmountInallBills - ($saleData->remaining_amount);
            $CurrentBalance =  $totalReceivedAmountInallBills - ($saleData->remaining_amount) + ($saleData->remaining_amount);
            $view = view('pdf.studentSalesInvoice-pdf', compact('bussinessData', 'saleData', 'itemList', 'studentData', 'numberToWrods', 'termAndCondition', 'PreviousBalance', 'CurrentBalance', 'stapWithSign'))->render();

            // Create a new instance of Dompdf

            $options = new Options();
            $options->set('defaultFont', 'Courier');

            $dompdf = new Dompdf($options);
            $dompdf->setPaper('A4', 'portrait');
            // Set the page margins
            $dompdf->set_option('margin', 0);
            $dompdf->set_option('padding', 0);

            $dompdf->loadHtml($view);
            // Set paper size and orientation
            $dompdf->setPaper('A4', 'portrait');
            // Render the HTML content to PDF
            $dompdf->render();
            // Generate a unique filename for the PDF
            $filename = 'SaleInvoice_InvoiceNo' . $invoiceNo . '_' . implode("_", explode(" ", $studentData->candidatename)) . '_' . now()->format('YmdHis') . '.pdf';
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/pdf/StudentSalesInvoice/' . $filename);
            file_put_contents($filePath, $output);
            $link = asset('assets/uploads/pdf/StudentSalesInvoice/' . $filename);
            $fullMonthName = $fullMonthName == "" ? Carbon::parse($saleData->invoicedate)->format('F') : "";

            $whatsppPermissionCheck = $this->checkWhatsAppPermission();
            return response()->json([$link, $candidatMobile, $bussinessData->businessname, $studentData->candidatename, $fullMonthName, $whatsppPermissionCheck]);
        }
        ///////////////////////////////custom sale invoice share
        public function customInvoiceSale(Request $request)
        {
            $roomNo = $request['roomNo'];
            $invoiceNo = $request['invoiceNo'];
            $fullMonthName = "";
            // Path to the Dompdf library
            $dompdfPath = public_path('assets/dompdf');
            // Add the Dompdf library to the include path
            set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
            require_once $dompdfPath . '/autoload.inc.php';
            ob_start(); // Start output buffering
            $value = session()->get('user');
            $profile = DB::table('businessregisters')->where('username', $value)->get();
            if ($profile) {
                foreach ($profile as $details) {
                    $bussinessName = $details->businessname;
                    $bussinessemail = $details->email;
                    $mobile1 = $details->mobile;
                    $mobile2 = $details->mobile_secondary;
                    $address = $details->address . "," . $details->city . "," . $details->state . "," . $details->pincode;
                    $img = $details->pic;
                    $imgPath = public_path('assets/uploads/' . $img);
                }
            }
            $signatureImg = DB::table("uploadprofiles")
                ->select('upload_pdf')
                ->where('id', 2)
                ->first();
            $signaturePath = public_path('assets/uploads/pdf/' . $signatureImg->upload_pdf);
            $sales = Customsalesale::where('roomno', $roomNo)
                ->where('invoiceno', $invoiceNo)
                ->get();
            if ($sales->isNotEmpty()) {
                foreach ($sales as $sale) {
                    $id = $sale->roomno;
                    $customerId = $sale->id;
                    $invoiceno = $sale->invoiceno;
                    $invoicedate = $sale->invoicedate;
                    $items = $sale->item;
                    $quantitys = $sale->quantity;
                    $rates =  $sale->rate;
                    $taxs =  $sale->tax;
                    $amounts =  $sale->amount;
                    $discounts = $sale->discount;
                    $receivedAmount = $sale->receive_amount;
                    $totalAmount =    $sale->total;
                    $remainingAmount = $sale->remaining_amount;
                    $candidatName = $sale->name;
                    $candidatMobile = $sale->mobile;
                    $candidatAdd = $sale->address;
                };
                $PreviousBalance = $totalAmount - $remainingAmount;
                $CurrentBalance =  $totalAmount - $remainingAmount;
            }
            ?>
                <html>

                <head>
                    <meta charset="utf-8">
                    <meta content="width=device-width, initial-scale=1" name="viewport">
                </head>

                <body>
                    <style>
                        @import url(https://fonts.googleapis.com/css?family=Lato:400,300,300italic,400italic,700,700italic);

                        /** GLOBAL **/
                        html,
                        * {
                            font-family: "Lato", "Helvetica Neue", "Helvetica", Helvetica, Arial, sans-serif;
                            color: #333447;
                            line-height: 1.5;
                        }

                        /* TYPOGRAPHY */
                        h1 {
                            font-size: 2.5rem;
                        }

                        h2 {
                            font-size: 2rem;
                        }

                        h3 {
                            font-size: 1.375rem;
                        }

                        h4 {
                            font-size: 1.125rem;
                        }

                        h5 {
                            font-size: 1rem;
                        }

                        h6 {
                            font-size: 0.875rem;
                            margin: 0;
                            padding: 0;
                        }

                        p {
                            font-size: 1.125rem;
                            font-weight: 200;
                            line-height: 1.8;
                        }

                        span {
                            padding: 0;
                            font-size: 0.8rem;
                        }

                        .font-light {
                            font-weight: 300;
                        }

                        .font-regular {
                            font-weight: 400;
                        }

                        .font-heavy {
                            font-weight: 700;
                        }

                        /* POSITIONING */
                        .left {
                            text-align: left;
                        }

                        .right {
                            float: right;
                        }

                        .center {
                            text-align: center;
                            margin-left: auto;
                            margin-right: auto;
                        }

                        .justify {
                            text-align: justify;
                        }

                        /** standard padding**/
                        .no-padding {
                            padding: 0px;
                        }

                        .standard-padding {
                            padding: 20px;
                        }

                        .standard-padding-right {
                            padding-right: 20px;
                        }

                        .standard-padding-left {
                            padding-left: 20px;
                        }

                        .standard-padding-right {
                            padding-left: 20px;
                        }

                        .standard-padding-top {
                            padding-top: 20px;
                        }

                        .standard-padding-bottom {
                            padding-bottom: 20px;
                        }

                        .container {
                            width: 100%;
                            margin-left: auto;
                            margin-right: auto;
                        }

                        .row {
                            position: relative;
                            width: 100%;
                        }

                        .row [class^="col"] {
                            float: left;
                            margin: 0.5rem 2%;
                            min-height: 0.125rem;
                        }

                        .row::after {
                            content: "";
                            display: table;
                            clear: both;
                        }

                        .hidden-sm {
                            display: none;
                        }

                        .invoice-box {
                            background: #ffffff;
                            margin: 60px auto;
                            /*max-width: 900px;*/
                            padding: 30px;
                            border: 1px solid #002336;
                            box-shadow: 0 0 10px rgba(0, 0, 0, 0.15);
                            font-size: 16px;
                            line-height: 24px;
                            color: #002336;
                        }

                        .title {
                            margin-bottom: 0px;
                            padding-bottom: 0px;
                            margin-left: 10px;
                            margin-right: 10px;
                            font-weight: bold;
                            border-bottom: 1px solid #8B8B8B;
                            margin-bottom: 4px;
                        }

                        .infoblock {
                            margin-left: 10px;
                            margin-right: 10px;
                            margin-top: 0px;
                            padding-top: 0px;
                        }

                        .titles {
                            padding-top: 4px;
                            margin-top: 20px;
                            background: #000;
                            color: #fff;
                        }

                        ul {
                            font-size: 0.8rem;
                            color: #bababa;
                        }

                        @media only screen and (max-width: 600px) {
                            .invoice-box table tr.top table td {
                                width: 100%;
                                display: block;
                                text-align: center;
                            }

                            .invoice-box table tr.information table td {
                                width: 100%;
                                display: block;
                                text-align: center;
                            }
                        }

                        /** RTL **/
                        .rtl {
                            direction: rtl;
                            font-family: "Lato", Tahoma, "Helvetica Neue", "Helvetica", Helvetica, Arial, sans-serif;
                        }

                        .rtl table {
                            text-align: right;
                        }

                        .rtl table tr td:nth-child(2) {
                            text-align: left;
                        }

                        .eqWrap {
                            display: flex;
                        }

                        .eq {
                            padding: 10px;
                        }

                        .item:nth-of-type(odd) {
                            background: #F9F9F9;
                        }

                        .item:nth-of-type(even) {
                            background: #fff;
                        }

                        .equalHW {
                            flex: 1;
                        }

                        .equalHM {
                            width: 32%;
                        }

                        .equalHMR {
                            width: 32%;
                            margin-bottom: 2%;
                        }

                        table.table {
                            width: 100%;
                            margin-top: 20px;
                            border-collapse: collapse;
                        }

                        .table th,
                        .table td {
                            text-align: left;
                            padding: 0.25em;
                        }

                        .table tr {
                            border-bottom: 1px solid #DDD;
                        }

                        button:hover {
                            box-shadow: 0 0 4px rgba(3, 3, 3, 0.8);
                            opacity: 0.9;
                        }
                    </style>
                    <div class="invoice-box">
                        <div class="container">
                            <table style="width: inherit;">
                                <tr>
                                    <td>
                                        <h3 style="margin:0px;"><?php echo $bussinessName; ?></h3>
                                        <span id="AccountEmail">Email: <?php echo $bussinessemail; ?>
                                        </span>
                                        <span id="AccountPhone">Mobile No: <?php echo $mobile1 . ", " . $mobile2; ?>
                                        </span>
                                        <br>
                                        <span id="AccountEmail">Address:<?php echo $address; ?>
                                        </span>
                                        <br>
                                    </td>
                                    <td>
                                        <div>
                                            <?php
                                            if (file_exists(public_path($imgPath))) {
                                                $imgData = file_get_contents($imgPath);
                                                $imgBase64 = base64_encode($imgData);
                                            } else {
                                                $imgPath = public_path('assets/images/logoimage.jpeg');
                                                $imgData = file_get_contents($imgPath);
                                                $imgBase64 = base64_encode($imgData);
                                            }
                                            ?>
                                            <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="Business Logo" style="width:100%; margin-top:-25px; max-width:150px;float:right">
                                        </div>
                        </div>
                        </td>
                        </tr>
                        </table>
                        <div class="row">
                            <hr>
                            <div class="row">
                                <h4 class="center" style="margin:0">TAX INVOICE</h4>
                            </div>
                            <div class="row">
                                <div class="equalHWrap eqWrap">
                                    <table>
                                        <tr>
                                            <td>
                                                <div class="equalHW eq infoblock to-block" style="width:450px">
                                                    <h6 id="CustomerName">Bill To</h6>
                                                    <h6 id="AccountProject"><?php echo $candidatName; ?></h6>
                                                    <span id="CustomerAddress"><?php echo $candidatAdd; ?></span><br>
                                                    <span id="CustomerPhone"><?php echo $candidatMobile; ?></span><br>
                                                </div>
                                            </td>
                                            <td style="width:16%">
                                                <span id="CreatedDate">Invoice No #<?php echo $invoiceno; ?></span><br>
                                                <span id="DueDate">Date:<?php echo \Carbon\Carbon::parse($invoicedate)->format('d-M-Y') ?></span><br>
                                                <span id="KidNumber">RoomNo: <?php echo $id; ?></span><br>
                                            </td>
                                        </tr>
                                    </table>
                                </div>
                            </div>
                            <table class="table">
                                <tr class="titles">
                                    <th style="color:white">#</th>
                                    <th style="color:white">Item Name</th>
                                    <th style="color:white">Quantity</th>
                                    <th style="color:white">Rate</th>
                                    <th style="color:white">Tax</th>
                                    <th style="color:white">Discount</th>
                                    <th style="color:white">Amount</th>
                                </tr>
                                <?php
                                $itemArray = explode(',', $items);
                                $quantityArray = explode(',', $quantitys);
                                $ratesArray = explode(',', $rates);
                                $taxsArray = explode(',', $taxs);
                                $amountsArray = explode(',', $amounts);
                                $discountsArray = explode(',', $discounts);
                                $i = 1;
                                foreach ($itemArray as $index => $item) {
                                    // fetch the item name 
                                    $itemname = Item::where('id', $item)->pluck('itemname')->first();;
                                    $itemId = $itemArray[$index];
                                    if ($itemId == 0) {
                                        // fetch the room rent
                                        $roomDetails = DB::table('roomseats')->where('roomno', $roomNo)->first();
                                ?>
                                        <tr class="item">
                                            <td><span id="ProuductName"><?php echo $i; ?><span></span></span></td>
                                            <td><span id="ProductNumUnits">Room Rent<span></span></span></td>
                                            <td><span id="ProductUnit"><?php echo $quantityArray[$index]; ?><span></span></span>
                                            </td>
                                            <td><span id="ProductUnitPrice">Rs.<?php echo $ratesArray[$index]; ?></span></td>
                                            <td><span id="ProductDiscount"><?php echo $taxsArray[$index]; ?>%</span></td>
                                            <td><span id="ProductTax"><?php echo $discountsArray[$index]; ?>%</span></td>
                                            <td><span id="ProductCost">Rs.<?php echo $amountsArray[$index]; ?></span></td>
                                        </tr>
                                    <?php } else {
                                    ?>
                                        <tr class="item">
                                            <td><span id="ProuductName"><?php echo $i; ?><span></span></span></td>
                                            <td><span id="ProductNumUnits"><?php echo $itemname; ?><span></span></span></td>
                                            <td><span id="ProductUnit"><?php echo $quantityArray[$index]; ?><span></span></span>
                                            </td>
                                            <td><span id="ProductUnitPrice">Rs.<?php echo $ratesArray[$index]; ?></span></td>
                                            <td><span id="ProductDiscount"><?php echo $taxsArray[$index]; ?>%</span></td>
                                            <td><span id="ProductTax"><?php echo $discountsArray[$index]; ?>%</span></td>
                                            <td><span id="ProductCost">Rs.<?php echo $amountsArray[$index]; ?></span></td>
                                        </tr>
                                <?php
                                    }
                                    $i++;
                                } ?>
                            </table>
                        </div>
                        <div class="row">
                            <div class="equalHW eq">
                                <table style="width: -webkit-fill-available;">
                                    <tr>
                                        <td>
                                            <div>
                                                <h6 style="margin:0;padding:0;">Amount In words</h6>
                                                <p style="background-color:#e6e6fa;margin:0;padding:0 ;font-size:0.8rem"> <?php echo $this->numberToWords($totalAmount) . " Rupees only";  ?>
                                                </p>
                                            </div>
                                            <h4 style="margin:0;padding:0">Term And Condition</h4>
                                            <ul style="margin:0;padding:0">
                                                <?php
                                                $termsAndConditions = DB::table('termsandconditions')
                                                    ->select('termsandconditions')
                                                    ->get();
                                                foreach ($termsAndConditions as $row) {
                                                    echo "<li>" . $row->termsandconditions . "</li>";
                                                }
                                                ?>
                                            </ul>
                                        </td>
                                        <td style="width:30%">
                                            <table class="right">
                                                <tr>
                                                    <td>
                                                        <span><strong>Total:</strong></span>
                                                        <span id="InvoiceCurrency1">Rs.<?php echo $totalAmount; ?></span>
                                                        <br>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td><span><strong>Received:</strong></span>
                                                        <span id="InvoiceCurrency2">Rs.
                                                            <?php echo $receivedAmount; ?></span>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td><span><strong>Remaining:</strong></span>
                                                        <span id="InvoiceCurrency3">Rs.<?php echo $remainingAmount; ?></span>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td><span><strong>Previous Balance:</strong></span>
                                                        <span id="InvoiceCurrency3">Rs.<?php echo $PreviousBalance; ?></span>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td><span><strong>Current Balance:</strong>
                                                        </span><span id="InvoiceCurrency3">Rs.<?php echo $CurrentBalance; ?></span>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td colspan="2">
                                                        <h6 style="text-align:center" style="margin-bottom:30px">For, <?php echo $bussinessName ?></h6>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td colspan="2"><span>
                                                            <?php
                                                            if (file_exists($signaturePath)) {
                                                                $imgDataSign = file_get_contents($signaturePath);
                                                                $imgBaseSign64 = base64_encode($imgDataSign);
                                                            ?>
                                                                <img src="data:image/png;base64,<?php echo $imgBaseSign64; ?>" alt="" style="width:100%; max-width:150px;display:block;margin:auto;">
                                                            <?php
                                                            }
                                                            ?>
                                                        </span>
                                                    </td>
                                                </tr>
                                                <tr>
                                                    <td colspan="2">
                                                        <h6 class="center">Authorized Signature</h6>
                                                    </td>
                                                </tr>
                                            </table>
                                        </td>
                                    </tr>
                                </table>
                            </div>
                        </div>
                    </div>
                    <hr>
                    <div>
                        <table class="right">
                            <tr>
                                <td> <span class="">Visit Us</span></td>
                                <?php
                                $signaturePathVisit = public_path('assets/images/new_hostal_logo.jpg');
                                $imgDataSignVisit = file_get_contents($signaturePathVisit);
                                $imgBaseSignVisit64 = base64_encode($imgDataSignVisit);
                                ?>
                                <td> <span class=""><img src="data:image/png;base64,<?php echo $imgBaseSignVisit64; ?>" alt="" style="width:100%; max-width:50px;height:15px;"></span></td>
                            </tr>
                        </table>
                    </div>
                    </div>
                    </div>
                </body>

                </html>
            <?php
            $html = ob_get_clean();
            $dompdf = new Dompdf(); // Create a new Dompdf instance
            $dompdf->loadHtml($html); // Load HTML content
            $dompdf->setPaper('A4', 'portrait'); // (Optional) Set paper size and orientation
            $dompdf->render(); // Render the HTML content to PDF
            // Generate a random file name
            $name = 'custom_sale_invoice' . implode("_", explode(" ", $candidatName)) . '_' .  uniqid() . '.pdf';
            $filename = public_path('share/' . $name);
            // Save the PDF file locally
            $output = $dompdf->output();
            //$dompdf->stream($filename, ['Attachment' => true]);
            file_put_contents($filename, $output);
            $fileUrl =  asset('/share/' . $name);
            $fullMonthName = $fullMonthName == "" ? Carbon::parse($invoicedate)->format('F') : "";
            $whatsppPermissionCheck = $this->checkWhatsAppPermission();
            return response()->json([$fileUrl, $candidatMobile, $bussinessName, $candidatName, $fullMonthName, $whatsppPermissionCheck]);
        }


        private function getInsertedBillDate()
        {
            $lastRecords =  ElectricityManagement::select('BillDate')
                ->orderBy('id', 'desc')
                ->first();
            return $lastRecords ? $lastRecords->BillDate : null;
        }
        protected function getFormattedBillDates($selectedMonth)
        {
            if (!$selectedMonth) {
                return [null, null];
            }

            $previousReadingDate = ElectricityManagement::select('BillDate')
                ->where('BillDate', '<', $selectedMonth)
                ->orderBy('BillDate', 'DESC')
                ->groupBy('BillDate')
                ->first();

            $presentReadingDate = DB::table('electicity_management')
                ->where('BillDate', '=', $selectedMonth)
                ->first();

            $PrenseBillDate = $presentReadingDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : '';
            $PreviousBillDate = $previousReadingDate ? Carbon::parse($previousReadingDate->BillDate)->format('d-M-Y') : ($PrenseBillDate ? Carbon::parse($presentReadingDate->BillDate)->format('d-M-Y') : "");

            return [$PrenseBillDate, $PreviousBillDate];
        }

        protected function getElectricityRecordsByDateRange($startDate, $endDate)
        {
            return $lastRecords = ElectricityManagement::with('studentElectricityRecords.student')
                ->whereBetween('BillDate', [$startDate, $endDate])
                ->get();
        }

        protected function getElectricityRecordsByMonth($selectedMonth)
        {
            DB::enableQueryLog();
            return ElectricityManagement::with(['studentElectricityRecords' => function ($query) {
                $query->with('student');
            }])
                ->where('BillDate', $selectedMonth)
                ->get();
            DB::getQueryLog();
        }

        public function DownloadElectricityReports(Request $request)
        {
            try {
                $dompdfPath = public_path('assets/dompdf');
                set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
                require_once $dompdfPath . '/autoload.inc.php';
                $bussinessData = Businessregister::firstOrFail();

                $selectedMonth = $request->input('selected_month') ?? $this->getInsertedBillDate();

                list($PrenseBillDate, $PreviousBillDate) = $this->getFormattedBillDates($selectedMonth);
                $roomSeats = RoomSeat::with('reserveRoom')->get();
                if ($selectedMonth == 'custom') {
                    $PrenseBillDate = Carbon::parse($startDate)->format('d-M-Y');
                    $PreviousBillDate = Carbon::parse($endDate)->format('d-M-Y');
                    $electricityRecords = $this->getElectricityRecordsByDateRange($request->input('startDate'), $request->input('endDate'));
                } else {
                    $electricityRecords = $this->getElectricityRecordsByMonth($selectedMonth);
                }
                $allBillDates = ElectricityManagement::select('BillDate')->orderBy('BillDate', 'Asc')->groupBy('BillDate')->get();

                $view = view('pdf.electricityReport-pdf', compact('bussinessData', 'selectedMonth', 'PrenseBillDate', 'PreviousBillDate', 'roomSeats', 'allBillDates', 'electricityRecords'))->render();

                $dompdf = new Dompdf();
                $dompdf->loadHtml($view);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();
                $filename = 'Electricity_Report_' . now()->format('YmdHis') . '.pdf';
                $output = $dompdf->output();
                $filePath = public_path('assets/uploads/' . $filename);

                file_put_contents($filePath, $output);
                $link = asset('assets/uploads/' . $filename);
                return response()->json([
                    'status' => true,
                    'downloadLink' => $link,
                    'fileName' => $filename,
                    'message' => 'Electricity report created.'
                ], 200);
            } catch (Exception $e) {
                return response()->json([
                    'status' => false,
                    'error' => $e->getMessage()
                ], 500);
            }
        }
        public function createInvoiceForPaymentIn(Request $request)
        {
            $invoiceDate = $request['invoiceDate'];
            $fullMonthName = "";
            $invoiceNo = $request['invoiceNo'];
            // Path to the Dompdf library
            $dompdfPath = public_path('assets/dompdf');
            // Add the Dompdf library to the include path
            set_include_path($dompdfPath . PATH_SEPARATOR . get_include_path());
            require_once $dompdfPath . '/autoload.inc.php';
            ob_start(); // Start output buffering
            $value = session()->get('user');
            $profile = DB::table('businessregisters')->where('username', $value)->get();
            if ($profile) {
                foreach ($profile as $details) {
                    $bussinessName = $details->businessname;
                    $bussinessemail = $details->email;
                    $mobile1 = $details->mobile;
                    $mobile2 = $details->mobile_secondary;
                    $address = $details->address . "," . $details->city . "," . $details->state . "," . $details->pincode;
                    $img = $details->pic;
                    $imgPath = public_path('assets/uploads/' . $img);
                }
            }
            $signatureImg = DB::table("uploadprofiles")
                ->select('upload_pdf')
                ->where('upload_name', 'Stamp With Signature')
                ->first();
            $signaturePath = public_path('assets/uploads/pdf/' . $signatureImg->upload_pdf);
            $sales = Sale::where("invoiceno", $invoiceNo)->first();
            if (!empty($sales)) {
                $roomNo =  $sales->roomno;
                $totalAmount = $sales->total;
            }
            $paymentDetails = Recieve_amount::where('invoiceNo', $invoiceNo)
                ->where('receiveDate', $invoiceDate)
                ->first();
            if (!empty($paymentDetails)) {
                $invoiceNo = $paymentDetails->invoiceNo;
                $invoicedate = $paymentDetails->receiveDate;
                $receiveAmount = $paymentDetails->receiveAmount;
                $paymentMode = $paymentDetails->paymentMode;
            }
            $candidatDetails = Studentregister::where("id", $paymentDetails->candidateID)->first();
            if (!empty($candidatDetails)) {
                $candidatName = $candidatDetails->candidatename;
                $candidatAdd = $candidatDetails->fullAddress . "," . $candidatDetails->pinno . "," . $candidatDetails->state;
                $candidatMobile = $candidatDetails->phone;
            }
            ?>
                <html>

                <head>
                    <meta charset="utf-8">
                    <meta content="width=device-width, initial-scale=1" name="viewport">
                </head>

                <body>
                    <style>
                        @import url(https://fonts.googleapis.com/css?family=Lato:400,300,300italic,400italic,700,700italic);

                        /** GLOBAL **/
                        html,
                        * {
                            font-family: "Lato", "Helvetica Neue", "Helvetica", Helvetica, Arial, sans-serif;
                            color: #333447;
                            line-height: 1.5;
                        }

                        /* TYPOGRAPHY */
                        h1 {
                            font-size: 2.5rem;
                        }

                        h2 {
                            font-size: 2rem;
                        }

                        h3 {
                            font-size: 1.375rem;
                        }

                        h4 {
                            font-size: 1.125rem;
                        }

                        h5 {
                            font-size: 1rem;
                        }

                        h6 {
                            font-size: 0.875rem;
                            margin: 0;
                            padding: 0;
                        }

                        p {
                            font-size: 1.125rem;
                            font-weight: 200;
                            line-height: 1.8;
                        }

                        span {
                            padding: 0;
                            font-size: 0.8rem;
                        }

                        .font-light {
                            font-weight: 300;
                        }

                        .font-regular {
                            font-weight: 400;
                        }

                        .font-heavy {
                            font-weight: 700;
                        }

                        /* POSITIONING */
                        .left {
                            text-align: left;
                        }

                        .right {
                            float: right;
                        }

                        .center {
                            text-align: center;
                            margin-left: auto;
                            margin-right: auto;
                        }

                        .justify {
                            text-align: justify;
                        }

                        /** standard padding**/
                        .no-padding {
                            padding: 0px;
                        }

                        .standard-padding {
                            padding: 20px;
                        }

                        .standard-padding-right {
                            padding-right: 20px;
                        }

                        .standard-padding-left {
                            padding-left: 20px;
                        }

                        .standard-padding-right {
                            padding-left: 20px;
                        }

                        .standard-padding-top {
                            padding-top: 20px;
                        }

                        .standard-padding-bottom {
                            padding-bottom: 20px;
                        }

                        .container {
                            width: 100%;
                            margin-left: auto;
                            margin-right: auto;
                        }

                        .row {
                            position: relative;
                            width: 100%;
                        }

                        .row [class^="col"] {
                            float: left;
                            margin: 0.5rem 2%;
                            min-height: 0.125rem;
                        }

                        .row::after {
                            content: "";
                            display: table;
                            clear: both;
                        }

                        .hidden-sm {
                            display: none;
                        }

                        .invoice-box {
                            background: #ffffff;
                            margin: 60px auto;
                            /*max-width: 900px;*/
                            padding: 30px;
                            border: 1px solid #002336;
                            box-shadow: 0 0 10px rgba(0, 0, 0, 0.15);
                            font-size: 16px;
                            line-height: 24px;
                            color: #002336;
                            position: relative;
                        }

                        .title {
                            margin-bottom: 0px;
                            padding-bottom: 0px;
                            margin-left: 10px;
                            margin-right: 10px;
                            font-weight: bold;
                            border-bottom: 1px solid #8B8B8B;
                            margin-bottom: 4px;
                        }

                        .infoblock {
                            margin-left: 10px;
                            margin-right: 10px;
                            margin-top: 0px;
                            padding-top: 0px;
                        }

                        .titles {
                            padding-top: 4px;
                            margin-top: 20px;
                            background: #000;
                            color: #fff;
                        }

                        ul {
                            font-size: 0.8rem;
                            color: #bababa;
                        }

                        @media only screen and (max-width: 600px) {
                            .invoice-box table tr.top table td {
                                width: 100%;
                                display: block;
                                text-align: center;
                            }

                            .invoice-box table tr.information table td {
                                width: 100%;
                                display: block;
                                text-align: center;
                            }
                        }

                        /** RTL **/
                        .rtl {
                            direction: rtl;
                            font-family: "Lato", Tahoma, "Helvetica Neue", "Helvetica", Helvetica, Arial, sans-serif;
                        }

                        .rtl table {
                            text-align: right;
                        }

                        .rtl table tr td:nth-child(2) {
                            text-align: left;
                        }

                        .eqWrap {
                            display: flex;
                        }

                        .eq {
                            padding: 10px;
                        }

                        .item:nth-of-type(odd) {
                            background: #F9F9F9;
                        }

                        .item:nth-of-type(even) {
                            background: #fff;
                        }

                        .equalHW {
                            flex: 1;
                        }

                        .equalHM {
                            width: 32%;
                        }

                        .equalHMR {
                            width: 32%;
                            margin-bottom: 2%;
                        }

                        .right-side-content {
                            /* padding-left: 73%; */
                            display: inline-block;
                        }

                        .left-side-content {
                            display: inline-block;
                            max-width: 73%;
                            width: 500px;
                        }
                    </style>
                    <div class="invoice-box">
                        <div class="container">
                            <table style="width: inherit;">
                                <tr>
                                    <td>
                                        <h3 style="margin:0px;"><?php echo $bussinessName; ?></h3>
                                        <span id="AccountEmail">Email: <?php echo $bussinessemail; ?>
                                        </span>
                                        <span id="AccountPhone"> Mobile No: <?php echo $mobile1 . ", " . $mobile2; ?>
                                        </span>
                                        <br>
                                        <span id="AccountEmail"><?php echo $address; ?>
                                        </span>
                                    </td>
                                    <td>
                                        <div>
                                            <?php
                                            if (file_exists(public_path($imgPath))) {
                                                $imgData = file_get_contents($imgPath);
                                                $imgBase64 = base64_encode($imgData);
                                            } else {
                                                $imgPath = public_path('assets/images/logoimage.jpeg');
                                                $imgData = file_get_contents($imgPath);
                                                $imgBase64 = base64_encode($imgData);
                                            }
                                            ?>
                                            <img src="data:image/png;base64,<?php echo $imgBase64; ?>" alt="Business Logo" style="width:100%; margin-top:-25px; max-width:150px;float:right">
                                        </div>
                        </div>
                        </td>
                        </tr>
                        </table>
                        <div class="row">
                            <hr>
                            <div class="row">
                                <h4 class="center" style="margin:0">Payment-In</h4>
                            </div>
                            <div class="row">
                                <div class="equalHWrap eqWrap">
                                    <table>
                                        <tr>
                                            <td>
                                                <div class="equalHW eq infoblock to-block" style="width:450px">
                                                    <h6 id="CustomerName">Received From,</h6>
                                                    <h6 id="AccountProject"><?php echo $candidatName; ?></h6>
                                                    <span id="CustomerAddress"><?php echo $candidatAdd; ?></span><br>
                                                    <span id="CustomerPhone"><?php echo $candidatMobile; ?></span><br>
                                                </div>
                                            </td>
                                            <td style="width:20%">
                                                <span id="CreatedDate">Invoice No #<?php echo $invoiceNo; ?></span><br>
                                                <span id="DueDate">Date:<?php echo \Carbon\Carbon::parse($invoicedate)->format('d-M-Y') ?></span><br>
                                                <span id="KidNumber">RoomNo: <?php echo $roomNo ?></span><br>
                                            </td>
                                        </tr>
                                    </table>
                                </div>
                            </div>

                        </div>
                        <div class="right-side-box">
                            <table class="table">
                                <tr>
                                    <td style="vertical-align: top;">
                                        <div style="width:450px;height:max-content">
                                            <h6 style="padding:12px 15px;">Amount In words</h6>
                                            <p style="background-color:#e6e6fa;margin:0;padding:12px 15px;font-size:1rem"> <?php echo $this->numberToWords($receiveAmount) . " Rupees only";  ?>
                                            </p>
                                        </div>
                                    </td>
                                    <td>
                                        <div class="right-side-content">
                                            <div class="">
                                                <span><strong>Received:</strong></span>
                                                <span style="margin-left: 30px;" id="InvoiceCurrency2">Rs.<?php echo $receiveAmount; ?></span>
                                            </div>
                                            <div>
                                                <span><strong>Payment Mode:</strong></span>
                                                <span id="InvoiceCurrency3"><?php echo $paymentMode; ?></span>
                                            </div>
                                            <div>
                                                <span><strong>Previous Balance:</strong></span>
                                                <span id="InvoiceCurrency3">Rs.<?php echo $totalAmount; ?></span>
                                            </div>
                                            <div>
                                                <span><strong>Current Balance:</strong>
                                                </span><span id="InvoiceCurrency3">Rs.<?php echo $totalAmount - $receiveAmount; ?></span>
                                            </div>
                                            <div>
                                                <h6 style="text-align:center">For, <?php echo $bussinessName ?></h6>
                                            </div>
                                            <div style="margin-top: 30px;">
                                                <span>
                                                    <?php
                                                    if (file_exists($signaturePath)) {
                                                        $imgDataSign = file_get_contents($signaturePath);
                                                        $imgBaseSign64 = base64_encode($imgDataSign);
                                                    ?>
                                                        <img src="data:image/png;base64,<?php echo $imgBaseSign64; ?>" alt="" style="width:100%; max-width:150px;display:block;margin:auto;">
                                                    <?php
                                                    }

                                                    ?>
                                                </span>
                                            </div>
                                            <div>
                                                <h6 class="center">Authorized Signature</h6>
                                            </div>
                                        </div>
                                    </td>
                                </tr>
                            </table>
                        </div>
                        <hr>
                        <div>
                            <table class="right">
                                <tr>
                                    <td> <span class="">Visit Us</span></td>
                                    <?php
                                    $signaturePathVisit = public_path('assets/images/new_hostal_logo.jpg');
                                    if (file_exists($signaturePathVisit)) {
                                        $imgDataSignVisit = file_get_contents($signaturePathVisit);
                                        $imgBaseSignVisit64 = base64_encode($imgDataSignVisit);
                                    ?>
                                        <td> <span class=""><img src="data:image/png;base64,<?php echo $imgBaseSignVisit64; ?>" alt="" style="width:100%; max-width:50px;height:15px;"></span></td>
                                    <?php
                                    }
                                    ?>
                                </tr>
                            </table>
                        </div>
                    </div>
                    </div>
                </body>

                </html>
        <?php
            $html = ob_get_clean();
            $dompdf = new Dompdf(); // Create a new Dompdf instance
            $dompdf->loadHtml($html); // Load HTML content
            $dompdf->setPaper('A4', 'portrait'); // (Optional) Set paper size and orientation
            $dompdf->render(); // Render the HTML content to PDF
            // Generate a random file name
            $name = 'PaymentIn_Invoice_' . implode("_", explode(" ", $candidatName)) . '_' . uniqid() . '.pdf';
            $filename = public_path('share/' . $name);
            // Save the PDF file locally
            $output = $dompdf->output();
            file_put_contents($filename, $output);
            $fileUrl =  asset('/share/' . $name);
            $fullMonthName = $fullMonthName == "" ? Carbon::parse($invoicedate)->format('F') : "";
            $whatsppPermissionCheck = $this->checkWhatsAppPermission();
            return response()->json([$fileUrl, $candidatMobile, $bussinessName, $candidatName, $fullMonthName, $whatsppPermissionCheck], 200);
        }


        public function sendSaleStatement($studentId)
        {
            try {
                $bussinessData = Businessregister::firstOrFail();
                $salesData = [];
                $query = Sale::where('studentid', $studentId)->with(['student', 'receiveAmounts']);
                $totalTxn = Sale::where('studentid', $studentId)->count();
                $totalSale = Sale::where('studentid', $studentId)->sum("total");
                $dueBalance = Sale::where('studentid', $studentId)->sum('remaining_amount');

                $salesRecords = $query->get();
                foreach ($salesRecords as $row) {
                    $salesData[] = [
                        'invoiceno' => $row->invoiceno,
                        'saleType' => 'Reg. Sale',
                        'saleDate' => $row->tpdate ? Carbon::parse($row->tpdate)->format('d-M-Y') : Carbon::parse($row->invoicedate)->format('d-M-Y'),
                        'studentName' => $row->student->candidatename,
                        'roomNo' => $row->roomno,
                        'totalAmount' => $row->total,
                        'Balance' => $row->remaining_amount,
                        'paymentMode' =>  optional($row->receiveAmounts->first())->paymentMode,
                        'items' => $this->prepareSalesItemList($row),
                        'paymentInDetails' => $row->receiveAmounts,
                    ];
                }

                $view = view('pdf.sendStatement-pdf', compact('bussinessData', 'totalTxn', 'totalSale', 'dueBalance', 'salesData'))->render();
                $dompdf = new Dompdf();
                $dompdf->loadHtml($view);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();
                $filename = 'SaleStatement_' . now()->format('YmdHis') . '.pdf';
                $output = $dompdf->output();
                $filePath = public_path('assets/uploads/' . $filename);

                file_put_contents($filePath, $output);
                $link = asset('assets/uploads/' . $filename);
                $whatsppPermissionCheck = $this->checkWhatsAppPermission();
                return response()->json([
                    'status' => true,
                    'data' => [
                        'message' => "Sales Report created successfully.",
                        'download_link' => $link,
                        'businessName' => $bussinessData->businessname,
                        'whatsppPermissionCheck' => $whatsppPermissionCheck,
                    ]
                ], 200);
            } catch (Exception $e) {
                return response()->json([
                    'status' => false,
                    "error" => $e->getMessage(),
                    "message" => 'server Error'
                ], 500);
            }
        }

        public function downloadEmpForm($id)
        {
            try {
                $bussinessData = Businessregister::firstOrFail();
                $stampWithSingDoc = Uploadprofile::where('upload_name', 'Stamp With Signature')->first();
                if (!empty($stampWithSingDoc) && file_exists(public_path('assets/uploads/pdf/' . $stampWithSingDoc->upload_pdf))) {
                    $stampWithSingDocUrl = asset('assets/uploads/pdf/' . $stampWithSingDoc->upload_pdf);
                } else {
                    $stampWithSingDocUrl = "";
                }
                $emp = EmpManagement::findOrFail($id);
                $empName = $emp->candidate_name;
                $view = view('pdf.empFormDownload-pdf', compact('bussinessData', 'emp', 'stampWithSingDocUrl'))->render();
                $options = new Options();
                $options->set('DOMPDF_PDF_IMAGE_QUALITY', 30);
                $dompdf = new Dompdf($options);
                $dompdf->loadHtml($view);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();
                $filename = 'EmpForm_' . implode('_', explode(' ', $empName)) . "_" . now()->format('YmdHis') . '.pdf';
                $output = $dompdf->output();
                $filePath = public_path('assets/uploads/' . $filename);
                file_put_contents($filePath, $output);
                $link = asset('assets/uploads/' . $filename);
                return response()->json([
                    'status' => true,
                    'downloadLink' => $link,
                    'fileName' => $filename,
                    'message' => 'emp form created Successfully.'
                ], 200);
            } catch (ModelNotFoundException $e) {
                return response()->json([
                    'status' => false,
                    'error' => $e->getMessage(),
                ], 404);
            } catch (Exception $e) {
                return response()->json([
                    'error' => $e->getMessage(),
                ], 500);
            }
        }

        public function DownloadEmployeeDocument($id)
        {
            $bussinessData = Businessregister::firstOrFail();
            $emp = EmpManagement::findOrFail($id);
            $view = view('pdf.empDownloadDocuments-pdf', compact('bussinessData', 'emp'))->render();
            $dompdf = new Dompdf();
            $dompdf->loadHtml($view);
            $dompdf->setPaper('A4', 'portrait');
            $dompdf->render();
            $filename = 'Employee_Documents_' . implode('_', explode(' ', $emp->candidate_name)) . "_" . now()->format('YmdHis') . '.pdf';
            $output = $dompdf->output();
            $filePath = public_path('assets/uploads/' . $filename);

            file_put_contents($filePath, $output);
            $link = asset('assets/uploads/' . $filename);
            return response()->json([
                'status' => true,
                'downloadLink' => $link,
                'fileName' => $filename,
                'message' => 'emp form created Successfully.'
            ], 200);
        }

        public function  downloadSubscriptionBill(Request $request)
        {
            try {   

                $planId = $request->bill_id;

                $profileDetails = SubscriptionBillSetting::first();
                $bussinessData = Businessregister::firstOrFail();
                // now get the plan details 
                $planDetails = SaleSubscription::with(['activeUserAddOns', 'planInfo'])->findOrFail($planId);
                $numberToWordAmount = $planDetails->total_amount;
                $numberToWordAmountText = $this->numberToWords($numberToWordAmount);
                $view = view('pdf.subscriptonBill-genrate', compact('profileDetails', 'bussinessData', 'planDetails','numberToWordAmountText'))->render();
                $dompdf = new Dompdf();
                $dompdf->loadHtml($view);
                $dompdf->setPaper('A4', 'portrait');
                $dompdf->render();
                $filename = 'Bill_' . implode('_', explode(' ', ($bussinessData->fname . "_" . $bussinessData->lname))) . "_" . now()->format('YmdHis') . '.pdf';
                $output = $dompdf->output();
                $filePath = public_path('assets/uploads/' . $filename);

                file_put_contents($filePath, $output);
                $link = asset('assets/uploads/' . $filename);
                return response()->json([
                    'status' => true,
                    'downloadLink' => $link,
                    'fileName' => $filename,
                    'message' => 'emp form created Successfully.'
                ], 200);
            } catch (ModelNotFoundException $e) {
                return response()->json([
                    'status' => false,
                    'error' => $e->getMessage()
                ], 500);
            } catch (Exception $e) {

                return response()->json([
                    'status' => false,
                    'error' => $e->getMessage()
                ], 500);
            }
        }
    }
