<?php

namespace App\Http\Controllers;

use App\Models\ReferEarnSetup;
use Exception;
use Illuminate\Http\Request;
use Validator;
use App\Models\ReferHostel;
use Illuminate\Database\Eloquent\ModelNotFoundException;

class ReferEarnSetupController extends Controller
{

    public function view()
    {
        $referEarnSetup = ReferEarnSetup::where('is_active',1)->first();
        $referedHosteled = ReferHostel::with('studentDetails')->get();
        return view('fronted.refer_earn', compact('referEarnSetup', 'referedHosteled'));
    }

    public function createSetup(Request $request)
    {
        $validator = Validator::make($request->all(), [
            'ref_allowed' => 'required|numeric',
            'ref_discounts' => 'required|array',
            'ref_total_months' => 'required|integer',
            'ref_start_date' => 'required|date',
            'ref_end_date' => 'required|date|after_or_equal:ref_start_date',
            'ref_start_time' => 'required|date_format:H:i',
            'ref_end_time' => 'required|date_format:H:i',
            'terms_and_conditions' => 'required|string',
            'is_active' => 'required|boolean',
            'message' => 'required',
        ]);
        // Check for validation errors
        if ($validator->fails()) {
            return back()->withErrors($validator)->withInput();
        }

        try {
            // Get the validated data
            $validated = $validator->validated();
            // now check if the referEarnSetup already exist then get the current active setup make it inactive then insert new 
            if (ReferEarnSetup::exists()) {
                $setup = ReferEarnSetup::orderBy('id', 'desc')->where('is_active', 1)->first();
                $setup->update([
                    'is_active' => 0
                ]);
            }
            $setup = ReferEarnSetup::create($validated);

            return back()->with('success', 'Setting has been saved successfully');
        } catch (Exception $e) {
            return back()->with("error", $e->getMessage());
        }
    }

    public function deleteReferRecord($id)
    {
        try {
            $referedHosteled = ReferHostel::findOrFail($id);
            $referedHosteled->delete();

            return response()->json([
                'status' => true,
                'message' => 'records delete successfully',
            ], 200);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage(),
            ], 500);
        }
    }
}
