<?php

namespace App\Http\Controllers;

use App\Models\ReferHostel;
use Illuminate\Http\Request;
use Validator;
use App\Models\ApiModels\StudentRegister;
use App\Models\AdminReferal;
use Exception;
use App\Models\Businessregister;
use App\Models\ReferEarnSetup;
use App\Models\AdminReferalSetup;
use Illuminate\Database\Eloquent\ModelNotFoundException;

class ReferHostelController extends Controller
{
    public function store(Request $request, $studentId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'refer_name' => 'required',
                'refer_mobile' => 'required|digits:10',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Failed',
                    'errors' => $validator->errors(),
                ], 422);
            }

            $student = StudentRegister::findOrFail($studentId);

            $referEarnSetup = ReferEarnSetup::where('is_active', 1)->firstOrFail();

            $refer_start_date = $referEarnSetup->ref_start_date;
            $refer_end_date = $referEarnSetup->ref_end_date;
            $referallowedInReferProgram = $referEarnSetup->ref_allowed;

            $isExistRefer = ReferHostel::where('student_id', $studentId)
                ->where('refer_setup_id',$referEarnSetup->id)
                ->count();

            //check if the refered form the user is greter then refer setup then send error
            if($isExistRefer + 1 > $referallowedInReferProgram){
                return response()->json([
                    'status'=>false,
                    'statusCode' => 409,
                    'message'  => "In this refer program only $referallowedInReferProgram refer allowed. You already referd."
                ],409);
            } 
            //check if student refer to  hostel if same mobile number

            $checkReferedAlready = ReferHostel::where('student_id', $studentId)->where('refered_mobile', $request->refer_mobile)->count();
            if ($checkReferedAlready > 0) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 409,
                    'message' => "This mobile number is already used for refer a friend by You."
                ], 409);
            }


            $referalCode = strtoupper(substr($student->candidatename, 0, 3)) . rand(999, 9999);
            $data = [
                'refer_setup_id' => $referEarnSetup->id, 
                'referal_code' => $referalCode,
                'student_id' => $studentId,
                'studentName' => $student->candidatename,
                'mobile' => $student->phone,
                'refered_name' => $request->input('refer_name'),
                'refered_mobile' => $request->input('refer_mobile'),
                'referal_count' => $isExistRefer + 1,
                'discount' => 0,
                'for_month' => $referEarnSetup->ref_total_months,
                'active_refer_message' => json_encode($referEarnSetup->message),
                'status' => 0,
                'refer_start_date' => $refer_start_date,
                'refer_end_date' => $refer_end_date
            ];

            ReferHostel::create($data);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Successfully Referred!',
            ]);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Bed request',
                'error'=>$e->getMessage()
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred',
            ], 500);
        }
    }

    public function  getAllReferral($studentId)
    {
        try {
            $allreferal = ReferHostel::where('student_id', $studentId)->get();

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'referal data get successfully',
                'data' => $allreferal,
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => true,
                'statusCode' => 500,
                'message' => $e->getMessage()
            ], 500);
        }
    }

    public function checkReferalCode(Request $request)
    {
        try {
            $referCode = $request['referCode'];
            $referMobileNo = $request['referMobileNo'];
            // check referCode is exist or not and  verify it.
            $refer = ReferHostel::where('referal_code', strtoupper($referCode))
            ->where('refered_mobile', $referMobileNo)
            ->where('status', 0)->first();
            // status 0 mean that refer entery is valid 1 is for expired , 2 is for reedem

            if ($refer == null || empty($refer)) {
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'refer_code_is_valid' => false,
                    'message' => 'refer code is invalid',
                ]);
            } else {
                return response()->json([
                    'status' => true,
                    'statusCode' => 200,
                    'refer_code_is_valid' => true,
                    'message' => 'added refercode',
                    'data' => $refer,
                ], 200);
            }
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'server Error',
                'error' => $e->getMessage(),
            ], 500);
        }
    }

    public function adminReferals()
    {
        try {
            $hostelId = session("liseneceNo");
            $referedList = AdminReferal::where('hostel_id', $hostelId)->get();
            return view('fronted.admin_referal', compact('referedList'));
        } catch (Exception $e) {
            return back()->with('error', 'somthing went wronge');
        }
    }


    public function getAdminReferalsSetupData()
    {
        try {
            $adminRefealData = AdminReferalSetup::where('is_active', 1)->first();
            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Admin referal setup',
                'data' => $adminRefealData
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => 'false',
                'statusCode' => 500,
                'message' => 'server Error',
                'error' => $e->getMessage()
            ], 500);
        }
    }
    public function InsertAdminRefer(Request $request)
    {
        try {
            $validator = Validator::make($request->all(), [
                'referal_name' => 'required|string',
                'referal_mobile' => 'required|digits:10',
            ]);

            if ($validator->fails()) {
                return back()->with('error', $validator->errors()->first());
            }
            $hostelId = session("liseneceNo");

            // check this referal number is used by or not this admins
            $checkReferal = AdminReferal::where('hostel_id', $hostelId)->where('refered_mobile', $request->referal_mobile)->count();

            if ($checkReferal > 0) {
                throw new Exception("`You have already refer to this $request->referal_mobile mobile number`");
            }
            // check referal program is active or not 
            $checkReferalIsActive = AdminReferalSetup::where('is_active', 1)->first();

            if ($checkReferalIsActive == null) {
                throw new Exception('Referal Program Is Not Active');
            }

            // get the referal dicount or poinst value 
            $referDicount = $checkReferalIsActive->from_ref_point;
            $toReferDicount = $checkReferalIsActive->to_ref_point;

            // check how many referal it can did  by one admin.
            $referalCountByAdmin = AdminReferal::where('hostel_id', $hostelId)->count();
            // get admin data  by licence no 
            $adminDetails = Businessregister::first();

            // now collect all the data
            $referalCode = strtoupper(substr($adminDetails->fname, 0, 3)) . rand(999, 9999);

            $data = [
                'hostel_id' => session("liseneceNo"),
                'adminName' =>  $adminDetails->fname . " " . $adminDetails->lname,
                'mobile' => $adminDetails->mobile,
                'refered_name' => $request->referal_name,
                'refered_mobile' => $request->referal_mobile,
                'referral_code' => $referalCode,
                'referral_discount' => $toReferDicount[$referalCountByAdmin] ?? null,
                'referral_count' => $referalCountByAdmin + 1,
                'discount' => $referDicount[$referalCountByAdmin] ?? null,
                'for_month' => $checkReferalIsActive->expired_days,
                'active_refer_message' => $checkReferalIsActive->message,
                'offer_type' => $checkReferalIsActive->selected_offer_type,
                'status' => 1
            ];

            AdminReferal::create($data);

            return back()->with('success', 'successfully refer to your friend');
        } catch (Exception $e) {
            return back()->with('error', $e->getMessage());
        }
    }
}
