<?php

namespace App\Http\Controllers;

use App\Models\ReferedApp;
use Illuminate\Http\Request;
use Validator;
use App\Models\ApiModels\StudentRegister;
use Exception;
use App\Models\Businessregister;
use App\Models\ReferEarnAdminSetup;
use Illuminate\Database\Eloquent\ModelNotFoundException;

class ReferedAppController extends Controller
{
    // Store a newly created resource in storage
    public function store(Request $request,$studentId)
    {
        try {
            $validator = Validator::make($request->all(), [
                'refer_name' => 'required',
                'refer_mobile' => 'required|digits:10',
            ]);

            if ($validator->fails()) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 422,
                    'message' => 'Validation Failed',
                    'errors' => $validator->errors(),
                ], 422);
            }

            $student = StudentRegister::findOrFail($studentId);
            $hostelId = Businessregister::value('license_no') ?? null;
            $isExistRefer = ReferedApp::where('student_id', $studentId)->count();
            $referEarnSetup = ReferEarnAdminSetup::where('is_active', 1)->firstOrFail();
            $referdPoints = $referEarnSetup->from_ref_point;

            //check if student refer to  hostel if same mobile number

            $checkReferedAlready = ReferedApp::where('student_id', $studentId)->where('refered_mobile', $request->refer_mobile)->count();
            if ($checkReferedAlready > 0) {
                return response()->json([
                    'status' => false,
                    'statusCode' => 409,
                    'message' => "This mobile number is already used for refer a friend by You."
                ], 409);
            }


            $referalCode = strtoupper(substr($student->candidatename, 0, 3)) . rand(999, 9999);
            $data = [
                'referal_code' => $referalCode,
                'hostel_id' => $hostelId,
                'student_id' => $studentId,
                'student_name' => $student->candidatename,
                'mobile' => $student->phone,
                'refered_name' => $request->input('refer_name'),
                'refered_mobile' => $request->input('refer_mobile'),
                'referal_count' => $isExistRefer + 1,
                'points' => $referdPoints[$isExistRefer] ?? null,
                'refered_points' => $referEarnSetup->to_ref_point,
                'for_days' => $referEarnSetup->expired_days,
                'active_refer_message' => json_encode($referEarnSetup->message),
                'status' => 1,
            ];

            ReferedApp::create($data);

            return response()->json([
                'status' => true,
                'statusCode' => 200,
                'message' => 'Successfully Referred!',
            ]);
        } catch (ModelNotFoundException $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 404,
                'message' => 'Student not found',
            ], 404);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'statusCode' => 500,
                'message' => 'An error occurred',
            ], 500);
        }
    }

    public function getReferedApp($studentId){
        try{
          $referedApp =  ReferedApp::where('student_id',$studentId)->get();
            return response()->json([
                'status'=>true,
                'statusCode'=>200,
                'message'=>'get the refered app list',
                'data' => $referedApp
            ],200);

        }catch(Exception $e){
            return response()->json([
                'status'=>false,
                'statusCode'=>500,
                'message'=>$e->getMessage(),
            ],500);
        }
    }

    public function getReferEarnAdminSetup(){
        try{
            $ReferEarnAdminSetup = ReferEarnAdminSetup::get();
            return response()->json([
                'status'=>true,
                'statusCode'=>200,
                'message'=>'get the admin share and earn setup',
                'data'=>$ReferEarnAdminSetup
            ],200);
        }catch(Exception $e){
            return response()->json([
                'status'=>false,
                'statusCode'=>500,
                'message'=> $e->getMessage(),
            ],500);
        }
    }
}
