<?php

namespace App\Http\Controllers;

use App\Models\leavingdate;
use App\Models\Roomseat;
// use Illuminate\Support\Facades\DB;
use DB;
use Illuminate\Http\Request;
use Carbon\Carbon;
use Exception;
use App\Models\Salesubscription;
use App\Models\Studentregister;

class Room_seatController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */


    public function allroomdetails(Request $request)
    {
        $roomnumberdetails = DB::table('roomseats')
            ->select('roomno')
            ->orderBy('roomno', 'ASC')
            ->get();
        $room = [];
        foreach ($roomnumberdetails as $tem) {
            $room[] = $tem->roomno;
        }
        return response()->json($room);
    }
    public function getItemDetails(Request $request)
    {
        $id = $request->query('itemID');
        $itemDetails = DB::table('items')->where('id', $id)->first();
        return response()->json([
            'itemcode' => $itemDetails->itemcode,
            'rate' => $itemDetails->rate,
            'tax' => $itemDetails->tax,
            'amount' => $itemDetails->amount,
        ]);
    }
    public function getCandidateDetails(Request $request)
    {
        $id = $request->query('candidateName');
        $candidate = DB::table('studentregisters')->where('id', $id)->first();
        $regDate = $candidate->join_date ? Carbon::parse($candidate->join_date) : Carbon::parse($candidate->date);

        // Update day and month with the current values

        $regDate->month(now()->month);
        $regDate->year(now()->year);
        // Format the updated date as "YYYY-MM-DD"
        $updatedDate = $regDate->format('Y-m-d');

        return response()->json([
            'mobile' => $candidate->phone,
            'address' => $candidate->fullAddress,
            'regDate' => $updatedDate
        ]);
    }
    public function getRoomDetails(Request $request)
    {
        $roomNo = $request->query('roomNo');
        $candidates = DB::table('studentregisters')->where('roomno', $roomNo)
            ->where('status', '1')
            ->get();
        return response()->json($candidates);
    }
    private function getSubscriptionDetails()
    {
        $licenseNo = session("liseneceNo");
        config(['database.default' => env('DB_CONNECTION')]);

        // Get the subscription details for the user
        $subscription = Salesubscription::where("user_id", $licenseNo)
            ->where("status", 1)
            ->first();

        // Switch back to the custom database
        config(['database.default' => "custom_db"]);

        if (!$subscription) {
            return [
                'status' => false,
                'message' => 'You have not added any subscription.',
            ];
        }

        // Get the total number of seats registered
        $totalRegisteredSeats = Roomseat::sum('numberseats');

        // Check if the total registered seats are greater than or equal to the purchased seats
        if ($totalRegisteredSeats >= $subscription->no_of_seats) {
            return [
                'status' => false,
                'message' => "You have reached the maximum number of seats ({$subscription->no_of_seats}) allowed by your subscription.",
                'purchedSeats' => $subscription->no_of_seats,
                'subscription_details' => $subscription,
            ];
        }

        // Check if the subscription end date is less than the current date
        $currentDate =  Carbon::now();
        $endDate = Carbon::parse($subscription->enddate);

        if ($endDate->lt($currentDate)) {
            return [
                'status' => false,
                'message' => 'Your subscription has expired.',
                'subscription_details' => $subscription,
            ];
        }

        // If there are available seats and the subscription is active
        return [
            'status' => true,
            'message' => 'You have available seats and your subscription is active.',
            'subscription_details' => $subscription
        ];
    }

    public function index()
    {
        $roomnumber = DB::table('roomseats')->count();
        $totalseats = DB::table('roomseats')->get()->sum("numberseats");
        $engagedseats = DB::table('studentregisters')->where('status', 1)->get()->count('id');
        $vacantSeats = $totalseats - $engagedseats;
        $subscriptionDetails = $this->getSubscriptionDetails();
        // dd($subscriptionDetails);
        return view('fronted.room_seat', compact('roomnumber', 'totalseats', 'engagedseats', 'vacantSeats', 'subscriptionDetails'));
    }
    /**
     * Show the form for creating a new resource.
     *
     * @return \Illuminate\Http\Response
     */
    public function create()
    {
        $masterroomtype = DB::table('masterroomseats')->get();
        $roomnumber = DB::table('roomseats')->get();
        return view('fronted.register', compact('roomnumber', 'masterroomtype'));
    }
    /**
     * Store a newly created resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @return \Illuminate\Http\Response
     */

    private function checkSeatAvailable($seats)
    {
        try {
            // Sum the number of seats already registered
            $totalRegisteredSeats = Roomseat::sum('numberseats');

            // Calculate the total seats after adding the new seats
            $totalSeatsAfterAdding = $totalRegisteredSeats + $seats;

            // Switch to the main database to get the number of purchased seats
            $licenseNo = session("liseneceNo");
            config(['database.default' => env('DB_CONNECTION')]);

            // Get the number of purchased seats for the user
            $purchasedSeats = Salesubscription::where("user_id", $licenseNo)
                ->where("status", 1)
                ->value('no_of_seats');

            // Switch back to the custom database
            config(['database.default' => "custom_db"]);

            // If no purchased seats record found, return an error message
            if ($purchasedSeats === null) {
                return [
                    'status' => true,
                    'message' =>  "Subscription not found."
                ];
            }

            // Check if the total seats after adding is less than or equal to the purchased seats
            $isSeatAvailable = intval($totalSeatsAfterAdding) <= $purchasedSeats;

            // If there are not enough seats, return an error message
            if (!$isSeatAvailable) {
                return [
                    'status' => false,
                    'message' =>  "Not enough seats available. You have purchased {$purchasedSeats} seats.",
                    'purchasedSeats' =>  $purchasedSeats,
                ];
            }

            // If everything is okay, return true
            return [
                'status' => true,
                'message' =>  "enough seats available."
            ];
        } catch (Exception $e) {
            return [
                'status' => false,
                'message' => "Something went wrong"
            ];
        }
    }

    public function store(Request $request)
    {
        $request->validate([
            'roomno' => 'required',
            'roomtype' => 'required',
            'numberseats' => 'required',
            'nocandidate' => 'required',
            'rent' => 'required',
        ]);
        // return $request->input();
        $checkRoomNo = Roomseat::where("roomno", $request->roomno)->first();
        if ($checkRoomNo) {
            return back()->with("roomerror", $request->roomno . " Room number already exist. ");
        }
        $totalSeats = $this->checkSeatAvailable($request->numberseats);
        if (!$totalSeats['status']) {
            return back()->with([
                'roomerror' => $totalSeats["message"],
                'activeUpgradeBtn' => true,
            ]);
        }
        $data = new Roomseat();
        $data->roomno = $request->roomno;
        $data->roomtype = $request->roomtype;
        $data->numberseats = $request->numberseats;
        $data->nocandidate = $request->nocandidate;
        $data->rent = $request->rent;
        $data->save();
        return back()->with('success', 'Room Added Successfully');
    }
    public function leavingstore(Request $request)
    {
        $request->validate([
            'leavingdate' => 'required',
        ]);
        $data = new leavingdate();
        $data->leavingdate = $request->leavingdate;
        $data->save();
        return back()->with('success', 'done');
    }
    /**
     * Display the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */

    /**
     * Show the form for editing the specified resource.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function edit($id)
    {
        $roomseatsdata = Roomseat::find($id);
        return response()->json([
            'status' => 200,
            'roomseats' => $roomseatsdata,
        ]);
    }
    /**
     * Update the specified resource in storage.
     *
     * @param  \Illuminate\Http\Request  $request
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function update(Request $request)
    {
        $roomseats_id = $request->input('roomseats_id');
        $data = Roomseat::find($roomseats_id);
        $wantToIncreseSeatsInRoom = ($data->numberseats >= $request->numberseats) ? ($data->numberseats - $request->numberseats) : ($request->numberseats - $data->numberseats);
        $totalSeats = $this->checkSeatAvailable($wantToIncreseSeatsInRoom);
        if (!$totalSeats['status']) {
            return back()->with([
                'roomerror' => $totalSeats["message"],
                'activeUpgradeBtn' => true,
            ]);
        }
        if ($data) {
            $data->roomno = $request->roomno;
            $data->roomtype = $request->roomtype;
            $data->numberseats = $request->numberseats;
            $data->nocandidate = $request->nocandidate;
            $data->rent = $request->rent;
            $data->update();
            return back()->with('success', 'Room Seat Update successful!');
        } else {
            return back()->with('error', 'Data not found.');
        }
        return redirect('room_seat')->with('success', 'Room Updated Succssfully !');
    }
    /**
     * Remove the specified resource from storage.
     *
     * @param  int  $id
     * @return \Illuminate\Http\Response
     */
    public function destroy($id)
    {
        $data = Roomseat::find($id);

        $studentsInRoom = Studentregister::where('roomno', $data->roomno)->count();

        if ($studentsInRoom > 0) {
            return back()->with('roomerror', 'Cannot delete room seat because there are students assigned to it');
        }
        $data->delete();
        return redirect('room_seat')->with('danger', 'Delete!');
    }
  
    public function getRoomRent(Request $request)
    {
        $RoomNo = $request->query('roomNo');
        $itemDetails = DB::table('roomseats')->where('roomno', $RoomNo)->first();
        return response()->json([
            'itemcode' => 0,
            'rate' => $itemDetails->rent,
            'tax' => 0,
            'amount' => $itemDetails->rent,
        ]);
    }
    public function getNumberSeats(Request $request)
    {
        $roomNo = $request->query('roomNo');
        // Perform your database query or any other necessary operations to get the number of seats
        $roomsDetails = DB::table('roomseats')->where('roomno', $roomNo)->first();
        // Create a response array with the number of seats
        $totalseats = array();
        for ($i = 1; $i <= $roomsDetails->numberseats; $i++) {
            $roomSeatExists = DB::table('studentregisters')
                ->where('roomno', $roomNo)
                ->where('seetno', $i)
                ->where('status', 1)
                ->exists();
            if ($roomSeatExists) {
                // Room number and seat number already exist in the studentregisters table
                // Skip this iteration
                continue;
            } else {
                $totalseats[] = $i;
            }
        }
        $response = [
            'numberSeats' => $totalseats,
        ];
        return response()->json($response);
    }
    public function getroomseats(Request $request)
    {
        // Logic to fetch the number of seats based on the provided roomNo
        $roomNo = $request->query('roomNo');
        // Perform your database query or any other necessary operations to get the number of seats
        $roomsDetails = DB::table('roomseats')->where('roomno', $roomNo)->first();
        // Create a response array with the number of seats
        $totalseats = array();
        for ($i = 1; $i <= $roomsDetails->numberseats; $i++) {
            $totalseats[] = $i;
        }
        $response = [
            'numberSeats' => $totalseats,
        ];
        return response()->json($response);
    }
    public function avalablesSeats(Request $request)
    {
        $roomNo = $request->query('roomNo');
        // Perform your database query or any other necessary operations to get the number of seats
        $roomsDetails = DB::table('roomseats')->where('roomno', $roomNo)->first();
        // Create a response array with the number of seats
        $totalseats = array();
        for ($i = 1; $i <= $roomsDetails->numberseats; $i++) {
            $checkBookinSeats = DB::table('bookings')->where('roomno', $roomNo)->where("seatno", $i)->where('status', 1)->first();
            if (!$checkBookinSeats) {
                $totalseats[] = $i;
            }
        }
        $response = [
            'numberSeats' => $totalseats,
        ];
        return response()->json($response);
    }


    public function checkRoomSeats()
    {
        try {
            $roomSeats = Roomseat::count();
            return response()->json([
                'status' => true,
                'roomSeats' => $roomSeats,
            ], 200);
        } catch (Exception $e) {
            return response(["error" => "Error in checking Room Seats"], 500);
        }
    }

    public function currentTotalSeats()
    {
        try {
            $totalseats = DB::table('roomseats')->get()->sum("numberseats");
            return response()->json([
                'status' => true,
                'totalSeats' => $totalseats,
            ], 200);
        } catch (Exception $e) {
            return response(['message' => 'Something went wrong'], 500);
        }
    }
}
