<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Sale;
use Exception;
use App\Models\Studentregister;
use App\Models\Item;
use App\Models\Customsalesale;
use App\Models\Expense;
use App\Models\Recieve_amount;
use DB;
use Carbon\Carbon;
use App\Models\StudentElecticityRecords;
use App\Models\Permissions;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use UnexpectedValueException;
use Illuminate\Support\Facades\Log;
use App\Models\MessageManager;
use App\Models\RoomSeat;
use App\Models\ClientPartyTableForCustomeSale;



class SaleController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     * 
     * 
     */

    public function view(Request $request)
    {
        // get the current month start date and end date
        $start_date = Carbon::now()->format('Y-m-01');
        $end_date = Carbon::now()->format('Y-m-t');
        $currentRegSale = Sale::whereBetween('tpdate', [$start_date, $end_date])->sum('total');
        $currentCustomeSale = Customsalesale::whereBetween('tpdate', [$start_date, $end_date])->sum('total');
        $totalCurrentSale = $currentRegSale + $currentCustomeSale;
        $totalRegSale = Sale::sum('total');
        $totalCustomeSale = Customsalesale::sum('total');
        $totalSale = $totalRegSale + $totalCustomeSale;
        $totalRegDues = Sale::sum('remaining_amount');
        $currentRegDue = Sale::where('tpdate', [$start_date, $end_date])->sum('remaining_amount');
        $totalCustomSaleDue = Customsalesale::sum('remaining_amount');
        $currentCustomeSaleDue = Customsalesale::where('tpdate', [$start_date, $end_date])->sum('remaining_amount');
        $totalDue = $totalRegDues + $totalCustomSaleDue;
        $totalCurrentDue = $currentRegDue + $currentCustomeSaleDue;

        $salesTableData = Sale::join('studentregisters', 'sales.studentid', '=', 'studentregisters.id')
            ->select(
                'studentregisters.id as studentid',
                'studentregisters.candidatename',
                'studentregisters.phone',
                'studentregisters.roomno',
                DB::raw('MAX(sales.tpdate) as tpdate'),
                DB::raw('MAX(sales.invoicedate) as invoicedate'),
                DB::raw('SUM(sales.remaining_amount) as totalAmount'),
                DB::raw('SUM(sales.unused) as unusedAmount')
            )
            ->groupBy('studentregisters.id', 'studentregisters.candidatename')
            ->orderBy(DB::raw('COALESCE(MAX(sales.tpdate), MAX(sales.invoicedate))'), 'desc')
            ->get();

        // get all active room number
        $roomNumber = RoomSeat::select('roomno')->get();

        $latestInvoiceNo = $this->getLatestInvoiceNo();
        // custome sales party detula
        $customeSalePartDetails = $this->getCustomeSalePartDetails();

        return view('fronted.sale', compact('currentRegSale', 'currentCustomeSale', 'totalCurrentSale', 'totalRegSale', 'totalCustomeSale', 'totalSale', 'totalDue', 'totalCurrentDue', 'salesTableData', 'roomNumber','latestInvoiceNo','customeSalePartDetails'));
    }


    private function getLatestInvoiceNo()
    {
        try {
            // Get the latest invoice numbers from both tables
            $invoiceNo = Sale::orderBy('id', 'desc')->value('invoiceno');
            $customeInvoiceNo = Customsalesale::orderBy('id', 'desc')->value('invoiceno');

            // Handle null cases
            $invoiceNo = $invoiceNo ?? 0;
            $customeInvoiceNo = $customeInvoiceNo ?? 0;

            // Determine the maximum invoice number
            $latestInvoiceNo = max($invoiceNo, $customeInvoiceNo);
        } catch (Exception $e) {
            // Handle exceptions and set a default invoice number
            $latestInvoiceNo = 0;
        }

        return ++$latestInvoiceNo;
    }


    private function getCustomeSalePartDetails(){
        $customeSalePartDetails = ClientPartyTableForCustomeSale::all();
        return $customeSalePartDetails;
    }

   

    private function checkWhatsAppPermission(): bool
    {
        try {
            $permission = Permissions::where('permissionType', 'Desktop Whatsapp')->value('permission');
            return $permission ?? false; // Return the permission value or false if not found

        } catch (ModelNotFoundException $e) {
            return false; // Handle the case when the permission is not found

        } catch (Exception $e) {
            // Log or handle other exceptions
            return false;
        }
    }
    private function getBussinessDetailsForPaymentInvoice($user)
    {
        $profile = DB::table('businessregisters')->where('username', $user)->first();
        if ($profile) {

            $bussinessName = $profile->businessname;
            $bussinessemail = $profile->email;
            $mobile1 = $profile->mobile;
            $mobile2 = $profile->mobile_secondary;
            $address = $profile->address;
            $img = $profile->pic;
            $imgPath = file_exists(public_path('assets/uploads/' . $img)) ? asset('assets/uploads/' . $img) : asset('assets/images/logoimage.jpeg');

            $signatureImg = DB::table("uploadprofiles")
                ->select('upload_pdf')
                ->where('upload_name', 'Stamp With Signature')
                ->first();

            $signaturePath = file_exists(public_path('assets/uploads/pdf/' . $signatureImg->upload_pdf)) ? asset('assets/uploads/pdf/' . $signatureImg->upload_pdf) : null;
            $userdata = [
                'bussinessName' => $bussinessName,
                'bussinessemail' => $bussinessemail,
                'mobile1' => $mobile1,
                'mobile2' => $mobile2,
                'address' => $address,
                'imgPath' => $imgPath,
                'signaturePath' => $signaturePath,
                'whatsAppPermission' => $this->checkWhatsAppPermission()
            ];
            return $userdata;
        } else {
            return [];
        }
    }
    private function numberToWords($number)
    {
        $words = [
            'Zero',
            'One',
            'Two',
            'Three',
            'Four',
            'Five',
            'Six',
            'Seven',
            'Eight',
            'Nine',
            'Ten',
            'Eleven',
            'Twelve',
            'Thirteen',
            'Fourteen',
            'Fifteen',
            'Sixteen',
            'Seventeen',
            'Eighteen',
            'Nineteen'
        ];

        $tens = [
            '',
            '',
            'Twenty',
            'Thirty',
            'Forty',
            'Fifty',
            'Sixty',
            'Seventy',
            'Eighty',
            'Ninety'
        ];

        if ($number < 20) {
            return $words[$number];
        } elseif ($number < 100) {
            return $tens[(int)($number / 10)] . (($number % 10 !== 0) ? ' ' . $words[$number % 10] : '');
        } elseif ($number < 1000) {
            return $words[(int)($number / 100)] . ' Hundred' . (($number % 100 !== 0) ? ' and ' . $this->numberToWords($number % 100) : '');
        } elseif ($number < 100000) {
            return $this->numberToWords((int)($number / 1000)) . ' Thousand' . (($number % 1000 !== 0) ? ' ' . $this->numberToWords($number % 1000) : '');
        } elseif ($number < 10000000) {
            return $this->numberToWords((int)($number / 100000)) . ' Lakh' . (($number % 100000 !== 0) ? ' ' . $this->numberToWords($number % 100000) : '');
        } else {
            return 'Number out of range';
        }
    }

    private function getStudentDetailsById($studentId)
    {
        try {
            $student = Studentregister::findOrFail($studentId);
            $studentdata = [
                'name' => $student->candidatename,
                'MobileNo' => $student->phone,
                'address' => $student->fullAddress . "," . $student->state . "," . $student->pinno
            ];
            return $studentdata;
        } catch (Exception $e) {
            throw $e;
        }
    }

    private function getSalesDetatils($invoiceNo)
    {
        try {

            $sales = Sale::where("invoiceno", $invoiceNo)->first();
            if (empty($sales)) {
                throw new Exception("Sales Not Found for thise invoice");
            }
            return [
                "RoomNo" => $sales->roomno,
                'totalAmount' => $sales->total,
            ];
        } catch (Exception $e) {
            throw $e;
        }
    }
    public function getPaymentInDetailsForShare(Request $request)
    {
        try {
            // get the bussiness details 
            $businessDetails = $this->getBussinessDetailsForPaymentInvoice($request->session()->get('user'));
            //now get the payment In details 
            $paymentDetails = Recieve_amount::where('invoiceNo', $request->invoiceNo)
                ->where('receiveDate', $request->invoiceDate)
                ->first();
            if (empty($paymentDetails)) {
                throw new UnexpectedValueException('Payment Details not Found');
            }
            // get the details of Student 
            $studentDetails = $this->getStudentDetailsById($paymentDetails->candidateID);
            // get the actual room no of the this sale of the payment
            $actualRoomNoForThisSale = $this->getSalesDetatils($request->invoiceNo);

            return response()->json([
                'businessDetails' => $businessDetails,
                'studentDetails' => $studentDetails,
                'slaeDetails' => $actualRoomNoForThisSale,
                'paymenyInDetails' => [
                    'invoiceNo' => $paymentDetails->invoiceNo,
                    'invoicedate' => Carbon::parse($paymentDetails->receiveDate)->format('d-M-Y'),
                    'receiveAmount' => $paymentDetails->receiveAmount,
                    'paymentMode' => $paymentDetails->paymentMode,
                    'AmountInWords' => $this->numberToWords($paymentDetails->receiveAmount)
                ]
            ]);
        } catch (UnexpectedValueException $e) {
            return response()->json([
                'error' => $e->getMessage()
            ]);
        } catch (Exception $e) {
            return response()->json([
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function CustomeDeleteSales(Request $request)
    {
        $invoiceNo = $request['invoiceNo'];
        $deleteSale = Customsalesale::where('invoiceNo', $invoiceNo);
        if ($deleteSale->delete()) {
            return response()->json(['message' => "success"], 200);
        } else {
            return response()->json(['message' => 'error'], 400);
        }
    }
    public function customereceiveamount(Request $request)
    {
        $data = $request->all();
        $invoiceno = $data['invoiceNo'];
        $totalAmount = $data['totalAmount'];
        $receiveAmountTake = $data['receiveAmount'];
        $paymentMode = $data['paymentMode'];
        $DBTotalReceivedAmount = Customsalesale::select("receive_amount")->where("invoiceno", $invoiceno)->first();
        $receiveAmount = $DBTotalReceivedAmount->receive_amount + $receiveAmountTake;
        if ($totalAmount == $receiveAmount) {
            $pandingAmount = 0;
            $unused = 0;
        } else if ($totalAmount > $receiveAmount) {
            $pandingAmount = $totalAmount - $receiveAmount;
            $unused = 0;
        } else {
            $pandingAmount = 0;
            $unused = $receiveAmount - $totalAmount;
        }
        $data = Customsalesale::where("invoiceno", $invoiceno)->first();
        $data->receive_amount = $receiveAmount;
        $data->remaining_amount = $pandingAmount;
        $data->unused = $unused;
        $data->paymentMode = $paymentMode;
        if ($data->update()) {
            return response()->json(["message" => "success"]);
        } else {
            return response()->json(['error' => 'Somting Wrong'], 400);
        }
    }
    public function CustomeSaleDetails(Request $request)
    {
        $invoiceno = $request['invoiceNo'];
        $data = Customsalesale::where("invoiceno", $invoiceno)->get();
        if ($data) {
            foreach ($data as $row) {
                $id = $row->id;
                $roomno = $row->roomno;
                $invoiceNo = $row->invoiceno;
                $name = $row->name;
                $pandingAmount = $row->remaining_amount;
                $paymentMode = $row->paymentMode;
                $totalAmount = $row->total;
            }
            $data = [
                'id' => $id,
                'roomno' => $roomno,
                'invoiceno' => $invoiceNo,
                'name' => $name,
                'pandingAmount' => $pandingAmount,
                'paymentMode' => $paymentMode,
                'totalAmount' => $totalAmount
            ];
            return response()->json(["data" => $data]);
        } else {
            return response()->json(["message" => 'error']);
        }
    }
    public function CustomeEditsalesave(Request $request)
    {
        try {
            $requestData = $request->all();
            $saleFormData = $requestData['saleFormData'];
            $listItemData = $requestData['EditListItemData'];
            // form data Accessed by saleformdata
            $invoiceNo = $saleFormData['invoiceNo'] ?? null;
            $date = $saleFormData['date'] ?? null;
            $CandidateId = $saleFormData['CandidateId'] ?? null;
            $roomNoSelect = $saleFormData['roomNoSelect'] ?? null;
            $TotalAmount = $saleFormData['itemAmount'] ?? null;
            $ReceivedAmount = $saleFormData['ReceivedAmount'] ?? null;
            $paymentMode = $saleFormData['paymentMode'] ?? null;
            $candidateName = $saleFormData['candidateName'] ?? null;
            $candidateMobile = $saleFormData['mobile'] ?? null;
            $CandidateAddress = $saleFormData['address'] ?? null;
            // create array for item data
            $itemIDs = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            // Add more arrays for other properties as needed
            if (!empty($listItemData)) {
                foreach ($listItemData as $itemData) {
                    $itemIDs[] = $itemData['id'] ?? null;
                    $itemRates[] = $itemData['Rate'] ?? null;
                    $itemQuantities[] = $itemData['Quantity'] ?? null;
                    $itemTax[] = $itemData['Tax'] ?? null;
                    $itemDiscount[] = $itemData['Discount'] ?? null;
                    $itemAmount[] = $itemData['Amount'] ?? null;
                }
            }
            // Save the data to the database
            $remainingAmount = $TotalAmount - $ReceivedAmount;
            if ($remainingAmount >= 0) {
                $paymentStatus = 1;
            } else {
                $paymentStatus = 0;
            }
            $sale = Customsalesale::where('invoiceno', $invoiceNo)->first();
            $sale->roomno = (int) $roomNoSelect;
            $sale->name = $candidateName;
            $sale->mobile = $candidateMobile;
            $sale->address = $CandidateAddress;
            $sale->invoiceno = $invoiceNo;
            $sale->invoicedate = $date;
            $sale->tpdate = $date;
            $sale->item = implode(',', $itemIDs);
            $sale->quantity = implode(',', $itemQuantities);
            $sale->rate = implode(',', $itemRates);
            $sale->tax = implode(',', $itemTax);
            $sale->amount = implode(',', $itemAmount);
            $sale->discount = implode(',', $itemDiscount);
            $sale->receive_amount = $ReceivedAmount;
            $sale->total = $TotalAmount;
            $sale->paymentMode = $ReceivedAmount != 0  ? $paymentMode : null;
            if ($remainingAmount >= 0) {
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = (string) $remainingAmount;
                $sale->unused = 0;
            } else if ($remainingAmount < 0) {
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = 0;
                $sale->unused = abs($remainingAmount);
            }
            if ($sale->update()) {
                return response()->json(['message' => 'Data saved successfully']);
            } else {
                return response()->json(['message' => 'Failed to save data'], 500);
            }
        } catch (\Exception $e) {
            // Log the error or handle it as needed
            return response()->json(['message' => "somting Wrong"], 500);
        }
    }
    public function customesaleinfo(Request $request)
    {
        $invoiceNo = $request['invoiceNo'];
        $sales = Customsalesale::where('invoiceno', $invoiceNo)
            ->get();
        if ($sales->isNotEmpty()) {
            foreach ($sales as $sale) {
                $id = $sale->roomno;
                $candidatName = $sale->name;
                $candidatAdd = $sale->address;
                $candidatMobile = $sale->mobile;
                $customerId = $sale->id;
                $invoiceno = $sale->invoiceno;
                $invoicedate = $sale->invoicedate;
                $items = $sale->item;
                $quantitys = $sale->quantity;
                $rates =  $sale->rate;
                $taxs =  $sale->tax;
                $amounts =  $sale->amount;
                $discounts = $sale->discount;
                $receivedAmount = $sale->receive_amount;
                $totalAmount =    $sale->total;
                $remainingAmount = $sale->remaining_amount;
                $paymentMode = $sale->paymentMode;
                $unusedAmount = $sale->unused;
            };
            $itemArray = explode(',', $items);
            $quantityArray = explode(',', $quantitys);
            $ratesArray = explode(',', $rates);
            $taxsArray = explode(',', $taxs);
            $amountsArray = explode(',', $amounts);
            $discountsArray = explode(',', $discounts);
            foreach ($itemArray as $index => $item) {
                // fetch the item name 
                $itemname = Item::where('id', $item)->pluck('itemname')->first();
                $itemId = $itemArray[$index];
                if ($itemId == 0) {
                    // fetch the room rent
                    $itemObject[$itemId] = [
                        'itemName' => "Room Rent",
                        'quantity' => $quantityArray[$index],
                        'amount' => $amountsArray[$index],
                        'rate' => $ratesArray[$index],
                        'tax' => $taxsArray[$index],
                        'discount' => $discountsArray[$index],
                    ];
                } else {
                    $itemObject[$itemId] = [
                        'itemName' => $itemname,
                        'quantity' => $quantityArray[$index],
                        'amount' => $amountsArray[$index],
                        'rate' => $ratesArray[$index],
                        'tax' => $taxsArray[$index],
                        'discount' => $discountsArray[$index],
                    ];
                }
            }
            $data = [
                'RoomNo' => $id,
                'customerId' => $customerId,
                'candidatName' => $candidatName,
                'candidatAdd' => $candidatAdd,
                'candidatMobile' => $candidatMobile,
                'invoiceno' => $invoiceno,
                'invoicedate' => $invoicedate,
                'items' => $itemObject,
                'receivedAmount' => $receivedAmount,
                'totalAmount' => $totalAmount,
                'amountInWords' => $this->numberToWords($totalAmount),
                'remainingAmount' => $remainingAmount,
                'paymentMode' => $paymentMode,
                'unusedAmount' => $unusedAmount
            ];
            return response()->json(['saleData' => $data]);
        } else {
            return response()->json(['message' => 'Sale data not found'], 404);
        }
    }
    public function getCustomSaleDetails(Request $request)
    {
        try {
            $candidateID = $request['id'];
            $name = $request['name'];
            $totalReceivedAmount = 0;
            $totalUnusedAmount = 0;
            $totalremaining_amount = 0;

            $customSales = [];
            $candidateDetails = Customsalesale::where('mobile', $candidateID)
                ->where('name', $name)
                ->get();
            if ($candidateDetails->isNotEmpty()) {
                foreach ($candidateDetails as $row) {
                    $roomNo = $row->roomno;
                    $candidateName = $row->name;
                    $candidateMobile = $row->mobile;
                    $totalReceivedAmount += $row->receive_amount ?? 0;
                    $totalUnusedAmount += $row->unused ?? 0;
                    $totalremaining_amount += $row->remaining_amount ?? 0;
                    $customSales[] = [
                        "salesId" => $row->id,
                        'saleRoomno' => $row->roomno,
                        'invoiceNo' => $row->invoiceno,
                        'date' => $invoicedate = $row->tpdate,
                        'receive_amount' =>  $row->receive_amount ?? 0,
                        'remaining_amount' => $row->remaining_amount ?? 0,
                        'unused' => $row->unused,
                        'totalAmount' => $row->total ?? 0,
                    ];
                }
                $paymentsDetails = [
                    'candidateDetails' => [
                        'candidateId' => $candidateID,
                        'candidateName' => $candidateName,
                        'candidateMobile' => $candidateMobile,
                        'totalReceivedAmount' => $totalReceivedAmount,
                        'totalUnusedAmount' => $totalUnusedAmount,
                        'totalremaining_amount' => $totalremaining_amount
                    ],
                    'salesdetails' =>  $customSales,
                ];
                return response()->json(['ViewMoreData' => $paymentsDetails]);
            } else {
                return response()->json(['message' => 'sale not found']);
            }
        } catch (\Exception $e) {
            response()->json(['message' => "data not found"]);
        }
    }
    public function CustomDatasale(Request $request)
    {
        $roomNo = $request['roomNo'];
        $invoiceNo = $request['invoiceNo'];
        try {
            $sales = Customsalesale::where('invoiceno', $invoiceNo)
                ->get();
            if ($sales->isNotEmpty()) {
                foreach ($sales as $sale) {
                    $id = $sale->roomno;
                    $customerId = $sale->id;
                    $invoiceno = $sale->invoiceno;
                    $invoicedate = $sale->invoicedate;
                    $items = $sale->item;
                    $quantitys = $sale->quantity;
                    $rates =  $sale->rate;
                    $taxs =  $sale->tax;
                    $amounts =  $sale->amount;
                    $discounts = $sale->discount;
                    $receivedAmount = $sale->receive_amount;
                    $totalAmount =    $sale->total;
                    $remainingAmount = $sale->remaining_amount;
                    $candidatName = $sale->name;
                    $candidatAdd = $sale->address;
                    $candidatMobile = $sale->mobile;
                };
                $totalReceivedAmount = Customsalesale::where('id', $customerId)
                    ->sum('remaining_amount');
                $itemArray = explode(',', $items);
                $quantityArray = explode(',', $quantitys);
                $ratesArray = explode(',', $rates);
                $taxsArray = explode(',', $taxs);
                $amountsArray = explode(',', $amounts);
                $discountsArray = explode(',', $discounts);
                foreach ($itemArray as $index => $item) {
                    // fetch the item name 
                    $itemname = Item::where('id', $item)->pluck('itemname')->first();
                    $itemId = $itemArray[$index];
                    if ($itemId == 0) {
                        // fetch the room rent
                        $roomDetails = DB::table('roomseats')->where('roomno', $roomNo)->first();
                        $itemObject[$itemId] = [
                            'itemName' => "Room Rent",
                            'quantity' => $quantityArray[$index],
                            'amount' => $amountsArray[$index],
                            'rate' => $ratesArray[$index],
                            'tax' => $taxsArray[$index],
                            'discount' => $discountsArray[$index],
                        ];
                    } else {
                        $itemObject[$itemId] = [
                            'itemName' => $itemname,
                            'quantity' => $quantityArray[$index],
                            'amount' => $amountsArray[$index],
                            'rate' => $ratesArray[$index],
                            'tax' => $taxsArray[$index],
                            'discount' => $discountsArray[$index],
                        ];
                    }
                }
                $data = [
                    'RoomNo' => $id,
                    'customerId' => $customerId,
                    'candidatName' => $candidatName,
                    'candidatAdd' => $candidatAdd,
                    'candidatMobile' => $candidatMobile,
                    'invoiceno' => $invoiceNo,
                    'invoicedate' => $invoicedate,
                    'items' => $itemObject,
                    'receivedAmount' => $receivedAmount,
                    'totalAmount' => $totalAmount,
                    'remainingAmount' => $remainingAmount,
                    'amountInWords' => $this->numberToWords($totalAmount),
                    'TotalRemainingAmount' => $totalReceivedAmount,
                ];
                return response()->json(['saleData' => $data]);
            } else {
                return response()->json(['message' => 'Sale data not found'], 404);
            }
        } catch (\Exception $e) {
            return response()->json(['message' => $e], 404);
        }
    }
    public function customeSaleSave(Request $request)
    {
        try {
            $requestData = $request->all();
            $saleFormData = $requestData['saleFormData'];
            $listItemData = $requestData['ListItemData'];
            // form data Accessed by saleformdata
            $invoiceNo = $saleFormData['invoiceNo'] ?? null;
            $date = $saleFormData['date'] ?? null;
            $CandidateId = $saleFormData['CandidateId'] ?? null;
            $roomNoSelect = $saleFormData['roomNoSelect'] ?? null;
            $TotalAmount = $saleFormData['itemAmount'] ?? null;
            $ReceivedAmount = $saleFormData['ReceivedAmount'] ?? null;
            $paymentMode = $saleFormData['paymentMode'] ?? null;
            $candidateName = $saleFormData['candidateName'] ?? null;
            $candidateMobile = $saleFormData['mobile'] ?? null;
            $CandidateAddress = $saleFormData['address'] ?? null;
            // create array for item data
            $itemIDs = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            // Add more arrays for other properties as needed
            if (!empty($listItemData)) {
                foreach ($listItemData as $itemData) {
                    $itemIDs[] = $itemData['id'] ?? null;
                    $itemRates[] = $itemData['Rate'] ?? null;
                    $itemQuantities[] = $itemData['Quantity'] ?? null;
                    $itemTax[] = $itemData['Tax'] ?? null;
                    $itemDiscount[] = $itemData['Discount'] ?? null;
                    $itemAmount[] = $itemData['Amount'] ?? null;
                }
            }
            // Save the data to the database
            $remainingAmount = $TotalAmount - $ReceivedAmount;
            if ($remainingAmount >= 0) {
                $paymentStatus = 1;
            } else {
                $paymentStatus = 0;
            }
            $sale = new Customsalesale();
            $sale->roomno = (int) $roomNoSelect;
            $sale->name = $candidateName;
            $sale->mobile = $candidateMobile;
            $sale->address = $CandidateAddress;
            $sale->invoiceno = $invoiceNo;
            $sale->invoicedate = $date;
            $sale->tpdate = $date;
            $sale->item = implode(',', $itemIDs);
            $sale->quantity = implode(',', $itemQuantities);
            $sale->rate = implode(',', $itemRates);
            $sale->tax = implode(',', $itemTax);
            $sale->amount = implode(',', $itemAmount);
            $sale->discount = implode(',', $itemDiscount);
            $sale->receive_amount = $ReceivedAmount;
            $sale->total = $TotalAmount;
            $sale->paymentMode = $paymentMode;
            if ($remainingAmount >= 0) {
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = (string) $remainingAmount;
            } else if ($remainingAmount < 0) {
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = 0;
                $sale->unused = abs($remainingAmount);
            }
            if ($sale->save()) {
                return response()->json(['message' => 'Data saved successfully']);
            } else {
                return response()->json(['message' => 'Failed to save data'], 500);
            }
        } catch (\Exception $e) {
            // Log the error or handle it as needed
            return response()->json(['message' => "somting Wrong"], 500);
        }
    }
    public function PaymentInDelete(Request $request)
    {
        $candidateID = $request['candidateID'];
        $remainingAmountID = $request['remainingAmountID'];
        $incoiceNo = $request['incoiceNo'];
        $sale = Sale::where('invoiceno', $incoiceNo)
            ->select("total")
            ->first();
        $totalAmountDB = $sale->total;
        $unusedAmountDB = $sale->unused;
        if ($sale) {
            Recieve_amount::where('id', $remainingAmountID)
                ->delete();
            $ReceivedAmountTotalDB = Recieve_amount::where('invoiceNo', $incoiceNo)
                ->select(DB::raw('SUM(receiveAmount) as TotalReceivedAmount'))
                ->get();
            foreach ($ReceivedAmountTotalDB as $receiveAmountDb) {
                $ReceivedAmounts = $receiveAmountDb->TotalReceivedAmount;
            }
            if ($totalAmountDB <= $ReceivedAmounts) {
                $NewUnusedAmt = $ReceivedAmounts - $totalAmountDB;
                $NewRemainingAmt = 0;
                $NewPendingAmt = 0;
                $paymentStatus = 0;
            } else {
                $NewRemainingAmt = $totalAmountDB - $ReceivedAmounts;
                $NewUnusedAmt = 0;
                $NewPendingAmt = $NewRemainingAmt;
                $paymentStatus = 1;
            }
            $saleRecord = Sale::where('invoiceno', $incoiceNo)->first();

            if ($saleRecord) {
                // Update the sale record
                $saleRecord->update([
                    'receive_amount' => abs($ReceivedAmounts),
                    'unused' => $NewUnusedAmt,
                    'remaining_amount' => $NewRemainingAmt,
                    'payment_status' => $paymentStatus
                ]);
                // Check the conditions and update the electricity record if needed
                if ($saleRecord->item == "e01") {
                    $electricityRecord = StudentElecticityRecords::where('invoiceno', $saleRecord->invoiceno)->first();

                    if ($electricityRecord) {
                        $electricityRecord->update([
                            'status' => ($saleRecord->remaining_amount == 0) ? 2 : 1 // 2 = paid, 1 = unpaid
                        ]);
                    }
                }
            }
        }
    }
    public function getTotalBussinessDetails(Request $request)
    {
        $filterKeyword = $request['type'];
        $TotalAmount = 0;
        $currentMonth = date('m');
        $currentYear = date('Y');
        $totalExpenseAmount = 0;
        if ($filterKeyword == 'month') {
            $totalAmount = DB::table('sales')
                ->select(DB::raw('MONTH(invoicedate) AS month'), DB::raw('SUM(total) AS totalAmount'))
                ->groupBy(DB::raw('MONTH(invoicedate)'))
                ->get();
            $totalCustomeAmount = DB::table("customsalesales")
                ->select(DB::raw('MONTH(invoicedate) AS month'), DB::raw('SUM(total) AS totalAmount'))
                ->groupBy(DB::raw('MONTH(invoicedate)'))
                ->get();
            $totalExpense = DB::table('expenses')
                ->join('expensedetails', 'expenses.id', '=', 'expensedetails.category_id')
                ->select(DB::raw('SUM(expensedetails.amount) as totalExpenses'))
                ->whereMonth('billdate', '=', $currentMonth)
                ->get();
        } elseif ($filterKeyword == 'quarter' || $filterKeyword == 'Months') {
            $totalAmount = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS quarter'), DB::raw('SUM(total) AS totalAmount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'))
                ->get();
            $totalCustomeAmount = DB::table("customsalesales")
                ->select(DB::raw('YEAR(invoicedate) AS quarter'), DB::raw('SUM(total) AS totalAmount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'))
                ->get();
            $totalExpense = DB::table('expenses')
                ->join('expensedetails', 'expenses.id', '=', 'expensedetails.category_id')
                ->select(DB::raw('SUM(expensedetails.amount) as totalExpenses'))
                ->whereRaw('YEAR(billdate) = YEAR(CURRENT_DATE)')
                ->whereYear('billdate', '=', $currentYear)
                ->get();
        } elseif ($filterKeyword == 'year') {
            $totalAmount = DB::table('sales')
                ->select(DB::raw('SUM(total) AS totalAmount'))
                ->get();
            $totalCustomeAmount = DB::table("customsalesales")
                ->select(DB::raw('SUM(total) AS totalAmount '))
                ->get();
            $totalExpense = DB::table('expenses')
                ->join('expensedetails', 'expenses.id', '=', 'expensedetails.category_id')
                ->select(DB::raw('SUM(expensedetails.amount) as totalExpenses'))
                ->get();
        } elseif ($filterKeyword == 'custom') {
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];
            if ($startDate && $endDate) {
                $totalAmount = DB::table('sales')
                    ->select(DB::raw('SUM(total) as totalAmount'))
                    ->whereBetween('invoicedate', [$startDate, $endDate])
                    ->get();
                $totalCustomeAmount = DB::table("customsalesales")
                    ->select(DB::raw('SUM(total) AS totalAmount '))
                    ->whereBetween('invoicedate', [$startDate, $endDate])
                    ->get();
                $totalExpense = DB::table('expensedetails')
                    ->select(DB::raw('SUM(amount) as totalExpenses'))
                    ->whereBetween('billdate', [$startDate, $endDate])
                    ->get();
            }
        } else {
            $totalAmount = DB::table('sales')
                ->select(DB::raw('SUM(total) AS totalAmount'))
                ->get();
            $totalCustomeAmount = DB::table("customsalesales")
                ->select(DB::raw('SUM(total) AS totalAmount '))
                ->get();
            $totalExpense = DB::table('expenses')
                ->join('expensedetails', 'expenses.id', '=', 'expensedetails.category_id')
                ->select(DB::raw('SUM(expensedetails.amount) as totalExpenses'))
                ->get();
        }
        if ($totalAmount) {
            foreach ($totalAmount as $sale) {
                $TotalAmount = $sale->totalAmount;
            }
        } else {
            $TotalAmount = 0;
        }
        if (!empty($totalCustomeAmount)) {
            foreach ($totalCustomeAmount as $customeSale) {
                $TotalAmount += $customeSale->totalAmount;
            }
        }
        if ($totalExpense) {
            foreach ($totalExpense as $expens) {
                $totalExpenseAmount = $expens->totalExpenses;
            }
        } else {
            $totalExpenseAmount = 0;
        }
        return response()->json([$totalExpenseAmount, $TotalAmount]);
    }
    public function getBussinessDetails(Request $request)
    {
        $value = $request->session()->get('user');
        $profile = DB::table('businessregisters')->where('username', $value)->get();
        if ($profile) {
            foreach ($profile as $details) {
                $bussinessName = $details->businessname;
                $bussinessemail = $details->email;
                $mobile1 = $details->mobile;
                $mobile2 = $details->mobile_secondary;
                $address = $details->address;
                $img = $details->pic;
                $imgPath = asset('assets/uploads/' . $img);
            }
            // terms and condition
            $termsAndConditions = DB::table('termsandconditions')
                ->select('termsandconditions')
                ->get();
            $terms = [];
            foreach ($termsAndConditions as $row) {
                $terms[] = $row->termsandconditions;
            }
            $signatureImg = DB::table("uploadprofiles")
                ->select('upload_pdf')
                ->where('upload_name', 'Stamp With Signature')
                ->first();
            $signaturePath = asset('assets/uploads/pdf/' . $signatureImg->upload_pdf);

            // get the message template 

            $messageTemplate =  MessageManager::where('msg_type', 'payment_pre_reminder')->value('msg');

            $userdata = [
                'bussinessName' => $bussinessName,
                'bussinessemail' => $bussinessemail,
                'mobile1' => $mobile1,
                'mobile2' => $mobile2,
                'address' => $address,
                'imgPath' => $imgPath,
                'terms' => $terms,
                'signaturePath' => $signaturePath,
                'whatsAppPermission' => $this->checkWhatsAppPermission(),
                'messageTemplate' => $messageTemplate,
            ];
            return response()->json(['userData' => $userdata]);
        } else {
            return response()->json(['message' => "error"]);
        }
    }
    public function paymentInUpdate(Request $request)
    {
        $candidateID = $request['candidateID'];
        $remainingAmountID = $request['remainingAmountID'];
        $incoiceNo = $request['incoiceNo'];
        $receivedAmount = $request['receivedAmount'];
        $date = $request['date'];
        $paymentMode = $request['paymentMode'];
        $sale = Sale::where('invoiceno', $incoiceNo)
            ->first();
        $totalAmountDB = $sale->total;
        $remainingAmountDB = $sale->remaining_amount;
        $recvAmountDB = $sale->receive_amount;
        $unusedAmountDB = $sale->unused;
        if ($sale) {
            Recieve_amount::where('id', $remainingAmountID)
                ->update(
                    [
                        'receiveAmount' => $receivedAmount,
                        'receiveDate' => $date,
                        'paymentMode' => $paymentMode,
                        // 'unused' => $NewUnusedAmt,
                        // 'pandingAmount' => $NewPendingAmt,
                    ]
                );
            $ReceivedAmountTotalDB = Recieve_amount::where('invoiceNo', $incoiceNo)
                ->select(DB::raw('SUM(receiveAmount) as TotalReceivedAmount'))
                ->get();
            foreach ($ReceivedAmountTotalDB as $receiveAmountDb) {
                $ReceivedAmounts = $receiveAmountDb->TotalReceivedAmount;
            }
            if ($totalAmountDB <= $ReceivedAmounts) {
                $NewUnusedAmt = $ReceivedAmounts - $totalAmountDB;
                $NewRemainingAmt = 0;
                $NewPendingAmt = 0;
            } else {
                $NewRemainingAmt = $totalAmountDB - $ReceivedAmounts;
                $NewUnusedAmt = 0;
                $NewPendingAmt = $NewRemainingAmt;
            }
            $saleRecord = Sale::where('invoiceno', $incoiceNo)->first();

            if ($saleRecord) {
                // Update the sale record
                $saleRecord->update([
                    'receive_amount' => abs($ReceivedAmounts),
                    'unused' => $NewUnusedAmt,
                    'remaining_amount' => $NewRemainingAmt,
                ]);
                // Check the conditions and update the electricity record if needed
                $electricityRecord = StudentElecticityRecords::where('invoiceno', $saleRecord->invoiceno)->first();
                if ($saleRecord->item == "e01" && $saleRecord->remaining_amount == 0) {
                    if ($electricityRecord) {
                        $electricityRecord->update([
                            'status' => 2
                        ]);
                    }
                } else {
                    if ($electricityRecord) {
                        $electricityRecord->update([
                            'status' => 1
                        ]);
                    }
                }
            }
            return response()->json([
                'message' => 'payment In Updated'
            ]);
        } else {
            return response()->json([
                'error' => 'Sale Records not Found'
            ], 500);
        }
    }
    public function getPaymentInDetailsEdit(Request $request)
    {
        $remaingAmountId = $request['remaningID'];
        $RemaingData = Recieve_amount::where('id', $remaingAmountId)->get();
        if ($RemaingData) {
            foreach ($RemaingData as $data) {
                $invoiceNo = $data->invoiceNo;
                $pandingAmount = $data->pandingAmount;
                $receiveAmount = $data->receiveAmount;
                $date = $data->receiveDate;
                $candidateID = $data->candidateID;
                $paymentMode = $data->paymentMode;
            }
            $data = [
                'id' => $remaingAmountId,
                'invoiceNo' => $invoiceNo,
                'pandingAmount' => $pandingAmount,
                'receiveAmount'  => $receiveAmount,
                'date'  => $date,
                'candidateID'  => $candidateID,
                'paymentMode' => $paymentMode
            ];
            return response()->json(['PaymentIndata' => $data]);
        } else {
            return response()->json(['message' => 'error']);
        }
    }
    private function checkElectricityRecords($invoiceNo)
    {
        try {
            $electricityRecords = StudentElecticityRecords::where("invoiceno", $invoiceNo);

            if ($electricityRecords->count() > 0) {
                $electricityRecords->update([
                    'status' => 0,
                    'invoiceno' => null,
                ]);
                return true;
            } else {
                return true;
            }
        } catch (Exception $e) {
            // Return false in case of an exception
            return false;
        }
    }
    public function DeleteSales(Request $request)
    {
        $invoiceNo = $request['invoiceNo'];
        $deleteSale = Sale::where('invoiceNo', $invoiceNo);
        if ($deleteSale->delete()) {
            $deleteReceiveAmoutsRecords = Recieve_amount::where("invoiceNo", $invoiceNo);
            if ($deleteReceiveAmoutsRecords->count() > 0) {
                $deleteReceiveAmoutsRecords->delete();
            }
            if ($this->checkElectricityRecords($invoiceNo)) {
                return response()->json(['message' => 'success']);
            } else {
                return response()->json(['message' => 'somting wronge'], 200);
            }
        } else {
            return response()->json(['message' => 'error'], 500);
        }
    }
    public function saleinfo(Request $request)
    {
        $invoiceNo = $request['invoiceNo'];
        $sales = Sale::where('invoiceno', $invoiceNo)
            ->get();

        $ReceivedAmount = Recieve_amount::where("invoiceNo", $invoiceNo)
            ->orderBy("receiveDate", "DESC")
            ->first();

        $paymentMode = $ReceivedAmount ? $ReceivedAmount->paymentMode : Null;

        if ($sales->isNotEmpty()) {
            foreach ($sales as $sale) {
                $id = $sale->roomno;
                $customerId = $sale->studentid;
                $invoiceno = $sale->invoiceno;
                $invoicedate = $sale->invoicedate;
                $items = $sale->item;
                $quantitys = $sale->quantity;
                $rates =  $sale->rate;
                $taxs =  $sale->tax;
                $amounts =  $sale->amount;
                $discounts = $sale->discount;
                $receivedAmount = $sale->receive_amount;
                $totalAmount =    $sale->total;
                $remainingAmount = $sale->remaining_amount;
                $unusedAmount = $sale->unused;
            };
            $candidatDetails = Studentregister::where("id", $customerId)->get();
            if ($candidatDetails->isNotEmpty()) {
                foreach ($candidatDetails as $candidat) {
                    $candidatName = $candidat->candidatename;
                    $candidatAdd = $candidat->fullAddress . "," . $candidat->state . "," . $candidat->pinno;
                    $candidatMobile = $candidat->phone;
                }
            }
            $itemArray = explode(',', $items);
            $quantityArray = explode(',', $quantitys);
            $ratesArray = explode(',', $rates);
            $taxsArray = explode(',', $taxs);
            $amountsArray = explode(',', $amounts);
            $discountsArray = explode(',', $discounts);
            $itemObject = [];
            foreach ($itemArray as $index => $item) {
                // fetch the item name 
                $itemname = Item::where('id', $item)->pluck('itemname')->first();
                $itemId = $itemArray[$index];
                if ($item == 0) {
                    // fetch the room rent
                    $itemObject[] = [
                        'itemName' => "Room Rent",
                        'quantity' => $quantityArray[$index],
                        'amount' => $amountsArray[$index],
                        'rate' => $ratesArray[$index],
                        'tax' => $taxsArray[$index],
                        'discount' => $discountsArray[$index],
                    ];
                } elseif ($item == "e01") {
                    $electricityDate = StudentElecticityRecords::where("invoiceno", $invoiceno)->first();
                    if ($electricityDate) {
                        $itemObject[] = [
                            'itemName' => "Electricity Bill( " . Carbon::parse($electricityDate->billDate)->format("d-M-Y") . " )",
                            'quantity' => $quantityArray[$index],
                            'amount' => $amountsArray[$index],
                            'rate' => $ratesArray[$index],
                            'tax' => $taxsArray[$index],
                            'discount' => $discountsArray[$index],
                        ];
                    }
                } else {
                    $itemObject[] = [
                        'itemName' => $itemname,
                        'quantity' => $quantityArray[$index],
                        'amount' => $amountsArray[$index],
                        'rate' => $ratesArray[$index],
                        'tax' => $taxsArray[$index],
                        'discount' => $discountsArray[$index],
                    ];
                }
            }
            $data = [
                'RoomNo' => $id,
                'customerId' => $customerId,
                'candidatName' => $candidatName,
                'candidatAdd' => $candidatAdd,
                'candidatMobile' => $candidatMobile,
                'invoiceno' => $invoiceno,
                'invoicedate' => $invoicedate,
                'items' => $itemObject,
                'receivedAmount' => $receivedAmount,
                'totalAmount' => $totalAmount,
                'amountInWords' => $this->numberToWords($totalAmount),
                'remainingAmount' => $remainingAmount,
                'paymentMode' => $paymentMode,
                'unusedAmount' => $unusedAmount
            ];
            return response()->json(['saleData' => $data]);
        } else {
            return response()->json(['message' => 'Sale data not found'], 404);
        }
    }
    public function getPaymentDetails(Request $request)
    {
        try {
            $candidateID = $request['studentId'];
            $candidateDetails = Studentregister::where('id', $candidateID)->get();
            if ($candidateDetails->isNotEmpty()) {
                foreach ($candidateDetails as $details) {
                    $regDate = !is_null($details->join_date) ? Carbon::parse($details->join_date)  : Carbon::parse($details->date);
                    $regDate->month(now()->month);
                    $regDate->year(now()->year);
                    $updatedDate = $regDate->format('Y-m-d');
                    $candidateName = $details->candidatename;
                    $candidateMobile = $details->phone;
                    $roomNo = $details->roomno;
                    $CandidateAddress = $details->fullAddress . "," . $details->state . "," . $details->pinno;
                }
                $remainingAmountTotal = DB::table('sales')->where('studentid', $candidateID)
                    ->select(DB::raw('SUM(sales.remaining_amount) as receiveAmount'), DB::raw('SUM(sales.unused) as unsedAmount'))
                    ->get();
                foreach ($remainingAmountTotal as $sale) {
                    $TotalReceived = $sale->receiveAmount;
                    $TotalUnused = $sale->unsedAmount;
                }
                $salesDetails = Sale::where("studentid", $candidateID)
                    ->orderBy("invoiceno", "desc")
                    ->get();
                if ($salesDetails) {
                    $salesData = [];
                    $itemObject = [];
                    foreach ($salesDetails as $sales) {
                        $salesData[$sales->invoiceno][] = [
                            'salesRoomNo' => $sales->roomno,
                            'amountInWords' => $this->numberToWords($sales->total),
                            'saleDate' => $sales->invoicedate,
                            'total' => $sales->total,
                            'receiveAmount' => $sales->receive_amount,
                            'pandingAmount' => $sales->remaining_amount,
                            'unused' => $sales->unused
                        ];
                        $candidateSaleDetails = Recieve_amount::where("invoiceNo", $sales->invoiceno)
                            ->get();
                        if ($candidateSaleDetails->isNotEmpty()) {
                            foreach ($candidateSaleDetails as $saleDetails) {
                                $itemObject[$sales->invoiceno][] = [
                                    'receivedId' => $saleDetails->id,
                                    'receiveDate' => $saleDetails->receiveDate,
                                    'unusedamount' => $saleDetails->unused,
                                    'receiveAmount' => $saleDetails->receiveAmount,
                                ];
                            }
                        }
                    }
                    $paymentsDetails = [
                        'roomNo' =>  $roomNo,
                        'candidateId' => $candidateID,
                        'candidateName' => $candidateName,
                        'candidateMobile' => $candidateMobile,
                        'candidateAddress' => $CandidateAddress,
                        'regdate' => $updatedDate,
                        'TotalReceived' => $TotalReceived,
                        'TotalUnused' => $TotalUnused,
                        'salesDetails' => $salesData,
                        'receivedData' => $itemObject,
                    ];
                    return response()->json(['ViewMoreData' => $paymentsDetails]);
                } else {
                    return response()->json(['message' => 'sale not found']);
                }
            }
        } catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage()], 500);
        }
    }
    public function getPaymentInDetails(Request $request)
    {
        // get info for the payment details 
        $invoiceNo = $request['invoiceNo'];
        $salesDetails = Sale::where('invoiceno', $invoiceNo)->get();
        if ($salesDetails->isNotEmpty()) {
            foreach ($salesDetails as $temp) {
                $Id =  $temp->studentid;
                $roomNo = $temp->roomno;
                $remaingAmount = $temp->remaining_amount;
            };
            $studentName = Studentregister::where('id', $Id)->pluck('candidatename')->first();
            $paymentInData = [
                'Id' => $Id,
                'invoiceNo' => $invoiceNo,
                'roomNo' => $roomNo,
                'remaingAmount' => $remaingAmount,
                'name' => $studentName,
            ];
            return response()->json(['data' => $paymentInData]);
        } else {
            return response()->json(['message', $salesDetails]);
        }
    }
    public function saleDetails(Request $request)
    {
        $customerId = $request['customerId'];
        $paymentDetails = Sale::where('studentid', $customerId)
            ->where('payment_status', '<>', 0)
            ->get();
        if ($paymentDetails->isNotEmpty()) {
            $paymentIds  = []; // Create a new collection object
            foreach ($paymentDetails as $payment) {
                $paymentIds[] = $payment->invoiceno;
            }
            return response()->json(['paymentData' => $paymentIds]);
        } else {
            return response()->json(['message' => 'Sale data not found'], 404);
        }
    }
    public function getremainingamount(Request $request)
    {
        $invoiceNo = $request['invoiceNo'];
        $candidatID = $request['candidateID'];
        $paymentDetails = Sale::where('studentid', $candidatID)
            ->where('invoiceno', $invoiceNo)
            ->where('payment_status', '<>', 0)
            ->get();
        if ($paymentDetails->isNotEmpty()) {
            $remainingAmount  = []; // Create a new collection object
            foreach ($paymentDetails as $payment) {
                $remainingAmount[] = $payment->remaining_amount;
            }
            return response()->json(['paymentData' => $remainingAmount]);
        } else {
            return response()->json(['message' => 'Sale data not found'], 404);
        }
    }
    public function receiveamount(Request $request)
    {
        try {
            $requestData = $request->all();
            $receiveDate = $requestData['receiveDate'];
            $candidateID = $requestData['candidateId'];
            $invoiceNo = $requestData['invoiceNo'];
            $pandingAmount = $requestData['pandingAmount'];
            $receiveAmount = $requestData['receiveAmount'];
            $paymentMode = $requestData['paymentMode'];
            $sale = Sale::where('invoiceno', $invoiceNo)
                ->first();
            $totalAmountDB = $sale->total;
            $remainingAmountDB = $sale->remaining_amount;
            $recvAmountDB = $sale->receive_amount;
            $unusedAmountDB = $sale->unused;
            if ($sale) {
                $Recieve_amount = new Recieve_amount();
                $Recieve_amount->receiveDate = $receiveDate;
                $Recieve_amount->candidateID = $candidateID;
                $Recieve_amount->invoiceNo = $invoiceNo;
                $Recieve_amount->receiveAmount = $receiveAmount;
                $Recieve_amount->receiveDate = $receiveDate;
                $Recieve_amount->paymentMode = $paymentMode;
                $Recieve_amount->save();
                $ReceivedAmountTotalDB = Recieve_amount::where('invoiceNo', $invoiceNo)
                    ->select(DB::raw('SUM(receiveAmount) as TotalReceivedAmount'))
                    ->get();
                foreach ($ReceivedAmountTotalDB as $receiveAmountDb) {
                    $ReceivedAmounts = $receiveAmountDb->TotalReceivedAmount;
                }
                if ($totalAmountDB <= $ReceivedAmounts) {
                    $NewUnusedAmt = $ReceivedAmounts - $totalAmountDB;
                    $NewRemainingAmt = 0;
                    $NewPendingAmt = 0;
                } else {
                    $NewRemainingAmt = $totalAmountDB - $ReceivedAmounts;
                    $NewUnusedAmt = 0;
                    $NewPendingAmt = $NewRemainingAmt;
                }
                // Retrieve the sale record
                $saleRecord = Sale::where('invoiceno', $invoiceNo)->first();

                if ($saleRecord) {
                    // Update the sale record
                    $saleRecord->update([
                        'receive_amount' => abs($ReceivedAmounts),
                        'unused' => $NewUnusedAmt,
                        'remaining_amount' => $NewRemainingAmt,
                    ]);
                    // Check the conditions and update the electricity record if needed
                    if ($saleRecord->item == "e01" && $saleRecord->remaining_amount == 0) {
                        $electricityRecord = StudentElecticityRecords::where('invoiceno', $saleRecord->invoiceno)->first();

                        if ($electricityRecord) {
                            $electricityRecord->update([
                                'status' => 2
                            ]);
                        }
                    }
                }
                return response()->json(['message' => 'Data saved successfully']);
            }
        } catch (Exception $e) {
            return response()->json(['error' => 'somthing went wronge'], 500);
        }
    }
    public function index(Request $request)
    {
        $roomNo = $request['roomNo'];
        $invoiceNo = $request['invoiceNo'];
        try {
            $sales = Sale::where('invoiceno', $invoiceNo)
                ->get();
            if ($sales->isNotEmpty()) {
                foreach ($sales as $sale) {
                    $id = $sale->roomno;
                    $customerId = $sale->studentid;
                    $invoiceno = $sale->invoiceno;
                    $invoicedate = $sale->invoicedate;
                    $items = $sale->item;
                    $quantitys = $sale->quantity;
                    $rates =  $sale->rate;
                    $taxs =  $sale->tax;
                    $amounts =  $sale->amount;
                    $discounts = $sale->discount;
                    $receivedAmount = $sale->receive_amount;
                    $totalAmount =    $sale->total;
                    $remainingAmount = $sale->remaining_amount;
                };
                $totalReceivedAmount = Sale::where('studentid', $customerId)
                    ->sum('remaining_amount');
                $candidatDetails = Studentregister::where("id", $customerId)->get();
                if ($candidatDetails->isNotEmpty()) {
                    foreach ($candidatDetails as $candidat) {
                        $candidatName = $candidat->candidatename;
                        $candidatAdd = $candidat->fullAddress . "," . $candidat->state . "," . $candidat->pinno;
                        $candidatMobile = $candidat->phone;
                    }
                }
                $itemArray = explode(',', $items);
                $quantityArray = explode(',', $quantitys);
                $ratesArray = explode(',', $rates);
                $taxsArray = explode(',', $taxs);
                $amountsArray = explode(',', $amounts);
                $discountsArray = explode(',', $discounts);
                foreach ($itemArray as $index => $item) {
                    // fetch the item name 
                    $itemname = Item::where('id', $item)->pluck('itemname')->first();
                    $itemId = $itemArray[$index];
                    if ($itemId == 0) {
                        // fetch the room rent
                        $roomDetails = DB::table('roomseats')->where('roomno', $roomNo)->first();
                        $itemObject[] = [
                            'itemName' => "Room Rent",
                            'quantity' => $quantityArray[$index],
                            'amount' => $amountsArray[$index],
                            'rate' => $ratesArray[$index],
                            'tax' => $taxsArray[$index],
                            'discount' => $discountsArray[$index],
                        ];
                    } elseif ($itemId == "e01") {
                        $electricityDate = StudentElecticityRecords::where("invoiceno", $invoiceno)->first();
                        if ($electricityDate) {
                            $itemObject[$itemId] = [
                                'itemName' => "Electricity Bill( " . Carbon::parse($electricityDate->billDate)->format("d-M-Y") . " )",
                                'quantity' => $quantityArray[$index],
                                'amount' => $amountsArray[$index],
                                'rate' => $ratesArray[$index],
                                'tax' => $taxsArray[$index],
                                'discount' => $discountsArray[$index],
                            ];
                        }
                    } else {
                        $itemObject[$itemId] = [
                            'itemName' => $itemname,
                            'quantity' => $quantityArray[$index],
                            'amount' => $amountsArray[$index],
                            'rate' => $ratesArray[$index],
                            'tax' => $taxsArray[$index],
                            'discount' => $discountsArray[$index],
                        ];
                    }
                }
                $data = [
                    'RoomNo' => $id,
                    'customerId' => $customerId,
                    'candidatName' => $candidatName,
                    'candidatAdd' => $candidatAdd,
                    'candidatMobile' => $candidatMobile,
                    'invoiceno' => $invoiceNo,
                    'invoicedate' => $invoicedate,
                    'items' => $itemObject,
                    'receivedAmount' => $receivedAmount,
                    'totalAmount' => $totalAmount,
                    'amountInWords' => $this->numberToWords($totalAmount),
                    'remainingAmount' => $remainingAmount,
                    'TotalRemainingAmount' => $totalReceivedAmount,
                ];
                return response()->json(['saleData' => $data]);
            } else {
                return response()->json(['message' => 'Sale data not found'], 404);
            }
        } catch (\Exception $e) {
            return response()->json(['message' => $e->getMessage()], 404);
        }
    }

    private function checkAndUpdateElectricityBill($itemList, $candidateId, $invoiceNo, $remainingAmount = null)
    {
        try {
            foreach ($itemList as $key => $value) {
                if ($value['id'] == "e01") {
                    $month = $value['billDate'] ?? null;
                    $amount = $value['Amount'] ?? null;

                    if (!$month) {
                        $studentElectricityRecords = StudentElecticityRecords::where('invoiceno', $invoiceNo)->first();
                    } else {
                        $studentElectricityRecords = StudentElecticityRecords::where("billDate", $month)
                            ->where("userId", $candidateId)
                            ->where("status", 0)
                            ->first();
                    }

                    if ($studentElectricityRecords) {
                        // Log the values for debugging
                        Log::info('Updating Student Electricity Record:', [
                            'remainingAmount' => $remainingAmount,
                            'currentStatus' => $studentElectricityRecords->status,
                        ]);

                        // Set the status based on the remaining amount
                        $studentElectricityRecords->status = ($remainingAmount !== null && $remainingAmount === 0) ? 2 : 1;

                        // Log the new status
                        Log::info('New Status:', [
                            'newStatus' => $studentElectricityRecords->status,
                        ]);

                        // Update the invoice number and save the record
                        $studentElectricityRecords->invoiceno = $invoiceNo;
                        $studentElectricityRecords->save();
                    }
                }
            }

            // Log final values for debugging
            Log::info('Final Remaining Amount and Status:', [
                'remainingAmount' => $remainingAmount,
                'computedStatus' => ($remainingAmount !== null && $remainingAmount === 0) ? 2 : 1,
            ]);
        } catch (Exception $e) {
            // Log the exception
            Log::error('Error in checkAndUpdateElectricityBill:', ['exception' => $e]);

            // Rethrow the exception
            throw $e;
        }
    }



    public function store(Request $request)
    {
        try {
            $requestData = $request->all();
            $saleFormData = $requestData['saleFormData'];
            $listItemData = $requestData['ListItemData'];
            // form data Accessed by saleformdata
            $invoiceNo = $saleFormData['invoiceNo'] ?? null;
            $date = $saleFormData['date'] ?? null;
            $CandidateId = $saleFormData['CandidateId'] ?? null;
            $roomNoSelect = $saleFormData['roomNoSelect'] ?? null;
            $TotalAmount = $saleFormData['itemAmount'] ?? null;
            $ReceivedAmount = $saleFormData['ReceivedAmount'] ?? null;
            $paymentMode = $saleFormData['paymentMode'] ?? null;

            function insertData($receiveDate, $candidateID, $invoiceNo, $pandingAmount, $receiveAmount, $paymentMode)
            {
                $Recieve_amount = new Recieve_amount();
                $Recieve_amount->receiveDate = $receiveDate;
                $Recieve_amount->candidateID = $candidateID;
                $Recieve_amount->invoiceNo = $invoiceNo;
                if ($pandingAmount >= 0) {
                    $Recieve_amount->pandingAmount = $pandingAmount;
                } else {
                    $Recieve_amount->unused = abs($pandingAmount);
                    $Recieve_amount->pandingAmount = 0;
                }
                $Recieve_amount->receiveAmount = $receiveAmount;
                $Recieve_amount->paymentMode = $paymentMode;
                if ($Recieve_amount->save()) {
                    return response()->json(['message' => 'Data saved successfully']);
                } else {
                    return response()->json(['message' => 'Failed to save data'], 500);
                }
            }
            // create array for item data
            $itemIDs = [];
            $itemRates = [];
            $itemQuantities = [];
            $itemTax = [];
            $itemDiscount = [];
            $itemAmount = [];
            // Add more arrays for other properties as needed
            if (!empty($listItemData)) {
                foreach ($listItemData as $itemData) {
                    $itemIDs[] = $itemData['id'] ?? null;
                    $itemRates[] = $itemData['Rate'] ?? null;
                    $itemQuantities[] = $itemData['Quantity'] ?? null;
                    $itemTax[] = $itemData['Tax'] ?? null;
                    $itemDiscount[] = $itemData['Discount'] ?? null;
                    $itemAmount[] = $itemData['Amount'] ?? null;
                }
            }
            // Save the data to the database
            $remainingAmount = $TotalAmount - $ReceivedAmount;
            if ($remainingAmount >= 0) {
                $paymentStatus = 1;
            } else {
                $paymentStatus = 0;
            }
            if ($remainingAmount >= 0) {
                // update the received amount table
                $sale = new Sale();
                $sale->StudentId = (int) $CandidateId;
                $sale->roomno = (int) $roomNoSelect;
                $sale->invoiceno = $invoiceNo;
                $sale->invoicedate = $date;
                $sale->tpdate = $date;
                $sale->item = implode(',', $itemIDs);
                $sale->quantity = implode(',', $itemQuantities);
                $sale->rate = implode(',', $itemRates);
                $sale->tax = implode(',', $itemTax);
                $sale->amount = implode(',', $itemAmount);
                $sale->discount = implode(',', $itemDiscount);
                $sale->receive_amount = $ReceivedAmount;
                $sale->total = $TotalAmount;
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = (string) $remainingAmount;
                if ($ReceivedAmount != 0) {
                    insertData($date, $CandidateId, $invoiceNo, $remainingAmount, $ReceivedAmount, $paymentMode);
                }
            } else if ($remainingAmount < 0) {
                $sale = new Sale();
                $sale->StudentId = (int) $CandidateId;
                $sale->roomno = (int) $roomNoSelect;
                $sale->invoiceno = $invoiceNo;
                $sale->invoicedate = $date;
                $sale->item = implode(',', $itemIDs);
                $sale->quantity = implode(',', $itemQuantities);
                $sale->rate = implode(',', $itemRates);
                $sale->tax = implode(',', $itemTax);
                $sale->amount = implode(',', $itemAmount);
                $sale->discount = implode(',', $itemDiscount);
                $sale->receive_amount = $ReceivedAmount;
                $sale->total = $TotalAmount;
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = 0;
                $sale->unused = abs($remainingAmount);
                insertData($date, $CandidateId, $invoiceNo, $remainingAmount, $ReceivedAmount, $paymentMode);
            }
            if ($sale->save()) {
                $checkElectricityRecords = $this->checkAndUpdateElectricityBill($listItemData, $CandidateId, $invoiceNo, $sale->remaining_amount);
                return response()->json(['message' => 'Data saved successfully']);
            } else {
                return response()->json(['message' => 'Failed to save data'], 500);
            }
        } catch (\Exception $e) {
            // Log the error or handle it as needed
            return response()->json(['message' => $e->getMessage()], 500);
        }
    }
    public function Editsalesave(Request $request)
    {
        $requestData = $request->all();
        $saleFormData = $requestData['saleFormData'];
        $listItemData = $requestData['EditListItemData'];
        // form data Accessed by saleformdata
        $invoiceNo = $saleFormData['invoiceNo'] ?? null;
        $date = $saleFormData['date'] ?? null;
        $CandidateId = $saleFormData['CandidateId'] ?? null;
        $roomNoSelect = $saleFormData['roomNoSelect'] ?? null;
        $TotalAmount = $saleFormData['itemAmount'] ?? null;
        $ReceivedAmount = $saleFormData['ReceivedAmount'] ?? null;
        // $oldReceived = $saleFormData['oldReceived'] ?? null;
        $paymentMode = $saleFormData['paymentMode'] ?? null;
        // $unusedAmt = $saleFormData['unusedAmt'] ?? null;

        $deleteReceivedAmountRecords = DB::table("recieve_amounts")->where("candidateID", $CandidateId)->where('invoiceNo', $invoiceNo)
            ->delete();
        $checkElectricityRecords = StudentElecticityRecords::where("invoiceno", $invoiceNo)->count();


        function insertData($receiveDate, $candidateID, $invoiceNo, $pandingAmount, $receiveAmount, $paymentMode, $UnusedAmountReceved)
        {
            $Recieve_amount = new Recieve_amount();
            $Recieve_amount->receiveDate = $receiveDate;
            $Recieve_amount->candidateID = $candidateID;
            $Recieve_amount->invoiceNo = $invoiceNo;
            if ($UnusedAmountReceved == 0) {
                $Recieve_amount->pandingAmount = abs($pandingAmount);
            } else {
                $Recieve_amount->unused = abs($UnusedAmountReceved);
                $Recieve_amount->pandingAmount = 0;
            }
            $Recieve_amount->receiveAmount = $receiveAmount;
            $Recieve_amount->paymentMode = $paymentMode;
            if ($Recieve_amount->save()) {
                return response()->json(['message' => 'Data saved successfully']);
            } else {
                return response()->json(['message' => 'Failed to save data'], 500);
            }
        }
        // create array for item data
        $itemIDs = [];
        $itemRates = [];
        $itemQuantities = [];
        $itemTax = [];
        $itemDiscount = [];
        $itemAmount = [];
        // Add more arrays for other properties as needed
        if (!empty($listItemData)) {
            foreach ($listItemData as $itemData) {
                $itemIDs[] = $itemData['id'] ?? null;
                $itemRates[] = $itemData['Rate'] ?? null;
                $itemQuantities[] = $itemData['Quantity'] ?? null;
                $itemTax[] = $itemData['Tax'] ?? null;
                $itemDiscount[] = $itemData['Discount'] ?? null;
                $itemAmount[] = $itemData['Amount'] ?? null;
            }
        }
        // Save the data to the database
        $UnusedAmountReceved = 0;
        $remainingAmount = 0;
        if ($ReceivedAmount >= $TotalAmount) {
            $UnusedAmountReceved = $ReceivedAmount - $TotalAmount;
            $remainingAmount = 0;
        } else {
            $remainingAmount = $TotalAmount - $ReceivedAmount;
            $UnusedAmountReceved = 0;
        }
        $sale = Sale::where('studentid', $CandidateId)
            ->where('invoiceno', $invoiceNo)
            ->first();
        if ($sale) {
            if ($remainingAmount >= 0) {
                $paymentStatus = 1;
            } else {
                $paymentStatus = 0;
            }
            if ($ReceivedAmount >=  $TotalAmount) {
                $sale->StudentId = (int) $CandidateId;
                $sale->roomno = (int) $roomNoSelect;
                $sale->invoiceno = $invoiceNo;
                $sale->invoicedate = $date;
                $sale->tpdate = $date;
                $sale->item = implode(',', $itemIDs);
                $sale->quantity = implode(',', $itemQuantities);
                $sale->rate = implode(',', $itemRates);
                $sale->tax = implode(',', $itemTax);
                $sale->amount = implode(',', $itemAmount);
                $sale->discount = implode(',', $itemDiscount);
                $sale->receive_amount = $ReceivedAmount;
                $sale->total = $TotalAmount;
                $sale->payment_status = $paymentStatus;
                $sale->remaining_amount = 0;
                $sale->unused = $UnusedAmountReceved;
                insertData($date, $CandidateId, $invoiceNo, $TotalAmount, $ReceivedAmount, $paymentMode, $UnusedAmountReceved);
            } else {
                $sale->StudentId = (int) $CandidateId;
                $sale->roomno = (int) $roomNoSelect;
                $sale->invoiceno = $invoiceNo;
                $sale->invoicedate = $date;
                $sale->tpdate = $date;
                $sale->item = implode(',', $itemIDs);
                $sale->quantity = implode(',', $itemQuantities);
                $sale->rate = implode(',', $itemRates);
                $sale->tax = implode(',', $itemTax);
                $sale->amount = implode(',', $itemAmount);
                $sale->discount = implode(',', $itemDiscount);
                $sale->receive_amount = $ReceivedAmount;
                $sale->total = $TotalAmount;
                $sale->unused = $UnusedAmountReceved;
                $sale->remaining_amount = abs($remainingAmount);
                if ($ReceivedAmount != 0) {
                    insertData($date, $CandidateId, $invoiceNo, $remainingAmount, $ReceivedAmount, $paymentMode, $UnusedAmountReceved);
                    // $UpdateElectricityRecords = $this->checkAndUpdateElectricityBill($listItemData, $CandidateId, $invoiceNo);
                }
            }
            if ($sale->update()) {
                try {
                    $UpdateElectricityRecords = $this->checkAndUpdateElectricityBill($listItemData, $CandidateId, $invoiceNo, $sale->remaining_amount);
                } catch (Exception $e) {
                    dd($e->getMessage());
                }
                return response()->json(['message' => 'Data stored successfully']);
            } else {
                return response()->json(['message' => 'Failed to save data'], 500);
            }
        } else {
            return response()->json(['message' => $sale], 500);
        }
    }

    public function getSalesGraphData()
    {
        try {
            $currentYear = date('Y');
            $currentMonth = date('m');
            if ($currentMonth < 4) {
                $startDate = ($currentYear - 1) . "-04-01";
                $endDate = $currentYear . "-03-31";
            } else {
                $startDate = $currentYear . "-04-01";
                $endDate = ($currentYear + 1) . "-03-31";
            }

            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $customSalesQuery = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $result = $query->unionAll($customSalesQuery)->get();

            $labels = [];
            $totalAmount = [];

            // Initialize totalAmount with 0 for each month
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $totalAmount[$monthNum] = 0;
            }

            // Populate labels starting from January
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $actualMonth = $monthNum % 12;
                $actualMonth = ($actualMonth == 0) ? 12 : $actualMonth;

                $labels[] = date("M", mktime(0, 0, 0, $actualMonth, 1));
            }

            // Sum up the total amounts for each month
            foreach ($result as $sale) {
                $monthNum = intval($sale->month_num);
                $totalAmount[$monthNum] += round($sale->total_amount, 2);
            }

            // Re-index totalAmount to match the labels order
            $orderedTotalAmount = [];
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $actualMonth = $monthNum % 12;
                $actualMonth = ($actualMonth == 0) ? 12 : $actualMonth;
                $orderedTotalAmount[] = $totalAmount[$actualMonth];
            }

            $response = [
                'labels' => $labels,
                'totalAmount' => $orderedTotalAmount
            ];
            return response()->json($response);
        } catch (Exception $e) {
            return response()->json([
                'error' => $e->getMessage(),
            ]);
        }
    }

    public function salesgraph(Request $request)
    {
        $filterKeyword = $request['type'];
        $labels = [];
        $totalAmount = [];
        // Initialize the query builder
        $query = DB::table('sales')
            ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
            ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
            ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

        $CustomSalesmonthlyExp = DB::table('customsalesales')
            ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
            ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
            ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

        $result = $query->unionAll($CustomSalesmonthlyExp)->get();

        if ($filterKeyword == 'year') {
            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('SUM(total) AS total_amount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'));

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('SUM(total) AS total_amount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'));

            $year_exp = $query->unionAll($CustomSalesmonthlyExp)->get();
            $labels = [];
            $yearTotal = [];

            foreach ($year_exp as $row) {
                $yearNum = $row->year_num;
                if (!in_array($yearNum, $labels)) {
                    $labels[] = $yearNum;
                    $yearTotal[$yearNum] = round($row->total_amount);
                } else {
                    $yearTotal[$yearNum] += round($row->total_amount);
                }
            }

            // Sort the labels array
            sort($labels);

            // Fill in missing years with 0 total sales
            foreach ($labels as $year) {
                if (!isset($yearTotal[$year])) {
                    $yearTotal[$year] = 0;
                }
            }

            // Sort the yearTotal array by keys (years)
            ksort($yearTotal);

            $response = [
                'labels' => $labels,
                'totalAmount' => array_values($yearTotal),
            ];
        } elseif ($filterKeyword == 'quarter') {
            $currentYear = date('Y');
            $currentMonth = date('m');
            if ($currentMonth < 4) {
                $startDate = ($currentYear - 1) . "-04-01";
                $endDate = $currentYear . "-03-31";
            } else {
                $startDate = $currentYear . "-04-01";
                $endDate = ($currentYear + 1) . "-03-31";
            }
            // Retrieve the quarterly sales data
            $query = DB::table('sales')
                ->select(DB::raw('QUARTER(invoicedate) AS quarter_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('QUARTER(invoicedate)'))
                ->orderBy(DB::raw('QUARTER(invoicedate)'));

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('QUARTER(invoicedate) AS quarter_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('QUARTER(invoicedate)'))
                ->orderBy(DB::raw('QUARTER(invoicedate)'));

            $quarter_exp = $query->unionAll($CustomSalesmonthlyExp)->get();
            // Create an array to store the quarterly totals
            $quarterTotals = array_fill(1, 4, 0); // Initialize all quarters with zero totals
            // Populate the array with the retrieved quarterly totals
            foreach ($quarter_exp as $row) {
                $quarterNum = $row->quarter_num;
                $quarterTotals[$quarterNum] = round($row->total_amount);
            }
            // Generate the labels for the quarters
            $labels = ['Q1', 'Q2', 'Q3', 'Q4'];
            // Prepare the data for the response
            $response = [
                'labels' => $labels,
                'totalAmount' => array_values($quarterTotals),
            ];
            return response()->json($response);
        } elseif ($filterKeyword == 'month') {
            $currentMonth = date('m');
            $currentDay = date('d');
            $query = DB::table('sales')
                ->select(DB::raw('DATE_FORMAT(invoicedate, "%d") AS days'), DB::raw('total AS total_amount'))
                ->whereMonth('invoicedate', '=', $currentMonth)
                ->whereDay('invoicedate', '<=', $currentDay);

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('DATE_FORMAT(invoicedate, "%d") AS days'), DB::raw('total AS total_amount'))
                ->whereMonth('invoicedate', '=', $currentMonth)
                ->whereDay('invoicedate', '<=', $currentDay);

            $monthlyExp = $query->unionAll($CustomSalesmonthlyExp)->get();

            $totalDays = Carbon::now()->daysInMonth;
            for ($day = 1; $day <= $totalDays; $day++) {
                $labels[] = date("d", mktime(0, 0, 0, $currentMonth, $day));
                $totalAmount[$day] = 0;
            }
            foreach ($monthlyExp as $expense) {
                $monthNum = intval($expense->days);
                $totalAmount[$monthNum] += round($expense->total_amount);
            }
            $totalAmount = array_values($totalAmount);
            $response = [
                'labels' => $labels,
                'totalAmount' => $totalAmount
            ];
        } else if ($filterKeyword == 'Months') {
            $currentYear = date('Y');
            $currentMonth = date('m');
            if ($currentMonth < 4) {
                $startDate = ($currentYear - 1) . "-04-01";
                $endDate = $currentYear . "-03-31";
            } else {
                $startDate = $currentYear . "-04-01";
                $endDate = ($currentYear + 1) . "-03-31";
            }

            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $customSalesQuery = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $result = $query->unionAll($customSalesQuery)->get();

            $labels = [];
            $totalAmount = [];

            // Initialize totalAmount with 0 for each month
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $totalAmount[$monthNum] = 0;
            }

            // Populate labels starting from January
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $actualMonth = $monthNum % 12;
                $actualMonth = ($actualMonth == 0) ? 12 : $actualMonth;

                $labels[] = date("M", mktime(0, 0, 0, $actualMonth, 1));
            }

            // Sum up the total amounts for each month
            foreach ($result as $sale) {
                $monthNum = intval($sale->month_num);
                $totalAmount[$monthNum] += round($sale->total_amount, 2);
            }

            // Re-index totalAmount to match the labels order
            $orderedTotalAmount = [];
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $actualMonth = $monthNum % 12;
                $actualMonth = ($actualMonth == 0) ? 12 : $actualMonth;
                $orderedTotalAmount[] = $totalAmount[$actualMonth];
            }

            $response = [
                'labels' => $labels,
                'totalAmount' => $orderedTotalAmount
            ];
        } else if ($filterKeyword == 'custom') {
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];

            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $result = $query->unionAll($CustomSalesmonthlyExp)->get();

            $labels = [];
            $totalAmount = [];

            // Initialize totalAmount with 0 for each month
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $totalAmount[$monthNum] = 0;
            }

            // Populate labels starting from January
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
            }

            // Sum up the total amounts for each month
            foreach ($result as $sale) {
                $monthNum = intval($sale->month_num);
                $totalAmount[$monthNum] += round($sale->total_amount);
            }

            // Re-index totalAmount to match the labels order
            $orderedTotalAmount = [];
            foreach ($labels as $index => $label) {
                $monthNum = $index + 1; // Month numbers start from 1
                $orderedTotalAmount[] = $totalAmount[$monthNum];
            }

            $response = [
                'labels' => $labels,
                'totalAmount' => $orderedTotalAmount
            ];
        } else {
            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $result = $query->unionAll($CustomSalesmonthlyExp)->get();

            $labels = [];
            $totalAmount = [];

            // Initialize totalAmount with 0 for each month
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $totalAmount[$monthNum] = 0;
            }

            // Populate labels starting from April
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $actualMonth = $monthNum % 12;
                $actualMonth = ($actualMonth == 0) ? 12 : $actualMonth;

                $labels[] = date("M", mktime(0, 0, 0, $actualMonth, 1));
            }

            // Sum up the total amounts for each month
            foreach ($result as $expense) {
                $monthNum = intval($expense->month_num);
                $totalAmount[$monthNum] += round($expense->total_amount);
            }

            // Re-index totalAmount to match the labels order
            $orderedTotalAmount = [];
            foreach ($labels as $index => $label) {
                $monthNum = date("n", mktime(0, 0, 0, $index + 4, 1)); // +4 to start from April
                $orderedTotalAmount[] = $totalAmount[$monthNum];
            }

            $response = [
                'labels' => $labels,
                'totalAmount' => $orderedTotalAmount
            ];
        };
        return response()->json($response);
    }


    public function salesExpensesgGraph(Request $request)
    {
        $filterKeyword = $request['type'];
        $labels = [];
        $totalAmount = [];
        // Initialize the query builder
        $SalesmonthlyExp = DB::table('sales')
            ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS sale_total'))
            ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
            ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

        $CustomSalesmonthlyExp = DB::table('customsalesales')
            ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS sale_total'))
            ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
            ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');
        $saleAndCusomeSaleData = $SalesmonthlyExp->unionAll($CustomSalesmonthlyExp)->get();

        $ExpensesmonthlyExp = DB::table('expensedetails')
            ->select(DB::raw('DATE_FORMAT(billdate, "%m") AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
            ->groupBy(DB::raw('DATE_FORMAT(billdate, "%m")'))
            ->orderBy(DB::raw('DATE_FORMAT(billdate, "%m")'))
            ->get();
        if ($filterKeyword == 'year') {
            $salesData = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('SUM(total) AS sale_total'))
                ->groupBy(DB::raw('YEAR(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('SUM(total) AS sale_total'))
                ->groupBy(DB::raw('YEAR(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $totalSalesData = $salesData->union($CustomSalesmonthlyExp)->get();

            $expensesData = DB::table('expensedetails')
                ->select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('SUM(amount) AS expenses_amount'))
                ->groupBy(DB::raw('YEAR(billdate)'))
                ->orderBy(DB::raw('YEAR(billdate)', 'asc'))
                ->get();
            $labels = [];
            $salesTotal = [];
            $expensesTotal = [];
            // Process sales data
            foreach ($totalSalesData as $salesRow) {
                $yearNum = $salesRow->year_num;
                if (!in_array($yearNum, $labels)) {
                    $labels[] = $yearNum;
                }
                if (!isset($salesTotal[$yearNum])) {
                    $salesTotal[$yearNum] = round($salesRow->sale_total);
                } else {
                    $salesTotal[$yearNum] += round($salesRow->sale_total);
                }
            }

            foreach ($expensesData as $expensesRow) {
                $yearNum = $expensesRow->year_num;
                if (!in_array($yearNum, $labels)) {
                    $labels[] = $yearNum;
                }
                if (!isset($expensesTotal[$yearNum])) {
                    $expensesTotal[$yearNum] = round($expensesRow->expenses_amount);
                } else {
                    $expensesTotal[$yearNum] += round($expensesRow->expenses_amount);
                }
            }
            // Fill in missing years in expenses data with 0 total
            foreach ($salesTotal as $yearNum => $saleTotal) {
                if (!array_key_exists($yearNum, $expensesTotal)) {
                    $expensesTotal[$yearNum] = 0;
                }
                if (!array_key_exists($yearNum, $salesTotal)) {
                    $salesTotal[$yearNum] = 0;
                }
            }
            // Sort the data by year
            ksort($salesTotal);
            ksort($expensesTotal);
            $response = [
                'labels' => $labels,
                'SalestotalAmount' => array_values($salesTotal),
                'ExpensestotalAmount' => array_values($expensesTotal),
            ];
        } elseif ($filterKeyword == 'quarter') {
            $currentYear = date('Y');
            $currentMonth = date('m');
            if ($currentMonth < 4) {
                $startDate = ($currentYear - 1) . "-04-01";
                $endDate = $currentYear . "-03-31";
            } else {
                $startDate = $currentYear . "-04-01";
                $endDate = ($currentYear + 1) . "-03-31";
            }            // Retrieve the quarterly sales data
            $query = DB::table('sales')
                ->select(DB::raw('QUARTER(invoicedate) AS quarter_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('QUARTER(invoicedate)'))
                ->orderBy(DB::raw('QUARTER(invoicedate)'));

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('QUARTER(invoicedate) AS quarter_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('QUARTER(invoicedate)'))
                ->orderBy(DB::raw('QUARTER(invoicedate)'));

            $saleAndCusomeSaleData = $query->union($CustomSalesmonthlyExp)->get();
            // Retrieve the quarterly expenses data
            $expensesData = DB::table('expensedetails')
                ->select(DB::raw('QUARTER(billdate) AS quarter_num'), DB::raw('SUM(amount) AS expenses_amount'))
                ->whereBetween('billdate', [$startDate, $endDate])
                ->groupBy(DB::raw('QUARTER(billdate)'))
                ->orderBy(DB::raw('QUARTER(billdate)'))
                ->get();
            // Create an array to store the quarterly sales and expenses totals
            $salesTotal = array_fill(1, 4, 0); // Initialize all quarters with zero sales total
            $expensesTotal = array_fill(1, 4, 0); // Initialize all quarters with zero expenses total
            // Populate the arrays with the retrieved quarterly totals
            foreach ($saleAndCusomeSaleData as $row) {
                $quarterNum = $row->quarter_num;
                $salesTotal[$quarterNum] = round($row->total_amount);
            }
            foreach ($expensesData as $row) {
                $quarterNum = $row->quarter_num;
                $expensesTotal[$quarterNum] = round($row->expenses_amount);
            }
            // Generate the labels for the quarters
            $labels = ['Q1', 'Q2', 'Q3', 'Q4'];
            // Prepare the data for the response
            $response = [
                'labels' => $labels,
                'SalestotalAmount' => array_values($salesTotal),
                'ExpensestotalAmount' => array_values($expensesTotal),
            ];
            return response()->json($response);
        } else if ($filterKeyword == 'month') {
            $currentMonth = date('m');
            $currentDay = date('d');
            $currentYear = date('Y');
            // Retrieve the daily sales data for the current month
            $salesData = DB::table('sales')
                ->select(DB::raw('DATE_FORMAT(invoicedate, "%d") AS days'), 'total AS sales_amount')
                ->whereMonth('invoicedate', '=', $currentMonth)
                ->whereDay('invoicedate', '<=', $currentDay)
                ->whereYear('invoicedate', "=", $currentYear);

            $CustomSalesmonthlyExp = DB::table('customsalesales')
                ->select(DB::raw('DATE_FORMAT(invoicedate, "%d") AS days'), 'total AS sales_amount')
                ->whereMonth('invoicedate', '=', $currentMonth)
                ->whereDay('invoicedate', '<=', $currentDay);

            $saleAndCusomeSaleData = $salesData->union($CustomSalesmonthlyExp)->get();
            // Retrieve the daily expenses data for the current month
            $expensesData = DB::table('expensedetails')
                ->select(DB::raw('DATE_FORMAT(billdate, "%d") AS days'), 'amount AS expenses_amount')
                ->whereMonth('billdate', '=', $currentMonth)
                ->whereDay('billdate', '<=', $currentDay)
                ->get();
            $totalDays = Carbon::now()->daysInMonth;
            // Initialize the labels, sales amounts, and expenses amounts arrays
            $labels = [];
            $salesAmounts = array_fill(1, $totalDays, 0);
            $expensesAmounts = array_fill(1, $totalDays, 0);
            // Populate the labels array and calculate the sales and expenses amounts
            for ($day = 1; $day <= $totalDays; $day++) {
                $labels[] = str_pad($day, 2, '0', STR_PAD_LEFT);
            }
            foreach ($saleAndCusomeSaleData as $sale) {
                $dayNum = intval($sale->days);
                $salesAmounts[$dayNum] += round($sale->sales_amount);
            }
            foreach ($expensesData as $expense) {
                $dayNum = intval($expense->days);
                $expensesAmounts[$dayNum] += round($expense->expenses_amount);
            }
            $response = [
                'labels' => $labels,
                'SalestotalAmount' => array_values($salesAmounts),
                'ExpensestotalAmount' => array_values($expensesAmounts),
            ];
        } else if ($filterKeyword == 'Months') {
            $currentYear = date('Y');
            $currentMonth = date('m');
            if ($currentMonth < 4) {
                $startDate = ($currentYear - 1) . "-04-01";
                $endDate = $currentYear . "-03-31";
            } else {
                $startDate = $currentYear . "-04-01";
                $endDate = ($currentYear + 1) . "-03-31";
            }
            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $customSalesQuery = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $saleAndCusomeSaleData = $query->unionAll($customSalesQuery)->get();

            $ExpensesmonthlyExp = DB::table('expensedetails')
                ->select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
                ->whereBetween('billdate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
                ->orderBy(DB::raw('YEAR(billdate)'), 'ASC')
                ->get();
            $labels = [];
            $salestotalAmount = [];
            $expensetotalAmount = [];
            // Initialize the total amount array for all months to 0
            $salestotalAmount = array_fill(1, 12, 0);
            $expensetotalAmount = array_fill(1, 12, 0);

            foreach ($saleAndCusomeSaleData as $expense) {
                $monthNum = intval($expense->month_num);
                $salestotalAmount[$monthNum] += round($expense->total_amount);
            }
            foreach ($ExpensesmonthlyExp as $expense) {
                $monthNum = intval($expense->month_num);
                $expensetotalAmount[$monthNum] = round($expense->expenses_amount);
            }
            // Generate the month labels and total amounts
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
            }
            $response = [
                'labels' => $monthLabels,
                'SalestotalAmount' => array_values($salestotalAmount),
                'ExpensestotalAmount' => array_values($expensetotalAmount)
            ];
        } elseif ($filterKeyword == 'custom') {
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];

            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $customSalesQuery = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $result = $query->unionAll($customSalesQuery)->get();

            $ExpensesmonthlyExp = DB::table('expensedetails')
                ->select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
                ->whereBetween('billdate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
                ->orderBy(DB::raw('YEAR(billdate)'), 'ASC')
                ->get();

            $monthLabels = [];
            $salestotalAmount = array_fill(1, 12, 0); // Initialize only if needed
            $expensetotalAmount = array_fill(1, 12, 0); // Initialize only if needed

            foreach ($result as $sale) {
                $monthNum = intval($sale->month_num);
                $salestotalAmount[$monthNum] += round($sale->total_amount);
            }

            foreach ($ExpensesmonthlyExp as $expense) {
                $monthNum = intval($expense->month_num);
                $expensetotalAmount[$monthNum] = round($expense->expenses_amount);
            }

            // Generate the month labels and total amounts
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
            }

            $response = [
                'labels' => $monthLabels,
                'SalestotalAmount' => array_values($salestotalAmount),
                'ExpensestotalAmount' => array_values($expensetotalAmount)
            ];
        } else {
            $monthLabels = [];
            $salestotalAmount = [];
            $expensetotalAmount = [];
            // Initialize the total amount array for all months to 0
            $salestotalAmount = array_fill(1, 12, 0);
            $expensetotalAmount = array_fill(1, 12, 0);
            foreach ($saleAndCusomeSaleData as $expense) {
                $monthNum = intval($expense->month_num);
                $salestotalAmount[$monthNum] += round($expense->sale_total);
            }
            foreach ($ExpensesmonthlyExp as $expense) {
                $monthNum = intval($expense->month_num);
                $expensetotalAmount[$monthNum] = round($expense->expenses_amount);
            }
            // Generate the month labels and total amounts
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
            }
            $response = [
                'labels' => $monthLabels,
                'SalestotalAmount' => array_values($salestotalAmount),
                'ExpensestotalAmount' => array_values($expensetotalAmount)
            ];
        }
        return response()->json($response);
    }

    public function saleExpensesCurrensSessionChart()
    {
        try {
            $currentYear = date('Y');
            $currentMonth = date('m');
            if ($currentMonth < 4) {
                $startDate = ($currentYear - 1) . "-04-01";
                $endDate = $currentYear . "-03-31";
            } else {
                $startDate = $currentYear . "-04-01";
                $endDate = ($currentYear + 1) . "-03-31";
            }
            $query = DB::table('sales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $customSalesQuery = DB::table('customsalesales')
                ->select(DB::raw('YEAR(invoicedate) AS year_num'), DB::raw('MONTH(invoicedate) AS month_num'), DB::raw('SUM(total) AS total_amount'))
                ->whereBetween('invoicedate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(invoicedate)'), DB::raw('MONTH(invoicedate)'))
                ->orderBy(DB::raw('YEAR(invoicedate)'), 'asc');

            $saleAndCusomeSaleData = $query->unionAll($customSalesQuery)->get();

            $ExpensesmonthlyExp = DB::table('expensedetails')
                ->select(DB::raw('YEAR(billdate) AS year_num'), DB::raw('MONTH(billdate) AS month_num'), DB::raw('SUM(amount) AS expenses_amount'))
                ->whereBetween('billdate', [$startDate, $endDate])
                ->groupBy(DB::raw('YEAR(billdate)'), DB::raw('MONTH(billdate)'))
                ->orderBy(DB::raw('YEAR(billdate)'), 'ASC')
                ->get();
            $labels = [];
            $salestotalAmount = [];
            $expensetotalAmount = [];
            // Initialize the total amount array for all months to 0
            $salestotalAmount = array_fill(1, 12, 0);
            $expensetotalAmount = array_fill(1, 12, 0);

            foreach ($saleAndCusomeSaleData as $expense) {
                $monthNum = intval($expense->month_num);
                $salestotalAmount[$monthNum] += round($expense->total_amount);
            }
            foreach ($ExpensesmonthlyExp as $expense) {
                $monthNum = intval($expense->month_num);
                $expensetotalAmount[$monthNum] = round($expense->expenses_amount);
            }
            // Generate the month labels and total amounts
            for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                $monthLabels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
            }
            $response = [
                'labels' => $monthLabels,
                'SalestotalAmount' => array_values($salestotalAmount),
                'ExpensestotalAmount' => array_values($expensetotalAmount)
            ];
            return response()->json($response);
        } catch (Exception $e) {
            return response()->json([
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
