<?php

namespace App\Http\Controllers;

use Dompdf\Dompdf;
use Illuminate\Support\Facades\DB;
use Illuminate\Http\Request;
use App\Models\StudentRegistertoken;
use Illuminate\Support\Facades\Mail;
use App\Mail\RegistrationFormMail;
use Illuminate\Support\Str;
use App\Models\SelfStudentRegister;
use Illuminate\Support\Facades\Config;
use App\Models\Businessregister;
use Illuminate\Support\Facades\File;
use App\Models\Studentregister;
use Exception;
use Illuminate\Support\Facades\View;
use App\Models\Uploadprofile;
use App\Models\StudentElecticityRecords;
use App\Models\Permissions;
use Illuminate\Database\Eloquent\ModelNotFoundException;
use Carbon\Carbon;
use App\Models\ReferHostel;
use App\Models\MessageManager;


// use Intervention\Image\Facades\Image;
// use Intervention\Image\ImageManagerStatic as Image;


use function PHPUnit\Framework\fileExists;

class ShareController extends Controller
{

    /**
     * Resize image if it exceeds the specified size.
     *
     * @param string $imageData
     * @param int $maxSize
     * @param string $extension
     * @return string
     */
    private function checkWhatsAppPermission(): bool
    {
        try {
            $permission = Permissions::where('permissionType', 'Desktop Whatsapp')->value('permission');
            return $permission ?? false; // Return the permission value or false if not found

        } catch (ModelNotFoundException $e) {
            return false; // Handle the case when the permission is not found

        } catch (Exception $e) {
            // Log or handle other exceptions
            return false;
        }
    }
    public function resizeBase64Image($base64Data, $targetSizeInKB)
    {
        // Decode base64 data
        $img = explode(',', $base64Data);
        // dd($img[1]);

        $imageData = base64_decode($img[1]);


        // Create an image resource from the decoded data
        $img = imagecreatefromstring($imageData);

        // Get the original image dimensions
        $width = imagesx($img);
        $height = imagesy($img);

        // Set the desired width and calculate the corresponding height to maintain the aspect ratio
        $targetWidth = 200; // Adjust this value based on your requirements
        $targetHeight = ($targetWidth / $width) * $height;

        // Create a new image resource for the resized image
        $resizedImg = imagecreatetruecolor($targetWidth, $targetHeight);

        // Resize the image
        imagecopyresampled($resizedImg, $img, 0, 0, 0, 0, $targetWidth, $targetHeight, $width, $height);

        // Create a buffer for the resized image
        ob_start();

        // Output the resized image to the buffer
        imagejpeg($resizedImg, null, 90); // Adjust the quality as needed

        // Get the buffer content
        $resizedImageData = ob_get_clean();

        // Check if the resized image meets the target size
        while (strlen($resizedImageData) > (50 * 1024)) {
            // If the size is still larger than the target, reduce the quality and update the buffer
            ob_start();
            imagejpeg($resizedImg, null, 90);
            $resizedImageData = ob_get_clean();
        }

        // Encode the resized image to base64
        $resizedBase64Data = base64_encode($resizedImageData);

        // Destroy the image resources
        imagedestroy($img);
        imagedestroy($resizedImg);

        return $resizedBase64Data;
    }

    private function MakeCustomeDataBase($db_username, $db_password, $db_name, $db_host, $db_port)
    {
        Config::set('database.connections.custom_db.username', $db_username);
        Config::set('database.connections.custom_db.password', $db_password);
        Config::set('database.connections.custom_db.database', $db_name);
        Config::set('database.connections.custom_db.host', $db_host);
        Config::set('database.connections.custom_db.port', $db_port);
        return true;
    }
    public function generatePDF()
    {
        // Fetch the expenses report data
        $expensesReport = DB::table('expenses')->get();

        // Generate HTML for the PDF report
        $html = '<h1>Expenses Report</h1>';
        $html .= '<table>';
        foreach ($expensesReport as $expense) {
            $html .= '<tr>';
            $html .= '<td>Category Name: ' . $expense->category_name . '</td>';
            $html .= '</tr>';

            // Fetch expense details for the current category
            $expenseDetails = DB::table('expensedetails')
                ->where('category_id', $expense->id)
                ->get();

            foreach ($expenseDetails as $detail) {
                $html .= '<tr>';
                $html .= '<td>' . $detail->itemname . '</td>';
                // Add other columns for expense details as needed
                $html .= '</tr>';
            }
        }
        $html .= '</table>';

        // Generate PDF using DOMPDF
        $dompdf = new Dompdf();
        $dompdf->loadHtml($html);
        $dompdf->setPaper('A4', 'portrait');
        $dompdf->render();

        // Generate a random file name
        $filename = 'expenses_report_' . uniqid() . '.pdf';

        // Save the PDF file locally
        $output = $dompdf->output();
        file_put_contents($filename, $output);

        // Return the file path for download
        return public_path($filename);
    }
    public function ShareForm(Request $request)
    {
        $whatsAppPermission = $this->checkWhatsAppPermission();
        $studentId = null;
        $studentId = $request->studentId ?? null;
        config(['database.default' => env('DB_CONNECTION')]);
        $request->validate([
            'shareType' => 'required',
            'shareEmail' => 'nullable|required_if:shareType,email|email',
            'Sharemobile' => 'nullable|required_if:shareType,mobile|numeric|digits:10',
        ]);
        $form = new StudentRegistertoken();
        $token = Str::random(65);
        $form->token = $token;
        $form->studentId = $studentId;
        if ($request->shareType === "email") {
            $form->email = $request->shareEmail;
        } else {
            $form->email = $request->Sharemobile;
        }
        $form->save();
        if ($request->shareType === "email") {
            try {
                Mail::send('emails.Registration-form', ['token' => $token], function ($message) use ($request) {
                    $message->to($request->shareEmail);
                    $message->subject("Registration Form");
                });
                return back()->with(['RegistrationForm' => 'Registration Form send Successfully']);
            } catch (\Exception $e) {
                return back()->with(['error' => $e->getMessage()]);
            }
        } else {
            $mobileno = $request->Sharemobile;
            $formPath = url(route('registration.form', ['token' => $token]));
            // get the message of the share form for whatsapp
            $message = MessageManager::where('msg_type', 'from_share')
            ->where('status',1)
            ->value('msg');
            
            return back()->with([
                "shareOnMobile" => $formPath, 
                "Mobileno" => $mobileno, 
                'whatsAppPermission' => $whatsAppPermission,
                'message' => $message
            ]);
        }
    }

    private function getStudentData($studentId)
    {
        try {
            $studentData = Studentregister::where('id', $studentId)->firstOrFail();
            return $studentData;
        } catch (Exception $e) {
            return [];
        }
    }

    public function registrationForm($token)
    {
        $pdfFilePath = "";
        $bussinesName = "";
        $bussineLogoPath = "";
        $studentData = '';
        $currentDate = Carbon::now()->format("Y-m-d");
        config(['database.default' => env('DB_CONNECTION')]);
        $tokendata = StudentRegistertoken::where('token', $token)->first();
        if (!empty($tokendata)) {
            $licence_no = $tokendata->licence_no;
            $getDataBaseData = Businessregister::where("license_no", $licence_no)->first();
            if (!empty($getDataBaseData)) {
                $bussinesName = $getDataBaseData->businessname;
                $bussinesImg = $getDataBaseData->pic;
                $bussineLogoPath = file_exists(asset("assets/uploads/" . $bussinesImg)) ? asset("assets/uploads/" . $bussinesImg) : asset("assets/images/logoimage.jpeg");
                $db_username = $getDataBaseData->db_username;
                $db_password = $getDataBaseData->db_password;
                $db_name = $getDataBaseData->db_name;
                $db_host = $getDataBaseData->db_host;
                $db_port = $getDataBaseData->db_port;
                $returnValue = $this->MakeCustomeDataBase($db_username, $db_password, $db_name, $db_host, $db_port);
                if ($returnValue) {
                    config(['database.default' => 'custom_db']);
                    $studentData = $this->getStudentData($tokendata->studentId) ?? null;
                    $termCanditions = Uploadprofile::where('upload_name', 'Rules & Regulations')->select('upload_pdf')->first();
                    $pdfFilePath = asset('assets/uploads/pdf/' . $termCanditions->upload_pdf);
                } else {
                    $pdfFilePath = "";
                }
            }
        }
        return view('RegistrationForm', compact("token", "pdfFilePath", "bussinesName", "bussineLogoPath", 'currentDate', 'studentData'));
    }
    public function ragister(Request $request)
    {
        $rules = [
            'regdate' => 'required|date',
            'candidatename' => 'required|string',
            'dob' => 'required|date',
            'aadhar' => 'required|numeric|digits:12',
            'phone' => 'required|numeric|digits:10',
            'email' => 'required|email',
            'blood' => 'required|string',
            'coursename' => 'required|string',
            'coursedetails' => 'nullable|required_if:coursename,coursetype|string',
            'jobdetails' => 'nullable|required_if:coursename,jobtype|string',
            'institute' => 'required|string',
            'institutename' => 'nullable|required_if:institute,instituteName|string',
            'companyname' => 'nullable|required_if:institute,ComapanyName|string',
            'stydrt' => 'required|string',
            'healthissue' => 'required|in:yes,no',
            'health_description' => 'nullable|required_if:healthissue,yes|string',
            'vehicalno' => 'required|in:yes,no',
            'vehicledescription' => 'nullable|required_if:vehicalno,yes|string',
            'fathername' => 'required|string',
            'fatheroocc' => 'required|string',
            'mothername' => 'required|string',
            'motheroocc' => 'required|string',
            'tel' => 'required|numeric|digits:10',
            'mobilenumber' => 'nullable|numeric|digits:10',
            'pemail' => 'nullable|email',
            'fullAddress' => 'required|string',
            'state' => 'required|string',
            'pinno' => 'required|numeric|digits:6',
            'guardianname' => 'required|string',
            'guardiannumber' => 'required|numeric|digits:10',
            'guardianaddress' => 'required|string',
            'checkbox' => 'required|accepted',
            'uploadaadharfront' => 'required|image|mimes:jpeg,jpg,png',
            'uploadaadharback' => 'required|image|mimes:jpeg,jpg,png',
            'candidatesign' => 'required|image|mimes:jpeg,jpg,png',
            'Parentsign' => 'required|image|mimes:jpeg,jpg,png',
            'uploadimage' => 'required|image|mimes:jpeg,jpg,png',
            'termConditions' => 'required|in:1'
        ];
        $request->validate(
            $rules
        );
        // token Validation 
        $token = $request->token;
        // check token 
        config(['database.default' => env('DB_CONNECTION')]);
        $tokendata = StudentRegistertoken::where('token', $token)->first();
        if (!empty($tokendata)) {
            //get student id 
            $licence_no = $tokendata->licence_no;
            $getDataBaseData = Businessregister::where("license_no", $licence_no)->first();

            if (!empty($getDataBaseData)) {
                $db_username = $getDataBaseData->db_username;
                $db_password = $getDataBaseData->db_password;
                $db_name = $getDataBaseData->db_name;
                $db_host = $getDataBaseData->db_host;
                $db_port = $getDataBaseData->db_port;
                $returnValue = $this->MakeCustomeDataBase($db_username, $db_password, $db_name, $db_host, $db_port);
                if ($returnValue) {
                    config(['database.default' => 'custom_db']);
                    $NewEntry = new SelfStudentRegister();
                    $NewEntry->StudentId = $request->studentId;
                    $NewEntry->regdate = $request->regdate;
                    $NewEntry->candidatename = $request->candidatename;
                    $NewEntry->dob = $request->dob;
                    $NewEntry->aadhar = $request->aadhar;
                    $NewEntry->phone = $request->phone;
                    $NewEntry->email = $request->email;
                    $NewEntry->blood = $request->blood;
                    $NewEntry->course_job = $request->coursename;
                    $NewEntry->coursedetails = $request->coursedetails;
                    $NewEntry->jobdetails = $request->jobdetails;
                    $NewEntry->institute_company = $request->institute;
                    $NewEntry->institutename = $request->institutename;
                    $NewEntry->companyname = $request->companyname;
                    $NewEntry->stydrt = $request->stydrt;
                    $NewEntry->healthissue = $request->healthissue;
                    $NewEntry->health_description = $request->health_description;
                    $NewEntry->vehicalno = $request->vehicalno;
                    $NewEntry->vehicledescription = $request->vehicledescription;
                    $NewEntry->fathername = $request->fathername;
                    $NewEntry->fatheroocc = $request->fatheroocc;
                    $NewEntry->mothername = $request->mothername;
                    $NewEntry->motheroocc = $request->motheroocc;
                    $NewEntry->parentmobile1 = $request->tel;
                    $NewEntry->parentmobile2 = $request->mobilenumber;
                    $NewEntry->pemail = $request->pemail;
                    $NewEntry->fullAddress = $request->fullAddress;
                    $NewEntry->state = $request->fathername;
                    $NewEntry->pinno = $request->pinno;
                    $NewEntry->guardianname = $request->guardianname;
                    $NewEntry->guardiannumber = $request->guardiannumber;
                    $NewEntry->guardianaddress = $request->guardianaddress;
                    $NewEntry->termsAndCondition = $request->termConditions;
                    $NewEntry->referCode = strtoupper($request->referCode);
                    if ($request->hasFile('uploadimage')) {
                        $file = $request->file('uploadimage');
                        $extension = $file->getClientOriginalExtension();
                        $filename = str_replace(' ', '_', $request->candidatename) . '_' . Carbon::now()->format("YmdHis") . '_' . uniqid() . '.' . $extension;
                        // $imageData = explode(',', $request->uploadCropImg);
                        $resizedBase64Data = $this->resizeBase64Image($request->uploadCropImg, 50);
                        File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                        $NewEntry->studentImg = $filename;
                    }
                    if ($request->hasFile('uploadaadharfront')) {
                        $file = $request->file('uploadaadharfront');
                        $extension = $file->getClientOriginalExtension();
                        $filename = str_replace(' ', '_', $request->candidatename) . '_' . Carbon::now()->format("YmdHis") . '_' . uniqid() . '.' . $extension;
                        $resizedBase64Data = $this->resizeBase64Image($request->AadharFrontCrop, 50);
                        File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                        $NewEntry->uploadaadharfront = $filename;
                    }
                    if ($request->hasFile('uploadaadharback')) {
                        $file = $request->file('uploadaadharback');
                        $extension = $file->getClientOriginalExtension();
                        $filename = str_replace(' ', '_', $request->candidatename) . '_' . Carbon::now()->format("YmdHis") . '_' . uniqid() . '.' . $extension;
                        $imageData = explode(',', $request->AadharBackCrop);
                        $resizedBase64Data = $this->resizeBase64Image($request->AadharBackCrop, 50);
                        File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                        $NewEntry->uploadaadharback = $filename;
                    }
                    if ($request->hasFile('candidatesign')) {
                        $file = $request->file('candidatesign');
                        $extension = $file->getClientOriginalExtension();
                        $filename = str_replace(' ', '_', $request->candidatename) . '_' . Carbon::now()->format("YmdHis") . '_' . uniqid() . '.' . $extension;
                        $imageData = explode(',', $request->candidatesignCropImg);
                        $resizedBase64Data = $this->resizeBase64Image($request->candidatesignCropImg, 50);
                        File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                        $NewEntry->candidatesign = $filename;
                    }
                    if ($request->hasFile('Parentsign')) {
                        $file = $request->file('Parentsign');
                        $extension = $file->getClientOriginalExtension();
                        $filename = str_replace(' ', '_', $request->candidatename) . '_' . Carbon::now()->format("YmdHis") . '_' . uniqid() . '.' . $extension;
                        $imageData = explode(',', $request->ParentCropImg);
                        $resizedBase64Data = $this->resizeBase64Image($request->ParentCropImg, 50);
                        File::put(public_path('assets/uploads/' . $filename), base64_decode($resizedBase64Data));
                        $NewEntry->Parentsign = $filename;
                    }
                    if ($NewEntry->save()) {
                        config(['database.default' => env('DB_CONNECTION')]);
                        $deleteToekn = $tokendata->delete();
                        if ($deleteToekn) {
                            return back()->with('success', "Registration Done...");
                        }
                    }
                }
            }
        }
        return back()->with('error', "Your Registration token has been Expired , Please Contact to your Hostel.");
    }

    public function registerToMain(Request $request)
    {

        $Validated = $request->validate([
            'formno' => 'required|numeric',
            'roomno' => 'required|numeric',
            'seetno' => 'required|numeric',
            'registrationDate' => 'required|date',
            'user_remark' => 'nullable|string',
        ]);
        $newRegsterData = SelfStudentRegister::find($request->studentId);
        if (empty($newRegsterData)) {
            return back()->with('registerError', 'Student Data is not Found');
        }
        try {
            $newRegsterData = SelfStudentRegister::find($request->studentId);
            $data = $newRegsterData->StudentId ? Studentregister::where('id', $newRegsterData->StudentId)->firstOrFail() : new Studentregister();
            $data->formno = $request->formno;
            $data->roomno = $request->roomno;
            $data->seetno = $request->seetno;
            $data->date = $request->registrationDate;
            $data->candidatename = $newRegsterData->candidatename;
            $data->dob =  $newRegsterData->dob;
            $data->valid =  $newRegsterData->aadhar;
            $data->phone =  $newRegsterData->phone;
            $data->email =  $newRegsterData->email;
            $data->blood =  $newRegsterData->blood;
            $data->healthissue =  $newRegsterData->healthissue;
            $data->health_description =  $newRegsterData->health_description;
            $data->stydrt =  $newRegsterData->stydrt;
            $data->vehicalno =  $newRegsterData->vehicalno;
            $data->vehicledescription =  $newRegsterData->vehicledescription;
            $data->coursename =  $newRegsterData->course_job;
            $data->coursedetails =  $newRegsterData->coursedetails;
            $data->jobdetails =  $newRegsterData->jobdetails;
            $data->institute =  $newRegsterData->institute_company;
            $data->companyname =  $newRegsterData->companyname;
            $data->institutename =  $newRegsterData->institutename;
            $data->fathername =  $newRegsterData->fathername;
            $data->fatheroocc =  $newRegsterData->fatheroocc;
            $data->mothername =  $newRegsterData->mothername;
            $data->motheroocc =  $newRegsterData->motheroocc;
            $data->tel =  $newRegsterData->parentmobile1;
            $data->mobilenumber =  $newRegsterData->parentmobile2;
            $data->pemail =  $newRegsterData->pemail;
            $data->fullAddress =  $newRegsterData->fullAddress;
            $data->state =  $newRegsterData->state;
            $data->pinno =  $newRegsterData->pinno;
            $data->guardianname =  $newRegsterData->guardianname;
            $data->guardiannumber =  $newRegsterData->guardiannumber;
            $data->guardianaddress =  $newRegsterData->guardianaddress;
            $data->uploadimage =  $newRegsterData->studentImg;
            $data->uploadaadharfront = $newRegsterData->uploadaadharfront ??= '';
            $data->uploadaadharback =  $newRegsterData->uploadaadharback ??= '';
            $data->candidatesign =  $newRegsterData->candidatesign ??= '';
            $data->Parentsign =  $newRegsterData->Parentsign ??= '';
            if ($newRegsterData->StudentId ? $data->update() : $data->save()) {
                if ($newRegsterData->referCode != "") {
                    $refer = ReferHostel::where('referal_code', strtoupper($newRegsterData->referCode))->first();
                    if (!empty($refer)) {
                        $refer->update([
                            'status' => 2,
                            'refered_student_id' => $data->id,
                        ]);
                    }
                }
                $newRegsterData->delete();
                return back()->with("RegistrationComplete", "Registration has been completed.");
            } else {
                throw Exception('Form not submited');
            }
        } catch (Exception $e) {
            return back()->with('error', "Somthing wrong");
        }
    }

    public function delete($id)
    {
        $newRegsterData = SelfStudentRegister::find($id);
        if ($newRegsterData->delete()) {
            return response()->json(['success', "delete success"], 200);
        } else {
            return response()->json(['error', 'not delete'], 500);
        }
    }

    public function studentData($id)
    {
        $newRegsterData = SelfStudentRegister::find($id);
        if ($newRegsterData) {
            $FormImgs = [
                'CandidateImg' => asset("assets/uploads/" . $newRegsterData->studentImg),
                'uploadaadharfront' => asset("assets/uploads/" . $newRegsterData->uploadaadharfront),
                'uploadaadharback' => asset("assets/uploads/" . $newRegsterData->uploadaadharback),
                'candidatesign' => asset("assets/uploads/" . $newRegsterData->candidatesign),
                'parentSingImg' => asset("assets/uploads/" . $newRegsterData->Parentsign),
            ];
        }
        return response()->json(['data' => $newRegsterData, 'Images' => $FormImgs]);
    }
}
