<?php

namespace App\Http\Controllers;

use Carbon\Carbon;
use Illuminate\Http\Request;
use App\Models\Expense;
use App\Models\StockItem;
use App\Models\stock_detail;
use App\Models\Expensedetail;
use App\Models\StockCategory;
use Exception;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;


class StockManagementController extends Controller
{
    /**
     * Display a listing of the resource.
     *
     * @return \Illuminate\Http\Response
     */


    public function view(Request $request)
    {
        $filterMonth = $request->MontheFilter ?: Carbon::now()->format('Y-m');
        $previousMonthDate = Carbon::parse($filterMonth)->subMonth()->format('Y-m');
        $categoriesWithItemsAndDetails = StockCategory::with(['stockItem' => function ($query) use ($filterMonth) {
            $query->with(['itemDetails' => function ($query) use ($filterMonth) {
                $query->where('month', $filterMonth);
            }]);
        }])->get();
        $categoriesWithItemsAndDetails->each(function ($category) use ($previousMonthDate) {
            $category->stockItem->each(function ($item) use ($category, $previousMonthDate) {
                // Access each item's itemDetails collection
                $previousMonthdata = stock_detail::where('month', $previousMonthDate)->where('item_id', $item->id)->where('category_id', $category->id)->first();
                $item->setAttribute('previousMonthData', $previousMonthdata);
                $itemDetails = $item->itemDetails;
                if (!empty($itemDetails)) {
                    $itemDetails->each(function ($detail) use (&$category, $previousMonthDate, &$item, &$itemDetails) {
                        $NeworderDates = $itemDetails->orderDate;
                        $previousItemOrderDate = $category->getAttribute('orderDates');
                        if (!is_array($previousItemOrderDate) && !empty($NeworderDates)) {
                            $category->setAttribute('orderDates', $NeworderDates);
                        } elseif (!empty($NeworderDates) && count($NeworderDates) > count($previousItemOrderDate)) {
                            $category->setAttribute('orderDates', $NeworderDates);
                        }
                    });
                }
            });
        });
        return view('fronted.StockManagement', compact('categoriesWithItemsAndDetails', 'filterMonth'));
    }

    function DeleteStockITems(Request $request)
    {
        try {
            $data = $request['itemDetails'];
            $data = explode(",", $data);
            $updateData = stock_detail::where("item_id", $data[0])
                ->where("category_id", $data[1])
                ->where("month", $data[2])
                ->first();
            if ($updateData) {
                $updateData->delete();
                return response()->json("success");
            } else {
                return response()->json("error", 500);
            }
        } catch (Exception $e) {
            return response()->json("error", "Somthing Wrong");
        }
    }
    function AddUsedStockData(Request $request)
    {
        try {
            $data = $request['EstData'];
            $estimationValues = $data['estimationValues'];
            $itemCategoryIds = $data['itemCategoryIds'];
            $lenth = sizeof($itemCategoryIds);
            for ($i = 0; $i < $lenth; $i++) {
                $ItemIDArry = explode(",", $itemCategoryIds[$i]);
                $item = $estimationValues[$i];
                $UsedStockAdd = stock_detail::where("item_id", $ItemIDArry[0])
                    ->where("category_id", $ItemIDArry[1])
                    ->where("month", $ItemIDArry[2])
                    ->first();
                if ($UsedStockAdd) {
                    $UsedStockAdd->used_stock = $item;
                    $UsedStockAdd->save();
                } else {
                    return response()->json("error", 500);
                }
            }
        } catch (Exception $e) {
            return response()->json("error", 500);
        }
    }
    function AddStockCetegory(Request $request)
    {
        $categoryName = $request["NewCategory"];
        $data = new StockCategory;
        $data->category_name = $categoryName;
        if ($data->save()) {
            return response()->json(["category_name" => $categoryName, "id" => $data->id]);
        } else {
            return response()->json("error", 500);
        }
    }
    public function StoreEstimationOrder(Request $request)
    {
        try {
            $estOrderdata = $request['EstOrdeData'];
            $tableOrder = $estOrderdata['tableOrder'];
            $itemCategoryIdsMonth = $estOrderdata['itemCategoryIds'];
            $estOrderValue = $estOrderdata['estimationValues'];
            $lenth = sizeof($itemCategoryIdsMonth);
            $date = Carbon::now();
            $formattedDate = $date->format('Y-m-d');
            for ($i = 0; $i < $lenth; $i++) {
                $ItemIDArry = explode(",", $itemCategoryIdsMonth[$i]);
                $item = $estOrderValue[$i];
                $orderSave = stock_detail::where("item_id", $ItemIDArry[0])
                    ->where("category_id", $ItemIDArry[1])
                    ->where("month", $ItemIDArry[2])
                    ->first();
                if ($orderSave) {
                    $existingOrderDates = $orderSave->orderDate;
                    if ($tableOrder == 1) {
                        $orderSave->order1 = $item;
                    } elseif ($tableOrder == 2) {
                        $orderSave->order2 = $item;
                    } elseif ($tableOrder == 3) {
                        $orderSave->order3 = $item;
                    } else {
                        break;
                        return response()->json(["error" => "This month All Order Done"], 500);
                    }
                    if ($tableOrder == 1) {
                        if (empty($existingOrderDates)) {
                            $updatedOrderDates = [$formattedDate];
                        } else {
                            $explodeOrderDates = $existingOrderDates;
                            $explodeOrderDates[0] = $formattedDate;
                            $updatedOrderDates = $explodeOrderDates;
                        }
                    } elseif ($tableOrder >= 2 && $tableOrder <= 3) {
                        $updatedOrderDates = $existingOrderDates;
                        $updatedOrderDates[$tableOrder - 1] = $formattedDate;
                    }
                    $updatedOrderDate = $updatedOrderDates;
                    // Update the orderDate column with the new value
                    $orderSave->orderDate = implode(',', $updatedOrderDate);
                    if ($orderSave->save()) {
                        $response = true;
                    } else {
                        $response = false;
                        break;
                    }
                } else {
                    $response = false;
                }
            }
            if ($response == true) {
                return response()->json("success", 200);
            } else {
                // throw  new Exception("Data not saved");
            }
        } catch (Exception $e) {
            return response()->json($e, 500);
        }
    }
    public function StoreEstimationStock(Request $request)
    {
        try {
            $estData = $request['EstData'];
            $month = $estData['month'];
            $itemCategoryIds = $estData['itemCategoryIds'];
            $StockEstItemValue = $estData['estimationValues'];
            $lenth = sizeof($itemCategoryIds);
            $date =   Carbon::now();
            for ($i = 0; $i < $lenth; $i++) {
                $ItemIDArry =  explode(",", $itemCategoryIds[$i]);
                $item =   $StockEstItemValue[$i];
                $checkDetails = stock_detail::where("item_id", $ItemIDArry[0])
                    ->where("category_id", $ItemIDArry[1])
                    ->where("month", $month)
                    ->first();
                if ($checkDetails) {
                    $checkDetails->item_id = $ItemIDArry[0];
                    $checkDetails->category_id = $ItemIDArry[1];
                    $checkDetails->month = $month;
                    $checkDetails->est_stock = $item;
                    $checkDetails->Date = $date;
                    $checkDetails->update();
                } else {
                    $storeEst = new stock_detail;
                    $storeEst->item_id = $ItemIDArry[0];
                    $storeEst->category_id = $ItemIDArry[1];
                    $storeEst->month = $month;
                    $storeEst->est_stock = $item;
                    $storeEst->Date = $date;
                    $storeEst->save();
                }
            }
            return response()->json("Success", 200);
        } catch (Exception $e) {
            return response()->json("error", 500);
        }
    }
    public function StoreStockItems(Request $request)
    {
        $tableData = $request['TableData'];
        $categoryName = $request['categoryName'];
        if ($categoryName != "") {
            $data = new StockCategory;
            $data->category_name = $categoryName;
            $data->save();
            $categoryID = $data->id;
        }
        foreach ($tableData as $data) {
            $string = $data['category'];
            if (preg_match('/\((\d+)\)/', $string, $matches)) {
                $intValue = intval($matches[1]);
            } else {
                $intValue =  $categoryID;
            }
            $StockItem = new StockItem;
            $StockItem->category_id = $intValue;
            $StockItem->itemName = $data['name'];
            $StockItem->itemUnite = $data['units'];
            $StockItem->save();
        }
        return response()->json("DataSubmit");
    }
    public function getGetegoryNames()
    {
        $expenseCategory = StockCategory::select("id", "category_name")->get();
        if ($expenseCategory) {
            return response()->json($expenseCategory);
        } else {
            return response()->json("erro", 500);
        }
    }
}
