<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use App\Models\Feature;
use App\Models\Subscription;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Str;
use Config;
use App\Models\Salesubscription;
use Exception;
// use Illuminate\Support\Carbon;
use App\Models\Studentregister;
use Carbon\Carbon;
use App\Models\SubscriptionFeatures;
use App\Models\SubscriptionPermission;
use App\Models\Businessregister;
use App\Models\AddOnItem;
use App\Models\UserAddOnItem;
use App\Models\BusinessSubscriptionPermission;
use App\Models\AddOnPermission;
use PhpParser\Node\Expr\FuncCall;

class SubscriptionController extends Controller
{

  public function CheckPlansDetails()
  {
    config(['database.default' => env('DB_CONNECTION')]);
    $liseneceNo = session("liseneceNo");
    $plansDetails = Salesubscription::where("user_id", $liseneceNo)->where("status", 1)->first();
    if ($plansDetails) {
      $endDate = Carbon::createFromFormat('Y-m-d', $plansDetails->enddate);
      $today = Carbon::today();
      $daysDifference = $today->diffInDays($endDate);
      if ($daysDifference <= 5 && $daysDifference > 0) {
        return response()->json(['PlansExpiredIn' => $daysDifference . " Days"], 200);
      } else if ($daysDifference == 0) {
        $plansDetails->status = 0;
        if ($plansDetails->update()) {
          return response()->json(['PlansExpiredIn' => " Today"], 200);
        }
      } else {
        return response()->json(['PlansExpiredIn' => $daysDifference], 500);
      };
    } else {
      return response()->json("error", 500);
    }
  }

  public function getParchsesSeats()
  {
    try {
      $totalRegister = Studentregister::count();
      config(['database.default' => env('DB_CONNECTION')]);
      $liseneceNo = session("liseneceNo");
      $parchesSeats = Salesubscription::select('no_of_seats', 'package_type', 'startdate')->where("user_id", $liseneceNo)->where("status", 1)->first();
      if (empty($parchesSeats)) {
        return response()->json([
          'status' => false,
          'message' => 'Subscription Records is not Found for this account.',
        ], 200);
      }
      return response()->json([
        'status' => true,
        'parchesesSeats' => (int) $parchesSeats->no_of_seats,
        'totalRegisterSeats' => $totalRegister,
        'purchaseDate' => Carbon::parse($parchesSeats->startdate)->format('d-M-Y'),
        'package_type' => $parchesSeats->package_type,
        'message' => 'Data Loaded Successfully.'

      ]);
    } catch (Exception $e) {
      return response()->json([
        'error' => $e->getMessage()
      ], 500);
    }
  }

  public function PakageDetails()
  {
    config(['database.default' => env('DB_CONNECTION')]);
    $package_title = DB::table('subscriptions')
      ->where('status', 1)
      ->whereNotIn('package_duration', ['7 Days', '15 Days'])
      ->select('package_title', 'package_duration')
      ->distinct()
      ->get();
    config(['database.default' => "custom_db"]);
    if ($package_title) {
      $package_duration = [];
      $packages = [];
      foreach ($package_title as $details) {
        if (!in_array($details->package_duration, $package_duration)) {
          $package_duration[] = $details->package_duration;
        }
        // Add the package_title to the $packages array if it's not already there
        if (!in_array($details->package_title, $packages)) {
          $packages[] = $details->package_title;
        }
      }

      return response()->json(['packages' => $packages, "package_duration" => $package_duration], 200);
    } else {
      return  response()->json(["error" => "data not here"], 500);
    }
  }

  private function calculateEndDateAndDays($startDate, $planType, $numberOfUnits)
  {
    $startDateObj = Carbon::createFromFormat('Y-m-d', $startDate);
    $endDateObj = $startDateObj->copy(); // Create a copy for calculating end date

    switch ($planType) {
      case 'Month':
        $endDateObj->addMonths($numberOfUnits);
        break;
      case 'Years':
      case 'Year':
        $endDateObj->addYears($numberOfUnits);
        // Handle leap year (add an extra day for each leap year)
        $numberOfLeapYears = intval($endDateObj->diffInYears($startDateObj) / 4); // Count leap years
        $endDateObj->addDays($numberOfLeapYears); // Add extra days for leap years
        break;
      case 'Months':
        $endDateObj->addMonths($numberOfUnits);
        break;
      case 'Days':
        $endDateObj->addDays($numberOfUnits);
        break;
      default:
        echo 'Invalid plan type';
        return null;
    }

    // Calculate total number of days
    $numberOfDays = $endDateObj->diffInDays($startDateObj);

    // Adjust the end date to be the last day of the subscription period
    $endDateObj->subDay(); // Subtract 1 day to get the last day of the period
    $endDate = $endDateObj->format('Y-m-d');

    return ['endDate' => $endDate, 'numberOfDays' => $numberOfDays];
  }


  public function StoreSubscriptionData(Request $request)
  {
    DB::beginTransaction();

    try {
      // Decode JSON strings
      $subscriptionDetails = json_decode($request->input('subscription_details'), true);
      $addOns = json_decode($request->input('add_ons'), true);
      $calculationData = $subscriptionDetails["calculationData"];
      $perRemaningAmount = 0;
      $totalAmount = 0;
      $cartAmount = 0;

      if ($calculationData['status']) {
        $perRemaningAmount = $calculationData['remaningUpgradedAmount'];
        $totalAmount = $calculationData['totalUpgradeAmount'];
      } else if ($subscriptionDetails['openTrailPlanInfo']) {
        $totalAmount = 1;
      } else {
        $totalAmount = $subscriptionDetails['subs_price'] * $subscriptionDetails['totalSeat'];
      }

      // Check and update the current subscription plan status
      $licenseNo = $request->input('business_id');
      $checkSubscription = Salesubscription::where('user_id', $licenseNo)->where('status', 1)->first();
      if ($checkSubscription) {
        $checkSubscription->status = 0;
        $checkSubscription->save();
      }

      // Extract data from the decoded subscription details
      $userID = $request->input('business_id');
      $totalAmountPayble = $request->input('totalPaybleAmount');
      $PlanName = $subscriptionDetails['PlanName'];
      $PlanDuration = $subscriptionDetails['openTrailPlanInfo'] ? $subscriptionDetails['trailPlanInfo'] : $subscriptionDetails['PlanDuration'];
      $PlansProductId = $subscriptionDetails['packagecode'];
      $plansPrice = $subscriptionDetails['subs_price'];

      $month = explode(" ", $PlanDuration);
      if ($month[1] == "Days" || $month[1] == "days") {
        $startDate = Carbon::today()->toDateString();
        $startDateObj = Carbon::createFromFormat('Y-m-d', $startDate);
        $futureDate = $startDateObj->addDays($month[0]);
        $endDate = $futureDate->format('Y-m-d');
        $noOfDays = $month[0];
      } else {
        $startDate = Carbon::today()->toDateString();
        $result = $this->calculateEndDateAndDays($startDate, $month[1], $month[0]);
        $endDate = $result['endDate'];
        $noOfDays = $result['numberOfDays'];
      }

      $saveData = new Salesubscription();
      $saveData->user_id = $userID;
      $saveData->plan_id = $PlansProductId;
      $saveData->startdate = $startDate;
      $saveData->enddate = $endDate;
      $saveData->package_type = $PlanName;
      $saveData->counter = $noOfDays;
      $saveData->discountOnPlan = $subscriptionDetails['sub_discount'];
      $saveData->no_of_seats = $subscriptionDetails['totalSeat'];
      $saveData->plan_price = $plansPrice;
      $saveData->total_amount = $totalAmount;
      $saveData->CartAmount = $subscriptionDetails['remaningCartAmount'];
      $saveData->AddOn_itemAmount =  $subscriptionDetails['totalAddOnItemAmount'];
      $saveData->plan_gst = $subscriptionDetails['tax_rate'];
      $saveData->paid_amount = $totalAmountPayble;
      $saveData->pre_remaning_amount = $perRemaningAmount;
      $saveData->plan_action = $subscriptionDetails['upgradePlandInfo'];
      $saveData->status = 1;

      // Store Add-Ons
      $this->storeAddOn($addOns, $licenseNo);
      $this->addAllPermissions($addOns, $licenseNo, $PlanName);

      if ($saveData->save()) {
        DB::commit();
        return response()->json([
          'status' => true,
          'message' => 'Plans activated successfully.',
          'id' => $saveData->id,
        ], 200);
      } else {
        DB::rollBack();
        return response()->json([
          'status' => false,
          "error" => "Failed to save subscription data"
        ], 500);
      }
    } catch (Exception $e) {
      DB::rollBack();
      return response()->json([
        'status' => false,
        "error" => $e->getMessage()
      ], 500);
    }
  }

  private function addAllPermissions($addOn, $licenseNo, $planName)
  {
    // Replace spaces and hyphens with underscores in the planName
    $planName = str_replace([' ', '-'], '_', $planName);

    // Fetch the main permission for the given planName from SubscriptionPermission
    $subscriptionPlanMainPermissions = SubscriptionPermission::select($planName)->value($planName);

    $groupedPermissions = [];

    // Iterate through each addOn item to get their permissions
    if (!empty($addOn)) {
      foreach ($addOn as $item) {
        $addOnItemId = $item['id'];
        $permissionWithAddOnItem = AddOnItem::find($addOnItemId);

        if (!empty($permissionWithAddOnItem)) {
          $addOnMainPermissionName = AddOnPermission::find($permissionWithAddOnItem->permission_id);

          if (!empty($addOnMainPermissionName)) {
            $selectedPermission = $addOnMainPermissionName->main_permissions;
            $allSelectedPermissions = AddOnPermission::where('main_permissions', $selectedPermission)->get();

            if (!empty($allSelectedPermissions)) {
              foreach ($allSelectedPermissions as $permission) {
                $mainPermission = $permission->main_permissions;
                $subPermission = $permission->sub_permissions;
                $partPermission = $permission->part_permissions;

                if (!isset($groupedPermissions[$mainPermission])) {
                  $groupedPermissions[$mainPermission] = [];
                }

                if ($subPermission) {
                  if (!isset($groupedPermissions[$mainPermission][$subPermission])) {
                    $groupedPermissions[$mainPermission][$subPermission] = [];
                  }

                  if ($partPermission) {
                    $groupedPermissions[$mainPermission][$subPermission][$partPermission] = "1";
                  } else {
                    $groupedPermissions[$mainPermission][$subPermission] = "1";
                  }
                } else {
                  if ($partPermission) {
                    $groupedPermissions[$mainPermission][$partPermission] = "1";
                  } else {
                    $groupedPermissions[$mainPermission] = "1";
                  }
                }
              }
            }
          }
        }
      }
    } else {
      // find the already addom items permission
      $addOnPermission =  BusinessSubscriptionPermission::where('business_id', $licenseNo)->value('addOn_permissions');
      $groupedPermissions = $addOnPermission;
    }

    // Create a new BusinessSubscriptionPermission entry with the gathered permissions
    $businessPermission = BusinessSubscriptionPermission::updateOrCreate(
      ['business_id' => $licenseNo],
      [
        'main_permissions' => $subscriptionPlanMainPermissions,
        'addOn_permissions' => json_encode($groupedPermissions)
      ]
    );

    return $businessPermission;
  }


  private function storeAddOn($addOns, $licenseNo)
  {
    try {
      // Ensure addOns is an array
      if (!is_array($addOns)) {
        throw new Exception("Invalid add-ons data.");
      }

      if (!empty($addOn)) {
        return;
      }
      // before create we want to change the stauts of the last actives addons
      $activeAddOns = UserAddOnItem::where('user_id', $licenseNo)->where('status', 1)->first();

      if (!empty($activeAddOns)) { // get the old addons items
        $activeAddOns->update(['status' => 0]);
        // now get the oldActiveAddonItems 
        $oldActiveAddonItems = $activeAddOns->add_ons;
        // now compare the addOnitem
        foreach ($oldActiveAddonItems as $item => $itemValues) {
          if (!empty($addOns)) {
            array_push($addOns, $itemValues);
          } else {
            $addOns[] = $itemValues;
          }
        }
      }

      // remove duplicate entry
      $addOns = $this->removeDuplicatesById($addOns);

      // Create a new AddOnItem
      $addOn = UserAddOnItem::create([
        'add_ons' => $addOns,
        'user_id' => $licenseNo,
        'status' => 1,
      ]);
      return true;
    } catch (Exception $e) {
      throw new Exception($e->getMessage());
    }
  }

  private function removeDuplicatesById($addOns)
  {
    $uniqueAddOns = [];
    $ids = [];

    foreach ($addOns as $addOn) {
      if (!in_array($addOn['id'], $ids)) {
        $uniqueAddOns[] = $addOn;
        $ids[] = $addOn['id'];
      }
    }

    return $uniqueAddOns;
  }


  private function checkLastSubscriptionUpdate()
  {
    try {
      $liseneceNo = session("liseneceNo");
      $subscription = Salesubscription::where("user_id", $liseneceNo)
        ->whereNot('counter', '<', 30)
        ->where('status', 1)
        ->first();
      if (!empty($subscription)) {
        $lastUpdateDate = Carbon::parse($subscription->updated_at);
        $oneMonthAgo = Carbon::now()->subMonth();
        // Check if the last update was more than a month ago
        return $lastUpdateDate < $oneMonthAgo;
      } else {
        return true;
      }
    } catch (Exception $e) {
      return false;
    }
  }

  private function checkSubscriptionPlanIsUpgradeOrNot($oldPlanCode, $newPlanName)
  {
    try {
      // Define the allowed upgrade paths
      $allowedUpgrades = [
        "S_mart_Basic" => ["S_mart_Silver", "S_mart_Gold", "S_mart_Platinum", "S_mart_Premium"],
        "S_mart_Silver" => ["S_mart_Gold", "S_mart_Platinum", "S_mart_Premium"],
        "S_mart_Gold" => ["S_mart_Platinum", "S_mart_Premium"],
        "S_mart_Platinum" => ["S_mart_Premium"],
        "S_mart_Premium" => [], // No upgrades available from premium
      ];

      // Map display names to internal names
      $inputFields = [
        "S-mart Basic" => "S_mart_Basic",
        "S-mart Silver" => "S_mart_Silver",
        "S-mart Gold" => "S_mart_Gold",
        "S-mart Platinum" => "S_mart_Platinum",
        "S-mart Premium" => "S_mart_Premium",
      ];

      if ($oldPlanCode !== null) {
        // Get the old plan name based on the old plan code
        $oldPlanName = Subscription::where('packagecode', $oldPlanCode)->value('package_category');

        if (!empty($oldPlanName) && isset($inputFields[$oldPlanName], $inputFields[$newPlanName])) {
          // Internal names of the old and new plans
          $oldPlanInternal = $inputFields[$oldPlanName];
          $newPlanInternal = $inputFields[$newPlanName];

          // Check if the new plan name is in the allowed upgrades for the old plan
          if (isset($allowedUpgrades[$oldPlanInternal]) && in_array($newPlanInternal, $allowedUpgrades[$oldPlanInternal])) {
            return true;
          }
        }
      }
    } catch (Exception $e) {
      return false;
    }

    return false;
  }


  public function getSubscriptionAmout(Request $request)
  {
    try {

      $data = $request->all();
      $subscriptionDetails = [];
      $calculatationData = [];
      $plansDetailsData = json_decode($data['planDetails'], true);
      $planName = $plansDetailsData['planName'];
      $duration = $plansDetailsData['duration'];
      $plan = $plansDetailsData['plan'];
      $currentPlan = $data['currentPlan'];
      $registerSeats = $data['registerSeats'];
      $wantToUpgradeSeats = $data['wantToupgradeSeats'];
      $upgradePlandInfo = $data['upgradePlan'];
      $calculatationData = [];
      $openTrialPlanDetails = false;
      $trailPlanInfo = null;

      $getsubscriptiondata = Subscription::where("package_category", $planName)
        ->where("package_title", $plan)
        ->where("package_duration", $duration)
        ->first();

      $totalSeats = $registerSeats + $wantToUpgradeSeats;

      // $this->checkSubscriptionPlanIsUpgradeOrNot($currentPlan, $getsubscriptiondata->package_category);

      if ($totalSeats > 25 &&  $planName === "S-mart Basic") {
        return redirect()->back()->with('error', 'Plase select another plan, you have above 25 seats you can go with another plans.');
      }

      if ($currentPlan !== null) {
        // check if the user have same plans or not have any plans
        if ($getsubscriptiondata) {
          if (($currentPlan === $getsubscriptiondata->packagecode) && ($wantToUpgradeSeats != null)) {
            //if user want to upgrade seats then calculate amount based on number of seats
            $calculatationData = $this->calculateAmountForSeatUpgrade($wantToUpgradeSeats, $getsubscriptiondata->subs_price);
          } else {
            if ($this->checkSubscriptionPlanIsUpgradeOrNot($currentPlan, $getsubscriptiondata->package_category)) {
              if ($this->checkLastSubscriptionUpdate()) {
                $calculatationData = $this->calculateAmountForPlanUpgrade($totalSeats, $planName, $duration, $plan);
              } else {
                return redirect()->back()->with('error', 'You can updagrade your plan after 1 month');
              }
            } else {
              return redirect()->back()->with('error', 'Please upgrade your Valid Plan.');
            }
          }
        }
      } else {
        
        // getTrailPlans Details 
        $tralPlanDetails = Subscription::where("package_title", "Trial")->where('status', 1)->first();
        if ($tralPlanDetails) {
          $openTrialPlanDetails = true;
          $trailPlanInfo = $tralPlanDetails->package_duration;
        }
      }
      if (empty($calculatationData)) {
        $calculatationData = [
          'status' => false
        ];
      }

      $subscriptionDetails = [
        "PlanName" => $getsubscriptiondata->package_category,
        "PlanDuration" => $getsubscriptiondata->package_duration,
        "PlanType" => $getsubscriptiondata->package_title,
        "subscription_price" => round($getsubscriptiondata->subscription_price, 2),
        "sub_discount" => $getsubscriptiondata->sub_discount,
        "subs_price" => round($getsubscriptiondata->subs_price, 2),
        "tax_rate" => $getsubscriptiondata->tax_rate,
        "total_price" => round($getsubscriptiondata->total_price, 2),
        "packagecode" =>  $getsubscriptiondata->packagecode,
        'calculationData' => $calculatationData,
        'totalSeat' => $totalSeats,
        'upgradePlandInfo' => $upgradePlandInfo,
        'openTrailPlanInfo' => $openTrialPlanDetails,
        'trailPlanInfo' => $trailPlanInfo,
      ];

      // add on plans details
      $addOnItems = AddOnItem::with('permission')->where('plan_duration', $getsubscriptiondata->package_duration)->get();
      //check AddonItems if have
      $alreadyHaveAddOnItems = $this->checkAddOnItems();
      return view('fronted.subscriptionCheckOut', compact('subscriptionDetails', 'addOnItems', 'alreadyHaveAddOnItems'));
    } catch (Exception $e) {
      return redirect()->back()->with('error', 'Something went wrong.');
    }
  }


  private function checkAddOnItems()
  {
    $lisenceNo = session("liseneceNo");
    $addOnITems = UserAddOnItem::where('user_id', $lisenceNo)->where('status', 1)->first();
    return $addOnITems;
  }

  public function index(Request $request)
  {
    config(['database.default' => env('DB_CONNECTION')]);
    $package_title = $request['plan'];
    $package_duration = $request['duration'];
    $getsubscriptiondata = Subscription::where("status", 1)
      ->where('package_title', $package_title)
      ->where('package_duration', $package_duration)
      ->orWhere('package_title', 'trial')
      ->orderBy('subs_price', 'asc')
      ->get();
    $trialPlanDetails = Subscription::where('status', 1)
      ->where('package_title', 'trial')
      ->first();
    $planDescriptionTitle = [
      "S-mart Basic" => "Quick Start",
      "S-mart Silver" => "Value For Many",
      "S-mart Gold" => "Most Populer",
      "S-mart Platinum" => "Extreemly Popular",
      "S-mart Premium" => "Premium Plan",
    ];
    $selectedPlandMessage = "Upgrade seats and  duration with same plan from here.";
    $plansFooterMessages = [
      "S-mart Basic" => "Change Your plan in S-mart Basic with same selection of current plan and also upgrade seats, duration and type of plan from here.",
      "S-mart Silver" => "Change Your plan in S-mart Silver with same selection of current plan and also upgrade seats, duration and  type of plan from here.",
      "S-mart Gold" => "Change Your plan in S-mart Gold with same selection of current plan and also upgrade seats, duration and  type of plan from here.",
      "S-mart Platinum" => "Change Your plan in S-mart Platinum with Premium selection of current plan and also upgrade seats, duration and  type of plan from here.",
      "S-mart Premium" => "Change Your plan in S-mart Premium with same selection of current plan and also upgrade seats, duration and  type of plan from here.",
    ];
    $plansData = [];
    if ($getsubscriptiondata) {
      foreach ($getsubscriptiondata as $data) {
        $planObject = [
          'PackageCategory' => $data->package_category,
          'subs_price' => $data->subs_price,
          'selectedPlandMessage' => $selectedPlandMessage,
          'plansFooterMessages' => $plansFooterMessages,
          'planTitel' => $planDescriptionTitle[$data->package_category],
          'subscription_price' => $data->subscription_price,
          'sub_discount' => $data->sub_discount . "% off",
          'features' =>  $data->subscription_features,
          'package_title' => $data->package_title,
          'package_duration' => $data->package_duration,
          "packagecode" =>  $data->packagecode,

        ];
        // Use the subscription ID ($data->id) as the key for the $plansData array
        $plansData[] = $planObject;
      }
    }
    config(['database.default' => "custom_db"]);
    return response()->json([
      'plansDetails' => $plansData,
      'trailPlanDetails' => $trialPlanDetails,
    ]);
  }

  public function getAllSubscriptionPlansDetails()
  {
    try {
      $subscriptionData = [];
      $planDescriptionTitle = [
        "S-mart Basic" => [
          'planTitle' => "Quick Start",
          'backgroundColor' => '#92cddb',
          'borderColor' => '#92cddf',
          'footerBackgroundColor' => '#dbeef4',
        ],
        "S-mart Silver" => [
          'planTitle' => "Value For Many",
          'backgroundColor' => '#d8d8d8',
          'borderColor' => '#a3ad89',
          'footerBackgroundColor' => "#f2f2f2",
        ],
        "S-mart Gold" => [
          'planTitle' => "Most Populer",
          'backgroundColor' => '#fcf276',
          'borderColor' => '#ffd83a',
          'footerBackgroundColor' => "#fafad4",
        ],
        "S-mart Platinum" => [
          'planTitle' => "Extreemly Popular",
          'backgroundColor' => '#fcf276',
          'borderColor' => '#ffd83a',
          'footerBackgroundColor' => "#fff",
        ],
        "S-mart Premium" => [
          'planTitle' => "Premium Plan",
          'backgroundColor' => '#fcf276',
          'borderColor' => '#ffd83a',
          'footerBackgroundColor' => "#fff",
        ],
      ];
      $getsubscriptiondata = Subscription::where("status", 1)
        ->orderBy('subs_price', 'asc')
        ->whereNotIn('package_duration', ['7 Days', '15 Days'])
        ->groupBy('package_category')
        ->get();

      if (empty($getsubscriptiondata)) {
        throw new Exception("Subscription Modal are not created yet");
      }

      foreach ($getsubscriptiondata as $key => $value) {
        $subscriptionData[] = [
          'styleAndTitel' => $planDescriptionTitle[$value->package_category],
          'planName' => $value->package_category,
          'perBedAmount' => $value->subscription_price,
          'perBedDisAmount' => $value->subs_price,
          'discount' => $value->sub_discount,
          'features' => $this->makeSubscriptionFeaturesModify($value->subscription_features),
        ];
      }
      return response()->json([
        'message' => 'Successfully Get All Subscription Plans Details!',
        'code' => 200,
        'data' => $subscriptionData,
      ], 200);
    } catch (Exception $e) {
      return response()->json([
        'error' => $e->getMessage()
      ], 500);
    }
  }

  private function makeSubscriptionFeaturesModify($allFeatures)
  {
    if (!is_array($allFeatures) || count($allFeatures) <= 6) {
      return $allFeatures; // No modification needed
    }

    // Extract first 5 features and last feature
    $firstFive = array_slice($allFeatures, 0, 5);
    $lastFeature = end($allFeatures);

    // Combine first five features with the last one
    $Features = array_merge($firstFive, [$lastFeature]);

    return $Features;
  }

  public function store(Request $request)
  {
    $data = new Subscription();
    //$data->packagecode = (string) Str::uuid();
    $data->package_category = $request->package_category;
    $data->package_title = $request->package_title;
    $data->package_duration = $request->package_duration;
    $data->subscription_price = $request->subscription_price;
    $data->sub_discount = $request->sub_discount;
    $data->subs_price = $request->subs_price;
    $selectedFeatures = $request->input('features', []);
    $commaSeparatedFeatures = implode(',', $selectedFeatures);
    $data->features = $commaSeparatedFeatures;

    // Generate a unique ID using UUID
    $uniqueId = (string) Str::uuid();

    // Combine the custom prefix, unique ID, and "001"
    $data->packagecode = 'smart' . $uniqueId;
    if ($data->save()) {
      return back()->with('success', 'success');
    } else {
      return back()->with("error", "Subscritio");
    }
  }

  public function feature(Request $request)
  {
    $data = new Feature();
    $data->feature = $request->feature;
    $data->save();
    return back()->with('feature', 'save feature addded');
  }
  public function destroy($id)
  {
    $data = Subscription::find($id);
    $data->delete();
    return back()->with('destroy', 'delete successfully');
  }
  public function featuredelete($id)
  {

    $data = Feature::find($id);
    $data->delete();
    return back()->with('delete', 'delete successfully');
  }


  private function calculateAmountForSeatUpgrade($newUpgradeSeats, $plansPrice)
  {
    try {
      $liseneceNo = session("liseneceNo");
      // Retrieve subscription data
      $subscription = Salesubscription::where("user_id", $liseneceNo)->where('status', 1)->firstOrFail();

      // Extract relevant subscription data
      $oldSeats = $subscription->no_of_seats;
      $oldPlanAmount = $subscription->total_amount;
      $oldPlanAmountRemaning = 0;
      $addOnItemAmount = $subscription->AddOn_itemAmount;
      $planGstPaid = $subscription->plan_gst;
      $paidAmount = $subscription->paid_amount - $addOnItemAmount;
      $removeGstAmount = ($paidAmount * $planGstPaid) / 100;
      $paidAmount -= $removeGstAmount;
      $planStartDate = Carbon::parse($subscription->startdate);
      $planEndDate = Carbon::Parse($subscription->enddate);
      $prSetAmount = $subscription->plan_price;

      // Calculate total days in the subscription plan
      $totalDays = $planStartDate->diffInDays($planEndDate) + 1; // Including both start and end dates

      // Calculate days used in the subscription plan
      $daysUsed = $planStartDate->diffInDays(now()) + 1; // Including current day

      // Calculate remaining days in the subscription plan
      $remainingDays = max(0, $totalDays - $daysUsed);
      //calculate per day amount 
      $prdayAmount = $paidAmount / $totalDays;
      // calculate cost of used day
      $usedCost = round(($prdayAmount * $daysUsed), 2);
      //calculate reamanign amount 
      $remaningAmontOfplans = round(($paidAmount - $usedCost), 2) < 0 ? 0 : round(($paidAmount - $usedCost), 2);

      // oldPlan Remaning amount 
      $oldPrDayAmount = $oldPlanAmount / $totalDays;
      $oldUsedCost = round(($oldPrDayAmount * $daysUsed), 2);
      $oldPlanAmountRemaning = round(($oldPlanAmount - $oldUsedCost), 2);

      // calcualte  new amount for total seat (old + new);
      $totalSeats = $oldSeats + $newUpgradeSeats;
      $newAmount = round(($totalSeats * $plansPrice), 2);
      $cartAmount = 0;
      $payAbleAmountForUpgrade = round(($newAmount - $remaningAmontOfplans), 2);

      if ($payAbleAmountForUpgrade < 0) {
        $cartAmount = $remaningAmontOfplans - $newAmount;
        $payAbleAmountForUpgrade = 0;
      }

      return [
        "status" => true,
        "totalUpgradedSeats" => $totalSeats,
        "totalUpgradeAmount" => round($newAmount, 2),
        "remaningUpgradedAmount" => round($oldPlanAmountRemaning, 2),
        'payAbleAmountForUpgrade' => round(($newAmount - $oldPlanAmountRemaning), 2),
        'cartAmount' => $cartAmount,
      ];
    } catch (Exception $e) {
      return ["status" => false, "message" => $e->getMessage()];
    }
  }


  private function calculateAmountForPlanUpgrade($registerSeats, $planName, $duration, $plan)
  {
    try {
      $liseneceNo = session("liseneceNo");
      // Retrieve subscription data
      config(['database.default' => env('DB_CONNECTION')]);
      $subscription = Salesubscription::where("user_id", $liseneceNo)->where('status', 1)->firstOrFail();
      $UpgradePlanDetails = Subscription::where('package_category', $planName)->where('package_title', $plan)->where('package_duration', $duration)->where("status", 1)->first();
      $upgradePlanPrice = $UpgradePlanDetails->subs_price;
      $oldPlanAmount = $subscription->total_amount;
      $oldPlanAmountRemaning = 0;
      // Extract relevant subscription data
      $addOnItemAmount = $subscription->AddOn_itemAmount;
      $planGstPaid = $subscription->plan_gst;
      $paidAmount = $subscription->paid_amount - $addOnItemAmount;
      $removeGstAmount = ($paidAmount * $planGstPaid) / 100;
      $paidAmount -= $removeGstAmount;
      $planStartDate = Carbon::parse($subscription->startdate);
      $planEndDate = Carbon::Parse($subscription->enddate);
      // Calculate total days in the subscription plan
      $totalDays = $planStartDate->diffInDays($planEndDate) + 1; // Including both start and end dates

      // Calculate days used in the subscription plan
      $daysUsed = $planStartDate->diffInDays(now()) + 1; // Including current day

      // Calculate remaining days in the subscription plan
      $remainingDays = max(0, $totalDays - $daysUsed);
      //calculate per day amount 
      $prdayAmount = $paidAmount / $totalDays;
      // calculate cost of used day
      $usedCost = round(($prdayAmount * $daysUsed), 2);
      //calculate reamanign amount 
      $remaningAmontOfplans = round(($paidAmount - $usedCost), 2) < 0 ? 0 : round(($paidAmount - $usedCost), 2);
      // remove gst percentage amount
      // calcualte  new amount for total seat (old + new);

      // oldPlan Remaning amount 
      $oldPrDayAmount = $oldPlanAmount / $totalDays;
      $oldUsedCost = round(($oldPrDayAmount * $daysUsed), 2);
      $oldPlanAmountRemaning = round(($oldPlanAmount - $oldUsedCost), 2);

      $totalSeats = $registerSeats;
      $newAmount = round(($totalSeats * $upgradePlanPrice), 2);
      $cartAmount = 0;
      $payAbleAmountForUpgrade = round(($newAmount - $oldPlanAmountRemaning), 2);

      if ($payAbleAmountForUpgrade < 0) {
        $cartAmount = $oldPlanAmountRemaning - $newAmount;
        $payAbleAmountForUpgrade = 0;
      }
      return [
        "status" => true,
        "totalUpgradedSeats" => $totalSeats,
        "totalUpgradeAmount" => round($newAmount, 2),
        "remaningUpgradedAmount" => round($oldPlanAmountRemaning, 2),
        'payAbleAmountForUpgrade' => $payAbleAmountForUpgrade,
        'cartAmount' => $cartAmount,
      ];
    } catch (Exception $e) {
      return ["status" => false, "message" => $e->getMessage()];
    } finally {
      config(['database.default' => 'custom_db']);
    }
  }

  public function upgradeSeats(Request $request)
  {
    try {
      $liseneceNo = session("liseneceNo");
      // Retrieve subscription data
      config(['database.default' => env('DB_CONNECTION')]);
      $subscription = Salesubscription::where("user_id", $liseneceNo)->where('status', 1)->firstOrFail();
      $subscription->no_of_seats = $request->totalSeats;
      $subscription->paid_amount = $request->totalAmount;
      if ($subscription->save()) {
        return back()->with("SubscriptionAdd", "Successfully updated seats");
      } else {
        throw new Exception("Plan not Upgrade");
      }
    } catch (Exception $e) {
      return back()->with('roomerror', $e->getMessage());
    } finally {
      config(['database.default' => 'custom_db']);
    };
  }


  public function getActivePlanDetails()
  {
    try {
      $liseneceNo = session("liseneceNo");
      $plansDetails = Salesubscription::where("user_id", $liseneceNo)->where("status", 1)->first();
      if ($plansDetails) {
        return response()->json([
          'status' => true,
          'data' => $plansDetails
        ]);
      } else {
        return response()->json([
          'status' => false,
          'message' => "Plane details are not available."
        ], 406);
      }
    } catch (Exception $e) {
      return response()->json([
        'status' => false,
        'message' => 'server Error',
      ], 500);
    }
  }


  public function subscriptionPlanCompare()
  {
    try {
      $allFeatures = SubscriptionFeatures::whereNotNull('all_features')->where('all_features', '<>', '')->get();
      $S_mart_Basic = SubscriptionFeatures::whereNotNull('S_mart_Basic')
        ->where('S_mart_Basic', '<>', '')
        ->pluck('S_mart_Basic')
        ->toArray();
      $S_mart_Silver = SubscriptionFeatures::whereNotNull('S_mart_Silver')
        ->where('S_mart_Silver', '<>', '')
        ->pluck('S_mart_Silver')->toArray();

      $S_mart_Gold = SubscriptionFeatures::whereNotNull('S_mart_Gold')
        ->where('S_mart_Gold', '<>', '')
        ->pluck('S_mart_Gold')->toArray();
      $S_mart_Platinum = SubscriptionFeatures::whereNotNull('S_mart_Premium')
        ->where('S_mart_Premium', '<>', '')

        ->pluck('S_mart_Platinum')->toArray();
      $S_mart_Premium = SubscriptionFeatures::whereNotNull('S_mart_Premium')
        ->where('S_mart_Premium', '<>', '')
        ->pluck('S_mart_Premium')->toArray();

      $allPermissions = SubscriptionPermission::whereNotNull('all_permissions')->where('all_permissions', '<>', '')->get();

      $S_mart_Basic_permission = SubscriptionPermission::whereNotNull('S_mart_Basic')
        ->where('S_mart_Basic', '<>', '')
        ->pluck('S_mart_Basic')
        ->toArray();
      $S_mart_Silver_permission = SubscriptionPermission::whereNotNull('S_mart_Silver')
        ->where('S_mart_Silver', '<>', '')
        ->pluck('S_mart_Silver')->toArray();

      $S_mart_Gold_permission = SubscriptionPermission::whereNotNull('S_mart_Gold')
        ->where('S_mart_Gold', '<>', '')
        ->pluck('S_mart_Gold')->toArray();
      $S_mart_Platinum_permission = SubscriptionPermission::whereNotNull('S_mart_Premium')
        ->where('S_mart_Premium', '<>', '')

        ->pluck('S_mart_Platinum')->toArray();
      $S_mart_Premium_permission = SubscriptionPermission::whereNotNull('S_mart_Premium')
        ->where('S_mart_Premium', '<>', '')
        ->pluck('S_mart_Premium')->toArray();

      return view('fronted.compareSubscriptionPlan', compact(
        'allFeatures',
        'S_mart_Basic',
        'S_mart_Silver',
        'S_mart_Gold',
        'S_mart_Platinum',
        'S_mart_Premium',
        'allPermissions',
        'S_mart_Basic_permission',
        'S_mart_Silver_permission',
        'S_mart_Gold_permission',
        'S_mart_Platinum_permission',
        'S_mart_Premium_permission',
      ));
    } catch (Exception $e) {
      return back()->with('error', 'somthing wronge');
    }
  }

  public function finalCheckOut(Request $request)
  {
    // Validate the incoming request data
    $validatedData = $request->validate([
      'subscriptionDetails' => 'required|string',
      'addOns' => 'required|string',
      'remaningCartAmount' => 'required|numeric',
      'totalAddOnItemAmount' => 'required|numeric',
    ]);

    // Decode the JSON data
    $subscriptionDetails = json_decode($validatedData['subscriptionDetails'], true);
    $addOns = json_decode($validatedData['addOns'], true);
    $totalRowSpansValue = count($addOns);
    $subscriptionDetails['remaningCartAmount'] = $request->remaningCartAmount;
    $subscriptionDetails['totalAddOnItemAmount'] = $request->totalAddOnItemAmount;

    // Check if decoding was successful
    if (json_last_error() !== JSON_ERROR_NONE) {
      return redirect()->back()->withErrors(['error' => 'Invalid JSON data.']);
    }

    // Fetch the business register details
    $businessRegister = Businessregister::first();

    // Ensure business register details are found
    if (!$businessRegister) {
      return redirect()->back()->withErrors(['error' => 'Business register details not found.']);
    }
    // Pass data to the view
    return view('fronted.finalCheckOutSubscription', compact('subscriptionDetails', 'addOns', 'businessRegister', 'totalRowSpansValue'));
  }
}
