<?php

namespace App\Http\Controllers;

use Illuminate\Http\Request;
use Illuminate\Database\Eloquent\Model;
use DB;
use App\Models\TanantComplaint;
use Exception;
use App\Models\TanantNotice;
use Carbon\Carbon;

class TanantComplaintController extends Controller
{
    public function complaintView(Request $request)
    {
        try {
            $filterType = $request['complaint-fillter-type'];
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];
            $complaintStatus = $request['complaint'];
            if (!empty($filterType) && $filterType !== "all") {
                if ($filterType == "custom" && !empty($startDate) && !empty($endDate)) {
                    $complaints = TanantComplaint::with('student')
                        ->whereBetween('complaint_date', [$startDate, $endDate])
                        ->get();
                } else {
                    $complaints = TanantComplaint::with('student')
                        ->where('complaint_type', $filterType)
                        ->get();
                }
            } elseif (!empty($complaintStatus)) {
                $complaints = TanantComplaint::with('student')
                    ->where('action', $complaintStatus)
                    ->get();
            } else {
                $complaints = TanantComplaint::with('student')->get();
            }

            $activeComplaint = TanantComplaint::where('action', 1)->count();
            $closedComplaint = TanantComplaint::where('action', 2)->count();
            $unseenComplaint = TanantComplaint::where('action', 0)->count();
            $totalComplaint = TanantComplaint::count();

            return view('fronted.complaintboard', [
                'complaints' => $complaints,
                'totalComplaint' => $totalComplaint ?? 0,
                'activeComplaint' => $activeComplaint ?? 0,
                'closedComplaint' => $closedComplaint ?? 0,
                'unseenComplaint' => $unseenComplaint ?? 0,
                'filterType' => $filterType
            ]);
        } catch (Exception $e) {
            return view('fronted.complaintboard')->with('error', $e->getMessage());
        }
    }


    public function updateComplaintAction(Request $request, $complaintId)
    {
        try {
            $complaint = TanantComplaint::findOrFail($complaintId);

            $complaint->action = $request->action;
            $complaint->update();
            return response()->json(['message' => 'Action updated successfully'], 200);
        } catch (Exception $e) {
            return response()->json(['message' => $e->getMessage()], 500);
        }
    }

    public function giveComplaintResponse(Request $request, $complaintId)
    {
        try {
            $complaint = TanantComplaint::findOrFail($complaintId);
            $complaint->responseMsg = $request->response;
            $complaint->update();
            return back()->with(['complaintSuccess' => 'Complaint response add successfully']);
        } catch (Exception $e) {
            return back()->withErrors(['complaintError' => $e->getMessage()]);
        }
    }

    public function noticeView(Request $request)
    {
        try {
            $filterType = $request['notice-fillter-type'];
            $startDate = $request['startDate'];
            $endDate = $request['endDate'];
            $noticeStatus = $request['notice'];

            if (!empty($filterType) && $filterType !== "all") {
                if ($filterType == "custom" && !empty($startDate) && !empty($endDate)) {
                    $notices = TanantNotice::with('student')
                        ->whereBetween('noticeDate', [$startDate, $endDate])
                        ->get();
                } else {
                    $notices = TanantNotice::with('student')
                        ->where('noticeType', $filterType)
                        ->get();
                }
            } elseif (!empty($noticeStatus)) {
                $notices = TanantNotice::with('student')
                    ->where('action', $noticeStatus)
                    ->get();
            } else {
                $notices = TanantNotice::with('student')->get();
            }

            $activeNotice = TanantNotice::where('action', 1)->count();
            $closedNotice = TanantNotice::where('action', 2)->count();
            $unseenNotice = TanantNotice::where('action', 0)->count();
            $totalNotice = TanantNotice::count();

            return view('fronted.noticeBoard', [
                'notices' => $notices,
                'totalNotice' => $totalNotice ?? 0,
                'activeNotice' => $activeNotice ?? 0,
                'closedNotice' => $closedNotice ?? 0,
                'unseenNotice' => $unseenNotice ?? 0,
                'filterType' => $filterType
            ]);
        } catch (Exception $e) {
            return view('fronted.noticeBoard')->with('error', $e->getMessage());
        }
    }

    public function updateNoticeAction(Request $request, $noticeId)
    {
        try {
            $TanantNotice = TanantNotice::findOrFail($noticeId);

            $TanantNotice->action = $request->action;
            $TanantNotice->update();
            return response()->json(['message' => 'Action updated successfully'], 200);
        } catch (Exception $e) {
            return response()->json(['message' => $e->getMessage()], 500);
        }
    }

    public function giveNoticeResponse(Request $request, $noticeId)
    {
        try {
            $notice = TanantNotice::findOrFail($noticeId);
            $notice->responseMsg = $request->response;
            $notice->update();
            return back()->with(['noticeSuccess' => 'Notice response add successfully']);
        } catch (Exception $e) {
            return back()->with(['noticeError' => $e->getMessage()]);
        }
    }

    public function genrateComplainBoardGraph(Request $request)
    {
        try {
            $filterKeyword = $request['type'];
            if ($filterKeyword == 'year') {
                $query = TanantComplaint::select(DB::raw('YEAR(complaint_date) AS year_num'), DB::raw('COUNT(*) AS total_complaints'))
                    ->groupBy(DB::raw('YEAR(complaint_date)'))
                    ->orderBy(DB::raw('YEAR(complaint_date)'))
                    ->get();

                $labels = [];
                $totalComplaints = [];

                foreach ($query as $row) {
                    $yearNum = $row->year_num;
                    if (!in_array($yearNum, $labels)) {
                        $labels[] = $yearNum;
                        $totalComplaints[$yearNum] = round($row->total_complaints);
                    } else {
                        $totalComplaints[$yearNum] += round($row->total_complaints);
                    }
                }

                // Sort the labels array
                sort($labels);

                // Fill in missing years with 0 total sales
                foreach ($labels as $year) {
                    if (!isset($totalComplaints[$year])) {
                        $totalComplaints[$year] = 0;
                    }
                }
                // Sort the totalComplaints array by keys (years)
                ksort($totalComplaints);
            } elseif ($filterKeyword == 'quarter') {
                $currentYear = date('Y');
                $currentMonth = date('m');
                if ($currentMonth < 4) {
                    $startDate = ($currentYear - 1) . "-04-01";
                    $endDate = $currentYear . "-03-31";
                } else {
                    $startDate = $currentYear . "-04-01";
                    $endDate = ($currentYear + 1) . "-03-31";
                }

                $query = TanantComplaint::select(DB::raw('QUARTER(complaint_date) AS quarter_num'), DB::raw('COUNT(*) AS total_complaints'))
                    ->whereBetween('complaint_date', [$startDate, $endDate])
                    ->groupBy(DB::raw('QUARTER(complaint_date)'))
                    ->orderBy(DB::raw('QUARTER(complaint_date)'))
                    ->get();

                $totalComplaints = array_fill(1, 4, 0); // Initialize all quarters with zero totals
                // Populate the array with the retrieved quarterly totals
                foreach ($query as $row) {
                    $quarterNum = $row->quarter_num;
                    $totalComplaints[$quarterNum] = round($row->total_complaints);
                }
                $labels = ['Q1', 'Q2', 'Q3', 'Q4'];
            } elseif ($filterKeyword == 'month') {
                $currentMonth = date('m');
                $currentDay = date('d');

                $query = TanantComplaint::select(DB::raw('DATE_FORMAT(complaint_date, "%d") AS days'), DB::raw('COUNT(*) AS total_complaints'))
                    ->whereMonth('complaint_date', '=', $currentMonth)
                    ->whereDay('complaint_date', '<=', $currentDay)
                    ->get();

                $totalDays = Carbon::now()->daysInMonth;
                $totalComplaints = [];

                for ($day = 1; $day <= $totalDays; $day++) {
                    $labels[] = $day;
                    $totalComplaints[$day] = 0;
                }

                foreach ($query as $complaint) {
                    $day = intval($complaint->days);
                    $totalComplaints[$day] = ($complaint->total_complaints);
                }


            } elseif ($filterKeyword == "Months") {
                $currentYear = date('Y');
                $currentMonth = date('m');
                if ($currentMonth < 4) {
                    $startDate = ($currentYear - 1) . "-04-01";
                    $endDate = $currentYear . "-03-31";
                } else {
                    $startDate = $currentYear . "-04-01";
                    $endDate = ($currentYear + 1) . "-03-31";
                }
                $complaints = TanantComplaint::select(
                    DB::raw('YEAR(complaint_date) as year_num'),
                    DB::raw('MONTH(complaint_date) AS month_num'),
                    DB::raw('COUNT(*) AS total_complaints')
                )
                    ->whereBetween('complaint_date', [$startDate, $endDate])
                    ->groupBy(DB::raw('YEAR(complaint_date)'), DB::raw('MONTH(complaint_date)'))
                    ->get();

                $labels = [];
                $totalComplaints = [];
                $data = [];

                // Initialize arrays for 12 months
                for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                    $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                    $totalComplaints[$monthNum] = 0;
                }

                // Populate total complaints array from the query result
                foreach ($complaints as $complaint) {
                    $monthNum = intval($complaint->month_num);
                    $totalComplaints[$monthNum] = $complaint->total_complaints;
                }
            } elseif ($filterKeyword == 'custom') {
                $startDate = $request['startDate'];
                $endDate = $request['endDate'];

                $complaints = TanantComplaint::select(
                    DB::raw('YEAR(complaint_date) as year_num'),
                    DB::raw('MONTH(complaint_date) AS month_num'),
                    DB::raw('COUNT(*) AS total_complaints')
                )
                    ->whereBetween('complaint_date', [$startDate, $endDate])
                    ->groupBy(DB::raw('YEAR(complaint_date)'), DB::raw('MONTH(complaint_date)'))
                    ->get();

                $labels = [];
                $totalComplaints = [];
                $data = [];

                // Initialize arrays for 12 months
                for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                    $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                    $totalComplaints[$monthNum] = 0;
                }

                // Populate total complaints array from the query result
                foreach ($complaints as $complaint) {
                    $monthNum = intval($complaint->month_num);
                    $totalComplaints[$monthNum] = $complaint->total_complaints;
                }
            } else {
                $complaints = TanantComplaint::select(
                    DB::raw('YEAR(complaint_date) as year_num'),
                    DB::raw('MONTH(complaint_date) AS month_num'),
                    DB::raw('COUNT(*) AS total_complaints')
                )
                    ->groupBy(DB::raw('YEAR(complaint_date)'), DB::raw('MONTH(complaint_date)'))
                    ->get();

                $labels = [];
                $totalComplaints = [];
                $data = [];

                // Initialize arrays for 12 months
                for ($monthNum = 1; $monthNum <= 12; $monthNum++) {
                    $labels[] = date("M", mktime(0, 0, 0, $monthNum, 1));
                    $totalComplaints[$monthNum] = 0;
                }

                // Populate total complaints array from the query result
                foreach ($complaints as $complaint) {
                    $monthNum = intval($complaint->month_num);
                    $totalComplaints[$monthNum] = $complaint->total_complaints;
                }
            }
            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => [
                    'labels' => $labels,
                    'data' => array_values($totalComplaints),
                ]
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }

    public function compaintGraphCategoryType(Request $request)
    {
        try {
            $filterKeyword = $request['type'];

            if ($filterKeyword == "year") {
                $complaintsByType = TanantComplaint::select('complaint_type', DB::raw('COUNT(*) AS count'))
                    ->groupBy('complaint_type')
                    ->get();
            } elseif ($filterKeyword == "quarter") {
                $currentYear = date('Y');
                $currentMonth = date('m');
                if ($currentMonth < 4) {
                    $startDate = ($currentYear - 1) . "-04-01";
                    $endDate = $currentYear . "-03-31";
                } else {
                    $startDate = $currentYear . "-04-01";
                    $endDate = ($currentYear + 1) . "-03-31";
                }

                $complaintsByType = TanantComplaint::select('complaint_type', DB::raw('COUNT(*) AS count'))
                    ->whereBetween('complaint_date', [$startDate, $endDate])
                    ->groupBy('complaint_type')
                    ->get();
            } elseif ($filterKeyword == "month") {
                $currentMonth = date('m');
                $currentDay = date('d');

                $complaintsByType = TanantComplaint::select(DB::raw('complaint_type'), DB::raw('COUNT(*) AS count'))
                    ->whereMonth('complaint_date', '=', $currentMonth)
                    ->whereDay('complaint_date', '<=', $currentDay)
                    ->groupBy('complaint_type')
                    ->get();
            } elseif ($filterKeyword == "Months") {
                $currentYear = date('Y');
                $currentMonth = date('m');
                if ($currentMonth < 4) {
                    $startDate = ($currentYear - 1) . "-04-01";
                    $endDate = $currentYear . "-03-31";
                } else {
                    $startDate = $currentYear . "-04-01";
                    $endDate = ($currentYear + 1) . "-03-31";
                }
                $complaintsByType = TanantComplaint::select(
                    'complaint_type',
                    DB::raw('COUNT(*) AS count')
                )
                    ->whereBetween('complaint_date', [$startDate, $endDate])
                    ->groupBy('complaint_type')
                    ->get();
            } elseif ($filterKeyword == "custom") {
                $startDate = $request['startDate'];
                $endDate = $request['endDate'];
                $complaintsByType = TanantComplaint::select(
                    'complaint_type',
                    DB::raw('COUNT(*) AS count')
                )
                    ->whereBetween('complaint_date', [$startDate, $endDate])
                    ->groupBy('complaint_type')
                    ->get();
            } else {
                $complaintsByType = TanantComplaint::select('complaint_type', DB::raw('COUNT(*) as count'))
                    ->groupBy('complaint_type')
                    ->get();
            }
            $labels = [];
            $data = [];
            foreach ($complaintsByType as $key => $complaint) {
                $data[] = [
                    'label' => $complaint->complaint_type,
                    'value' => $complaint->count,
                ];
            }
            return response()->json([
                'status' => true,
                'message' => "Data Loaded Successfully",
                'data' => $data
            ], 200);
        } catch (Exception $e) {
            return response()->json([
                'status' => false,
                'error' => $e->getMessage()
            ], 500);
        }
    }
}
