<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Support\Facades\DB;
use Illuminate\Support\Facades\Log;
use App\Models\ApiModels\UserAuthTokens;
use App\Models\ApiModels\BussinessModel;

class AuthenticateToken
{
    protected $except = [
        'api/hostelname',
        'api/login',
        'api/businessRegister',
        'api/checkBusinessEmail/*',
        'api/checkBusinessMobile/*',
        'api/checkBusinessName/*',
        "api/addSubscription",
        "api/logoutUserSession/*",
        'api/getSubscriptionDetails',
        'api/forgetUserPassword/*',
        'api/checkToken',
        'api/getRegisterHostelList',
        'api/studentSingUp',
        'api/createStudentPassword/*',
        'api/studentLogin',
        'api/forgetStudentPassword',
        'api/verifyForgetPasswordOtp',
        'api/resetStudentPassword/*',
        'api/checkStudentPasswordIsCreatedOrNot/*',
    ];

    public function handle($request, Closure $next)
    {
        $token = $request->header('Authorization');
        if ($this->shouldPassThrough($request)) {
            return $next($request);
        }
        if (!$token) {
            return response()->json(['error' => "Unauthorized"], 401);
        }

        $userData = UserAuthTokens::where('token', $token)->first();
        if (!$userData) {
            return response()->json(['error' => 'Invalid token'], 401);
        }
        $licenceno = $userData->licenceno;
        $bussinessData = BussinessModel::where("license_no", $licenceno)->first();
        if ($bussinessData) {
            try {
                $connectionName = 'custom_db';
                config([
                    "database.connections.$connectionName" => [
                        'driver'   => 'mysql',
                        'host'     => $bussinessData->db_host,
                        'port'     => $bussinessData->db_port,
                        'database' => $bussinessData->db_name,
                        'username' => $bussinessData->db_username,
                        'password' => $bussinessData->db_password,
                    ],
                ]);
                DB::setDefaultConnection($connectionName);
            } catch (\Exception $e) {
                Log::error('Database connection error', ['exception' => $e->getMessage()]);
                return response()->json(['error' => 'Database connection error'], 500);
            }

            // Log successful authentication
            Log::info('User authenticated successfully', ['user_id' => $userData->id]);
        } else {
            // Log invalid token
            Log::warning('Invalid token', ['user_id' => $userData->id]);
            return response()->json(['error' => 'Invalid token'], 401);
        }

        return $next($request);
    }

    protected function shouldPassThrough($request)
    {
        foreach ($this->except as $except) {
            if ($request->is($except)) {
                return true;
            }
        }
        return false;
    }
}
