<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Contracts\Auth\Factory as Auth;
use Illuminate\Session\Store;

class AutoLogout
{
    protected $auth;
    protected $session;

    public function __construct(Auth $auth, Store $session)
    {
        $this->auth = $auth;
        $this->session = $session;
    }

    public function handle($request, Closure $next, $guard = null)
    {
        if (!$this->auth->guard($guard)->check()) {
            return $next($request);
        }

        $lastActivityTime = $this->session->get('last_activity_time');
        $idleTimeout = 600; // seconds, adjust as needed

        if ($lastActivityTime && time() - $lastActivityTime > $idleTimeout) {
            $this->auth->guard($guard)->logout();
            $request->session()->invalidate();
            $request->session()->regenerateToken();

            return redirect('/login')->with('error', 'You have been logged out due to inactivity.');
        }

        $this->session->put('last_activity_time', time());

        return $next($request);
    }
}

?>