<?php

namespace App\Http\Middleware;

use Closure;
use Illuminate\Support\Facades\Auth;
use App\Models\Salesubscription;
use App\Models\Subscription;
use App\Models\EmployeRegister;
use App\Models\BusinessSubscriptionPermission;

class CheckSubscriptionPermission
{
    public function handle($request, Closure $next, $permission)
    {
        $user = session('liseneceNo'); // Assuming 'liseneceNo' is the session key for user ID
        $defaultPermission = ['home' => 1, 'room/seat' =>1, 'Subscription' => 1];
        $permissionsArray = $defaultPermission;

        // Fetch the plan and permissions
        $plan = Salesubscription::select('plan_id')->where('user_id', $user)->where('status', 1)->first();
        if (!empty($plan)) {
            // Get permissions from the database
            $subscriptionPermissions = BusinessSubscriptionPermission::where('business_id', $user)->first();
            if ($subscriptionPermissions) {
                $mainPermission = json_decode($subscriptionPermissions->main_permissions, true);
                $addOnPermission = json_decode($subscriptionPermissions->addOn_permissions, true);
                // Merge permissions
                $permissionsArray = array_merge_recursive($mainPermission, $addOnPermission);
            }
        }
        session(['permissions' => $permissionsArray]);
        session(['mainPermissions' => $mainPermission ?? []]);
        session(['addOnPermissions' => $addOnPermission ?? []]);

        if (!empty(session('UserPermission')) && strtolower(session('userRoll')) !== "admin") {            
            if (!$this->hasPermission(session('UserPermission'), $permission)) {
                if ($request->expectsJson()) {
                    return response()->json(['error' => "Access denile: You do not have permission for this resourse by admin."], 403);
                } else {
                    return redirect()->route('subscription.details')->with('subscriptionError', "You do not have permission to access this resource by admin.");
                }
            }
        } else {
            if (!$this->hasPermission($permissionsArray, $permission)) {
                if ($request->expectsJson()) {
                    return response()->json(['error' => "Access denile: You do not have permission for this resourse."], 403);
                } else {
                    return redirect()->route('subscription.details')->with('subscriptionError', "You do not have permission to access this resource as per your plan.");
                }
            }
        }
        return $next($request);
    }

    private function hasPermission($permissionsArray, $permission)
    {

        $keys = explode('.', $permission);
        $current = $permissionsArray;
        // dd($keys,$current);  
        foreach ($keys as $key) {
            if (!isset($current[$key])) {
                return false;
            }
            $current = $current[$key];
        }

        return true;
    }
}
